Package: systemfit
Version: 1.1-30
Date: 2023-03-22
Title: Estimating Systems of Simultaneous Equations
Author: Arne Henningsen and Jeff D. Hamann
Maintainer: Arne Henningsen <arne.henningsen@gmail.com>
Depends: R (>= 3.2.0), Matrix, car (>= 2.0-0), lmtest
Suggests: knitr, plm (>= 1.0-1), sem (>= 2.0-0)
Imports: stats (>= 2.14.0), sandwich (>= 2.2-9), MASS, methods
Description: Econometric estimation of simultaneous
 systems of linear and nonlinear equations using Ordinary Least
 Squares (OLS), Weighted Least Squares (WLS), Seemingly Unrelated
 Regressions (SUR), Two-Stage Least Squares (2SLS), Weighted
 Two-Stage Least Squares (W2SLS), and Three-Stage Least Squares (3SLS)
 as suggested, e.g., by Zellner (1962) <doi:10.2307/2281644>,
 Zellner and Theil (1962) <doi:10.2307/1911287>, and
 Schmidt (1990) <doi:10.1016/0304-4076(90)90127-F>.
License: GPL (>= 2)
URL: https://r-forge.r-project.org/projects/systemfit/
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2023-03-22 13:15:35 UTC; gsl324
Repository: CRAN
Date/Publication: 2023-03-22 17:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 14:06:55 UTC; unix

export( "correlation.systemfit" )
export( "createSystemfitModel" )
export( "hausman.systemfit" )
export( "nlsystemfit" )
export( "se.ratio.systemfit" )
export( "systemfit" )
export( "systemfit.control" )

import( "Matrix" )
importFrom( "car", "linear.hypothesis" )
importFrom( "car", "linearHypothesis" )
importFrom( "lmtest", "lrtest" )
importFrom( "lmtest", "lrtest.default" )
importFrom( "MASS", "mvrnorm" )
importFrom( "methods", "as" )
importFrom( "sandwich", "bread" )
importFrom( "sandwich", "estfun" )
importFrom( "stats", "nobs" )
importFrom( "stats", "as.formula" )
importFrom( "stats", "coef" )
importFrom( "stats", "confint" )
importFrom( "stats", "cor" )
importFrom( "stats", "delete.response" )
importFrom( "stats", "deriv" )
importFrom( "stats", "df.residual" )
importFrom( "stats", "fitted" )
importFrom( "stats", "formula" )
importFrom( "stats", "model.extract" )
importFrom( "stats", "model.frame" )
importFrom( "stats", "model.matrix" )
importFrom( "stats", "nlm" )
importFrom( "stats", "optim" )
importFrom( "stats", "pchisq" )
importFrom( "stats", "pf" )
importFrom( "stats", "predict" )
importFrom( "stats", "printCoefmat" )
importFrom( "stats", "pt" )
importFrom( "stats", "qt" )
importFrom( "stats", "residuals" )
importFrom( "stats", "rnorm" )
importFrom( "stats", "symnum" )
importFrom( "stats", "terms" )
importFrom( "stats", "var" )
importFrom( "stats", "vcov" )

S3method( "bread", "systemfit" )
S3method( "coef", "systemfit" )
S3method( "coef", "systemfit.equation" )
S3method( "coef", "summary.systemfit" )
S3method( "coef", "summary.systemfit.equation" )
S3method( "confint", "systemfit" )
S3method( "confint", "systemfit.equation" )
S3method( "estfun", "systemfit" )
S3method( "fitted", "systemfit" )
S3method( "fitted", "systemfit.equation" )
S3method( "formula", "systemfit" )
S3method( "formula", "systemfit.equation" )
S3method( "linear.hypothesis", "systemfit" )
S3method( "linearHypothesis", "systemfit" )
S3method( "logLik", "systemfit" )
S3method( "lrtest", "systemfit" )
S3method( "model.frame", "systemfit" )
S3method( "model.frame", "systemfit.equation" )
S3method( "model.matrix", "systemfit" )
S3method( "model.matrix", "systemfit.equation" )
S3method( "nobs", "systemfit" )
S3method( "predict", "systemfit" )
S3method( "predict", "systemfit.equation" )
S3method( "print", "confint.systemfit" )
S3method( "print", "systemfit" )
S3method( "print", "systemfit.equation" )
S3method( "print", "summary.systemfit" )
S3method( "print", "summary.systemfit.equation" )
S3method( "residuals", "systemfit" )
S3method( "residuals", "systemfit.equation" )
S3method( "summary", "systemfit" )
S3method( "summary", "systemfit.equation" )
S3method( "terms", "systemfit" )
S3method( "terms", "systemfit.equation" )
S3method( "vcov", "systemfit" )
S3method( "vcov", "systemfit.equation" )

## ----include=FALSE------------------------------------------------------------
library(knitr)
opts_chunk$set(
engine='R'
)

## ----echo=FALSE---------------------------------------------------------------
options( prompt = "R> ", ctinue = "+  " )

## ----eval=FALSE---------------------------------------------------------------
#  sigmaInv <- solve( residCov )
#  xtOmegaInv <- crossprod( xMat, kronecker( sigmaInv, Diagonal( nObs ) ) )
#  coef <- solve( xtOmegaInv %*% xMat, xtOmegaInv %*%  yVec )

## ----message=FALSE------------------------------------------------------------
library( "systemfit" )
data( "Kmenta" )
attach( Kmenta )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
eqSystem <- list( demand = eqDemand, supply = eqSupply )
fitols <- systemfit( eqSystem )
print( fitols )

## -----------------------------------------------------------------------------
fitsur <- systemfit( eqSystem, method = "SUR" )

## -----------------------------------------------------------------------------
fit3sls  <- systemfit( eqSystem, method = "3SLS",
   inst = ~ income + farmPrice + trend )
fit3sls2 <- systemfit( eqSystem, method = "3SLS",
   inst = list( ~ farmPrice + trend, ~ income + farmPrice + trend ) )

## -----------------------------------------------------------------------------
fitsur <- systemfit( eqSystem, method = "SUR", data = Kmenta )

## ----tidy=FALSE---------------------------------------------------------------
restrict <- "demand_price + supply_farmPrice = 0"
fitsurRmat <- systemfit(eqSystem, method = "SUR",
   restrict.matrix = restrict)

## ----tidy=FALSE---------------------------------------------------------------
Rmat <- matrix(0, nrow = 1, ncol = 7)
Rmat[1, 2] <- 1
Rmat[1, 6] <- 1
qvec <- c(0)
fitsurRmatNum <- systemfit(eqSystem, method = "SUR",
   restrict.matrix = Rmat, restrict.rhs = qvec)

## ----tidy=FALSE---------------------------------------------------------------
modRegMat <- matrix(0, nrow = 7, ncol = 6)
modRegMat[1:5, 1:5] <- diag(5)
modRegMat[6, 2] <- -1
modRegMat[7, 6] <- 1
fitsurRegMat <- systemfit(eqSystem, method = "SUR",
   restrict.regMat = modRegMat)

## -----------------------------------------------------------------------------
all.equal( coef( fitsurRmat ), coef( fitsurRmatNum ) )
all.equal( coef( fitsurRmat ), coef( fitsurRegMat ) )

## -----------------------------------------------------------------------------
fitsurit <- systemfit( eqSystem, method = "SUR", maxiter = 500 )

## -----------------------------------------------------------------------------
summary( fitsur )

## -----------------------------------------------------------------------------
summary( fitsur, residCov = FALSE, equations = FALSE )

## ----message=FALSE,eval = requireNamespace("plm", quietly = TRUE)-------------
### this code chunk is evaluated only if the 'plm' package is available
data( "GrunfeldGreene" )
library( "plm" )
GGPanel <- pdata.frame( GrunfeldGreene, c( "firm", "year" ) )
greeneSur <- systemfit( invest ~ value + capital, method = "SUR",
   data = GGPanel )

## ----eval = requireNamespace("plm", quietly = TRUE)---------------------------
### this code chunk is evaluated only if the 'plm' package is available
greeneSurPooled <- systemfit( invest ~ value + capital, method = "SUR",
   data = GGPanel, pooled = TRUE )

## -----------------------------------------------------------------------------
linearHypothesis( fitsur, Rmat, qvec, test = "FT" )

linearHypothesis( fitsur, Rmat, qvec, test = "F" )

linearHypothesis( fitsur, Rmat, qvec, test = "Chisq" )

lrtest( fitsurRmat, fitsur )

## -----------------------------------------------------------------------------
fit2sls  <- systemfit( eqSystem, method = "2SLS",
   inst = ~ income + farmPrice + trend, data = Kmenta )
fit3sls <- systemfit( eqSystem, method = "3SLS",
   inst = ~ income + farmPrice + trend, data = Kmenta )
hausman.systemfit( fit2sls, fit3sls )

## ----echo=FALSE---------------------------------------------------------------
hausmantest <- hausman.systemfit( fit2sls, fit3sls )

## ----echo=FALSE---------------------------------------------------------------
library( "systemfit" )

## ----results='hide'-----------------------------------------------------------
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## ----results='hide'-----------------------------------------------------------
fitOls <- systemfit( system, data = Kmenta )
round( coef( summary( fitOls ) ), digits = 4 )

## ----results='hide'-----------------------------------------------------------
fit2sls <- systemfit( system, method = "2SLS", inst = inst, data = Kmenta )
round( coef( summary( fit2sls ) ), digits = 4 )

## ----results='hide'-----------------------------------------------------------
fit3sls <- systemfit( system, method = "3SLS", inst = inst, data = Kmenta )
round( coef( summary( fit3sls ) ), digits = 4 )

## ----results='hide'-----------------------------------------------------------
fitI3sls <- systemfit( system, method = "3SLS", inst = inst, data = Kmenta,
   maxit = 250 )
round( coef( summary( fitI3sls ) ), digits = 4 )

## -----------------------------------------------------------------------------
data( "KleinI" )
eqConsump  <- consump ~ corpProf + corpProfLag + wages
eqInvest   <- invest ~ corpProf + corpProfLag + capitalLag
eqPrivWage <- privWage ~ gnp + gnpLag + trend
inst <- ~ govExp + taxes + govWage + trend + capitalLag + corpProfLag + gnpLag
system <- list( Consumption = eqConsump, Investment = eqInvest,
   PrivateWages = eqPrivWage )

## ----results='hide'-----------------------------------------------------------
kleinOls <- systemfit( system, data = KleinI )
round( coef( summary( kleinOls ) ), digits = 3 )

## ----results='hide'-----------------------------------------------------------
klein2sls <- systemfit( system, method = "2SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor" )
round( coef( summary( klein2sls ) ), digits = 3 )

## ----results='hide'-----------------------------------------------------------
klein3sls <- systemfit( system, method = "3SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor" )
round( coef( summary( klein3sls ) ), digits = 3 )

## ----results='hide'-----------------------------------------------------------
kleinI3sls <- systemfit( system, method = "3SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor", maxit = 500 )
round( coef( summary( kleinI3sls ) ), digits = 3 )

## ----message=FALSE,eval = requireNamespace("plm", quietly = TRUE)-------------
### this code chunk is evaluated only if the 'plm' package is available
data( "GrunfeldGreene" )
library( "plm" )
GGPanel <- pdata.frame( GrunfeldGreene, c( "firm", "year" ) )
formulaGrunfeld <- invest ~ value + capital

## ----results='hide',eval = requireNamespace("plm", quietly = TRUE)------------
### this code chunk is evaluated only if the 'plm' package is available
greeneOls <- systemfit( formulaGrunfeld,
   data = GGPanel )
round( coef( summary( greeneOls ) ), digits = 4 )
round( sapply( greeneOls$eq, function(x){return(summary(x)$ssr/20)} ), digits = 3 )

## ----results='hide',eval = requireNamespace("plm", quietly = TRUE)------------
### this code chunk is evaluated only if the 'plm' package is available
greeneOlsPooled <- systemfit( formulaGrunfeld,
   data = GGPanel, pooled = TRUE )
round( coef( summary( greeneOlsPooled$eq[[1]] ) ), digits = 4 ) #$
sum( sapply( greeneOlsPooled$eq, function(x){return(summary(x)$ssr)}) )/100

## ----results='hide',eval = requireNamespace("plm", quietly = TRUE)------------
### this code chunk is evaluated only if the 'plm' package is available
greeneSur <- systemfit( formulaGrunfeld, method = "SUR",
   data = GGPanel, methodResidCov = "noDfCor" )
round( coef( summary( greeneSur ) ), digits = 4 )
round( greeneSur$residCov, digits = 3 ) #$
round( summary( greeneSur )$residCor, digits = 3 ) #$

## ----results='hide',eval = requireNamespace("plm", quietly = TRUE)------------
### this code chunk is evaluated only if the 'plm' package is available
greeneSurPooled <- systemfit( formulaGrunfeld, method = "SUR",
   data = GGPanel, pooled = TRUE, methodResidCov = "noDfCor",
   residCovWeighted = TRUE )
round( coef( summary( greeneSurPooled$eq[[1]] ) ), digits = 4 ) #$

round( greeneSurPooled$residCov, digits = 3 ) #$
round( cov( residuals( greeneSurPooled ) ), digits = 3 )
round( summary( greeneSurPooled )$residCor, digits = 3 ) #$

## ----eval = requireNamespace("plm", quietly = TRUE)---------------------------
### this code chunk is evaluated only if the 'plm' package is available
GrunfeldTheil <- subset( GrunfeldGreene,
   firm %in% c( "General Electric", "Westinghouse" ) )
GTPanel <- pdata.frame( GrunfeldTheil, c( "firm", "year" ) )
formulaGrunfeld <- invest ~ value + capital

## ----results='hide',eval = requireNamespace("plm", quietly = TRUE)------------
### this code chunk is evaluated only if the 'plm' package is available
theilOls <- systemfit( formulaGrunfeld,
   data = GTPanel )
round( coef( summary( theilOls ) ), digits = 3 )

## ----results='hide',eval = requireNamespace("plm", quietly = TRUE)------------
### this code chunk is evaluated only if the 'plm' package is available
theilSur <- systemfit( formulaGrunfeld, method = "SUR",
   data = GTPanel, methodResidCov = "noDfCor" )
round( coef( summary( theilSur ) ), digits = 3 )

## ----results='hide',eval = requireNamespace("plm", quietly = TRUE)------------
### this code chunk is evaluated only if the 'plm' package is available
RMatrix <- rbind( c( 0, 1, 0, 0, -1, 0 ), c( 0, 0, 1, 0, 0, -1 ) )
linearHypothesis( theilSur, RMatrix )

## ----results='hide',tidy=FALSE,eval = requireNamespace("plm", quietly = TRUE)----
### this code chunk is evaluated only if the 'plm' package is available
theilSurRestr <- systemfit(formulaGrunfeld, method = "SUR",
   data = GTPanel, methodResidCov = "noDfCor", restrict.matrix = RMatrix,
   residCovRestricted = FALSE)
round(coef(summary(theilSurRestr)), digits = 3)

## ----message=FALSE------------------------------------------------------------
library( "systemfit" )
set.seed( 1 )
systemfitModel <- createSystemfitModel( nEq = 8, nReg = 10, nObs = 750 )
system.time(
   fitMatrix <- systemfit( systemfitModel$formula, method = "SUR",
      data = systemfitModel$data )
)
system.time(
   fitTrad <- systemfit( systemfitModel$formula, method = "SUR",
      data = systemfitModel$data, useMatrix = FALSE )
)
all.equal( fitMatrix, fitTrad )

## -----------------------------------------------------------------------------
smallModel <- createSystemfitModel( nEq = 3, nReg = 4, nObs = 50 )
system.time(
   fitSmallMatrix <- systemfit( smallModel$formula, method = "SUR",
      data = smallModel$data, maxit = 500 )
)
system.time(
   fitSmallTrad <- systemfit( smallModel$formula, method = "SUR",
      data = smallModel$data, maxit = 500, useMatrix = FALSE )
)
all.equal( fitSmallMatrix, fitSmallTrad )

## ----echo=FALSE----------------------------------------------------------
options( width = 75 )

## ----message=FALSE,eval = requireNamespace("sem", quietly = TRUE)--------
### this code chunk is evaluated only if the 'sem' package is available
library( "sem" )
library( "systemfit" )
data( "KleinI" )

## ----tidy=FALSE,eval = requireNamespace("sem", quietly = TRUE)-----------
### this code chunk is evaluated only if the 'sem' package is available
limlRam <- matrix(c(
   "consump  <-  corpProf",    "consump_corpProf",    NA,
   "consump  <-  corpProfLag", "consump_corpProfLag", NA,
   "consump  <-  wages",       "consump_wages",       NA,
   "invest   <-  corpProf",    "invest_corpProf",     NA,
   "invest   <-  corpProfLag", "invest_corpProfLag",  NA,
   "invest   <-  capitalLag",  "invest_capitalLag",   NA,
   "privWage <-  gnp",         "privWage_gnp",        NA,
   "privWage <-  gnpLag",      "privWage_gnpLag",     NA,
   "privWage <-  trend",       "privWage_trend",      NA,
   "consump  <-> consump",     "s11", NA,
   "privWage <-> privWage",    "s22", NA,
   "invest   <-> invest",      "s33", NA),
   ncol = 3, byrow = TRUE)
class(limlRam) <- "mod"
exogVar <- c("corpProf", "corpProfLag", "wages", "capitalLag", "trend",
   "gnp", "gnpLag")
endogVar <- c("consump", "invest", "privWage")
allVar <- c(exogVar, endogVar)

limlResult <- sem(model = limlRam, S = cov(KleinI[ -1, allVar ]),
   N = (nrow(KleinI) - 1), fixed.x = exogVar)
print(limlResult)

## ----tidy=FALSE----------------------------------------------------------
eqConsump <- consump ~ corpProf + corpProfLag + wages
eqInvest <- invest ~ corpProf + corpProfLag + capitalLag
eqPrivWage <- privWage ~ gnp + gnpLag + trend
system <- list(consump = eqConsump, invest = eqInvest,
   privWage = eqPrivWage)
olsResult <- systemfit(system, data = KleinI)
print(olsResult)

## ----tidy=FALSE,eval = requireNamespace("sem", quietly = TRUE)-----------
### this code chunk is evaluated only if the 'sem' package is available
fimlRam <- rbind(limlRam,
   c("consump  <-> invest",   "s12", NA),
   c("consump  <-> privWage", "s13", NA),
   c("privWage <-> invest",   "s23", NA))
class(fimlRam) <- "mod"

fimlResult <- sem(model = fimlRam, S = cov(KleinI[ -1, allVar ]),
   N = (nrow(KleinI) - 1), fixed.x = exogVar)
print(fimlResult)

## ------------------------------------------------------------------------
surResult <- systemfit( system, method = "SUR", data = KleinI,
   methodResidCov = "noDfCor", maxit = 500 )
print( surResult )

.ftest.systemfit <- function( object, restrict.matrix,
   restrict.rhs = NULL, vcov. = NULL ){

   coef <- coef( object )

   # coefficient covariance matrix
   if( is.null( vcov. ) ){
      vcov <- vcov( object )
   } else if( is.function( vcov. ) ){
      vcov <- vcov.( object )
   } else {
      vcov <- vcov.
   }

   resid <- unlist( residuals( object ) )
   resid <- resid[ !is.na( resid ) ]
   nEq   <- length( object$eq )
   if( is.null( object$residCovEst ) ) {
      rcov <- diag( nEq )
   } else {
      rcov <- object$residCovEst
   }

   validObsEq <- matrix( NA, nrow = nrow( residuals( object ) ), ncol = nEq )
   for( i in 1:nEq ) {
      validObsEq[ , i ] <- !is.na( residuals( object$eq[[ i ]] ) )
   }

   result <- list()

   result$nRestr <- nrow( restrict.matrix )
   result$df.residual.sys  <- object$df.residual

   numerator <- t( restrict.matrix %*% coef - restrict.rhs ) %*%
      solve( restrict.matrix %*% vcov %*% t( restrict.matrix ) ) %*%
      ( restrict.matrix %*% coef - restrict.rhs )

   resid <- matrix( resid, ncol = 1 )
   if( object$control$useMatrix ) {
      resid <- as( resid, "CsparseMatrix" )
      rcov <- as( rcov, "symmetricMatrix" )
   }

   denominator <- as.numeric( .calcXtOmegaInv( xMat = resid, sigma = rcov,
      validObsEq = validObsEq, useMatrix = object$control$useMatrix ) %*% resid )

   result$statistic <- ( numerator / result$nRestr ) /
      ( denominator / result$df.residual.sys )

   result$p.value <- 1 - pf( result$statistic, result$nRestr, result$df.residual.sys )

   return( result )
}

estfun.systemfit <- function ( x, residFit = TRUE, ... ) {
   if( !is.null( x$restrict.matrix ) || !is.null( x$restrict.rhs ) ||
        !is.null( x$restrict.regMat ) ) {
      stop( "returning the estimation function for models with restrictions",
            " has not yet been implemented.")
   }
   
   # residuals
   res <- unlist(  residuals( x ) )
   
   # model matrix
   if( is.null( x$eq[[1]]$inst ) ) {
      mm <- model.matrix( x )
   } else {
      mm <- model.matrix( x, which = "xHat" )
      if( residFit ) {
         res[ !is.na( res ) ] <- res[ !is.na( res ) ] +
            ( model.matrix( x ) - mm ) %*% coef( x )
         #       resid_fit = y - x_fit b
         #       resid = y - x b
         #       resid_fit - resid = - x_fit b + x b
         #       resid_fit = resid + ( x - x_fit ) b
      }
   }
   
   if( sum( !is.na( res ) ) != nrow( mm ) ) {
      stop( "internal error: the number of residuals is not equal to the",
         " number of rows of the model matrix. Please contact the maintainer." )
   }

   if( is.null( x$residCovEst ) ) {
      omegaInvXmat <- mm
   } else {
      omegaInvXmat <- t( .calcXtOmegaInv( xMat = mm, sigma = x$residCovEst, 
         validObsEq = !is.na( residuals( x ) ), invertSigma = TRUE ) )
   }
   
   result <- res[ !is.na( res ) ] * omegaInvXmat
   
   dimnames( result ) <- dimnames( mm )
   
   if( max( abs( colSums( result ) ) ) > 1e-6 ) {
      warning( "the columns of the returned estimating function",
         " do not all sum up to zero,",
         " which indicates that the wrong estimating function is returned" )
   }
   
   return( result )
}

.prepareData.systemfit <- function( data )
{
   callNoDots <- match.call( expand.dots = FALSE ) #-"- without ...-expansion

   # model frame (without formula)
   modelFrame <- callNoDots[ c( 1, match( "data", names( callNoDots ), 0 ) ) ]
   modelFrame$na.action <- as.name( "na.pass" )
   modelFrame[[1]] <- as.name( "model.frame" )

   return( modelFrame )
}

library( systemfit )
options( digits = 3 )

data( "Kmenta" )
useMatrix <- FALSE

demand <- consump ~ price + income
supply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
inst1  <- ~ income + farmPrice
instlist <- list( inst1, inst )
system <- list( demand = demand, supply = supply )
restrm <- matrix(0,1,7)  # restriction matrix "R"
restrm[1,3] <-  1
restrm[1,7] <- -1
restrict <- "demand_income - supply_trend = 0"
restr2m <- matrix(0,2,7)  # restriction matrix "R" 2
restr2m[1,3] <-  1
restr2m[1,7] <- -1
restr2m[2,2] <- -1
restr2m[2,5] <-  1
restr2q <- c( 0, 0.5 )  # restriction vector "q" 2
restrict2 <- c( "demand_income - supply_trend = 0",
   "- demand_price + supply_price = 0.5" )
tc <- matrix(0,7,6)
tc[1,1] <- 1
tc[2,2] <- 1
tc[3,3] <- 1
tc[4,4] <- 1
tc[5,5] <- 1
tc[6,6] <- 1
tc[7,3] <- 1
restr3m <- matrix(0,1,6)  # restriction matrix "R" 2
restr3m[1,2] <- -1
restr3m[1,5] <-  1
restr3q <- c( 0.5 )  # restriction vector "q" 2
restrict3 <- "- C2 + C5 = 0.5"


## *************** 3SLS estimation ************************
fit3sls <- list()
formulas <- c( "GLS", "IV", "Schmidt", "GMM", "EViews" )
for( i in seq( along = formulas ) ) {
   fit3sls[[ i ]] <- list()

   print( "***************************************************" )
   print( paste( "3SLS formula:", formulas[ i ] ) )
   print( "************* 3SLS *********************************" )
   fit3sls[[ i ]]$e1 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, method3sls = formulas[ i ], useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e1 ) )

   print( "********************* 3SLS EViews-like *****************" )
   fit3sls[[ i ]]$e1e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", method3sls = formulas[ i ],
      useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e1e, useDfSys = TRUE ) )

   print( "********************* 3SLS with methodResidCov = Theil *****************" )
   fit3sls[[ i ]]$e1c <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "Theil", method3sls = formulas[ i ],
      x = TRUE, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e1c, useDfSys = TRUE ) )

   print( "*************** W3SLS with methodResidCov = Theil *****************" )
   fit3sls[[ i ]]$e1wc <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "Theil", method3sls = formulas[ i ],
      residCovWeighted = TRUE, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e1wc, useDfSys = TRUE ) )


   print( "*************** 3SLS with restriction *****************" )
   fit3sls[[ i ]]$e2 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.matrix = restrm, method3sls = formulas[ i ],
      x = TRUE, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e2 ) )
   # the same with symbolically specified restrictions
   fit3sls[[ i ]]$e2Sym <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.matrix = restrict, method3sls = formulas[ i ],
      x = TRUE, useMatrix = useMatrix )
   print( all.equal( fit3sls[[ i ]]$e2, fit3sls[[ i ]]$e2Sym ) )

   print( "************** 3SLS with restriction (EViews-like) *****************" )
   fit3sls[[ i ]]$e2e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.matrix = restrm,
      method3sls = formulas[ i ], useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e2e, useDfSys = TRUE ) )
   print( nobs( fit3sls[[i]]$e2e ))

   print( "*************** W3SLS with restriction *****************" )
   fit3sls[[ i ]]$e2w <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.matrix = restrm, method3sls = formulas[ i ],
      residCovWeighted = TRUE, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e2w ) )


   print( "*************** 3SLS with restriction via restrict.regMat ********************" )
   fit3sls[[ i ]]$e3 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, method3sls = formulas[ i ],
      useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e3 ) )

   print( "*************** 3SLS with restriction via restrict.regMat (EViews-like) *******" )
   fit3sls[[ i ]]$e3e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.regMat = tc,
      method3sls = formulas[ i ], x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e3e, useDfSys = TRUE ) )

   print( "**** W3SLS with restriction via restrict.regMat (EViews-like) ****" )
   fit3sls[[ i ]]$e3we <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.regMat = tc,
      method3sls = formulas[ i ], residCovWeighted = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e3we, useDfSys = TRUE ) )


   print( "*************** 3SLS with 2 restrictions **********************" )
   fit3sls[[ i ]]$e4 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.matrix = restr2m, restrict.rhs = restr2q,
      method3sls = formulas[ i ], x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e4 ) )
   # the same with symbolically specified restrictions
   fit3sls[[ i ]]$e4Sym <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.matrix = restrict2, method3sls = formulas[ i ],
      x = TRUE, useMatrix = useMatrix )
   print( all.equal( fit3sls[[ i ]]$e4, fit3sls[[ i ]]$e4Sym ) )

   print( "*************** 3SLS with 2 restrictions (EViews-like) ************" )
   fit3sls[[ i ]]$e4e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.matrix = restr2m,
      restrict.rhs = restr2q, method3sls = formulas[ i ],
      useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e4e, useDfSys = TRUE ) )

   print( "********** W3SLS with 2 (symbolic) restrictions ***************" )
   fit3sls[[ i ]]$e4wSym <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.matrix = restrict2, method3sls = formulas[ i ],
      residCovWeighted = TRUE, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e4wSym ) )


   print( "*************** 3SLS with 2 restrictions via R and restrict.regMat **********" )
   fit3sls[[ i ]]$e5 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, restrict.matrix = restr3m,
      restrict.rhs = restr3q, method3sls = formulas[ i ],
      useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e5 ) )
   # the same with symbolically specified restrictions
   fit3sls[[ i ]]$e5Sym <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, restrict.matrix = restrict3,
      method3sls = formulas[ i ], useMatrix = useMatrix )
   print( all.equal( fit3sls[[ i ]]$e5, fit3sls[[ i ]]$e5Sym ) )

   print( "******** 3SLS with 2 restrictions via R and restrict.regMat (EViews-like)*****" )
   fit3sls[[ i ]]$e5e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, methodResidCov = "noDfCor",
      restrict.matrix = restr3m, restrict.rhs = restr3q,
      method3sls = formulas[ i ], x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e5e, useDfSys = TRUE ) )

   print( "*** W3SLS with 2 restrictions via R and restrict.regMat (EViews-like) ***" )
   fit3sls[[ i ]]$e5we <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, methodResidCov = "noDfCor",
      restrict.matrix = restr3m, restrict.rhs = restr3q, method3sls = formulas[ i ],
      residCovWeighted = TRUE, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$e5we, useDfSys = TRUE ) )

   ## *********** estimations with a single regressor ************
   fit3sls[[ i ]]$S1 <- systemfit(
      list( farmPrice ~ consump - 1, price ~ consump + trend ), "3SLS",
      data = Kmenta, inst = ~ trend + income, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$S1 ) )
   fit3sls[[ i ]]$S2 <- systemfit(
      list( consump ~ farmPrice - 1, consump ~ trend - 1 ), "3SLS",
      data = Kmenta, inst = ~ price + income, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$S2 ) )
   fit3sls[[ i ]]$S3 <- systemfit(
      list( consump ~ trend - 1, farmPrice ~ trend - 1 ), "3SLS",
      data = Kmenta, inst = instlist, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$S3 ) )
   fit3sls[[ i ]]$S4 <- systemfit(
      list( consump ~ farmPrice - 1, price ~ trend - 1 ), "3SLS",
      data = Kmenta, inst = ~ farmPrice + trend + income,
      restrict.matrix = matrix( c( 1, -1 ), nrow = 1 ), useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$S4 ) )
   fit3sls[[ i ]]$S5 <- systemfit(
      list( consump ~ 1, price ~ 1 ), "3SLS",
      data = Kmenta, inst = ~ income, useMatrix = useMatrix )
   print( summary( fit3sls[[ i ]]$S5 ) )
}

## ******************** iterated 3SLS **********************
fit3slsi <- list()
formulas <- c( "GLS", "IV", "Schmidt", "GMM", "EViews" )
for( i in seq( along = formulas ) ) {
   fit3slsi[[ i ]] <- list()

   print( "***************************************************" )
   print( paste( "3SLS formula:", formulas[ i ] ) )
   print( "************* 3SLS *********************************" )
   fit3slsi[[ i ]]$e1 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, method3sls = formulas[ i ], maxiter = 100,
      useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e1 ) )

   print( "********************* iterated 3SLS EViews-like ****************" )
   fit3slsi[[ i ]]$e1e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", method3sls = formulas[ i ],
      maxiter = 100, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e1e, useDfSys = TRUE ) )

   print( "************** iterated 3SLS with methodResidCov = Theil **************" )
   fit3slsi[[ i ]]$e1c <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "Theil", method3sls = formulas[ i ],
      maxiter = 100, x = TRUE, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e1c, useDfSys = TRUE ) )

   print( "**************** iterated W3SLS EViews-like ****************" )
   fit3slsi[[ i ]]$e1we <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", method3sls = formulas[ i ],
      maxiter = 100, residCovWeighted = TRUE, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e1we, useDfSys = TRUE ) )


   print( "******* iterated 3SLS with restriction *****************" )
   fit3slsi[[ i ]]$e2 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.matrix = restrm, method3sls = formulas[ i ],
      maxiter = 100, x = TRUE, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e2 ) )

   print( "********* iterated 3SLS with restriction (EViews-like) *********" )
   fit3slsi[[ i ]]$e2e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.matrix = restrm,
      method3sls = formulas[ i ], maxiter = 100, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e2e, useDfSys = TRUE ) )

   print( "******** iterated W3SLS with restriction (EViews-like) *********" )
   fit3slsi[[ i ]]$e2we <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.matrix = restrm,
      method3sls = formulas[ i ], maxiter = 100, residCovWeighted = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e2we, useDfSys = TRUE ) )


   print( "********* iterated 3SLS with restriction via restrict.regMat *****************" )
   fit3slsi[[ i ]]$e3 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, method3sls = formulas[ i ],
      maxiter = 100, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e3 ) )

   print( "********* iterated 3SLS with restriction via restrict.regMat (EViews-like) ***" )
   fit3slsi[[ i ]]$e3e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.regMat = tc,
      method3sls = formulas[ i ], maxiter = 100, x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e3e, useDfSys = TRUE ) )

   print( "***** iterated W3SLS with restriction via restrict.regMat ********" )
   fit3slsi[[ i ]]$e3w <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, method3sls = formulas[ i ], maxiter = 100,
      residCovWeighted = TRUE, x = TRUE, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e3w ) )


   print( "******** iterated 3SLS with 2 restrictions *********************" )
   fit3slsi[[ i ]]$e4 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.matrix = restr2m, restrict.rhs = restr2q,
      method3sls = formulas[ i ], maxiter = 100, x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e4 ) )

   print( "********* iterated 3SLS with 2 restrictions (EViews-like) *******" )
   fit3slsi[[ i ]]$e4e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.matrix = restr2m,
      restrict.rhs = restr2q, method3sls = formulas[ i ], maxiter = 100,
      useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e4e, useDfSys = TRUE ) )

   print( "******** iterated W3SLS with 2 restrictions (EViews-like) *******" )
   fit3slsi[[ i ]]$e4we <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, methodResidCov = "noDfCor", restrict.matrix = restr2m,
      restrict.rhs = restr2q, method3sls = formulas[ i ], maxiter = 100,
      residCovWeighted = TRUE, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e4we, useDfSys = TRUE ) )


   print( "******** iterated 3SLS with 2 restrictions via R and restrict.regMat *********" )
   fit3slsi[[ i ]]$e5 <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, restrict.matrix = restr3m,
      restrict.rhs = restr3q, method3sls = formulas[ i ], maxiter = 100,
      useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e5 ) )

   print( "*** iterated 3SLS with 2 restrictions via R and restrict.regMat (EViews-like)**" )
   fit3slsi[[ i ]]$e5e <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, methodResidCov = "noDfCor",
      restrict.matrix = restr3m, restrict.rhs = restr3q,
      method3sls = formulas[ i ], maxiter = 100, useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e5e, useDfSys = TRUE ) )

   print( "** iterated W3SLS with 2 restrictions via R and restrict.regMat ***" )
   fit3slsi[[ i ]]$e5w <- systemfit( system, "3SLS", data = Kmenta,
      inst = inst, restrict.regMat = tc, restrict.matrix = restr3m,
      restrict.rhs = restr3q, method3sls = formulas[ i ], maxiter = 100,
      residCovWeighted = TRUE, x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsi[[ i ]]$e5w ) )
}

## **************** 3SLS with different instruments *************
fit3slsd <- list()
formulas <- c( "GLS", "IV", "Schmidt", "GMM", "EViews" )
for( i in seq( along = formulas ) ) {
   fit3slsd[[ i ]] <- list()

   print( "***************************************************" )
   print( paste( "3SLS formula:", formulas[ i ] ) )
   print( "************* 3SLS with different instruments **************" )
   fit3slsd[[ i ]]$e1 <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, method3sls = formulas[ i ], useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e1 ) )

   print( "******* 3SLS with different instruments (EViews-like) **********" )
   fit3slsd[[ i ]]$e1e <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, methodResidCov = "noDfCor", method3sls = formulas[ i ],
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e1e, useDfSys = TRUE ) )

   print( "**** 3SLS with different instruments and methodResidCov = Theil ***" )
   fit3slsd[[ i ]]$e1c <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, methodResidCov = "Theil", method3sls = formulas[ i ],
      x = TRUE, useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e1c, useDfSys = TRUE ) )

   print( "************* W3SLS with different instruments **************" )
   fit3slsd[[ i ]]$e1w <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, method3sls = formulas[ i ], residCovWeighted = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e1w ) )


   print( "******* 3SLS with different instruments and restriction ********" )
   fit3slsd[[ i ]]$e2 <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, restrict.matrix = restrm, method3sls = formulas[ i ],
      x = TRUE, useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e2 ) )

   print( "** 3SLS with different instruments and restriction (EViews-like) *" )
   fit3slsd[[ i ]]$e2e <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, methodResidCov = "noDfCor", restrict.matrix = restrm,
      method3sls = formulas[ i ], useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e2e, useDfSys = TRUE ) )

   print( "** W3SLS with different instruments and restriction (EViews-like) *" )
   fit3slsd[[ i ]]$e2we <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, methodResidCov = "noDfCor", restrict.matrix = restrm,
      method3sls = formulas[ i ], residCovWeighted = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e2we, useDfSys = TRUE ) )


   print( "** 3SLS with different instruments and restriction via restrict.regMat *******" )
   fit3slsd[[ i ]]$e3 <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, restrict.regMat = tc, method3sls = formulas[ i ],
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e3 ) )

   print( "3SLS with different instruments with restriction via restrict.regMat (EViews-like)" )
   fit3slsd[[ i ]]$e3e <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, methodResidCov = "noDfCor", restrict.regMat = tc,
      method3sls = formulas[ i ], x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e3e, useDfSys = TRUE ) )

   print( "** W3SLS with different instr. and restr. via restrict.regMat ****" )
   fit3slsd[[ i ]]$e3w <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, restrict.regMat = tc, method3sls = formulas[ i ],
      residCovWeighted = TRUE, x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e3w ) )


   print( "****** 3SLS with different instruments and 2 restrictions *********" )
   fit3slsd[[ i ]]$e4 <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, restrict.matrix = restr2m, restrict.rhs = restr2q,
      method3sls = formulas[ i ], x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e4 ) )

   print( "** 3SLS with different instruments and 2 restrictions (EViews-like) *" )
   fit3slsd[[ i ]]$e4e <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, methodResidCov = "noDfCor", restrict.matrix = restr2m,
      restrict.rhs = restr2q, method3sls = formulas[ i ],
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e4e, useDfSys = TRUE ) )

   print( "**** W3SLS with different instruments and 2 restrictions *********" )
   fit3slsd[[ i ]]$e4w <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, restrict.matrix = restr2m, restrict.rhs = restr2q,
      method3sls = formulas[ i ], residCovWeighted = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e4w ) )


   print( " 3SLS with different instruments with 2 restrictions via R and restrict.regMat" )
   fit3slsd[[ i ]]$e5 <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, restrict.regMat = tc, restrict.matrix = restr3m,
      restrict.rhs = restr3q, method3sls = formulas[ i ],
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e5 ) )

   print( "3SLS with diff. instruments and 2 restr. via R and restrict.regMat (EViews-like)" )
   fit3slsd[[ i ]]$e5e <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, restrict.regMat = tc, methodResidCov = "noDfCor",
      restrict.matrix = restr3m, restrict.rhs = restr3q,
      method3sls = formulas[ i ], x = TRUE,
      useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e5e, useDfSys = TRUE ) )

   print( "W3SLS with diff. instr. and 2 restr. via R and restrict.regMat (EViews-like)" )
   fit3slsd[[ i ]]$e5we <- systemfit( system, "3SLS", data = Kmenta,
      inst = instlist, restrict.regMat = tc, methodResidCov = "noDfCor",
      restrict.matrix = restr3m, restrict.rhs = restr3q, method3sls = formulas[ i ],
      residCovWeighted = TRUE, useMatrix = useMatrix )
   print( summary( fit3slsd[[ i ]]$e5we, useDfSys = TRUE ) )
}


## **************** shorter summaries **********************
print( summary( fit3sls[[ 2 ]]$e1c, equations = FALSE ) )

print( summary( fit3sls[[ 3 ]]$e2e ), residCov = FALSE )

print( summary( fit3sls[[ 4 ]]$e3, useDfSys = FALSE ), residCov = FALSE )

print( summary( fit3sls[[ 5 ]]$e4e, equations = FALSE ),
   equations = FALSE )

print( summary( fit3sls[[ 1 ]]$e4wSym, residCov = FALSE ),
   equations = FALSE )

print( summary( fit3sls[[ 2 ]]$e5, residCov = FALSE ), residCov = TRUE )

print( summary( fit3slsi[[ 3 ]]$e3e, residCov = FALSE,
   equations = FALSE ) )

print( summary( fit3slsi[[ 4 ]]$e1we ), equations = FALSE, residCov = FALSE )

print( summary( fit3slsd[[ 5 ]]$e4, residCov = FALSE ) )

print( summary( fit3slsd[[ 1 ]]$e2we, equations = FALSE ) )


## ****************** residuals **************************
print( residuals( fit3sls[[ 1 ]]$e1c ) )
print( residuals( fit3sls[[ 1 ]]$e1c$eq[[ 1 ]] ) )

print( residuals( fit3sls[[ 4 ]]$e1wc ) )
print( residuals( fit3sls[[ 4 ]]$e1wc$eq[[ 1 ]] ) )

print( residuals( fit3sls[[ 2 ]]$e2e ) )
print( residuals( fit3sls[[ 2 ]]$e2e$eq[[ 2 ]] ) )

print( residuals( fit3sls[[ 3 ]]$e3 ) )
print( residuals( fit3sls[[ 3 ]]$e3$eq[[ 1 ]] ) )

print( residuals( fit3sls[[ 4 ]]$e4e ) )
print( residuals( fit3sls[[ 4 ]]$e4e$eq[[ 2 ]] ) )

print( residuals( fit3sls[[ 5 ]]$e5 ) )
print( residuals( fit3sls[[ 5 ]]$e5$eq[[ 1 ]] ) )

print( residuals( fit3slsi[[ 2 ]]$e3e ) )
print( residuals( fit3slsi[[ 2 ]]$e3e$eq[[ 1 ]] ) )

print( residuals( fit3slsi[[ 1 ]]$e2we ) )
print( residuals( fit3slsi[[ 1 ]]$e2we$eq[[ 1 ]] ) )

print( residuals( fit3slsd[[ 3 ]]$e4 ) )
print( residuals( fit3slsd[[ 3 ]]$e4$eq[[ 2 ]] ) )

print( residuals( fit3slsd[[ 5 ]]$e5we ) )
print( residuals( fit3slsd[[ 5 ]]$e5we$eq[[ 2 ]] ) )


## *************** coefficients *********************
print( round( coef( fit3sls[[ 3 ]]$e1c ), digits = 6 ) )
print( round( coef( fit3sls[[ 4 ]]$e1c$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fit3slsi[[ 4 ]]$e2 ), digits = 6 ) )
print( round( coef( fit3slsi[[ 5 ]]$e2$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fit3sls[[ 2 ]]$e2w ), digits = 6 ) )
print( round( coef( fit3sls[[ 3 ]]$e2w$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fit3slsd[[ 5 ]]$e3e ), digits = 6 ) )
print( round( coef( fit3slsd[[ 5 ]]$e3e, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fit3slsd[[ 1 ]]$e3e$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fit3slsd[[ 4 ]]$e3w ), digits = 6 ) )
print( round( coef( fit3slsd[[ 4 ]]$e3w, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fit3slsd[[ 5 ]]$e3w$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fit3sls[[ 1 ]]$e4 ), digits = 6 ) )
print( round( coef( fit3sls[[ 2 ]]$e4$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fit3slsi[[ 2 ]]$e4we ), digits = 6 ) )
print( round( coef( fit3slsi[[ 1 ]]$e4we$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fit3slsi[[ 2 ]]$e5e ), digits = 6 ) )
print( round( coef( fit3slsi[[ 2 ]]$e5e, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fit3slsi[[ 3 ]]$e5e$eq[[ 2 ]] ), digits = 6 ) )


## *************** coefficients with stats *********************
print( round( coef( summary( fit3sls[[ 3 ]]$e1c, useDfSys = FALSE ) ),
   digits = 6 ) )
print( round( coef( summary( fit3sls[[ 4 ]]$e1c$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fit3slsd[[ 2 ]]$e1w, useDfSys = FALSE ) ),
   digits = 6 ) )
print( round( coef( summary( fit3slsd[[ 3 ]]$e1w$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 3 ) )

print( round( coef( summary( fit3slsi[[ 4 ]]$e2 ) ), digits = 6 ) )
print( round( coef( summary( fit3slsi[[ 5 ]]$e2$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fit3slsd[[ 5 ]]$e3e, useDfSys = FALSE ) ),
   digits = 6 ) )
print( round( coef( summary( fit3slsd[[ 5 ]]$e3e, useDfSys = FALSE ),
   modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fit3slsd[[ 1 ]]$e3e$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fit3slsi[[ 4 ]]$e3w, useDfSys = FALSE ) ),
   digits = 6 ) )
print( round( coef( summary( fit3slsi[[ 4 ]]$e3w, useDfSys = FALSE ),
   modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fit3slsi[[ 5 ]]$e3w$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fit3sls[[ 1 ]]$e4 ) ), digits = 6 ) )
print( round( coef( summary( fit3sls[[ 2 ]]$e4$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fit3slsi[[ 2 ]]$e5e ) ), digits = 6 ) )
print( round( coef( summary( fit3slsi[[ 2 ]]$e5e ), modified.regMat = TRUE ),
   digits = 6 ) )
print( round( coef( summary( fit3slsi[[ 3 ]]$e5e$eq[[ 2 ]] ) ), digits = 6 ) )

print( round( coef( summary( fit3sls[[ 2 ]]$e5we ) ), digits = 6 ) )
print( round( coef( summary( fit3sls[[ 2 ]]$e5we ), modified.regMat = TRUE ),
   digits = 6 ) )
print( round( coef( summary( fit3sls[[ 3 ]]$e5we$eq[[ 2 ]] ) ), digits = 6 ) )


## *********** variance covariance matrix of the coefficients *******
print( round( vcov( fit3sls[[ 3 ]]$e1c ), digits = 5 ) )
print( round( vcov( fit3sls[[ 4 ]]$e1c$eq[[ 2 ]] ), digits = 5 ) )

print( round( vcov( fit3sls[[ 4 ]]$e2 ), digits = 5 ) )
print( round( vcov( fit3sls[[ 5 ]]$e2$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3sls[[ 5 ]]$e3e ), digits = 5 ) )
print( round( vcov( fit3sls[[ 5 ]]$e3e, modified.regMat = TRUE ), digits = 5 ) )
print( round( vcov( fit3sls[[ 1 ]]$e3e$eq[[ 2 ]] ), digits = 5 ) )

print( round( vcov( fit3sls[[ 1 ]]$e4 ), digits = 5 ) )
print( round( vcov( fit3sls[[ 2 ]]$e4$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3sls[[ 3 ]]$e4wSym ), digits = 5 ) )
print( round( vcov( fit3sls[[ 4 ]]$e4wSym$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3sls[[ 2 ]]$e5e ), digits = 5 ) )
print( round( vcov( fit3sls[[ 2 ]]$e5e, modified.regMat = TRUE ), digits = 5 ) )
print( round( vcov( fit3sls[[ 3 ]]$e5e$eq[[ 2 ]] ), digits = 5 ) )

print( round( vcov( fit3slsi[[ 4 ]]$e1e ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 3 ]]$e1e$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3slsi[[ 5 ]]$e1we ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 1 ]]$e1we$eq[[ 2 ]] ), digits = 5 ) )

print( round( vcov( fit3slsi[[ 5 ]]$e2e ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 4 ]]$e2e$eq[[ 2 ]] ), digits = 5 ) )

print( round( vcov( fit3slsi[[ 1 ]]$e3 ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 1 ]]$e3, modified.regMat = TRUE ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 5 ]]$e3$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3slsi[[ 2 ]]$e4e ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 1 ]]$e4e$eq[[ 2 ]] ), digits = 5 ) )

print( round( vcov( fit3slsi[[ 3 ]]$e5 ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 3 ]]$e5, modified.regMat = TRUE ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 2 ]]$e5$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3slsi[[ 5 ]]$e5w ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 5 ]]$e5w, modified.regMat = TRUE ), digits = 5 ) )
print( round( vcov( fit3slsi[[ 4 ]]$e5w$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3slsd[[ 5 ]]$e1c ), digits = 5 ) )
print( round( vcov( fit3slsd[[ 2 ]]$e1c$eq[[ 2 ]] ), digits = 5 ) )

print( round( vcov( fit3slsd[[ 1 ]]$e2 ), digits = 5 ) )
print( round( vcov( fit3slsd[[ 3 ]]$e2$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3slsd[[ 5 ]]$e2we ), digits = 5 ) )
print( round( vcov( fit3slsd[[ 3 ]]$e2we$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3slsd[[ 2 ]]$e3 ), digits = 5 ) )
print( round( vcov( fit3slsd[[ 2 ]]$e3, modified.regMat = TRUE ), digits = 5 ) )
print( round( vcov( fit3slsd[[ 4 ]]$e3$eq[[ 2 ]] ), digits = 5 ) )

print( round( vcov( fit3slsd[[ 3 ]]$e4 ), digits = 5 ) )
print( round( vcov( fit3slsd[[ 5 ]]$e4$eq[[ 1 ]] ), digits = 5 ) )

print( round( vcov( fit3slsd[[ 4 ]]$e5e ), digits = 5 ) )
print( round( vcov( fit3slsd[[ 4 ]]$e5e, modified.regMat = TRUE ), digits = 5 ) )
print( round( vcov( fit3slsd[[ 1 ]]$e5e$eq[[ 2 ]] ), digits = 5 ) )


## *********** confidence intervals of coefficients *************
print( confint( fit3sls[[ 1 ]]$e1c, useDfSys = TRUE ) )
print( confint( fit3sls[[ 1 ]]$e1c$eq[[ 1 ]], level = 0.9, useDfSys = TRUE ) )

print( confint( fit3sls[[ 2 ]]$e2e, level = 0.9, useDfSys = TRUE ) )
print( confint( fit3sls[[ 2 ]]$e2e$eq[[ 2 ]], level = 0.99, useDfSys = TRUE ) )

print( confint( fit3sls[[ 3 ]]$e3, level = 0.99 ) )
print( confint( fit3sls[[ 3 ]]$e3$eq[[ 1 ]], level = 0.5 ) )

print( confint( fit3sls[[ 5 ]]$e3we, level = 0.99 ) )
print( confint( fit3sls[[ 5 ]]$e3we$eq[[ 1 ]], level = 0.5 ) )

print( confint( fit3sls[[ 4 ]]$e4e, level = 0.5, useDfSys = TRUE ) )
print( confint( fit3sls[[ 4 ]]$e4e$eq[[ 2 ]], level = 0.25, useDfSys = TRUE ) )

print( confint( fit3sls[[ 5 ]]$e5, level = 0.25 ) )
print( confint( fit3sls[[ 5 ]]$e5$eq[[ 1 ]], level = 0.975 ) )

print( confint( fit3slsi[[ 2 ]]$e3e, level = 0.975, useDfSys = TRUE ) )
print( confint( fit3slsi[[ 2 ]]$e3e$eq[[ 1 ]], level = 0.999, useDfSys = TRUE ) )

print( confint( fit3slsi[[ 1 ]]$e5w, level = 0.975, useDfSys = TRUE ) )
print( confint( fit3slsi[[ 1 ]]$e5w$eq[[ 1 ]], level = 0.999, useDfSys = TRUE ) )

print( confint( fit3slsd[[ 3 ]]$e4, level = 0.999 ) )
print( confint( fit3slsd[[ 3 ]]$e4$eq[[ 2 ]] ) )

print( confint( fit3slsd[[ 2 ]]$e4w, level = 0.999 ) )
print( confint( fit3slsd[[ 2 ]]$e4w$eq[[ 2 ]] ) )


## *********** fitted values *************
print( fitted( fit3sls[[ 2 ]]$e1c ) )
print( fitted( fit3sls[[ 2 ]]$e1c$eq[[ 1 ]] ) )

print( fitted( fit3sls[[ 1 ]]$e1wc ) )
print( fitted( fit3sls[[ 1 ]]$e1wc$eq[[ 1 ]] ) )

print( fitted( fit3sls[[ 3 ]]$e2e ) )
print( fitted( fit3sls[[ 3 ]]$e2e$eq[[ 2 ]] ) )

print( fitted( fit3sls[[ 4 ]]$e3 ) )
print( fitted( fit3sls[[ 4 ]]$e3$eq[[ 1 ]] ) )

print( fitted( fit3sls[[ 5 ]]$e4e ) )
print( fitted( fit3sls[[ 5 ]]$e4e$eq[[ 2 ]] ) )

print( fitted( fit3sls[[ 1 ]]$e5 ) )
print( fitted( fit3sls[[ 1 ]]$e5$eq[[ 1 ]] ) )

print( fitted( fit3slsi[[ 3 ]]$e3e ) )
print( fitted( fit3slsi[[ 3 ]]$e3e$eq[[ 1 ]] ) )

print( fitted( fit3slsd[[ 4 ]]$e4 ) )
print( fitted( fit3slsd[[ 4 ]]$e4$eq[[ 2 ]] ) )

print( fitted( fit3slsd[[ 2 ]]$e3w ) )
print( fitted( fit3slsd[[ 2 ]]$e3w$eq[[ 2 ]] ) )


## *********** predicted values *************
predictData <- Kmenta
predictData$consump <- NULL
predictData$price <- Kmenta$price * 0.9
predictData$income <- Kmenta$income * 1.1

print( predict( fit3sls[[ 2 ]]$e1c, se.fit = TRUE, interval = "prediction",
   useDfSys = TRUE ) )
print( predict( fit3sls[[ 2 ]]$e1c$eq[[ 1 ]], se.fit = TRUE, interval = "prediction",
   useDfSys = TRUE ) )

print( predict( fit3sls[[ 3 ]]$e2e, se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData, useDfSys = TRUE ) )
print( predict( fit3sls[[ 3 ]]$e2e$eq[[ 2 ]], se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData, useDfSys = TRUE ) )

print( predict( fit3sls[[ 5 ]]$e2w, se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData, useDfSys = TRUE ) )
print( predict( fit3sls[[ 5 ]]$e2w$eq[[ 2 ]], se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData, useDfSys = TRUE ) )

print( predict( fit3sls[[ 4 ]]$e3, se.pred = TRUE, interval = "prediction",
   level = 0.975 ) )
print( predict( fit3sls[[ 4 ]]$e3$eq[[ 1 ]], se.pred = TRUE, interval = "prediction",
   level = 0.975 ) )

print( predict( fit3sls[[ 5 ]]$e4e, se.fit = TRUE, interval = "confidence",
   level = 0.25, useDfSys = TRUE ) )
print( predict( fit3sls[[ 5 ]]$e4e$eq[[ 2 ]], se.fit = TRUE, interval = "confidence",
   level = 0.25, useDfSys = TRUE ) )

print( predict( fit3sls[[ 1 ]]$e5, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )
print( predict( fit3sls[[ 1 ]]$e5$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )

print( predict( fit3slsi[[ 3 ]]$e3e, se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99, useDfSys = TRUE ) )
print( predict( fit3slsi[[ 3 ]]$e3e$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99, useDfSys = TRUE ) )

print( predict( fit3slsi[[ 1 ]]$e5w, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )
print( predict( fit3slsi[[ 1 ]]$e5w$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )

print( predict( fit3slsd[[ 4 ]]$e4, se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )
print( predict( fit3slsd[[ 4 ]]$e4$eq[[ 2 ]], se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )

print( predict( fit3slsd[[ 2 ]]$e3w, se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99, useDfSys = TRUE ) )
print( predict( fit3slsd[[ 2 ]]$e3w$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99, useDfSys = TRUE ) )


# predict just one observation
smallData <- data.frame( price = 130, income = 150, farmPrice = 120,
   trend = 25 )

print( predict( fit3sls[[ 3 ]]$e1c, newdata = smallData ) )
print( predict( fit3sls[[ 3 ]]$e1c$eq[[ 1 ]], newdata = smallData ) )

print( predict( fit3sls[[ 4 ]]$e2e, se.fit = TRUE, level = 0.9,
   newdata = smallData ) )
print( predict( fit3sls[[ 5 ]]$e2e$eq[[ 1 ]], se.pred = TRUE, level = 0.99,
   newdata = smallData ) )

print( predict( fit3sls[[ 1]]$e3, interval = "prediction", level = 0.975,
   newdata = smallData ) )
print( predict( fit3sls[[ 1 ]]$e3$eq[[ 1 ]], interval = "confidence", level = 0.8,
   newdata = smallData ) )

print( predict( fit3sls[[ 4]]$e3we, interval = "prediction", level = 0.975,
   newdata = smallData ) )
print( predict( fit3sls[[ 4 ]]$e3we$eq[[ 1 ]], interval = "confidence", level = 0.8,
   newdata = smallData ) )

print( predict( fit3sls[[ 2 ]]$e4e, se.fit = TRUE, interval = "confidence",
   level = 0.999, newdata = smallData ) )
print( predict( fit3sls[[ 2 ]]$e4e$eq[[ 2 ]], se.pred = TRUE, interval = "prediction",
   level = 0.75, newdata = smallData ) )

print( predict( fit3sls[[ 3 ]]$e5, se.fit = TRUE, interval = "prediction",
   newdata = smallData ) )
print( predict( fit3sls[[ 3 ]]$e5$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   newdata = smallData ) )

print( predict( fit3slsi[[ 4 ]]$e3e, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fit3slsd[[ 5 ]]$e4$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )

print( predict( fit3slsd[[ 2 ]]$e2we, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fit3slsi[[ 3 ]]$e4we$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )


## ************ correlation of predicted values ***************
print( correlation.systemfit( fit3sls[[ 1 ]]$e1c, 2, 1 ) )

print( correlation.systemfit( fit3sls[[ 2 ]]$e2e, 1, 2 ) )

print( correlation.systemfit( fit3sls[[ 5 ]]$e2w, 2, 1 ) )

print( correlation.systemfit( fit3sls[[ 3 ]]$e3, 2, 1 ) )

print( correlation.systemfit( fit3sls[[ 4 ]]$e4e, 1, 2 ) )

print( correlation.systemfit( fit3sls[[ 5 ]]$e5, 2, 1 ) )

print( correlation.systemfit( fit3slsi[[ 2 ]]$e3e, 1, 2 ) )

print( correlation.systemfit( fit3slsi[[ 4 ]]$e5w, 1, 2 ) )

print( correlation.systemfit( fit3slsd[[ 3 ]]$e4, 2, 1 ) )


## ************ Log-Likelihood values ***************
print( logLik( fit3sls[[ 1 ]]$e1c ) )
print( logLik( fit3sls[[ 1 ]]$e1c, residCovDiag = TRUE ) )

print( logLik( fit3sls[[ 2 ]]$e2e ) )
print( logLik( fit3sls[[ 2 ]]$e2e, residCovDiag = TRUE ) )

print( logLik( fit3sls[[ 3 ]]$e3 ) )
print( logLik( fit3sls[[ 3 ]]$e3, residCovDiag = TRUE ) )

print( logLik( fit3sls[[ 4 ]]$e4e ) )
print( logLik( fit3sls[[ 4 ]]$e4e, residCovDiag = TRUE ) )

print( logLik( fit3sls[[ 2 ]]$e4wSym ) )
print( logLik( fit3sls[[ 2 ]]$e4wSym, residCovDiag = TRUE ) )

print( logLik( fit3sls[[ 5 ]]$e5 ) )
print( logLik( fit3sls[[ 5 ]]$e5, residCovDiag = TRUE ) )

print( logLik( fit3slsi[[ 2 ]]$e3e ) )
print( logLik( fit3slsi[[ 2 ]]$e3e, residCovDiag = TRUE ) )

print( logLik( fit3slsi[[ 1 ]]$e1we ) )
print( logLik( fit3slsi[[ 1 ]]$e1we, residCovDiag = TRUE ) )

print( logLik( fit3slsd[[ 3 ]]$e4 ) )
print( logLik( fit3slsd[[ 3 ]]$e4, residCovDiag = TRUE ) )

print( logLik( fit3slsd[[ 5 ]]$e2we ) )
print( logLik( fit3slsd[[ 5 ]]$e2we, residCovDiag = TRUE ) )


## ************** F tests ****************
# testing first restriction
print( linearHypothesis( fit3sls[[ 1 ]]$e1, restrm ) )
linearHypothesis( fit3sls[[ 1 ]]$e1, restrict )

print( linearHypothesis( fit3sls[[ 2 ]]$e1e, restrm ) )
linearHypothesis( fit3sls[[ 2 ]]$e1e, restrict )

print( linearHypothesis( fit3sls[[ 3 ]]$e1c, restrm ) )
linearHypothesis( fit3sls[[ 3 ]]$e1c, restrict )

print( linearHypothesis( fit3slsi[[ 4 ]]$e1, restrm ) )
linearHypothesis( fit3slsi[[ 4 ]]$e1, restrict )

print( linearHypothesis( fit3slsd[[ 5 ]]$e1e, restrm ) )
linearHypothesis( fit3slsd[[ 5 ]]$e1e, restrict )

print( linearHypothesis( fit3slsd[[ 2 ]]$e1w, restrm ) )
linearHypothesis( fit3slsd[[ 2 ]]$e1w, restrict )

# testing second restriction
restrOnly2m <- matrix(0,1,7)
restrOnly2q <- 0.5
restrOnly2m[1,2] <- -1
restrOnly2m[1,5] <-  1
restrictOnly2 <- "- demand_price + supply_price = 0.5"
# first restriction not imposed
print( linearHypothesis( fit3sls[[ 5 ]]$e1c, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3sls[[ 5 ]]$e1c, restrictOnly2 )

print( linearHypothesis( fit3slsi[[ 1 ]]$e1e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3slsi[[ 1 ]]$e1e, restrictOnly2 )

print( linearHypothesis( fit3slsi[[ 3 ]]$e1we, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3slsi[[ 3 ]]$e1we, restrictOnly2 )

print( linearHypothesis( fit3slsd[[ 2 ]]$e1, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3slsd[[ 2 ]]$e1, restrictOnly2 )

# first restriction imposed
print( linearHypothesis( fit3sls[[ 4 ]]$e2, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3sls[[ 4 ]]$e2, restrictOnly2 )

print( linearHypothesis( fit3sls[[ 4 ]]$e3, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3sls[[ 4 ]]$e3, restrictOnly2 )

print( linearHypothesis( fit3sls[[ 1 ]]$e2w, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3sls[[ 1 ]]$e2w, restrictOnly2 )

print( linearHypothesis( fit3sls[[ 1 ]]$e3we, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3sls[[ 1 ]]$e3we, restrictOnly2 )

print( linearHypothesis( fit3slsi[[ 5 ]]$e2e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3slsi[[ 5 ]]$e2e, restrictOnly2 )

print( linearHypothesis( fit3slsi[[ 5 ]]$e3e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3slsi[[ 5 ]]$e3e, restrictOnly2 )

print( linearHypothesis( fit3slsd[[ 1 ]]$e2, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3slsd[[ 1 ]]$e2, restrictOnly2 )

print( linearHypothesis( fit3slsd[[ 1 ]]$e3, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit3slsd[[ 1 ]]$e3, restrictOnly2 )

# testing both of the restrictions
print( linearHypothesis( fit3sls[[ 2 ]]$e1e, restr2m, restr2q ) )
linearHypothesis( fit3sls[[ 2 ]]$e1e, restrict2 )

print( linearHypothesis( fit3slsi[[ 3 ]]$e1, restr2m, restr2q ) )
linearHypothesis( fit3slsi[[ 3 ]]$e1, restrict2 )

print( linearHypothesis( fit3slsd[[ 4 ]]$e1e, restr2m, restr2q ) )
linearHypothesis( fit3slsd[[ 4 ]]$e1e, restrict2 )

print( linearHypothesis( fit3slsd[[ 5 ]]$e1w, restr2m, restr2q ) )
linearHypothesis( fit3slsd[[ 5 ]]$e1w, restrict2 )


## ************** Wald tests ****************
# testing first restriction
print( linearHypothesis( fit3sls[[ 1 ]]$e1, restrm, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 1 ]]$e1, restrict, test = "Chisq" )

print( linearHypothesis( fit3sls[[ 2 ]]$e1e, restrm, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 2 ]]$e1e, restrict, test = "Chisq" )

print( linearHypothesis( fit3sls[[ 3 ]]$e1c, restrm, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 3 ]]$e1c, restrict, test = "Chisq" )

print( linearHypothesis( fit3slsi[[ 4 ]]$e1, restrm, test = "Chisq" ) )
linearHypothesis( fit3slsi[[ 4 ]]$e1, restrict, test = "Chisq" )

print( linearHypothesis( fit3slsi[[ 2 ]]$e1we, restrm, test = "Chisq" ) )
linearHypothesis( fit3slsi[[ 2 ]]$e1we, restrict, test = "Chisq" )

print( linearHypothesis( fit3slsd[[ 5 ]]$e1e, restrm, test = "Chisq" ) )
linearHypothesis( fit3slsd[[ 5 ]]$e1e, restrict, test = "Chisq" )

# testing second restriction
# first restriction not imposed
print( linearHypothesis( fit3sls[[ 5 ]]$e1c, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 5 ]]$e1c, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3sls[[ 3 ]]$e1wc, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 3 ]]$e1wc, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3slsi[[ 1 ]]$e1e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3slsi[[ 1 ]]$e1e, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3slsd[[ 2 ]]$e1, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3slsd[[ 2 ]]$e1, restrictOnly2, test = "Chisq" )

# first restriction imposed
print( linearHypothesis( fit3sls[[ 4 ]]$e2, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 4 ]]$e2, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3sls[[ 4 ]]$e3, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 4 ]]$e3, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3slsi[[ 5 ]]$e2e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3slsi[[ 5 ]]$e2e, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3slsi[[ 5 ]]$e3e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3slsi[[ 5 ]]$e3e, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3slsd[[ 1 ]]$e2, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3slsd[[ 1 ]]$e2, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3slsd[[ 1 ]]$e3, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3slsd[[ 1 ]]$e3, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3slsd[[ 2 ]]$e2we, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3slsd[[ 2 ]]$e2we, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit3slsd[[ 3 ]]$e3w, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit3slsd[[ 3 ]]$e3w, restrictOnly2, test = "Chisq" )

# testing both of the restrictions
print( linearHypothesis( fit3sls[[ 2 ]]$e1e, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 2 ]]$e1e, restrict2, test = "Chisq" )

print( linearHypothesis( fit3sls[[ 5 ]]$e1wc, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fit3sls[[ 5 ]]$e1wc, restrict2, test = "Chisq" )

print( linearHypothesis( fit3slsi[[ 3 ]]$e1, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fit3slsi[[ 3 ]]$e1, restrict2, test = "Chisq" )

print( linearHypothesis( fit3slsd[[ 4 ]]$e1e, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fit3slsd[[ 4 ]]$e1e, restrict2, test = "Chisq" )


## *********** model frame *************
print( mf <- model.frame( fit3sls[[ 3 ]]$e1c ) )
print( mf1 <- model.frame( fit3sls[[ 3 ]]$e1c$eq[[ 1 ]] ) )
print( attributes( mf1 )$terms )
print( mf2 <- model.frame( fit3sls[[ 3 ]]$e1c$eq[[ 2 ]] ) )
print( attributes( mf2 )$terms )

print( all.equal( mf, model.frame( fit3sls[[ 3 ]]$e1wc ) ) )
print( all.equal( mf2, model.frame( fit3sls[[ 3 ]]$e1wc$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fit3sls[[ 4 ]]$e2e ) ) )
print( all.equal( mf2, model.frame( fit3sls[[ 4 ]]$e2e$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fit3sls[[ 5 ]]$e3 ) ) )
print( all.equal( mf1, model.frame( fit3sls[[ 5 ]]$e3$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fit3sls[[ 1 ]]$e4e ) ) )
print( all.equal( mf2, model.frame( fit3sls[[ 1 ]]$e4e$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fit3sls[[ 2 ]]$e5 ) ) )
print( all.equal( mf1, model.frame( fit3sls[[ 3 ]]$e5$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fit3slsi[[ 4 ]]$e3e ) ) )
print( all.equal( mf1, model.frame( fit3slsi[[ 4 ]]$e3e$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fit3slsd[[ 5 ]]$e4 ) ) )
print( all.equal( mf2, model.frame( fit3slsd[[ 5 ]]$e4$eq[[ 2 ]] ) ) )

fit3sls[[ 3 ]]$e1c$eq[[ 1 ]]$modelInst
fit3sls[[ 3 ]]$e1c$eq[[ 2 ]]$modelInst

fit3sls[[ 1 ]]$e3$eq[[ 1 ]]$modelInst
fit3sls[[ 1 ]]$e3$eq[[ 2 ]]$modelInst

fit3slsd[[ 5 ]]$e4$eq[[ 1 ]]$modelInst
fit3slsd[[ 5 ]]$e4$eq[[ 2 ]]$modelInst


## **************** model matrix ************************
# with x (returnModelMatrix) = TRUE
print( !is.null( fit3sls[[ 4 ]]$e1c$eq[[ 1 ]]$x ) )
print( mm <- model.matrix( fit3sls[[ 4 ]]$e1c ) )
print( mm1 <- model.matrix( fit3sls[[ 4 ]]$e1c$eq[[ 1 ]] ) )
print( mm2 <- model.matrix( fit3sls[[ 4 ]]$e1c$eq[[ 2 ]] ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit3sls[[ 4 ]]$e1wc ) ) )
print( all.equal( mm1, model.matrix( fit3sls[[ 4 ]]$e1wc$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3sls[[ 4 ]]$e1wc$eq[[ 2 ]] ) ) )
print( !is.null( fit3sls[[ 4 ]]$e1wc$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit3sls[[ 5 ]]$e2$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit3sls[[ 5 ]]$e2 ) ) )
print( all.equal( mm1, model.matrix( fit3sls[[ 5 ]]$e2$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3sls[[ 5 ]]$e2$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit3sls[[ 5 ]]$e2e ) ) )
print( all.equal( mm1, model.matrix( fit3sls[[ 5 ]]$e2e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3sls[[ 5 ]]$e2e$eq[[ 2 ]] ) ) )
print( !is.null( fit3sls[[ 5 ]]$e1wc$e2e[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit3sls[[ 1 ]]$e3e$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit3sls[[ 1 ]]$e3e ) ) )
print( all.equal( mm1, model.matrix( fit3sls[[ 1 ]]$e3e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3sls[[ 1 ]]$e3e$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit3sls[[ 1 ]]$e3 ) ) )
print( all.equal( mm1, model.matrix( fit3sls[[ 1 ]]$e3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3sls[[ 1 ]]$e3$eq[[ 2 ]] ) ) )
print( !is.null( fit3sls[[ 1 ]]$e3$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit3slsi[[ 2 ]]$e4$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit3slsi[[ 2 ]]$e4 ) ) )
print( all.equal( mm1, model.matrix( fit3slsi[[ 2 ]]$e4$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3slsi[[ 2 ]]$e4$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit3slsi[[ 2 ]]$e4we ) ) )
print( all.equal( mm1, model.matrix( fit3slsi[[ 2 ]]$e4we$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3slsi[[ 2 ]]$e4we$eq[[ 2 ]] ) ) )
print( !is.null( fit3slsi[[ 2 ]]$e1wc$e4we[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit3slsi[[ 5 ]]$e5w$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit3slsi[[ 5 ]]$e5w ) ) )
print( all.equal( mm1, model.matrix( fit3slsi[[ 5 ]]$e5w$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3slsi[[ 5 ]]$e5w$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit3slsi[[ 5 ]]$e5 ) ) )
print( all.equal( mm1, model.matrix( fit3slsi[[ 5 ]]$e5$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3slsi[[ 5 ]]$e5$eq[[ 2 ]] ) ) )
print( !is.null( fit3slsi[[ 5 ]]$e5$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit3slsd[[ 3 ]]$e5e$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit3slsd[[ 3 ]]$e5e ) ) )
print( all.equal( mm1, model.matrix( fit3slsd[[ 3 ]]$e5e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3slsd[[ 3 ]]$e5e$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit3slsd[[ 3 ]]$e5we ) ) )
print( all.equal( mm1, model.matrix( fit3slsd[[ 3 ]]$e5we$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3slsd[[ 3 ]]$e5we$eq[[ 2 ]] ) ) )
print( !is.null( fit3sls[[ 3 ]]$e5we$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit3slsd[[ 2 ]]$e3w$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit3slsd[[ 2 ]]$e3w ) ) )
print( all.equal( mm1, model.matrix( fit3slsd[[ 2 ]]$e3w$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3slsd[[ 2 ]]$e3w$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit3slsd[[ 2 ]]$e3 ) ) )
print( all.equal( mm1, model.matrix( fit3slsd[[ 2 ]]$e3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit3slsd[[ 2 ]]$e3$eq[[ 2 ]] ) ) )
print( !is.null( fit3slsd[[ 2 ]]$e3$eq[[ 1 ]]$x ) )

# matrices of instrumental variables
model.matrix( fit3sls[[ 1 ]]$e1c, which = "z" )
model.matrix( fit3sls[[ 3 ]]$e1c$eq[[ 1 ]], which = "z" )
model.matrix( fit3sls[[ 4 ]]$e1c$eq[[ 2 ]], which = "z" )

# matrices of fitted regressors
model.matrix( fit3slsd[[ 1 ]]$e3w, which = "xHat" )
model.matrix( fit3slsd[[ 3 ]]$e3w$eq[[ 1 ]], which = "xHat" )
model.matrix( fit3slsd[[ 4 ]]$e3w$eq[[ 2 ]], which = "xHat" )


## **************** formulas ************************
formula( fit3sls[[ 2 ]]$e1c )
formula( fit3sls[[ 2 ]]$e1c$eq[[ 1 ]] )

formula( fit3sls[[ 3 ]]$e2e )
formula( fit3sls[[ 3 ]]$e2e$eq[[ 2 ]] )

formula( fit3sls[[ 4 ]]$e3 )
formula( fit3sls[[ 4 ]]$e3$eq[[ 1 ]] )

formula( fit3sls[[ 5 ]]$e4e )
formula( fit3sls[[ 5 ]]$e4e$eq[[ 2 ]] )

formula( fit3sls[[ 1 ]]$e5 )
formula( fit3sls[[ 1 ]]$e5$eq[[ 1 ]] )

formula( fit3slsi[[ 3 ]]$e3e )
formula( fit3slsi[[ 3 ]]$e3e$eq[[ 1 ]] )

formula( fit3slsd[[ 4 ]]$e4 )
formula( fit3slsd[[ 4 ]]$e4$eq[[ 2 ]] )

formula( fit3slsd[[ 2 ]]$e1w )
formula( fit3slsd[[ 2 ]]$e1w$eq[[ 1 ]] )


## **************** model terms *******************
terms( fit3sls[[ 2 ]]$e1c )
terms( fit3sls[[ 2 ]]$e1c$eq[[ 1 ]] )

terms( fit3sls[[ 3 ]]$e2e )
terms( fit3sls[[ 3 ]]$e2e$eq[[ 2 ]] )

terms( fit3sls[[ 4 ]]$e3 )
terms( fit3sls[[ 4 ]]$e3$eq[[ 1 ]] )

terms( fit3sls[[ 5 ]]$e4e )
terms( fit3sls[[ 5 ]]$e4e$eq[[ 2 ]] )

terms( fit3sls[[ 1 ]]$e5 )
terms( fit3sls[[ 1 ]]$e5$eq[[ 1 ]] )

terms( fit3sls[[ 2 ]]$e4wSym )
terms( fit3sls[[ 2 ]]$e4wSym$eq[[ 1 ]] )

terms( fit3slsi[[ 3 ]]$e3e )
terms( fit3slsi[[ 3 ]]$e3e$eq[[ 1 ]] )

terms( fit3slsd[[ 4 ]]$e4 )
terms( fit3slsd[[ 4 ]]$e4$eq[[ 2 ]] )

terms( fit3slsd[[ 5 ]]$e5we )
terms( fit3slsd[[ 5 ]]$e5we$eq[[ 2 ]] )


## **************** terms of instruments *******************
fit3sls[[ 2 ]]$e1c$eq[[ 1 ]]$termsInst

fit3sls[[ 3 ]]$e2e$eq[[ 2 ]]$termsInst

fit3sls[[ 4 ]]$e3$eq[[ 1 ]]$termsInst

fit3sls[[ 5 ]]$e4e$eq[[ 2 ]]$termsInst

fit3sls[[ 1 ]]$e5$eq[[ 1 ]]$termsInst

fit3sls[[ 2 ]]$e4wSym$eq[[ 1 ]]$termsInst

fit3slsi[[ 3 ]]$e3e$eq[[ 1 ]]$termsInst

fit3slsd[[ 4 ]]$e4$eq[[ 2 ]]$termsInst

fit3slsd[[ 5 ]]$e5we$eq[[ 2 ]]$termsInst


## **************** estfun ************************
library( "sandwich" )

estfun( fit3sls[[ 1 ]]$e1 )
round( colSums( estfun( fit3sls[[ 1 ]]$e1 ) ), digits = 7 )

estfun( fit3sls[[ 2 ]]$e1e )
round( colSums( estfun( fit3sls[[ 2 ]]$e1e ) ), digits = 7 )

estfun( fit3sls[[ 3 ]]$e1c )
round( colSums( estfun( fit3sls[[ 3 ]]$e1c ) ), digits = 7 )

estfun( fit3sls[[ 4 ]]$e1wc )

round( colSums( estfun( fit3sls[[ 5 ]]$e1wc ) ), digits = 7 )
round( colSums( estfun( fit3sls[[ 5 ]]$e1wc, residFit = FALSE ) ), digits = 7 )

round( colSums( estfun( fit3sls[[ 4 ]]$e1wc ) ), digits = 7 )
round( colSums( estfun( fit3sls[[ 4 ]]$e1wc, residFit = FALSE ) ), digits = 7 )

round( colSums( estfun( fit3sls[[ 3 ]]$e1wc ) ), digits = 7 )
round( colSums( estfun( fit3sls[[ 3 ]]$e1wc, residFit = FALSE ) ), digits = 7 )

round( colSums( estfun( fit3sls[[ 2 ]]$e1wc ) ), digits = 7 )
round( colSums( estfun( fit3sls[[ 2 ]]$e1wc, residFit = FALSE ) ), digits = 7 )

round( colSums( estfun( fit3sls[[ 1 ]]$e1wc ) ), digits = 7 )
round( colSums( estfun( fit3sls[[ 1 ]]$e1wc, residFit = FALSE ) ), digits = 7 )

estfun( fit3slsd[[ 5 ]]$e1w )
estfun( fit3slsd[[ 5 ]]$e1w, residFit = FALSE )

round( colSums( estfun( fit3slsd[[ 5 ]]$e1w ) ), digits = 7 )
round( colSums( estfun( fit3slsd[[ 5 ]]$e1w, residFit = FALSE ) ), digits = 7 )

round( colSums( estfun( fit3slsd[[ 4 ]]$e1w ) ), digits = 7 )
round( colSums( estfun( fit3slsd[[ 4 ]]$e1w, residFit = FALSE ) ), digits = 7 )

round( colSums( estfun( fit3slsd[[ 3 ]]$e1w ) ), digits = 7 )
round( colSums( estfun( fit3slsd[[ 3 ]]$e1w, residFit = FALSE ) ), digits = 7 )

round( colSums( estfun( fit3slsd[[ 2 ]]$e1w ) ), digits = 7 )
round( colSums( estfun( fit3slsd[[ 2 ]]$e1w, residFit = FALSE ) ), digits = 7 )

round( colSums( estfun( fit3slsd[[ 1 ]]$e1w ) ), digits = 7 )
round( colSums( estfun( fit3slsd[[ 1 ]]$e1w, residFit = FALSE ) ), digits = 7 )


## **************** bread ************************
bread( fit3sls[[ 1 ]]$e1 )

bread( fit3sls[[ 2 ]]$e1e )

bread( fit3sls[[ 3 ]]$e1c )

bread( fit3sls[[ 4 ]]$e1wc )

bread( fit3slsd[[ 5 ]]$e1w )

bread( fit3slsd[[ 4 ]]$e1w )

bread( fit3slsd[[ 3 ]]$e1w )

bread( fit3slsd[[ 2 ]]$e1w )

bread( fit3slsd[[ 1 ]]$e1w )

createSystemfitModel <- function( nEq, nRegEq, nObs, coef = NULL, sigma = NULL ){

   result <- list()
   nCoef <- nEq * ( nRegEq + 1 )

   if( is.null( coef ) ){
      coef <- round( rnorm( nCoef ), 1 )
   } else {
      if( length( coef ) != nCoef ){
         stop( "argument 'coef' must have length ", nCoef )
      }
   }

   if( is.null( sigma ) ){
      sigma <- matrix( rnorm( nEq^2 ), nEq, nEq )
      sigma <- crossprod( sigma )
   } else {
      if( all.equal( dim( sigma ), c( nEq, nEq ) ) != TRUE ){
         stop( "argument 'sigma' must be a ", nEq, "x", nEq, " matrix" )
      }
   }


   disturbances <- mvrnorm( nObs, rep( 0, nEq ), sigma )
   result$data <- data.frame( obsNo = c( 1:nObs ) )
   result$formula <- list()

   for( eqNo in 1:nEq ){
      result$formula[[ eqNo ]] <- as.formula( paste( "y.", eqNo, " ~ ",
         paste( paste( "x.", eqNo, ".", sep = "" ),
         c( 1:nRegEq ), sep = "", collapse = " + " ), sep = "" ) )
      for( regNo in 1:nRegEq ){
         result$data[[ paste( "x", eqNo, regNo, sep = "." ) ]] <-
            rnorm( nObs )
      }
      result$data[[ paste( "y", eqNo, sep = "." ) ]] <- 0
      xMatEq <- model.matrix( result$formula[[ eqNo ]], result$data )
      result$data[[ paste( "y", eqNo, sep = "." ) ]] <-
         drop( xMatEq %*% coef[ ( ( eqNo - 1 ) * ( nRegEq + 1 ) + 1 ):
            ( eqNo * ( nRegEq + 1 ) ) ] ) + disturbances[ , eqNo ]
   }

   result$coef <- coef
   result$sigma <- sigma

   return( result )
}

.onAttach <- function( lib, pkg ) {
   packageStartupMessage(
      paste0( "\nPlease cite the 'systemfit' package as:\n",
         "Arne Henningsen and Jeff D. Hamann (2007). ",
         "systemfit: A Package for Estimating Systems of Simultaneous Equations in R. ",
         "Journal of Statistical Software 23(4), 1-40. ",
         "http://www.jstatsoft.org/v23/i04/.\n\n",
         "If you have questions, suggestions, or comments ",
         "regarding the 'systemfit' package, ",
         "please use a forum or 'tracker' at systemfit's R-Forge site:\n",
         "https://r-forge.r-project.org/projects/systemfit/"),
      domain = NULL,  appendLF = TRUE )
}

# .onLoad <- function( lib, pkg ) {
#    options(Matrix.warnDeprecatedCoerce = 2) # where n >= 1
# }

systemfit.control <- function(
      maxiter = 1,
      tol = 1e-5,
      methodResidCov = "geomean",
      centerResiduals = FALSE,
      residCovRestricted = TRUE,
      residCovWeighted = FALSE,
      method3sls = "GLS",
      singleEqSigma = NULL,
      useMatrix = TRUE,
      solvetol = .Machine$double.eps,
      model = TRUE,
      x = FALSE,
      y = FALSE,
      z = FALSE )
{
   result <- list()

   ## maxiter
   if( maxiter <= 0 || round( maxiter ) != maxiter ) {
      stop( "control parameter 'maxiter' must be a positive integer" )
   }
   result$maxiter <- maxiter

   ## tol
   if( tol <= 0 || !is.numeric( tol ) || length( tol ) != 1 ) {
      stop( "control parameter 'tol' must be a positive scalar" )
   }
   result$tol <- tol

   ## methodResidCov
   if( !( methodResidCov %in% c( "noDfCor", "geomean", "max", "Theil" ) ) ) {
      stop( "control parameter 'methodResidCov' must be either",
         " 'noDfCor', 'geomean', 'max', or 'Theil'" )
   }
   result$methodResidCov <- methodResidCov

   ## centerResiduals
   if( !is.logical( centerResiduals ) || length( centerResiduals ) != 1 ) {
      stop( "control parameter 'centerResiduals' must be logical" )
   }
   result$centerResiduals <- centerResiduals

   ## method3sls
   if( !( method3sls %in% c( "GLS", "IV", "GMM", "Schmidt", "EViews" ) ) ) {
      stop( "control parameter 'method3sls' must be either",
         " 'GLS', 'IV', 'GMM', 'Schmidt', or 'EViews'" )
   }
   result$method3sls <- method3sls

   ## singleEqSigma
   if( ( !is.logical( singleEqSigma ) || length( singleEqSigma ) != 1 )
         && !is.null( singleEqSigma ) ) {
      stop( "control parameter 'singleEqSigma' must be logical or NULL" )
   }
   result$singleEqSigma <- singleEqSigma

   ## useMatrix
   if( !is.logical( useMatrix ) || length( useMatrix ) != 1 ) {
      stop( "control parameter 'useMatrix' must be logical" )
   }
   result$useMatrix <- useMatrix

   ## solvetol
   if( solvetol <= 0 || !is.numeric( solvetol ) || length( solvetol ) != 1 ) {
      stop( "control parameter 'solvetol' must be a positive scalar" )
   }
   result$solvetol <- solvetol

   ## residCovRestricted
   if( !is.logical( residCovRestricted ) || length( residCovRestricted ) != 1 ) {
      stop( "control parameter 'residCovRestricted' must be logical" )
   }
   result$residCovRestricted <- residCovRestricted

   ## residCovWeighted
   if( !is.logical( residCovWeighted ) || length( residCovWeighted ) != 1 ) {
      stop( "control parameter 'residCovWeighted' must be logical" )
   }
   result$residCovWeighted <- residCovWeighted

   ## model (returnModelFrame)
   if( !is.logical( model ) || length( model ) != 1 ) {
      stop( "control parameter 'model' must be logical" )
   }
   result$model <- model

   ## x (returnModelMatrix)
   if( !is.logical( x ) || length( x ) != 1 ) {
      stop( "control parameter 'x' must be logical" )
   }
   result$x <- x

   ## y (returnResponse)
   if( !is.logical( y ) || length( y ) != 1 ) {
      stop( "control parameter 'y' must be logical" )
   }
   result$y <- y

   ## z (returnInstMatrix)
   if( !is.logical( z ) || length( z ) != 1 ) {
      stop( "control parameter 'z' must be logical" )
   }
   result$z <- z

   return( result )
}

.calcXtOmegaInv <- function( xMat, sigma, validObsEq, invertSigma = TRUE,
      useMatrix = FALSE, warnMatrix = TRUE, solvetol = 1e-5 ){

   nEq <- ncol( validObsEq )

   if( useMatrix && warnMatrix ){
      if( !inherits( sigma, "dsyMatrix" ) ){
         warning( "class of 'sigma' is '", 
            paste( class( sigma ), collapse = ", " ),
            "', but it should be 'dsyMatrix'" )
      }
      if( !inherits( xMat,"dgCMatrix" ) ){
         warning( "class of 'xMat' is '", 
            paste( class( xMat ), collapse = ", " ),
            "', but it should be 'dgCMatrix'" )
      }
   }

   if( invertSigma ) {
      sigmaInv <- solve( sigma, tol = solvetol )
   } else {
      sigmaInv <- sigma
   }

   if( useMatrix ){
      for( i in 1:nEq ) {
         for( j in 1:nEq ) {
            thisBlock <- sparseMatrix(
               i = which( validObsEq[ validObsEq[ , i ], j ] ),
               j = which( validObsEq[ validObsEq[ , j ], i ] ),
               x = sigmaInv[ i, j ],
               dims = c( sum( validObsEq[ , i ] ), sum( validObsEq[ , j ] ) ) )
            if( j == 1 ) {
               thisRow <- thisBlock
            } else {
               thisRow <- cbind( thisRow, thisBlock )
            }
         }
         if( i == 1 ) {
            omegaInv <- thisRow
         } else {
            omegaInv <- rbind( omegaInv, thisRow )
         }
      }
      result <- crossprod( xMat, omegaInv )
   } else {
      eqSelect <- rep( 0, nrow( xMat ) )
      for( i in 1:nEq ) {
         eqSelect[ ( sum( validObsEq[ , 0:( i - 1 ) ] ) + 1 ):sum( validObsEq[ , 1:i ] ) ] <- i
      }
      result <- matrix( 0, nrow = ncol( xMat ), ncol = nrow( xMat ) )
      for( i in 1:nEq ) {
         for( j in 1:nEq ) {
            colSelectI <- eqSelect == i
            colSelectI[ colSelectI ] <- validObsEq[ validObsEq[ , i ], j ]
            colSelectJ <- eqSelect == j
            colSelectJ[ colSelectJ ] <- validObsEq[ validObsEq[ , j ], i ]
            result[ , colSelectI ] <- result[ , colSelectI ] +
               t( xMat )[ , colSelectJ ] * sigmaInv[ i, j ]
         }
      }
   }

   return( result )
}

.calcGLS <- function( xMat, yVec = NULL, xMat2 = xMat, R.restr = NULL,
      q.restr = NULL, sigma, validObsEq, useMatrix = TRUE, warnMatrix = TRUE,
      solvetol = 1e-5 ){

   if( useMatrix && warnMatrix ){
      if( !inherits( xMat, "dgCMatrix" ) ){
         warning( "class of 'xMat' is '", 
            paste( class( xMat ), collapse = ", " ),
            "', but it should be 'dgCMatrix'" )
      }
      if( !inherits( xMat2, "dgCMatrix" ) ){
         warning( "class of 'xMat2' is '", 
            paste( class( xMat2 ), collapse = ", " ),
            "', but it should be 'dgCMatrix'" )
      }
      if( !inherits( sigma, "dsyMatrix" ) ){
         warning( "class of 'sigma' is '", 
            paste( class( sigma ), collapse = ", " ),
            "', but it should be 'dsyMatrix'" )
      }
   }

   xtOmegaInv <- .calcXtOmegaInv( xMat = xMat, sigma = sigma, validObsEq = validObsEq,
      useMatrix = useMatrix, solvetol = solvetol )
   if( is.null( R.restr ) ) {
      if( is.null( yVec ) ) {
         result <- solve( xtOmegaInv %*% xMat2, tol = solvetol )
      } else {
         result <- as.numeric( solve( xtOmegaInv %*% xMat2, xtOmegaInv %*% yVec,
            tol = solvetol ) )
      }
   } else {
      W <- rbind2( cbind2( xtOmegaInv %*% xMat2, t(R.restr) ),
                  cbind2( R.restr, matrix(0, nrow(R.restr), nrow(R.restr) )))
      if( is.null( yVec ) ) {
         result <- as.matrix(
            solve( W, tol=solvetol )[ 1:ncol(xMat), 1:ncol(xMat) ] )
      } else{
         V <- c( as.numeric( xtOmegaInv %*% yVec ), q.restr )
         result <- solve( W, V, tol=solvetol )[ 1:ncol( xMat ) ]
      }
   }
   return( result )
}

## calculate confidence intervals of the coefficients
confint.systemfit <- function( object, parm = NULL, level = 0.95,
      useDfSys = NULL, ... ) {

   if( is.null( useDfSys ) ) {
      useDfSys <- length( coef( object ) ) != object$rank
         # TRUE if there are restrictions imposed
   }

   probLower <- ( 1 - level ) / 2
   probBoth <- c( probLower, 1 - probLower )
   pct <- paste( round( 100 * probBoth, 1 ), "%" )
   ci <- matrix( NA, length( object$coefficients ), 2,
            dimnames = list( names( object$coefficients ), pct ) )
   j <- 1
   for( i in 1:length( object$eq ) ) {
      object$eq[[i]]$df.residual.sys <- object$df.residual
      ci[ j:(j+length( coef( object$eq[[ i ]] ) )-1), ] <- confint( object$eq[[ i ]],
         useDfSys = useDfSys )
      j <- j + length( coef( object$eq[[ i ]] ) )
   }
   class( ci ) <- "confint.systemfit"
   ci
}

## calculate confidence intervals of the coefficients of a single equation
confint.systemfit.equation <- function( object, parm = NULL, level = 0.95,
   useDfSys = NULL, ... ) {

   if( is.null( useDfSys ) ) {
      useDfSys <- object$nCoef.sys != object$rank.sys
         # TRUE if there are restrictions imposed
   }

   probLower <- ( 1 - level ) / 2
   probBoth <- c( probLower, 1 - probLower )
   pct <- paste( round( 100 * probBoth, 1 ), "%" )
   ci <- matrix( NA, length( object$coefficients ), 2,
            dimnames = list( names( object$coefficients ), pct ) )
   if( useDfSys ) {
      fac <- qt( probBoth, object$df.residual.sys )
   } else {
      fac <- qt( probBoth, object$df.residual )
   }
   coef <- summary( object )$coefficients
   ci[] <- coef[ , 1 ] + coef[ , 2 ] %o% fac
   class( ci ) <- "confint.systemfit"
   ci
}

## print the confidence intervals of the coefficients
print.confint.systemfit <- function( x, digits = 3, ... ) {
   print( unclass( round( x, digits = digits, ...) ) )
   invisible(x)
}

## Likelihood Ratio Test
lrtest.systemfit <- function( object, ... ) {

   thisCall <- match.call()

   if( !inherits( object, "systemfit" ) ){
      stop( "argument 'object' must be of class 'systemfit'" )
   }
   object$lrtest.systemfit.name <- deparse( substitute( object ) )
   objectList <- list( ... )
   if( length( objectList ) < 1 ){
      stop( "at least one further argument ('...') must be provided" )
   }
   if( !all( sapply( objectList, function(x) inherits( x, "systemfit" ) ) ) ){
      stop( "all further arguments ('...') must be of class 'systemfit'" )
   }
   dotsNames <- as.list( thisCall )[ -1 ]
   dotsNames$object <- NULL
   for( i in 1:length( objectList ) ){
      objectList[[ i ]]$lrtest.systemfit.name <- deparse( dotsNames[[ i ]] )
   }
   extractName <- function( object ){
      return( object$lrtest.systemfit.name )
   }

   result <- do.call( lrtest.default,
      c( list( object = object ), objectList, list( name = extractName ) ) )

   for( i in 2:nrow( result ) ){
      if( ( result[ i, "#Df" ] - result[ i - 1, "#Df" ] ) *
            ( result[ i, "LogLik" ] - result[ i - 1, "LogLik" ] ) < 0 ) {
         if( result[ i, "LogLik" ] > result[ i - 1, "LogLik" ] ) {
            compareLikelihood <- "larger"
            compareDf <- "less"
         } else {
            compareLikelihood <- "smaller"
            compareDf <- "more"
         }
         warning( "model '", i, "' has a ", compareLikelihood,
            " log-likelihood value than the ", compareDf,
            " restricted model '", i - 1, "'" )
      }
   }

   return( result )
}

.systemfitPanel <- function( formula, inst, data, pooled ) {


  if (inherits( data, "pdata.frame" )) {
       # current panel data format from pkg plm
       index <- index( data )
       eqnVar  <- names( index )[1]
       timeVar <- names( index )[2]
       
       # pdata.frames do not necessarily carry the index variables as columns.
       # attach index vars to data, if not in data (pdata.frame(..., drop.index = TRUE))
       if (! (eqnVar %in% colnames(data))) {
         data <- cbind(data, index)
       }
    } else {
      if (inherits( data, "plm.dim" )) {
        # deprecated panel data format from pkg plm
        eqnVar  <- names( data )[1]
        timeVar <- names( data )[2]
      } else {
        stop( "argument 'data' must be of class 'pdata.frame'",
              " (created with 'pdata.frame')" )
        }
   }
   
   result <- list()
  
   data[[ eqnVar ]] <- gsub( " |_", ".", data[[ eqnVar ]] )
   eqnLabels <- levels( as.factor( data[[ eqnVar ]] ) )
   nEqn <- length( eqnLabels )
   timeLabels <- levels( as.factor( data[[ timeVar ]] ) )
   nRegressors <- ncol( model.matrix( formula, data ) )
   eqnSystem <- list()

   wideData <- data.frame( time = timeLabels )
   rownames( wideData ) <- make.names( timeLabels )
   for( eqnNo in 1:nEqn ) {
      eqn <- eqnLabels[ eqnNo ]
      endogVar <- formula[2]
      exogVar <- formula[3]
      eqnData <- data[ data[[ eqnVar ]] == eqn, ]
      for( var in all.vars( formula ) ) {
         newVar <- paste( eqn, var, sep = "_" )
         newVar <- make.names( newVar )
         wideData[[ newVar ]] <- NA
         wideData[ make.names( eqnData[ , timeVar ] ), newVar ] <-
            eqnData[ , var ]
         endogVar <- gsub( paste0( "\\b", var, "\\b" ), newVar, endogVar )
         exogVar <- gsub( paste0( "\\b", var, "\\b" ), newVar, exogVar )
      }
      eqnSystem[[ eqnNo ]] <- as.formula( paste( endogVar, "~", exogVar ) )
   }
   names( eqnSystem ) <- eqnLabels
   #reshape( data, idvar=timevar, timevar=eqnVar,direction="wide")

   if( is.null( inst ) ) {
      instSystem <- NULL
   } else {   
      instSystem <- list()
      for( eqnNo in 1:nEqn ) {
         eqn <- eqnLabels[ eqnNo ]
         instVar <- inst[2]
         eqnData <- data[ data[[ eqnVar ]] == eqn, ]
         for( var in all.vars( inst ) ) {
            newVar <- paste( eqn, var, sep = "_" )
            newVar <- make.names( newVar )
            wideData[[ newVar ]] <- NA
            wideData[ make.names( eqnData[ , timeVar ] ), newVar ] <-
               eqnData[ , var ]
            instVar <- gsub( paste0( "\\b", var, "\\b" ), newVar, instVar )
         }
         instSystem[[ eqnNo ]] <- as.formula( paste( "~", instVar ) )
      }
      names( instSystem ) <- eqnLabels
   }
   
   restrict.regMat <- NULL
   if( pooled ) {
      for( eqnNo in 1:nEqn ) {
         restrict.regMat <- rbind( restrict.regMat, diag( 1, nRegressors ) )
      }
   }

   result$eqnSystem <- eqnSystem
   result$instSystem <- instSystem
   result$wideData  <- wideData
   result$restrict.regMat <- restrict.regMat

   return( result )
}

calcFittedRegMat<- function( xMatAll, zMatEq, nEq, nObsEq, useMatrix, solvetol ) {
   # fitted values of regressors for IV estimations
      xMatHatEq <- list()
      for(i in 1:nEq) {
         # rows that belong to the ith equation
         rowsEq <- c( (1+sum(nObsEq[1:i])-nObsEq[i]):(sum(nObsEq[1:i])) )
         # extract instrument matrix
         xMatAllThisEq <- xMatAll[ rowsEq, ]
         if( useMatrix ){
            xMatAllThisEq <- as( xMatAllThisEq, "denseMatrix")
         }
         xMatHatEq[[ i ]] <- zMatEq[[i]] %*%
            solve( crossprod( zMatEq[[i]] ),
            crossprod( zMatEq[[i]], xMatAllThisEq ), tol = solvetol )
      }
      # fitted values of all regressors
      xMatHatAll <- .stackMatList( xMatHatEq, way = "below",
         useMatrix = useMatrix )

      return( xMatHatAll )
}

## Calculate the residual covariance matrix
.calcResidCov <- function( resids, methodResidCov, validObsEq = NULL,
      nCoefEq = NULL, xEq = NULL, diag = FALSE, centered = FALSE,
      useMatrix = FALSE, solvetol = .Machine$double.eps ) {

   eqNames <- NULL
   if( inherits( resids, "data.frame" ) ) {
      resids <- as.matrix( resids )
      validObsEq <- !is.na( resids )
      eqNames <- names( resids )
   } else if( !is.null( validObsEq ) ) {
      residMat <- matrix( NA, nrow = nrow( validObsEq ),
         ncol = ncol( validObsEq ) )
      for( i in 1:ncol( validObsEq ) ) {
         residMat[ validObsEq[ , i ], i ] <- resids[
            ( 1 + sum( validObsEq[ , 0:(i-1) ] ) ):( sum(validObsEq[ , 1:i ] ) ) ]
      }
      resids <- residMat
      rm( residMat )
   } else {
      stop( "internal error in .calcResidCov: if argument 'validObsEq'",
         " is not provided, argument 'resids' must be a data.frame'" )
   }

   nEq <- ncol( validObsEq )
   result <- matrix( 0, nEq, nEq )
   if( centered ) {
      for( i in 1:nEq ) {
         resids[ , i ] <- resids[ , i ] - mean( resids[ validObsEq[ , i ], i ] )
      }
   }
   validObsAll <- rowSums( !validObsEq ) == 0
   for( i in 1:nEq ) {
      for( j in ifelse( diag, i, 1 ):ifelse( diag, i, nEq ) ) {
         if( methodResidCov == "noDfCor" ) {
            result[ i, j ] <-
               sum( resids[ validObsAll, i ] * resids[ validObsAll, j ] ) /
               sum( validObsAll )
         } else if( methodResidCov == "geomean" ) {
            result[ i, j ] <-
               sum( resids[ validObsAll, i ] * resids[ validObsAll, j ] ) /
               sqrt( ( sum( validObsAll ) - nCoefEq[i] ) * ( sum( validObsAll ) - nCoefEq[j] ) )
         } else if( methodResidCov == "Theil" ) {
            #result[ i, j ] <- sum( residi[[i]] * residi[[j]] ) /
            #   ( sum( validObsAll ) - nCoefEq[i] - nCoefEq[j] + sum( diag(
            #   xEq[[i]] %*% solve( crossprod( xEq[[i]] ), tol=solvetol ) %*%
            #   crossprod( xEq[[i]], xEq[[j]]) %*%
            #   solve( crossprod( xEq[[j]] ), tol=solvetol ) %*%
            #   t( xEq[[j]] ) ) ) )
            result[ i, j ] <-
               sum( resids[ validObsAll, i ] * resids[ validObsAll, j ] ) /
               ( sum( validObsAll ) - nCoefEq[i] - nCoefEq[j] + sum( diag(
               solve( crossprod( xEq[[i]] ), tol=solvetol ) %*%
               crossprod( xEq[[i]], xEq[[j]]) %*%
               solve( crossprod( xEq[[j]] ), tol=solvetol ) %*%
               crossprod( xEq[[j]], xEq[[i]] ) ) ) )

         } else if( methodResidCov == "max" ) {
            result[ i, j ] <-
               sum( resids[ validObsAll, i ] * resids[ validObsAll, j ] ) /
               ( sum( validObsAll ) - max( nCoefEq[ i ], nCoefEq[ j ] ) )
         } else {
            stop( paste( "Argument 'methodResidCov' must be either 'noDfCor',",
                  "'geomean', 'max', or 'Theil'." ) )
         }
      }
   }
   if( !is.null( eqNames ) ) {
      rownames( result ) <- eqNames
      colnames( result ) <- eqNames
   }

   if( useMatrix ){
      result <- as( result, "symmetricMatrix" )
   }
   return( result )
}

## Calculate Sigma squared
.calcSigma2 <- function( resids, methodResidCov, nObs, nCoef ) {
   if( methodResidCov == "noDfCor" ) {
      result <- sum( resids^2 ) / nObs
   } else if( methodResidCov %in% c( "geomean", "max" ) ){
      result <- sum( resids^2 )/ ( nObs - nCoef )
   } else {
      stop( paste( "Sigma^2 can only be calculated if argument",
         "'methodResidCov' is either 'noDfCor', 'geomean', or 'max'" ) )
   }
}

## return model matrix of the entire system
model.matrix.systemfit <- function( object, which = "x", ... ){
   result <- matrix( NA, 0, 0 )
   mmRowNames <- NULL
   mmColNames <- NULL
   for( i in 1:length( object$eq ) ) {
      mmi <- model.matrix( object$eq[[ i ]], which = which ) 
      result <- rbind(
         cbind( result, matrix( 0, nrow( result ), ncol( mmi ) ) ),
         cbind( matrix( 0, nrow( mmi ), ncol( result ) ), mmi ) )
      mmRowNames <- c( mmRowNames,
         paste( object$eq[[ i ]]$eqnLabel, "_", rownames( mmi ), sep = "" ) )
      for( j in 1:ncol( mmi ) ){
         cName <- colnames( mmi )[ j ]
         if( object$panelLike && cName != "(Intercept)" ){
            mmColNames <- c( mmColNames, cName )
         } else {
            mmColNames <- c( mmColNames,
               paste( object$eq[[ i ]]$eqnLabel, "_", cName, sep = "" ) )
         }
      }
   }
   rownames( result ) <- mmRowNames
   colnames( result ) <- mmColNames
   return( result )
}

## return model matrix of a single equation
model.matrix.systemfit.equation <- function( object, which = "x", ... ){
   if( ! which %in% c( "x", "xHat", "z" ) ) {
      stop( "argument 'which' must be either \"x\", \"xHat\", or \"z\"" )
   } else if( which %in% c( "xHat", "z" ) && is.null( object$termsInst ) ) {
      stop( "argument 'which' can only be set to \"xHat\" or \"z\" if instruments were used" )
   }
   if( which == "xHat" ) {
      xMat <- model.matrix( object, which = "x" )
      zMat <- model.matrix( object, which = "z" )
      res <- residuals( object )
      if( sum( !is.na( res ) ) != nrow( xMat ) ) {
         stop( "internal error: number of non-NA residuals not equal to",
            " number of observations in xMat. Please contact the maintainer" )
      } else if( nrow( xMat ) != nrow( zMat) ) {
         stop( "internal error: number of observations in xMat is not equal to",
            " number of observations in zMat. Please contact the maintainer" )
      }
      result <- zMat %*% solve( crossprod( zMat ), crossprod( zMat, xMat ) )
   } else {
      if( !is.null( object[[ which ]] ) ) {
         result <- object[[ which ]]
      } else if( !is.null( model.frame( object ) ) ) {
         if( which == "x" ) {
            result <- model.matrix( object$terms, data = model.frame( object ) )
         } else {
            result <- model.matrix( object$termsIns, data = object$modelInst )
         }
         attrAssign <- attributes( result )$assign
         result <- result[ !is.na( residuals( object ) ), , drop = FALSE ]
         attributes( result )$assign <- attrAssign
      } else {
         if( which == "x" ) {
            stop( "returning model matrix not possible. Please re-estimate",
               " the system with either control variable",
               " 'x' or 'model' set to TRUE" )
         } else {
            stop( "returning matrix of instruments not possible. Please re-estimate",
               " the system with either control variable",
               " 'z' or 'model' set to TRUE" )
         }
      }
   }

   return( result )
}

bread.systemfit <- function ( x, ... ) {
   if( !is.null( x$restrict.matrix ) || !is.null( x$restrict.rhs ) ||
      !is.null( x$restrict.regMat ) ) {
      stop( "returning the 'bread' for models with restrictions",
            " has not yet been implemented.")
   }
   
   # model matrix
   if( is.null( x$eq[[1]]$inst ) ) {
      mm <- model.matrix( x )
   } else {
      mm <- model.matrix( x, which = "xHat" )
   }
   
   if( is.null( x$residCovEst ) ) {
      omegaInvXmat <- mm
   } else {
      omegaInvXmat <- t( .calcXtOmegaInv( xMat = mm, sigma = x$residCovEst, 
         validObsEq = !is.na( residuals( x ) ), invertSigma = TRUE ) )
   }
   
   result <- solve( crossprod( mm, omegaInvXmat ) / nrow( mm ) )
   
   return( result )
}

## print a few results of the whole system
print.systemfit <- function( x,
      digits = max( 3, getOption("digits") - 1 ),... ) {

   cat("\n")
   cat("systemfit results \n")
   cat("method: ")
   if(!is.null(x$iter)) if(x$iter>1) cat("iterated ")
   cat( paste( x$method, "\n\n"))
   if(!is.null(x$iter)) {
      if(x$iter>1) {
         if(x$iter<x$control$maxiter) {
            cat( paste( "convergence achieved after",x$iter,"iterations\n\n" ) )
         } else {
            cat( paste( "warning: convergence not achieved after", x$iter,
                        "iterations\n\n" ) )
         }
      }
   }
   cat( "Coefficients:\n" )
   print( x$coefficients, digits = digits )
   invisible( x )
}


## print a few results for a single equation
print.systemfit.equation <- function( x,
      digits = max( 3, getOption("digits") - 1 ), ... ) {

   cat("\n")
   cat( x$method, " estimates for '", x$eqnLabel,
            "' (equation ", x$eqnNo, ")\n", sep = "" )

   cat("Model Formula: ")
   print( formula( x$terms ) )
   if(!is.null(x$inst)) {
      cat("Instruments: ")
      print(x$inst)
   }

   cat("\nCoefficients:")
   print( x$coefficients, digits = digits )
   invisible( x )
}

## Likelihood function for FIML estimations
.systemfitFimlLik <- function( mlCoef, mlVars ) {

   nObs <- sum( mlVars$nObsEq )
   nEq <- length( mlVars$nObsEq )

   if( length( mlCoef ) != ncol( mlVars$xMat ) ){
      stop( "internal error: argument 'mlCoef' has length ",
         length( mlCoef ), " but must have length ", ncol( mlVars$xMat ) )
   }
   mlResids <- mlVars$yVec - mlVars$xMat %*% mlCoef
   mlSigma <- .calcResidCov( resids = mlResids, methodResidCov = mlVars$methodResidCov,
      validObsEq = mlVars$validObsEq, nCoefEq = mlVars$nCoefEq, xEq = mlVars$xEq,
      centered = mlVars$centerResiduals, diag = FALSE,
      solvetol = mlVars$solvetol )
   likValue <- - nObs * log( 2 * pi ) -
      ( nObs / 2 ) * log( det( mlSigma ) ) -
      ( 1 / 2 ) * .calcXtOmegaInv( mlResids, mlSigma, validObsEq = mlVars$validObsEq,
         solvetol = mlVars$solvetol ) %*% mlResids
   return( likValue )
}

## FIML estimation
.systemfitFiml <- function( systemfitCall, nObsEq, validObsEq, nCoefEq, yVec, xMat, xEq,
      methodResidCov, centerResiduals, solvetol, startCoef = "ITSUR" ) {

   nObs <- sum( nObsEq )
   nEq <- length( nObsEq )

   # starting values
   if( startCoef %in% c( "OLS", "SUR", "ITSUR" ) ) {
      if( startCoef == "OLS" ) {
         systemfitCall$method <- "OLS"
      } else if( startCoef == "SUR" ) {
         systemfitCall$method <- "SUR"
         systemfitCall$maxiter <- 1
      } else if( startCoef == "ITSUR" ) {
         systemfitCall$method <- "SUR"
         systemfitCall$maxiter <- 500
      }
      startResult <- eval( systemfitCall )
      startCoef <- startResult$b
   }

   # variables passed to .systemfitFimlLik
   mlVars <- list()
   mlVars$nObsEq  <- nObsEq
   mlVars$validObsEq <- validObsEq
   mlVars$nCoefEq <- nCoefEq
   mlVars$yVec    <- yVec
   mlVars$xMat    <- xMat
   mlVars$xEq     <- xEq
   mlVars$methodResidCov  <- methodResidCov
   mlVars$centerResiduals <- centerResiduals
   mlVars$sollvetol       <- solvetol

   # list for results of ML estimation
   mlResult <- list()

   mlResult$optim <- optim( startCoef, .systemfitFimlLik, method = "BFGS",
      control = list( fnscale = -1 ), mlVars = mlVars )
   # methods: "BFGS" "Nelder-Mead"
   mlResult$coefficients <- mlResult$optim$par
   mlResult$coefCov <- diag( 1, length( mlResult$coefficients ) )
   mlResult$resids  <- yVec - xMat %*% mlResult$coefficients

   return( mlResult )
}

library( systemfit )
options( digits = 3 )

data( "Kmenta" )
useMatrix <- FALSE

demand <- consump ~ price + income
supply <- consump ~ price + farmPrice + trend
system <- list( demand = demand, supply = supply )
restrm <- matrix(0,1,7)  # restriction matrix "R"
restrm[1,3] <-  1
restrm[1,7] <- -1
restrict <- "demand_income - supply_trend = 0"
restr2m <- matrix(0,2,7)  # restriction matrix "R" 2
restr2m[1,3] <-  1
restr2m[1,7] <- -1
restr2m[2,2] <- -1
restr2m[2,5] <-  1
restr2q <- c( 0, 0.5 )  # restriction vector "q" 2
restrict2 <- c( "demand_income - supply_trend = 0",
   "- demand_price + supply_price = 0.5" )
restrict2i <- c( "demand_income - supply_trend = 0",
   "- demand_price + supply_income = 0.5" )
tc <- matrix(0,7,6)
tc[1,1] <- 1
tc[2,2] <- 1
tc[3,3] <- 1
tc[4,4] <- 1
tc[5,5] <- 1
tc[6,6] <- 1
tc[7,3] <- 1
restr3m <- matrix(0,1,6)  # restriction matrix "R" 2
restr3m[1,2] <- -1
restr3m[1,5] <-  1
restr3q <- c( 0.5 )  # restriction vector "q" 2
restrict3 <- "- C2 + C5 = 0.5"

# the standard equations do not converge and lead to a singular weighting matrix
# both in R and in EViews, since both equations have the same endogenous variable
supply2 <- price ~ income + farmPrice + trend
system2 <- list( demand = demand, supply = supply2 )


## *************** SUR estimation ************************
fitsur1 <- systemfit( system, "SUR", data = Kmenta, useMatrix = useMatrix )
print( summary( fitsur1 ) )
nobs( fitsur1 )

## ********************* SUR (EViews-like) *****************
fitsur1e <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   useMatrix = useMatrix )
print( summary( fitsur1e, useDfSys = TRUE ) )
nobs( fitsur1e )

## ********************* SUR (methodResidCov="Theil") *****************
fitsur1r2 <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "Theil",
   useMatrix = useMatrix )
print( summary( fitsur1r2 ) )

## *************** SUR (methodResidCov="Theil", useDfSys = TRUE ) ***************
fitsur1e2 <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "Theil",
   x = TRUE, useMatrix = useMatrix )
print( summary( fitsur1e2, useDfSys = TRUE ) )

## ********************* SUR (methodResidCov="max") *****************
fitsur1r3 <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "max",
   useMatrix = useMatrix )
print( summary( fitsur1r3 ) )

## *************** WSUR estimation ************************
fitsur1w <- systemfit( system, "SUR", data = Kmenta, residCovWeighted = TRUE,
   useMatrix = useMatrix )
summary( fitsur1w )
nobs( fitsur1w )

## *************** WSUR (methodResidCov="Theil", useDfSys = TRUE ) ***************
fitsur1we2 <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "Theil",
   residCovWeighted = TRUE, useMatrix = useMatrix )
summary( fitsur1we2, useDfSys = TRUE )


## *************** SUR with cross-equation restriction **************
fitsur2 <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restrm,
   useMatrix = useMatrix )
print( summary( fitsur2 ) )
nobs( fitsur2 )
# the same with symbolically specified restrictions
fitsur2Sym <- systemfit( system, "SUR", data = Kmenta,
   restrict.matrix = restrict, useMatrix = useMatrix )
all.equal( fitsur2, fitsur2Sym )
nobs( fitsur2Sym )

## *************** SUR with cross-equation restriction (EViews-like) **
fitsur2e <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restrm,
   methodResidCov = "noDfCor", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitsur2e ) )

## *************** WSUR with cross-equation restriction (EViews-like) **
fitsur2we <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restrm,
   methodResidCov = "noDfCor", residCovWeighted = TRUE,
   x = TRUE, useMatrix = useMatrix )
summary( fitsur2we )


## *************** SUR with restriction via restrict.regMat *******************
fitsur3 <- systemfit( system, "SUR", data = Kmenta, restrict.regMat = tc,
   useMatrix = useMatrix )
print( summary( fitsur3 ) )
nobs( fitsur3 )

## *************** SUR with restriction via restrict.regMat (EViews-like) **************
fitsur3e <- systemfit( system, "SUR", data = Kmenta, restrict.regMat = tc,
   methodResidCov = "noDfCor", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitsur3e ) )

## *************** WSUR with restriction via restrict.regMat *******************
fitsur3w <- systemfit( system, "SUR", data = Kmenta, restrict.regMat = tc,
   residCovWeighted = TRUE, x = TRUE, useMatrix = useMatrix )
summary( fitsur3w )


## *************** SUR with 2 restrictions ***************************
fitsur4 <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, useMatrix = useMatrix )
print( summary( fitsur4 ) )
nobs( fitsur4 )
# the same with symbolically specified restrictions
fitsur4Sym <- systemfit( system, "SUR", data = Kmenta,
   restrict.matrix = restrict2, useMatrix = useMatrix )
all.equal( fitsur4, fitsur4Sym )

## *************** SUR with 2 restrictions (EViews-like) **************
fitsur4e <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr2m, restrict.rhs = restr2q, useMatrix = useMatrix )
print( summary( fitsur4e ) )

## *************** SUR with 2 restrictions (methodResidCov = "Theil") **************
fitsur4r2 <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "Theil",
   restrict.matrix = restr2m, restrict.rhs = restr2q, useMatrix = useMatrix )
print( summary( fitsur4r2 ) )

## *************** SUR with 2 restrictions (methodResidCov = "max") **************
fitsur4r3 <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "max",
   restrict.matrix = restr2m, restrict.rhs = restr2q,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitsur4r3 ) )

## *************** WSUR with 2 restrictions (EViews-like) **************
fitsur4we <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr2m, restrict.rhs = restr2q, residCovWeighted = TRUE,
   useMatrix = useMatrix )
summary( fitsur4we )


## *************** SUR with 2 restrictions via R and restrict.regMat ****************
fitsur5 <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitsur5 ) )
nobs( fitsur5 )
# the same with symbolically specified restrictions
fitsur5Sym <- systemfit( system, "SUR", data = Kmenta,
   restrict.matrix = restrict3, restrict.regMat = tc,
   x = TRUE, useMatrix = useMatrix )
all.equal( fitsur5, fitsur5Sym )

## *************** SUR with 2 restrictions via R and restrict.regMat (EViews-like) **************
fitsur5e <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   useMatrix = useMatrix )
print( summary( fitsur5e ) )

## ************ WSUR with 2 restrictions via R and restrict.regMat ************
fitsur5w <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, residCovWeighted = TRUE,
   useMatrix = useMatrix )
summary( fitsur5w )


## ************** iterated SUR ****************************
fitsuri1 <- systemfit( system2, "SUR", data = Kmenta, maxit = 100,
   useMatrix = useMatrix )
print( summary( fitsuri1 ) )
nobs( fitsuri1 )

## ************** iterated SUR (EViews-like) *****************
fitsuri1e <- systemfit( system2, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   maxit = 100, useMatrix = useMatrix )
print( summary( fitsuri1e, useDfSys = TRUE ) )

## ************** iterated SUR (methodResidCov = "Theil") ****************************
fitsuri1r2 <- systemfit( system2, "SUR", data = Kmenta, maxit = 100,
   methodResidCov = "Theil", useMatrix = useMatrix )
print( summary( fitsuri1r2 ) )

## ************** iterated SUR (methodResidCov="Theil", useDfSys=TRUE) *****************
fitsuri1e2 <- systemfit( system2, "SUR", data = Kmenta, methodResidCov = "Theil",
   maxit = 100, x = TRUE, useMatrix = useMatrix )
print( summary( fitsuri1e2, useDfSys = TRUE ) )

## ************** iterated SUR (methodResidCov = "max") ****************************
fitsuri1r3 <- systemfit( system2, "SUR", data = Kmenta, maxit = 100,
   methodResidCov = "max", useMatrix = useMatrix )
print( summary( fitsuri1r3 ) )

## ************** iterated WSUR (methodResidCov = "max") ****************************
fitsuri1wr3 <- systemfit( system2, "SUR", data = Kmenta, maxit = 100,
   methodResidCov = "max", residCovWeighted = TRUE, useMatrix = useMatrix )
summary( fitsuri1wr3 )


## *********** iterated SUR with restriction *******************
fitsuri2 <- systemfit( system2, "SUR", data = Kmenta, restrict.matrix = restrm,
   maxit = 100, useMatrix = useMatrix )
print( summary( fitsuri2 ) )

## *********** iterated SUR with restriction (EViews-like) ***************
fitsuri2e <- systemfit( system2, "SUR", data = Kmenta, restrict.matrix = restrm,
   methodResidCov = "noDfCor", maxit = 100, x = TRUE,
   useMatrix = useMatrix )
print( summary( fitsuri2e ) )

## *********** iterated WSUR with restriction *******************
fitsuri2w <- systemfit( system2, "SUR", data = Kmenta, restrict.matrix = restrm,
   maxit = 100, residCovWeighted = TRUE, useMatrix = useMatrix )
summary( fitsuri2w )


## *********** iterated SUR with restriction via restrict.regMat ********************
fitsuri3 <- systemfit( system2, "SUR", data = Kmenta, restrict.regMat = tc,
   maxit = 100, useMatrix = useMatrix )
print( summary( fitsuri3 ) )

## *********** iterated SUR with restriction via restrict.regMat (EViews-like) ***************
fitsuri3e <- systemfit( system2, "SUR", data = Kmenta, restrict.regMat = tc,
   methodResidCov = "noDfCor", maxit = 100, x = TRUE,
   useMatrix = useMatrix )
print( summary( fitsuri3e ) )

## *********** iterated WSUR with restriction via restrict.regMat (EViews-like) ***************
fitsuri3we <- systemfit( system2, "SUR", data = Kmenta, restrict.regMat = tc,
   methodResidCov = "noDfCor", maxit = 100, residCovWeighted = TRUE,
   useMatrix = useMatrix )
summary( fitsuri3we )


## *************** iterated SUR with 2 restrictions ***************************
fitsurio4 <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, maxit = 100, useMatrix = useMatrix )
print( summary( fitsurio4 ) )
fitsuri4 <- systemfit( system2, "SUR", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, maxit = 100, useMatrix = useMatrix )
print( summary( fitsuri4 ) )

## *************** iterated SUR with 2 restrictions (EViews-like) **************
fitsurio4e <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr2m, restrict.rhs = restr2q, maxit = 100,
   useMatrix = useMatrix )
print( summary( fitsurio4e ) )
fitsuri4e <- systemfit( system2, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr2m, restrict.rhs = restr2q, maxit = 100,
   useMatrix = useMatrix )
print( summary( fitsuri4e ) )

## *************** iterated WSUR with 2 restrictions ***************************
fitsurio4w <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, maxit = 100, residCovWeighted = TRUE,
   useMatrix = useMatrix )
summary( fitsurio4w )
fitsuri4w <- systemfit( system2, "SUR", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, maxit = 100, residCovWeighted = TRUE,
   useMatrix = useMatrix )
summary( fitsuri4w )


## *************** iterated SUR with 2 restrictions via R and restrict.regMat ****************
fitsurio5 <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, maxit = 100,
   useMatrix = useMatrix )
print( summary( fitsurio5 ) )
fitsuri5 <- systemfit( system2, "SUR", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, maxit = 100,
   useMatrix = useMatrix )
print( summary( fitsuri5 ) )

## ********* iterated SUR with 2 restrictions via R and restrict.regMat (EViews-like) **********
fitsurio5e <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   maxit = 100, useMatrix = useMatrix )
print( summary( fitsurio5e ) )
fitsuri5e <- systemfit( system2, "SUR", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   maxit = 100, useMatrix = useMatrix )
print( summary( fitsuri5e ) )
nobs( fitsuri5e )

## ********* iterated SUR with 2 restrictions via R and restrict.regMat (methodResidCov="Theil") **********
fitsurio5r2 <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "Theil",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   maxit = 100, x = TRUE, useMatrix = useMatrix )
print( summary( fitsurio5r2 ) )
fitsuri5r2 <- systemfit( system2, "SUR", data = Kmenta, methodResidCov = "Theil",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   maxit = 100, x = TRUE, useMatrix = useMatrix )
print( summary( fitsuri5r2 ) )

## ********* iterated SUR with 2 restrictions via R and restrict.regMat (methodResidCov="max") **********
# fitsuri5e <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "max",
#    restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
#    maxit = 100, useMatrix = useMatrix )
# print( summary( fitsuri5e ) )
# print( round( vcov( fitsuri5e ), digits = 6 ) )
# disabled, because the estimation does not converge

## ********* iterated WSUR with 2 restrictions via R and restrict.regMat (methodResidCov="Theil") **********
fitsurio5wr2 <- systemfit( system, "SUR", data = Kmenta, methodResidCov = "Theil",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   maxit = 100, residCovWeighted = TRUE, useMatrix = useMatrix )
summary( fitsurio5wr2 )
fitsuri5wr2 <- systemfit( system2, "SUR", data = Kmenta, methodResidCov = "Theil",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   maxit = 100, residCovWeighted = TRUE, useMatrix = useMatrix )
summary( fitsuri5wr2 )


## *********** estimations with a single regressor ************
fitsurS1 <- systemfit(
   list( price ~ consump - 1, farmPrice ~ consump + trend ), "SUR",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitsurS1 ) )
nobs( fitsurS1 )
fitsurS2 <- systemfit(
   list( consump ~ price - 1, consump ~ trend - 1 ), "SUR",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitsurS2 ) )
nobs( fitsurS2 )
fitsurS3 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "SUR",
   data = Kmenta, useMatrix = useMatrix )
nobs( fitsurS3 )
print( summary( fitsurS3 ) )
fitsurS4 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "SUR",
   data = Kmenta, restrict.matrix = matrix( c( 1, -1 ), nrow = 1 ),
   useMatrix = useMatrix )
print( summary( fitsurS4 ) )
nobs( fitsurS4 )
fitsurS5 <- systemfit(
   list( consump ~ 1, price ~ 1 ), "SUR",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitsurS5 ) )
nobs( fitsurS5 )


## **************** shorter summaries **********************
print( summary( fitsur1e2, useDfSys = TRUE, equations = FALSE ) )

print( summary( fitsur2e, useDfSys = FALSE, residCov = FALSE ) )

print( summary( fitsur3 ), equations = FALSE )

print( summary( fitsur4r3 ), residCov = FALSE, equations = FALSE )

print( summary( fitsur5, residCov = FALSE ), equations = FALSE )

print( summary( fitsur5w, equations = FALSE, residCov = FALSE ),
   equations = TRUE )

print( summary( fitsuri1r3, useDfSys = FALSE ), residCov = FALSE )

print( summary( fitsuri2 ), residCov = FALSE )

print( summary( fitsuri3e, residCov = FALSE, equations = FALSE ) )

print( summary( fitsurio4, residCov = FALSE ), equations = FALSE )
print( summary( fitsuri4, equations = FALSE ), residCov = FALSE )

print( summary( fitsuri4w, useDfSys = FALSE, equations = FALSE ) )

print( summary( fitsurio5r2, equations = FALSE ) )
print( summary( fitsuri5r2 ), residCov = FALSE )


## ****************** residuals **************************
print( residuals( fitsur1e2 ) )
print( residuals( fitsur1e2$eq[[ 2 ]] ) )

print( residuals( fitsur1w ) )
print( residuals( fitsur1w$eq[[ 2 ]] ) )

print( residuals( fitsur2e ) )
print( residuals( fitsur2e$eq[[ 1 ]] ) )

print( residuals( fitsur3 ) )
print( residuals( fitsur3$eq[[ 2 ]] ) )

print( residuals( fitsur4r3 ) )
print( residuals( fitsur4r3$eq[[ 1 ]] ) )

print( residuals( fitsur5 ) )
print( residuals( fitsur5$eq[[ 2 ]] ) )

print( residuals( fitsuri1r3 ) )
print( residuals( fitsuri1r3$eq[[ 1 ]] ) )

print( residuals( fitsuri2 ) )
print( residuals( fitsuri2$eq[[ 2 ]] ) )

print( residuals( fitsuri3e ) )
print( residuals( fitsuri3e$eq[[ 1 ]] ) )

print( residuals( fitsurio4 ) )
print( residuals( fitsurio4$eq[[ 2 ]] ) )
print( residuals( fitsuri4 ) )
print( residuals( fitsuri4$eq[[ 2 ]] ) )

print( residuals( fitsuri4w ) )
print( residuals( fitsuri4w$eq[[ 2 ]] ) )

print( residuals( fitsurio5r2 ) )
print( residuals( fitsurio5r2$eq[[ 1 ]] ) )
print( residuals( fitsuri5r2 ) )
print( residuals( fitsuri5r2$eq[[ 1 ]] ) )


## *************** coefficients *********************
print( round( coef( fitsur1r3 ), digits = 6 ) )
print( round( coef( fitsur1r3$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitsuri2 ), digits = 6 ) )
print( round( coef( fitsuri2$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitsur2we ), digits = 6 ) )
print( round( coef( fitsur2we$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitsur3 ), digits = 6 ) )
print( round( coef( fitsur3, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitsur3$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitsur4r2 ), digits = 6 ) )
print( round( coef( fitsur4r2$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitsuri5e ), digits = 6 ) )
print( round( coef( fitsuri5e, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitsuri5e$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitsur5w ), digits = 6 ) )
print( round( coef( fitsur5w, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitsur5w$eq[[ 1 ]] ), digits = 6 ) )


## *************** coefficients with stats *********************
print( round( coef( summary( fitsur1r3 ) ), digits = 6 ) )
print( round( coef( summary( fitsur1r3$eq[[ 2 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitsuri2, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitsuri2$eq[[ 1 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fitsur3 ) ), digits = 6 ) )
print( round( coef( summary( fitsur3 ), modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fitsur3$eq[[ 2 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitsuri3we ) ), digits = 6 ) )
print( round( coef( summary( fitsuri3we ), modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fitsuri3we$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitsur4r2 ) ), digits = 6 ) )
print( round( coef( summary( fitsur4r2$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitsur4we ) ), digits = 6 ) )
print( round( coef( summary( fitsur4we$eq[[ 2 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitsuri5e, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitsuri5e, useDfSys = FALSE ),
   modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fitsuri5e$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )


## *********** variance covariance matrix of the coefficients *******
print( round( vcov( fitsur1e2 ), digits = 6 ) )
print( round( vcov( fitsur1e2$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitsur1r3 ), digits = 6 ) )
print( round( vcov( fitsur1r3$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitsur2e ), digits = 6 ) )
print( round( vcov( fitsur2e$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitsur3 ), digits = 6 ) )
print( round( vcov( fitsur3, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitsur3$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitsur3w ), digits = 6 ) )
print( round( vcov( fitsur3w, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitsur3w$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitsur4r2 ), digits = 6 ) )
print( round( vcov( fitsur4r2$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitsur5e ), digits = 6 ) )
print( round( vcov( fitsur5e, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitsur5e$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitsuri1r3 ), digits = 6 ) )
print( round( vcov( fitsuri1r3$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitsuri2 ), digits = 6 ) )
print( round( vcov( fitsuri2$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitsuri3e ), digits = 6 ) )
print( round( vcov( fitsuri3e, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitsuri3e$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitsurio4e ), digits = 6 ) )
print( round( vcov( fitsurio4e$eq[[ 2 ]] ), digits = 6 ) )
print( round( vcov( fitsuri4e ), digits = 6 ) )
print( round( vcov( fitsuri4e$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitsurio5r2 ), digits = 6 ) )
print( round( vcov( fitsurio5r2, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitsurio5r2$eq[[ 1 ]] ), digits = 6 ) )
print( round( vcov( fitsuri5r2 ), digits = 6 ) )
print( round( vcov( fitsuri5r2, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitsuri5r2$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitsurio5wr2 ), digits = 6 ) )
print( round( vcov( fitsurio5wr2, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitsurio5wr2$eq[[ 2 ]] ), digits = 6 ) )


## *********** confidence intervals of coefficients *************
print( confint( fitsur1e2, useDfSys = TRUE ) )
print( confint( fitsur1e2$eq[[ 2 ]], level = 0.9, useDfSys = TRUE ) )

print( confint( fitsur1we2, useDfSys = TRUE ) )
print( confint( fitsur1we2$eq[[ 1 ]], level = 0.9, useDfSys = TRUE ) )

print( confint( fitsur2e, level = 0.9 ) )
print( confint( fitsur2e$eq[[ 1 ]], level = 0.99 ) )

print( confint( fitsur3, level = 0.99 ) )
print( confint( fitsur3$eq[[ 2 ]], level = 0.5 ) )

print( confint( fitsur4r3, level = 0.5 ) )
print( confint( fitsur4r3$eq[[ 1 ]], level = 0.25 ) )

print( confint( fitsur5, level = 0.25 ) )
print( confint( fitsur5$eq[[ 2 ]], level = 0.975 ) )

print( confint( fitsuri1r3, level = 0.975 ) )
print( confint( fitsuri1r3$eq[[ 1 ]], level = 0.999 ) )

print( confint( fitsuri2, level = 0.999 ) )
print( confint( fitsuri2$eq[[ 2 ]], level = 0.1 ) )

print( confint( fitsuri3e, level = 0.1 ) )
print( confint( fitsuri3e$eq[[ 1 ]], level = 0.01 ) )

print( confint( fitsurio4, level = 0.01 ) )
print( confint( fitsurio4$eq[[ 2 ]], level = 0.33 ) )
print( confint( fitsuri4, level = 0.01 ) )
print( confint( fitsuri4$eq[[ 2 ]], level = 0.33 ) )

print( confint( fitsurio4w, level = 0.01 ) )
print( confint( fitsurio4w$eq[[ 1 ]], level = 0.33 ) )

print( confint( fitsurio5r2, level = 0.33 ) )
print( confint( fitsurio5r2$eq[[ 1 ]] ) )
print( confint( fitsuri5r2, level = 0.33 ) )
print( confint( fitsuri5r2$eq[[ 1 ]] ) )


## *********** fitted values *************
print( fitted( fitsur1e2 ) )
print( fitted( fitsur1e2$eq[[ 2 ]] ) )

print( fitted( fitsur2e ) )
print( fitted( fitsur2e$eq[[ 1 ]] ) )

print( fitted( fitsur2we ) )
print( fitted( fitsur2we$eq[[ 2 ]] ) )

print( fitted( fitsur3 ) )
print( fitted( fitsur3$eq[[ 2 ]] ) )

print( fitted( fitsur4r3 ) )
print( fitted( fitsur4r3$eq[[ 1 ]] ) )

print( fitted( fitsur5 ) )
print( fitted( fitsur5$eq[[ 2 ]] ) )

print( fitted( fitsuri1r3 ) )
print( fitted( fitsuri1r3$eq[[ 1 ]] ) )

print( fitted( fitsuri1wr3 ) )
print( fitted( fitsuri1wr3$eq[[ 2 ]] ) )

print( fitted( fitsuri2 ) )
print( fitted( fitsuri2$eq[[ 2 ]] ) )

print( fitted( fitsuri3e ) )
print( fitted( fitsuri3e$eq[[ 1 ]] ) )

print( fitted( fitsurio4 ) )
print( fitted( fitsurio4$eq[[ 2 ]] ) )
print( fitted( fitsuri4 ) )
print( fitted( fitsuri4$eq[[ 2 ]] ) )

print( fitted( fitsurio5r2 ) )
print( fitted( fitsurio5r2$eq[[ 1 ]] ) )
print( fitted( fitsuri5r2 ) )
print( fitted( fitsuri5r2$eq[[ 1 ]] ) )


## *********** predicted values *************
predictData <- Kmenta
predictData$consump <- NULL
predictData$price <- Kmenta$price * 0.9
predictData$income <- Kmenta$income * 1.1

print( predict( fitsur1e2, se.fit = TRUE, interval = "prediction",
   useDfSys = TRUE ) )
print( predict( fitsur1e2$eq[[ 2 ]], se.fit = TRUE, interval = "prediction",
   useDfSys = TRUE ) )

print( predict( fitsur2e, se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )
print( predict( fitsur2e$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )

print( predict( fitsur3, se.pred = TRUE, interval = "prediction",
   level = 0.975 ) )
print( predict( fitsur3$eq[[ 2 ]], se.pred = TRUE, interval = "prediction",
   level = 0.975 ) )

print( predict( fitsur4r3, se.fit = TRUE, interval = "confidence",
   level = 0.25 ) )
print( predict( fitsur4r3$eq[[ 1 ]], se.fit = TRUE, interval = "confidence",
   level = 0.25 ) )

print( predict( fitsur4we, se.fit = TRUE, interval = "confidence",
   level = 0.25 ) )
print( predict( fitsur4we$eq[[ 2 ]], se.fit = TRUE, interval = "confidence",
   level = 0.25 ) )

print( predict( fitsur5, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )
print( predict( fitsur5$eq[[ 2 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )

print( predict( fitsuri1r3, se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99 ) )
print( predict( fitsuri1r3$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99 ) )

print( predict( fitsuri2, se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )
print( predict( fitsuri2$eq[[ 2 ]], se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )

print( predict( fitsuri2w, se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )
print( predict( fitsuri2w$eq[[ 2 ]], se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )

print( predict( fitsuri3e, interval = "prediction", level = 0.925 ) )
print( predict( fitsuri3e$eq[[ 1 ]], interval = "prediction", level = 0.925 ) )

print( predict( fitsurio4, interval = "confidence", newdata = predictData ) )
print( predict( fitsurio4$eq[[ 2 ]], interval = "confidence",
   newdata = predictData ) )
print( predict( fitsuri4, interval = "confidence", newdata = predictData ) )
print( predict( fitsuri4$eq[[ 2 ]], interval = "confidence",
   newdata = predictData ) )

print( predict( fitsurio5r2 ) )
print( predict( fitsurio5r2$eq[[ 1 ]] ) )
print( predict( fitsuri5r2 ) )
print( predict( fitsuri5r2$eq[[ 1 ]] ) )

# predict just one observation
smallData <- data.frame( price = 130, income = 150, farmPrice = 120,
   trend = 25 )

print( predict( fitsur1e2, newdata = smallData ) )
print( predict( fitsur1e2$eq[[ 1 ]], newdata = smallData ) )

print( predict( fitsur2e, se.fit = TRUE, level = 0.9,
   newdata = smallData ) )
print( predict( fitsur2e$eq[[ 1 ]], se.pred = TRUE, level = 0.99,
   newdata = smallData ) )

print( predict( fitsur3, interval = "prediction", level = 0.975,
   newdata = smallData ) )
print( predict( fitsur3$eq[[ 1 ]], interval = "confidence", level = 0.8,
   newdata = smallData ) )

print( predict( fitsur4r3, se.fit = TRUE, interval = "confidence",
   level = 0.999, newdata = smallData ) )
print( predict( fitsur4r3$eq[[ 2 ]], se.pred = TRUE, interval = "prediction",
   level = 0.75, newdata = smallData ) )

print( predict( fitsur5, se.fit = TRUE, interval = "prediction",
   newdata = smallData ) )
print( predict( fitsur5$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   newdata = smallData ) )

print( predict( fitsurio5r2, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fitsurio5r2$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )
print( predict( fitsuri5r2, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fitsuri5r2$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )

print( predict( fitsuri5wr2, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fitsuri5wr2$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )


## ************ correlation of predicted values ***************
print( correlation.systemfit( fitsur1e2, 2, 1 ) )

print( correlation.systemfit( fitsur2e, 1, 2 ) )

print( correlation.systemfit( fitsur3, 2, 1 ) )

print( correlation.systemfit( fitsur3w, 2, 1 ) )

print( correlation.systemfit( fitsur4r3, 1, 2 ) )

print( correlation.systemfit( fitsur5, 2, 1 ) )

print( correlation.systemfit( fitsuri1r3, 1, 2 ) )

print( correlation.systemfit( fitsuri2, 2, 1 ) )

print( correlation.systemfit( fitsuri2w, 1, 2 ) )

print( correlation.systemfit( fitsuri3e, 1, 2 ) )

print( correlation.systemfit( fitsurio4, 2, 1 ) )
print( correlation.systemfit( fitsuri4, 2, 1 ) )

print( correlation.systemfit( fitsurio5r2, 1, 2 ) )
print( correlation.systemfit( fitsuri5r2, 1, 2 ) )


## ************ Log-Likelihood values ***************
print( logLik( fitsur1e2 ) )
print( logLik( fitsur1e2, residCovDiag = TRUE ) )

print( logLik( fitsur2e ) )
print( logLik( fitsur2e, residCovDiag = TRUE ) )

print( logLik( fitsur3 ) )
print( logLik( fitsur3, residCovDiag = TRUE ) )

print( logLik( fitsur4r3 ) )
print( logLik( fitsur4r3, residCovDiag = TRUE ) )

print( logLik( fitsur5 ) )
print( logLik( fitsur5, residCovDiag = TRUE ) )

print( logLik( fitsur5w ) )
print( logLik( fitsur5w, residCovDiag = TRUE ) )

print( logLik( fitsuri1r3 ) )
print( logLik( fitsuri1r3, residCovDiag = TRUE ) )

print( logLik( fitsuri2 ) )
print( logLik( fitsuri2, residCovDiag = TRUE ) )

print( logLik( fitsuri3e ) )
print( logLik( fitsuri3e, residCovDiag = TRUE ) )

print( logLik( fitsurio4 ) )
print( logLik( fitsurio4, residCovDiag = TRUE ) )

print( logLik( fitsuri4 ) )
print( logLik( fitsuri4, residCovDiag = TRUE ) )

print( logLik( fitsuri4w ) )
print( logLik( fitsuri4w, residCovDiag = TRUE ) )

print( logLik( fitsurio5r2 ) )
print( logLik( fitsurio5r2, residCovDiag = TRUE ) )

print( logLik( fitsuri5r2 ) )
print( logLik( fitsuri5r2, residCovDiag = TRUE ) )


## *********** likelihood ratio tests *************
# testing first restriction
# non-iterating, methodResidCov = 1
print( lrtest( fitsur2, fitsur1 ) )
print( lrtest( fitsur3, fitsur1 ) )
# non-iterating, methodResidCov = 0
print( lrtest( fitsur2e, fitsur1e ) )
print( lrtest( fitsur3e, fitsur1e ) )
# iterating, methodResidCov = 1
print( lrtest( fitsuri2, fitsuri1 ) )
print( lrtest( fitsuri3, fitsuri1 ) )
# iterating, methodResidCov = 0
print( lrtest( fitsuri2e, fitsuri1e ) )
print( lrtest( fitsuri3e, fitsuri1e ) )
# non-iterating, methodResidCov = 1, WSUR
print( lrtest( fitsur3w, fitsur1w ) )

# testing second restriction
# non-iterating, methodResidCov = 1
print( lrtest( fitsur4, fitsur2 ) )
print( lrtest( fitsur4, fitsur3 ) )
print( lrtest( fitsur5, fitsur2 ) )
print( lrtest( fitsur5, fitsur3 ) )
# non-iterating, methodResidCov = 0
print( lrtest( fitsur4e, fitsur2e ) )
print( lrtest( fitsur4e, fitsur3e ) )
print( lrtest( fitsur5e, fitsur2e ) )
print( lrtest( fitsur5e, fitsur3e ) )
# iterating, methodResidCov = 1
print( lrtest( fitsurio4, fitsuri2 ) )
print( lrtest( fitsurio4, fitsuri3 ) )
print( lrtest( fitsurio5, fitsuri2 ) )
print( lrtest( fitsurio5, fitsuri3 ) )
   # corrected
print( lrtest( fitsuri2, fitsuri4 ) )
print( lrtest( fitsuri3, fitsuri4 ) )
print( lrtest( fitsuri2, fitsuri5 ) )
print( lrtest( fitsuri3, fitsuri5 ) )

# iterating, methodResidCov = 0
print( lrtest( fitsurio4e, fitsuri2e ) )
print( lrtest( fitsurio4e, fitsuri3e ) )
print( lrtest( fitsurio5e, fitsuri2e ) )
print( lrtest( fitsurio5e, fitsuri3e ) )
   # corrected
print( lrtest( fitsuri2e, fitsuri4e ) )
print( lrtest( fitsuri3e, fitsuri4e ) )
print( lrtest( fitsuri2e, fitsuri5e ) )
print( lrtest( fitsuri3e, fitsuri5e ) )

# non-iterating, methodResidCov = 0, WSUR
print( lrtest( fitsur4we, fitsur2we ) )

# iterating, methodResidCov = 1, WSUR
print( lrtest( fitsuri2w, fitsuri4w ) )

# testing both of the restrictions
# non-iterating, methodResidCov = 1
print( lrtest( fitsur4, fitsur1 ) )
print( lrtest( fitsur5, fitsur1 ) )
# non-iterating, methodResidCov = 0
print( lrtest( fitsur4e, fitsur1e ) )
print( lrtest( fitsur5e, fitsur1e ) )
# iterating, methodResidCov = 1
print( lrtest( fitsurio4, fitsuri1 ) )
print( lrtest( fitsurio5, fitsuri1 ) )
   # corrected
print( lrtest( fitsuri1, fitsuri4 ) )
print( lrtest( fitsuri1, fitsuri5 ) )
# iterating, methodResidCov = 0
print( lrtest( fitsurio4e, fitsuri1e ) )
print( lrtest( fitsurio5e, fitsuri1e ) )
   # corrected
print( lrtest( fitsuri1e, fitsuri4e ) )
print( lrtest( fitsuri1e, fitsuri5e ) )
# non-iterating, methodResidCov = 1, WSUR
print( lrtest( fitsur5w, fitsur1w ) )

# testing the two restrictions with one call
# non-iterating, methodResidCov = 1
print( lrtest( fitsur4, fitsur2, fitsur1 ) )
print( lrtest( fitsur5, fitsur3, fitsur1 ) )
print( lrtest( fitsur1, fitsur3, fitsur5 ) )
print( lrtest( object = fitsur5, fitsur3, fitsur1 ) )
print( lrtest( fitsur3, object = fitsur5, fitsur1 ) )
print( lrtest( fitsur3, fitsur1, object = fitsur5 ) )
# iterating, methodResidCov = 0
print( lrtest( fitsuri4e, fitsuri2e, fitsuri1e ) )
print( lrtest( fitsuri5e, fitsuri3e, fitsuri1e ) )

## ************** F tests ****************
# testing first restriction
print( linearHypothesis( fitsur1, restrm ) )
linearHypothesis( fitsur1, restrict )

print( linearHypothesis( fitsur1r2, restrm ) )
linearHypothesis( fitsur1r2, restrict )

print( linearHypothesis( fitsuri1e2, restrm ) )
linearHypothesis( fitsuri1e2, restrict )

print( linearHypothesis( fitsuri1r3, restrm ) )
linearHypothesis( fitsuri1r3, restrict )

print( linearHypothesis( fitsur1we2, restrm ) )
linearHypothesis( fitsur1we2, restrict )

print( linearHypothesis( fitsuri1wr3, restrm ) )
linearHypothesis( fitsuri1wr3, restrict )

# testing second restriction
restrOnly2m <- matrix(0,1,7)
restrOnly2q <- 0.5
restrOnly2m[1,2] <- -1
restrOnly2m[1,5] <-  1
restrictOnly2 <- "- demand_price + supply_price = 0.5"
restrictOnly2i <- "- demand_price + supply_income = 0.5"
# first restriction not imposed 
print( linearHypothesis( fitsur1e2, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitsur1e2, restrictOnly2 )

print( linearHypothesis( fitsuri1, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitsuri1, restrictOnly2i )

# first restriction imposed
print( linearHypothesis( fitsur2, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitsur2, restrictOnly2 )

print( linearHypothesis( fitsur3, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitsur3, restrictOnly2 )

print( linearHypothesis( fitsuri2e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitsuri2e, restrictOnly2i )

print( linearHypothesis( fitsuri3e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitsuri3e, restrictOnly2i )

print( linearHypothesis( fitsur2we, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitsur2we, restrictOnly2 )

print( linearHypothesis( fitsuri3we, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitsuri3we, restrictOnly2i )

# testing both of the restrictions
print( linearHypothesis( fitsur1r3, restr2m, restr2q ) )
linearHypothesis( fitsur1r3, restrict2 )

print( linearHypothesis( fitsuri1e2, restr2m, restr2q ) )
linearHypothesis( fitsuri1e2, restrict2i )

print( linearHypothesis( fitsur1w, restr2m, restr2q ) )
linearHypothesis( fitsur1w, restrict2 )

print( linearHypothesis( fitsuri1wr3, restr2m, restr2q ) )
linearHypothesis( fitsuri1wr3, restrict2i )


## ************** Wald tests ****************
# testing first restriction
print( linearHypothesis( fitsur1, restrm, test = "Chisq" ) )
linearHypothesis( fitsur1, restrict, test = "Chisq" )

print( linearHypothesis( fitsur1r2, restrm, test = "Chisq" ) )
linearHypothesis( fitsur1r2, restrict, test = "Chisq" )

print( linearHypothesis( fitsuri1e2, restrm, test = "Chisq" ) )
linearHypothesis( fitsuri1e2, restrict, test = "Chisq" )

print( linearHypothesis( fitsuri1r3, restrm, test = "Chisq" ) )
linearHypothesis( fitsuri1r3, restrict, test = "Chisq" )

print( linearHypothesis( fitsur1w, restrm, test = "Chisq" ) )
linearHypothesis( fitsur1w, restrict, test = "Chisq" )

# testing second restriction
# first restriction not imposed
print( linearHypothesis( fitsur1e2, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitsur1e2, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitsuri1, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitsuri1, restrictOnly2i, test = "Chisq" )

# first restriction imposed
print( linearHypothesis( fitsur2, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitsur2, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitsur3, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitsur3, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitsuri2e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitsuri2e, restrictOnly2i, test = "Chisq" )

print( linearHypothesis( fitsuri3e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitsuri3e, restrictOnly2i, test = "Chisq" )

print( linearHypothesis( fitsuri2w, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitsuri2w, restrictOnly2i, test = "Chisq" )

print( linearHypothesis( fitsur3w, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitsur3w, restrictOnly2, test = "Chisq" )


# testing both of the restrictions
print( linearHypothesis( fitsur1r3, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitsur1r3, restrict2, test = "Chisq" )

print( linearHypothesis( fitsuri1e2, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitsuri1e2, restrict2i, test = "Chisq" )

print( linearHypothesis( fitsur1we2, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitsur1we2, restrict2, test = "Chisq" )

print( linearHypothesis( fitsuri1wr3, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitsuri1wr3, restrict2i, test = "Chisq" )


## ****************** model frame **************************
print( mf <- model.frame( fitsur1e2 ) )
print( mf1 <- model.frame( fitsur1e2$eq[[ 1 ]] ) )
print( attributes( mf1 )$terms )
print( mf2 <- model.frame( fitsur1e2$eq[[ 2 ]] ) )
print( attributes( mf2 )$terms )

print( all.equal( mf, model.frame( fitsur1w ) ) )
print( all.equal( mf1, model.frame( fitsur1w$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitsur2e ) ) )
print( all.equal( mf1, model.frame( fitsur2e$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitsur3 ) ) )
print( all.equal( mf2, model.frame( fitsur3$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitsur4r3 ) ) )
print( all.equal( mf1, model.frame( fitsur4r3$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitsur4we ) ) )
print( all.equal( mf2, model.frame( fitsur4we$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitsur5 ) ) )
print( all.equal( mf2, model.frame( fitsur5$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitsuri1r3 ) ) )
print( all.equal( mf1, model.frame( fitsuri1r3$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitsuri2 ) ) )
print( all.equal( mf1, model.frame( fitsuri2$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitsuri3e ) ) )
print( all.equal( mf1, model.frame( fitsuri3e$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitsurio4 ) ) )
print( all.equal( mf2, model.frame( fitsurio4$eq[[ 2 ]] ) ) )
print( all.equal( mf, model.frame( fitsuri4 ) ) )
print( all.equal( mf1, model.frame( fitsuri4$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitsurio5r2 ) ) )
print( all.equal( mf1, model.frame( fitsurio5r2$eq[[ 1 ]] ) ) )
print( all.equal( mf, model.frame( fitsuri5r2 ) ) )
print( all.equal( mf1, model.frame( fitsuri5r2$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitsuri5wr2 ) ) )
print( all.equal( mf1, model.frame( fitsuri5wr2$eq[[ 1 ]] ) ) )


## **************** model matrix ************************
# with x (returnModelMatrix) = TRUE
print( !is.null( fitsur1e2$eq[[ 1 ]]$x ) )
print( mm <- model.matrix( fitsur1e2 ) )
print( mm1 <- model.matrix( fitsur1e2$eq[[ 1 ]] ) )
print( mm2 <- model.matrix( fitsur1e2$eq[[ 2 ]] ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitsur1r2 ) ) )
print( all.equal( mm1, model.matrix( fitsur1r2$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur1r2$eq[[ 2 ]] ) ) )
print( !is.null( fitsur1r2$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitsur2e$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitsur2e ) ) )
print( all.equal( mm1, model.matrix( fitsur2e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur2e$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitsur2 ) ) )
print( all.equal( mm1, model.matrix( fitsur2$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur2$eq[[ 2 ]] ) ) )
print( !is.null( fitsur2$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitsur2we$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitsur2we ) ) )
print( all.equal( mm1, model.matrix( fitsur2we$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur2we$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitsur2 ) ) )
print( all.equal( mm1, model.matrix( fitsur2$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur2$eq[[ 2 ]] ) ) )
print( !is.null( fitsuri2$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitsur3e$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitsur3e ) ) )
print( all.equal( mm1, model.matrix( fitsur3e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur3e$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitsur3 ) ) )
print( all.equal( mm1, model.matrix( fitsur3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur3$eq[[ 2 ]] ) ) )
print( !is.null( fitsur3$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitsur3w$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitsur3w ) ) )
print( all.equal( mm1, model.matrix( fitsur3w$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur3w$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitsur3 ) ) )
print( all.equal( mm1, model.matrix( fitsur3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur3$eq[[ 2 ]] ) ) )
print( !is.null( fitsuri3$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitsur4r3$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitsur4r3 ) ) )
print( all.equal( mm1, model.matrix( fitsur4r3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur4r3$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitsur4we ) ) )
print( all.equal( mm1, model.matrix( fitsur4we$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitsur4we$eq[[ 2 ]] ) ) )
print( !is.null( fitsur4we$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitsurio5r2$eq[[ 1 ]]$x ) )
print( !is.null( fitsur5$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitsurio5r2 ) ) )
print( all.equal( mm1, model.matrix( fitsurio5r2$eq[[ 1 ]] ) ) )
print( all.equal( mm, model.matrix( fitsur5 ) ) )
print( all.equal( mm1, model.matrix( fitsur5$eq[[ 1 ]] ) ) )
#print( all.equal( mm2, model.matrix( fitsuri5r2$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitsurio5 ) ) )
print( all.equal( mm1, model.matrix( fitsurio5$eq[[ 1 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitsur5w ) ) )
print( all.equal( mm1, model.matrix( fitsur5w$eq[[ 1 ]] ) ) )
#print( all.equal( mm2, model.matrix( fitsuri5r2$eq[[ 1 ]] ) ) )
print( !is.null( fitsurio5$eq[[ 1 ]]$x ) )
print( !is.null( fitsur5w$eq[[ 1 ]]$x ) )


## **************** formulas ************************
formula( fitsur1e2 )
formula( fitsur1e2$eq[[ 2 ]] )

formula( fitsur2e )
formula( fitsur2e$eq[[ 1 ]] )

formula( fitsur2we )
formula( fitsur2we$eq[[ 1 ]] )

formula( fitsur3 )
formula( fitsur3$eq[[ 2 ]] )

formula( fitsur4r3 )
formula( fitsur4r3$eq[[ 1 ]] )

formula( fitsur5 )
formula( fitsur5$eq[[ 2 ]] )

formula( fitsuri1r3 )
formula( fitsuri1r3$eq[[ 1 ]] )

formula( fitsuri2 )
formula( fitsuri2$eq[[ 2 ]] )

formula( fitsuri3e )
formula( fitsuri3e$eq[[ 1 ]] )

formula( fitsurio4 )
formula( fitsurio4$eq[[ 2 ]] )
formula( fitsuri4 )
formula( fitsuri4$eq[[ 2 ]] )

formula( fitsurio5r2 )
formula( fitsurio5r2$eq[[ 1 ]] )
formula( fitsuri5r2 )
formula( fitsuri5r2$eq[[ 1 ]] )

formula( fitsuri5wr2 )
formula( fitsuri5wr2$eq[[ 1 ]] )


## **************** model terms *******************
terms( fitsur1e2 )
terms( fitsur1e2$eq[[ 2 ]] )

terms( fitsur2e )
terms( fitsur2e$eq[[ 1 ]] )

terms( fitsur3 )
terms( fitsur3$eq[[ 2 ]] )

terms( fitsur3w )
terms( fitsur3w$eq[[ 2 ]] )

terms( fitsur4r3 )
terms( fitsur4r3$eq[[ 1 ]] )

terms( fitsur4we )
terms( fitsur4we$eq[[ 1 ]] )

terms( fitsur5 )
terms( fitsur5$eq[[ 2 ]] )

terms( fitsuri1r3 )
terms( fitsuri1r3$eq[[ 1 ]] )

terms( fitsuri2 )
terms( fitsuri2$eq[[ 2 ]] )

terms( fitsuri3e )
terms( fitsuri3e$eq[[ 1 ]] )

terms( fitsurio4 )
terms( fitsurio4$eq[[ 2 ]] )
terms( fitsuri4 )
terms( fitsuri4$eq[[ 2 ]] )

terms( fitsurio5r2 )
terms( fitsurio5r2$eq[[ 1 ]] )
terms( fitsuri5r2 )
terms( fitsuri5r2$eq[[ 1 ]] )


## **************** estfun ************************
library( "sandwich" )

estfun( fitsur1 )
round( colSums( estfun( fitsur1 ) ), digits = 7 )

estfun( fitsur1e2 )
round( colSums( estfun( fitsur1e2 ) ), digits = 7 )

estfun( fitsur1r3 )
round( colSums( estfun( fitsur1r3 ) ), digits = 7 )

estfun( fitsur1w )
round( colSums( estfun( fitsur1w ) ), digits = 7 )

estfun( fitsuri1e )
round( colSums( estfun( fitsuri1e ) ), digits = 7 )

estfun( fitsuri1wr3 )
round( colSums( estfun( fitsuri1wr3 ) ), digits = 7 )

estfun( fitsurS1 )
round( colSums( estfun( fitsurS1 ) ), digits = 7 )

estfun( fitsurS2 )
round( colSums( estfun( fitsurS2 ) ), digits = 7 )

estfun( fitsurS3 )
round( colSums( estfun( fitsurS3 ) ), digits = 7 )

try( estfun( fitsurS4 ) )

estfun( fitsurS5 )
round( colSums( estfun( fitsurS5 ) ), digits = 7 )


## **************** bread ************************
round( bread( fitsur1 ), digits = 7 )

round( bread( fitsur1e2 ), digits = 7 )

round( bread( fitsur1r3 ), digits = 7 )

round( bread( fitsur1w ), digits = 7 )

round( bread( fitsuri1e ), digits = 7 )

round( bread( fitsuri1wr3 ), digits = 7 )

round( bread( fitsurS1 ), digits = 7 )

round( bread( fitsurS2 ), digits = 7 )

round( bread( fitsurS3 ), digits = 7 )

try( bread( fitsurS4 ) )

library( systemfit )
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   library( plm )
   options( digits = 3 )
   useMatrix <- FALSE
}

## Repeating the OLS and SUR estimations in Theil (1971, pp. 295, 300)
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   data( "GrunfeldGreene" )
   GrunfeldTheil <- subset( GrunfeldGreene,
      firm %in% c( "General Electric", "Westinghouse" ) )
   GrunfeldTheil <- pdata.frame( GrunfeldTheil, c( "firm", "year" ) )
   formulaGrunfeld <- invest ~ value + capital
}

# OLS
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   theilOls <- systemfit( formulaGrunfeld, "OLS",
      data = GrunfeldTheil, useMatrix = useMatrix )
   print( theilOls )
   print( summary( theilOls ) )
   print( summary( theilOls, useDfSys = TRUE, residCov = FALSE,
      equations = FALSE ) )
   print( summary( theilOls, equations = FALSE ) )
   print( coef( theilOls ) )
   print( coef( summary(theilOls ) ) )
   print( vcov( theilOls ) )
   print( residuals( theilOls ) )
   print( confint( theilOls ) )
   print( fitted(theilOls ) )
   print( logLik( theilOls ) )
   print( logLik( theilOls, residCovDiag = TRUE ) )
   print( nobs( theilOls ) )
   print( model.frame( theilOls ) )
   print( model.matrix( theilOls ) )
   print( formula( theilOls ) )
   print( formula( theilOls$eq[[ 1 ]] ) )
   print( terms( theilOls ) )
   print( terms( theilOls$eq[[ 1 ]] ) )
}

# SUR
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   theilSur <- systemfit( formulaGrunfeld, "SUR",
      data = GrunfeldTheil, methodResidCov = "noDfCor", useMatrix = useMatrix )
   print( theilSur )
   print( summary( theilSur ) )
   print( summary( theilSur, useDfSys = TRUE, equations = FALSE ) )
   print( summary( theilSur, residCov = FALSE, equations = FALSE ) )
   print( coef( theilSur ) )
   print( coef( summary( theilSur ) ) )
   print( vcov( theilSur ) )
   print( residuals( theilSur ) )
   print( confint( theilSur ) )
   print( fitted( theilSur ) )
   print( logLik( theilSur ) )
   print( logLik( theilSur, residCovDiag = TRUE ) )
   print( nobs( theilSur ) )
   print( model.frame( theilSur ) )
   print( model.matrix( theilSur ) )
   print( formula( theilSur ) )
   print( formula( theilSur$eq[[ 2 ]] ) )
   print( terms( theilSur ) )
   print( terms( theilSur$eq[[ 2 ]] ) )
}

## Repeating the OLS and SUR estimations in Greene (2003, pp. 351)
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   GrunfeldGreene <- pdata.frame( GrunfeldGreene, c( "firm", "year" ) )
   formulaGrunfeld <- invest ~ value + capital
}

# OLS
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greeneOls <- systemfit( formulaGrunfeld, "OLS",
      data = GrunfeldGreene, useMatrix = useMatrix )
   print( greeneOls )
   print( summary( greeneOls ) )
   print( summary( greeneOls, useDfSys = TRUE, equations = FALSE ) )
   print( summary( greeneOls, residCov = FALSE ) )
   print( sapply( greeneOls$eq, function(x){return(summary(x)$ssr/20)} ) ) # sigma^2
   print( coef( greeneOls ) )
   print( coef( summary( greeneOls ) ) )
   print( vcov( greeneOls ) )
   print( residuals( greeneOls ) )
   print( confint(greeneOls  ) )
   print( fitted( greeneOls ) )
   print( logLik( greeneOls ) )
   print( logLik( greeneOls, residCovDiag = TRUE ) )
   print( nobs( greeneOls ) )
   print( model.frame( greeneOls ) )
   print( model.matrix( greeneOls ) )
   print( formula( greeneOls ) )
   print( formula( greeneOls$eq[[ 2 ]] ) )
   print( terms( greeneOls ) )
   print( terms( greeneOls$eq[[ 2 ]] ) )
}

# OLS Pooled
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greeneOlsPooled <- systemfit( formulaGrunfeld, "OLS",
      data = GrunfeldGreene, pooled = TRUE, useMatrix = useMatrix )
   print( greeneOlsPooled )
   print( summary( greeneOlsPooled ) )
   print( summary( greeneOlsPooled, useDfSys = FALSE, residCov = FALSE ) )
   print( summary( greeneOlsPooled, residCov = FALSE, equations = FALSE ) )
   print( sum( sapply( greeneOlsPooled$eq, function(x){return(summary(x)$ssr)}) )/97 ) # sigma^2
   print( coef( greeneOlsPooled ) )
   print( coef( greeneOlsPooled, modified.regMat = TRUE ) )
   print( coef( summary( greeneOlsPooled ) ) )
   print( coef( summary( greeneOlsPooled ), modified.regMat = TRUE ) )
   print( vcov( greeneOlsPooled ) )
   print( vcov( greeneOlsPooled, modified.regMat = TRUE ) )
   print( residuals( greeneOlsPooled ) )
   print( confint( greeneOlsPooled ) )
   print( fitted( greeneOlsPooled ) )
   print( logLik( greeneOlsPooled ) )
   print( logLik( greeneOlsPooled, residCovDiag = TRUE ) )
   print( nobs( greeneOlsPooled ) )
   print( model.frame( greeneOlsPooled ) )
   print( model.matrix( greeneOlsPooled ) )
   print( formula( greeneOlsPooled ) )
   print( formula( greeneOlsPooled$eq[[ 1 ]] ) )
   print( terms( greeneOlsPooled ) )
   print( terms( greeneOlsPooled$eq[[ 1 ]] ) )
}

# SUR
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greeneSur <- systemfit( formulaGrunfeld, "SUR",
      data = GrunfeldGreene, methodResidCov = "noDfCor", useMatrix = useMatrix )
   print( greeneSur )
   print( summary( greeneSur ) )
   print( summary( greeneSur, useDfSys = TRUE, residCov = FALSE ) )
   print( summary( greeneSur, equations = FALSE ) )
   print( coef( greeneSur ) )
   print( coef( summary( greeneSur ) ) )
   print( vcov( greeneSur ) )
   print( residuals( greeneSur ) )
   print( confint( greeneSur ) )
   print( fitted( greeneSur ) )
   print( logLik( greeneSur ) )
   print( logLik( greeneSur, residCovDiag = TRUE ) )
   print( nobs( greeneSur ) )
   print( model.frame( greeneSur ) )
   print( model.matrix( greeneSur ) )
   print( formula( greeneSur ) )
   print( formula( greeneSur$eq[[ 1 ]] ) )
   print( terms( greeneSur ) )
   print( terms( greeneSur$eq[[ 1 ]] ) )
}

# SUR Pooled
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greeneSurPooled <- systemfit( formulaGrunfeld, "SUR",
      data = GrunfeldGreene, pooled = TRUE, methodResidCov = "noDfCor",
      residCovWeighted = TRUE, useMatrix = useMatrix )
   print( greeneSurPooled )
   print( summary( greeneSurPooled ) )
   print( summary( greeneSurPooled, useDfSys = FALSE, equations = FALSE ) )
   print( summary( greeneSurPooled, residCov = FALSE, equations = FALSE ) )
   print( coef( greeneSurPooled ) )
   print( coef( greeneSurPooled, modified.regMat = TRUE ) )
   print( coef( summary( greeneSurPooled ) ) )
   print( coef( summary( greeneSurPooled ), modified.regMat = TRUE ) )
   print( vcov( greeneSurPooled ) )
   print( vcov( greeneSurPooled, modified.regMat = TRUE ) )
   print( residuals( greeneSurPooled ) )
   print( confint( greeneSurPooled ) )
   print( fitted( greeneSurPooled ) )
   print( logLik( greeneSurPooled ) )
   print( logLik( greeneSurPooled, residCovDiag = TRUE ) )
   print( nobs( greeneSurPooled ) )
   print( model.frame( greeneSurPooled ) )
   print( model.matrix( greeneSurPooled ) )
   print( formula( greeneSurPooled ) )
   print( formula( greeneSurPooled$eq[[ 1 ]] ) )
   print( terms( greeneSurPooled ) )
   print( terms( greeneSurPooled$eq[[ 1 ]] ) )
}


#########  IV estimation  #######################
###  2SLS  ###
# instruments = explanatory variables  ->  2SLS estimates = OLS estimates
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greene2sls <- systemfit( formulaGrunfeld, inst = ~ value + capital, "2SLS",
      data = GrunfeldGreene, useMatrix = useMatrix )
   print( greene2sls )
   print( summary( greene2sls ) )
   print( all.equal( coef( summary( greene2sls ) ), coef( summary( greeneOls ) ) ) )
   print( all.equal( greene2sls[ -c(1,2,6) ], greeneOls[ -c(1,2,6) ] ) )
   for( i in 1:length( greene2sls$eq ) ) {
      print( all.equal( greene2sls$eq[[i]][ -c(3,15:17) ], 
         greeneOls$eq[[i]][-3] ) )
   }
}
# 'real' IV/2SLS estimation
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greene2slsR <- systemfit( invest ~ capital, inst = ~ value, "2SLS",
      data = GrunfeldGreene, useMatrix = useMatrix )
   print( greene2slsR )
   print( summary( greene2slsR ) )
}

###  2SLS, pooled  ###
# instruments = explanatory variables  ->  2SLS estimates = OLS estimates
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greene2slsPooled <- systemfit( formulaGrunfeld, inst = ~ value + capital, "2SLS",
      data = GrunfeldGreene, pooled = TRUE, useMatrix = useMatrix )
   print( greene2slsPooled )
   print( summary( greene2slsPooled ) )
   print( all.equal( coef( summary( greene2slsPooled ) ),
      coef( summary( greeneOlsPooled ) ) ) )
   print( all.equal( greene2slsPooled[ -c(1,2,6) ], greeneOlsPooled[ -c(1,2,6) ] ) )
   for( i in 1:length( greene2slsPooled$eq ) ) {
      print( all.equal( greene2slsPooled$eq[[i]][ -c(3,15:17) ], 
         greeneOlsPooled$eq[[i]][-3] ) )
   }
}
# 'real' IV/2SLS estimation
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greene2slsRPooled <- systemfit( invest ~ capital, inst = ~ value, "2SLS",
      data = GrunfeldGreene, pooled = TRUE, useMatrix = useMatrix )
   print( greene2slsRPooled )
   print( summary( greene2slsRPooled ) )
}

###  3SLS  ###
# instruments = explanatory variables  ->  3SLS estimates = SUR estimates
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greene3sls <- systemfit( formulaGrunfeld, inst = ~ value + capital, "3SLS",
      data = GrunfeldGreene, useMatrix = useMatrix, methodResidCov = "noDfCor" )
   print( greene3sls )
   print( summary( greene3sls ) )
   print( all.equal( coef( summary( greene3sls ) ), coef( summary( greeneSur ) ) ) )
   print( all.equal( greene3sls[ -c(1,2,7) ], greeneSur[ -c(1,2,7) ] ) )
   for( i in 1:length( greene3sls$eq ) ) {
      print( all.equal( greene3sls$eq[[i]][ -c(3,15:17) ], 
         greeneSur$eq[[i]][-3] ) )
   }
}
# 'real' IV/3SLS estimation
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greene3slsR <- systemfit( invest ~ capital, inst = ~ value, "3SLS",
      data = GrunfeldGreene, useMatrix = useMatrix )
   print( greene3slsR )
   print( summary( greene3slsR ) )
}

###  3SLS, Pooled  ###
# instruments = explanatory variables  ->  3SLS estimates = SUR estimates
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greene3slsPooled <- systemfit( formulaGrunfeld, inst = ~ capital + value, "3SLS",
      data = GrunfeldGreene, pooled = TRUE, useMatrix = useMatrix, 
      residCovWeighted = TRUE, methodResidCov = "noDfCor" )
   print( greene3slsPooled )
   print( summary( greene3slsPooled ) )
   print( all.equal( coef( summary( greene3slsPooled ) ),
      coef( summary( greeneSurPooled ) ) ) )
   print( all.equal( greene3slsPooled[ -c(1,2,7) ], greeneSurPooled[ -c(1,2,7) ] ) )
   for( i in 1:length( greene3slsPooled$eq ) ) {
      print( all.equal( greene3slsPooled$eq[[i]][ -c(3,15:17) ], 
         greeneSurPooled$eq[[i]][-3] ) )
   }
}
# 'real' IV/3SLS estimation
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   greene3slsRPooled <- systemfit( invest ~ capital, inst = ~ value, "3SLS",
      data = GrunfeldGreene, useMatrix = useMatrix )
   print( greene3slsRPooled )
   print( summary( greene3slsRPooled ) )
}


## **************** estfun ************************
library( "sandwich" )

if(requireNamespace( 'plm', quietly = TRUE ) ) {
   print( estfun( theilOls ) )
   print( round( colSums( estfun( theilOls ) ), digits = 7 ) )
   
   print( estfun( theilSur ) )
   print( round( colSums( estfun( theilSur ) ), digits = 7 ) )
   
   print( estfun( greeneOls ) )
   print( round( colSums( estfun( greeneOls ) ), digits = 7 ) )
   
   print( try( estfun( greeneOlsPooled ) ) )
   
   print( estfun( greeneSur ) )
   print( round( colSums( estfun( greeneSur ) ), digits = 7 ) )
   
   print( try( estfun( greeneSurPooled ) ) )
}

## **************** bread ************************
if(requireNamespace( 'plm', quietly = TRUE ) ) {
   print( bread( theilOls ) )
   
   print( bread( theilSur ) )
   
   print( bread( greeneOls ) )
   
   print( try( bread( greeneOlsPooled ) ) )
   
   print( bread( greeneSur ) )
   
   print( try( bread( greeneSurPooled ) ) )
}

## this function returns a vector of the
## cross-equation corrlations between eq i and eq j
## from the results set for equation ij
correlation.systemfit <- function( results, eqni, eqnj ) {
  nCoefEq <- NULL
  for( i in 1:length( results$eq ) ) {
     nCoefEq <- c( nCoefEq, length( coef( results$eq[[ i ]] ) ) )
  }
  cij <- vcov( results )[(1+sum(nCoefEq[1:eqni])-nCoefEq[eqni]):(sum(nCoefEq[1:eqni])),
                      (1+sum(nCoefEq[1:eqnj])-nCoefEq[eqnj]):(sum(nCoefEq[1:eqnj]))]
  cii <- vcov( results )[(1+sum(nCoefEq[1:eqni])-nCoefEq[eqni]):(sum(nCoefEq[1:eqni])),
                      (1+sum(nCoefEq[1:eqni])-nCoefEq[eqni]):(sum(nCoefEq[1:eqni]))]
  cjj <- vcov( results )[(1+sum(nCoefEq[1:eqnj])-nCoefEq[eqnj]):(sum(nCoefEq[1:eqnj])),
                      (1+sum(nCoefEq[1:eqnj])-nCoefEq[eqnj]):(sum(nCoefEq[1:eqnj]))]
  rij <- NULL

  for( i in 1:nrow( residuals( results ) ) ) {
    xik    <- model.matrix( results$eq[[eqni]] )[i,]
    xjk    <- model.matrix( results$eq[[eqnj]] )[i,]
    top    <- xik %*% cij %*% xjk
    bottom <- sqrt( ( xik %*% cii %*% xik ) * ( xjk %*% cjj %*% xjk ) )
    rijk   <- top / bottom
    rij    <- rbind( rij, rijk )
  }
  rij
}

## determines the improvement of resultsj (3sls) over
## resultsi (2sls) for equation i and returns a matrix
## of the values, so you can examine the range, mean, etc
se.ratio.systemfit <- function( resultsi, resultsj, eqni ) {
  ratio <- NULL
  for( i in 1:nrow( residuals( resultsi ) ) ) {
    xik    <- model.matrix( resultsi$eq[[eqni]] )[i,]
    top    <- sqrt( xik %*% vcov( resultsi$eq[[eqni]] ) %*% xik )
    bottom <- sqrt( xik %*% vcov( resultsj$eq[[eqni]] ) %*% xik )
    rk     <- top / bottom
    ratio  <- rbind( ratio, rk )
  }
  ratio
}


## return all coefficients
coef.systemfit <- function( object, modified.regMat = FALSE, ... ) {
   if( modified.regMat ){
      if( is.null( object$restrict.regMat ) ){
         stop( "coefficients of the modified regressor matrix are not available,",
            " because argument 'restrict.regMat' has not been used in this estimation." )
      } else {
         return( drop( solve( crossprod( object$restrict.regMat ),
            t( object$restrict.regMat ) %*% coef( object ) ) ) )
      }
   } else {
      return( object$coefficients )
   }
}

## return all coefficients, std.errors, t-values and p-values
coef.summary.systemfit <- function( object, modified.regMat = FALSE, ... ) {
   if( modified.regMat ){
      if( is.null( object$coefModReg ) ){
         stop( "coefficients of the modified regressor matrix are not available,",
            " because argument 'restrict.regMat' has not been used in this estimation." )
      } else {
         return( object$coefModReg )
      }
   } else {
      return( object$coefficients )
   }
}

## return the coefficients of a single equation
coef.systemfit.equation <- function( object, ... ) {
   object$coefficients
}

## return coefficients, std.errors, t-values and p-values of a single equation
coef.summary.systemfit.equation <- function( object, ... ) {
   object$coefficients
}

## return all residuals
residuals.systemfit <- function( object, ... ) {
   result <- data.frame( obsNo = c( 1:length( residuals( object$eq[[1]] ) ) ) )
   for( i in 1:length( object$eq ) ) {
      result[[ object$eq[[i]]$eqnLabel ]] <- residuals( object$eq[[i]] )
   }
   result$obsNo <- NULL
   rownames( result ) <- names( residuals( object$eq[[ 1 ]] ) )
   return( result )
}

## return residuals of a single equation
residuals.systemfit.equation <- function( object, na.rm = FALSE, ... ) {
   if( na.rm ) {
      return( object$residuals[ !is.na( object$residuals ) ] )
   } else {
      return( object$residuals )
   }
}

## return the variance covariance matrix of the coefficients
vcov.systemfit <- function( object, modified.regMat = FALSE, ... ) {
   if( modified.regMat ){
      if( is.null( object$restrict.regMat ) ){
         stop( "coefficients of the modified regressor matrix",
            " and their covariance matrix are not available,",
            " because argument 'restrict.regMat' has not been used in this estimation." )
      } else {
         txtxInv <- solve( crossprod( object$restrict.regMat ) )
         result <- txtxInv %*% t( object$restrict.regMat ) %*% vcov( object ) %*%
            object$restrict.regMat %*% txtxInv
         return( result )
      }
   } else {
      return( object$coefCov )
   }
}

## return the variance covariance matrix of the coefficients of a single equation
vcov.systemfit.equation <- function( object, ... ) {
   object$coefCov
}


## return the fitted values
fitted.systemfit <- function( object, ... ) {
   nEq <- length( object$eq )
   fitted.values <- matrix( NA, length( object$eq[[1]]$fitted.values ), nEq )
   colnames( fitted.values ) <- as.character( 1:ncol( fitted.values ) )
   for(i in 1:nEq )  {
      fitted.values[ , i ]           <- object$eq[[ i ]]$fitted.values
      colnames( fitted.values )[ i ] <- object$eq[[ i ]]$eqnLabel
   }
   rownames( fitted.values ) <- names( fitted( object$eq[[ 1 ]] ) )
   return( as.data.frame( fitted.values ) )
}

## return the fitted values of e single euation
fitted.systemfit.equation <- function( object, na.rm = FALSE, ... ) {
   if( na.rm ) {
      return( object$fitted.values[ !is.na( object$fitted.values ) ] )
   } else {
      return( object$fitted.values )
   }
}


## return model frame of the entire system
model.frame.systemfit <- function( formula, ... ){
   mfColNames <- NULL
   for( i in 1:length( formula$eq ) ) {
      mfi <- model.frame( formula$eq[[ i ]] )
      if( i == 1 ) {
         result <- mfi
      } else {
         for( j in 1:ncol( mfi ) ) {
            if( ! names( mfi )[ j ] %in% names( result ) ) {
               result[[ names( mfi )[ j ] ]] <- mfi[ , j ]
            }
         }
      }
   }
   return( result )
}

## return model frame of a single equation
model.frame.systemfit.equation <- function( formula, ... ){
   if( !is.null( formula$model ) ) {
      result <- formula$model
   } else {
      stop( "returning model frame not possible. Please re-estimate",
         " the system with control variable 'model'",
         " set to TRUE" )
   }
   return( result )
}

library( systemfit )
options( digits = 3 )

data( "Kmenta" )
useMatrix <- FALSE

demand <- consump ~ price + income
supply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
inst1  <- ~ income + farmPrice
instlist <- list( inst1, inst )
system <- list( demand = demand, supply = supply )
restrm <- matrix(0,1,7)  # restriction matrix "R"
restrm[1,3] <-  1
restrm[1,7] <- -1
restrict <- "demand_income - supply_trend = 0"
restr2m <- matrix(0,2,7)  # restriction matrix "R" 2
restr2m[1,3] <-  1
restr2m[1,7] <- -1
restr2m[2,2] <- -1
restr2m[2,5] <-  1
restr2q <- c( 0, 0.5 )  # restriction vector "q" 2
restrict2 <- c( "demand_income - supply_trend = 0",
   "- demand_price + supply_price = 0.5" )
tc <- matrix(0,7,6)
tc[1,1] <- 1
tc[2,2] <- 1
tc[3,3] <- 1
tc[4,4] <- 1
tc[5,5] <- 1
tc[6,6] <- 1
tc[7,3] <- 1
restr3m <- matrix(0,1,6)  # restriction matrix "R" 2
restr3m[1,2] <- -1
restr3m[1,5] <-  1
restr3q <- c( 0.5 )  # restriction vector "q" 2
restrict3 <- "- C2 + C5 = 0.5"


## ********************* W2SLS *****************
fitw2sls1 <- systemfit( system, "W2SLS", data = Kmenta, inst = inst,
   useMatrix = useMatrix )
print( summary( fitw2sls1 ) )

## ********************* W2SLS (EViews-like) *****************
fitw2sls1e <- systemfit( system, "W2SLS", data = Kmenta, inst = inst,
   methodResidCov = "noDfCor", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitw2sls1e, useDfSys = TRUE ) )

## ********************* W2SLS with restriction *******************
fitw2sls2 <- systemfit( system, "W2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, useMatrix = useMatrix )
print( summary( fitw2sls2 ) )
# the same with symbolically specified restrictions
fitw2sls2Sym <- systemfit( system, "W2SLS", data = Kmenta,
   restrict.matrix = restrict, inst = inst, useMatrix = useMatrix )
all.equal( fitw2sls2, fitw2sls2Sym )

## ********************* W2SLS with restriction (EViews-like) **************
fitw2sls2e <- systemfit( system, "W2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, methodResidCov = "noDfCor", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitw2sls2e, useDfSys = TRUE ) )
nobs( fitw2sls2e )

## ********************* W2SLS with restriction via restrict.regMat *******************
fitw2sls3 <- systemfit( system, "W2SLS", data = Kmenta, restrict.regMat = tc,
   inst = inst, x = TRUE, useMatrix = useMatrix )
print( summary( fitw2sls3 ) )

## ********************* W2SLS with restriction via restrict.regMat (EViews-like) **************
fitw2sls3e <- systemfit( system, "W2SLS", data = Kmenta, restrict.regMat = tc,
   inst = inst, methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fitw2sls3e, useDfSys = TRUE ) )

## ***************** W2SLS with 2 restrictions ********************
fitw2sls4 <- systemfit( system, "W2SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, x = TRUE,
   useMatrix = useMatrix )
print( summary( fitw2sls4 ) )
# the same with symbolically specified restrictions
fitw2sls4Sym <- systemfit( system, "W2SLS", data = Kmenta,
   restrict.matrix = restrict2, inst = inst, x = TRUE,
   useMatrix = useMatrix )
all.equal( fitw2sls4, fitw2sls4Sym )

## ***************** W2SLS with 2 restrictions (EViews-like) **************
fitw2sls4e <- systemfit( system, "W2SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, methodResidCov = "noDfCor",
   useMatrix = useMatrix )
print( summary( fitw2sls4e, useDfSys = TRUE ) )

## ***************** W2SLS with 2 restrictions via R and restrict.regMat ******************
fitw2sls5 <- systemfit( system, "W2SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitw2sls5 ) )
# the same with symbolically specified restrictions
fitw2sls5Sym <- systemfit( system, "W2SLS", data = Kmenta,
   restrict.matrix = restrict3, restrict.regMat = tc, inst = inst,
   x = TRUE, useMatrix = useMatrix )
all.equal( fitw2sls5, fitw2sls5Sym )

## ***************** W2SLS with 2 restrictions via R and restrict.regMat (EViews-like) **************
fitw2sls5e <- systemfit( system, "W2SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fitw2sls5e, useDfSys = TRUE ) )

## ****** 2SLS estimation with different instruments **********************
fitw2slsd1 <- systemfit( system, "W2SLS", data = Kmenta, inst = instlist,
   useMatrix = useMatrix )
print( summary( fitw2slsd1 ) )

## ****** 2SLS estimation with different instruments (EViews-like)******************
fitw2slsd1e <- systemfit( system, "W2SLS", data = Kmenta, inst = instlist,
   methodResidCov = "noDfCor", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitw2slsd1e, useDfSys = TRUE ) )

## **** W2SLS estimation with different instruments and restriction ********
fitw2slsd2 <- systemfit( system, "W2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = instlist, useMatrix = useMatrix )
print( summary( fitw2slsd2 ) )

## **** W2SLS estimation with different instruments and restriction (EViews-like)*
fitw2slsd2e <- systemfit( system, "W2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = instlist, methodResidCov = "noDfCor", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitw2slsd2e, useDfSys = TRUE ) )

## ** W2SLS estimation with different instruments and restriction via restrict.regMat ****
fitw2slsd3 <- systemfit( system, "W2SLS", data = Kmenta, restrict.regMat = tc,
   inst = instlist, x = TRUE, useMatrix = useMatrix )
print( summary( fitw2slsd3 ) )

## W2SLS estimation with different instruments and restriction via restrict.regMat (EViews-like)
fitw2slsd3e <- systemfit( system, "W2SLS", data = Kmenta, restrict.regMat = tc,
   inst = instlist, methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fitw2slsd3e, useDfSys = TRUE ) )


## *********** estimations with a single regressor ************
fitw2slsS1 <- systemfit(
   list( consump ~ price - 1, price ~ consump + trend ), "W2SLS",
   data = Kmenta, inst = ~ farmPrice + trend + income, useMatrix = useMatrix )
print( summary( fitw2slsS1 ) )
fitw2slsS2 <- systemfit(
   list( consump ~ price - 1, consump ~ trend - 1 ), "W2SLS",
   data = Kmenta, inst = ~ farmPrice + price + income, useMatrix = useMatrix )
print( summary( fitw2slsS2 ) )
fitw2slsS3 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "W2SLS",
   data = Kmenta, inst = instlist, useMatrix = useMatrix )
print( summary( fitw2slsS3 ) )
fitw2slsS4 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "W2SLS",
   data = Kmenta, inst = ~ farmPrice + trend + income,
   restrict.matrix = matrix( c( 1, -1 ), nrow = 1 ), useMatrix = useMatrix )
print( summary( fitw2slsS4 ) )
fitw2slsS5 <- systemfit(
   list( consump ~ 1, price ~ 1 ), "W2SLS",
   data = Kmenta, inst = instlist, useMatrix = useMatrix )
print( summary( fitw2slsS5 ) )


## **************** shorter summaries **********************
print( summary( fitw2sls1e, residCov = FALSE ) )

print( summary( fitw2sls2, residCov = FALSE, equations = FALSE ) )

print( summary( fitw2sls3, useDfSys = FALSE ), equations = FALSE )

print( summary( fitw2sls4e ), residCov = FALSE )

print( summary( fitw2sls5, useDfSys = FALSE, residCov = FALSE ) )

print( summary( fitw2slsd1, useDfSys = TRUE ), residCov = FALSE,
   equations = FALSE )

print( summary( fitw2slsd2e, equations = TRUE ), equations = FALSE )

print( summary( fitw2slsd3e, equations = FALSE ), residCov = FALSE )


## ****************** residuals **************************
print( residuals( fitw2sls1e ) )
print( residuals( fitw2sls1e$eq[[ 1 ]] ) )

print( residuals( fitw2sls2 ) )
print( residuals( fitw2sls2$eq[[ 2 ]] ) )

print( residuals( fitw2sls3 ) )
print( residuals( fitw2sls3$eq[[ 1 ]] ) )

print( residuals( fitw2sls4e ) )
print( residuals( fitw2sls4e$eq[[ 2 ]] ) )

print( residuals( fitw2sls5 ) )
print( residuals( fitw2sls5$eq[[ 1 ]] ) )

print( residuals( fitw2slsd1 ) )
print( residuals( fitw2slsd1$eq[[ 2 ]] ) )

print( residuals( fitw2slsd2e ) )
print( residuals( fitw2slsd2e$eq[[ 1 ]] ) )

print( residuals( fitw2slsd3e ) )
print( residuals( fitw2slsd3e$eq[[ 2 ]] ) )


## *************** coefficients *********************
print( round( coef( fitw2sls1e ), digits = 6 ) )
print( round( coef( fitw2sls1e$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitw2slsd2e ), digits = 6 ) )
print( round( coef( fitw2slsd2e$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitw2slsd3e ), digits = 6 ) )
print( round( coef( fitw2slsd3e, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitw2slsd3e$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitw2sls4 ), digits = 6 ) )
print( round( coef( fitw2sls4$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitw2sls5 ), digits = 6 ) )
print( round( coef( fitw2sls5, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitw2sls5$eq[[ 2 ]] ), digits = 6 ) )


## *************** coefficients with stats *********************
print( round( coef( summary( fitw2sls1e, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitw2sls1e$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fitw2slsd2e ) ), digits = 6 ) )
print( round( coef( summary( fitw2slsd2e$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitw2slsd3e, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitw2slsd3e, useDfSys = FALSE ),
   modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fitw2slsd3e$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fitw2sls4 ) ), digits = 6 ) )
print( round( coef( summary( fitw2sls4$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitw2sls5 ) ), digits = 6 ) )
print( round( coef( summary( fitw2sls5 ), modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fitw2sls5$eq[[ 2 ]] ) ), digits = 6 ) )


## *********** variance covariance matrix of the coefficients *******
print( round( vcov( fitw2sls1e ), digits = 6 ) )
print( round( vcov( fitw2sls1e$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitw2sls2 ), digits = 6 ) )
print( round( vcov( fitw2sls2$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitw2sls3e ), digits = 6 ) )
print( round( vcov( fitw2sls3e, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitw2sls3e$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitw2sls4 ), digits = 6 ) )
print( round( vcov( fitw2sls4$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitw2sls5 ), digits = 6 ) )
print( round( vcov( fitw2sls5, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitw2sls5$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitw2slsd1 ), digits = 6 ) )
print( round( vcov( fitw2slsd1$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitw2slsd2e ), digits = 6 ) )
print( round( vcov( fitw2slsd2e$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitw2slsd3 ), digits = 6 ) )
print( round( vcov( fitw2slsd3, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitw2slsd3$eq[[ 1 ]] ), digits = 6 ) )


## *********** confidence intervals of coefficients *************
print( confint( fitw2sls1e, useDfSys = TRUE ) )
print( confint( fitw2sls1e$eq[[ 1 ]], level = 0.9, useDfSys = TRUE ) )

print( confint( fitw2sls2, level = 0.9 ) )
print( confint( fitw2sls2$eq[[ 2 ]], level = 0.99 ) )

print( confint( fitw2sls3, level = 0.99 ) )
print( confint( fitw2sls3$eq[[ 1 ]], level = 0.5 ) )

print( confint( fitw2sls4e, level = 0.5, useDfSys = TRUE ) )
print( confint( fitw2sls4e$eq[[ 2 ]], level = 0.25, useDfSys = TRUE ) )

print( confint( fitw2sls5, level = 0.25 ) )
print( confint( fitw2sls5$eq[[ 1 ]], level = 0.975 ) )

print( confint( fitw2slsd1, level = 0.975 ) )
print( confint( fitw2slsd1$eq[[ 2 ]], level = 0.999 ) )

print( confint( fitw2slsd2e, level = 0.999, useDfSys = TRUE ) )
print( confint( fitw2slsd2e$eq[[ 1 ]], level = 0.01, useDfSys = TRUE ) )

print( confint( fitw2slsd3e, level = 0.01, useDfSys = TRUE ) )
print( confint( fitw2slsd3e$eq[[ 2 ]], useDfSys = TRUE ) )


## *********** fitted values *************
print( fitted( fitw2sls1e ) )
print( fitted( fitw2sls1e$eq[[ 1 ]] ) )

print( fitted( fitw2sls2 ) )
print( fitted( fitw2sls2$eq[[ 2 ]] ) )

print( fitted( fitw2sls3 ) )
print( fitted( fitw2sls3$eq[[ 1 ]] ) )

print( fitted( fitw2sls4e ) )
print( fitted( fitw2sls4e$eq[[ 2 ]] ) )

print( fitted( fitw2sls5 ) )
print( fitted( fitw2sls5$eq[[ 1 ]] ) )

print( fitted( fitw2slsd1 ) )
print( fitted( fitw2slsd1$eq[[ 2 ]] ) )

print( fitted( fitw2slsd2e ) )
print( fitted( fitw2slsd2e$eq[[ 1 ]] ) )

print( fitted( fitw2slsd3e ) )
print( fitted( fitw2slsd3e$eq[[ 2 ]] ) )


## *********** predicted values *************
predictData <- Kmenta
predictData$consump <- NULL
predictData$price <- Kmenta$price * 0.9
predictData$income <- Kmenta$income * 1.1

print( predict( fitw2sls1e, se.fit = TRUE, interval = "prediction",
   useDfSys = TRUE ) )
print( predict( fitw2sls1e$eq[[ 1 ]], se.fit = TRUE, interval = "prediction",
   useDfSys = TRUE ) )

print( predict( fitw2sls2, se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )
print( predict( fitw2sls2$eq[[ 2 ]], se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )

print( predict( fitw2sls3, se.pred = TRUE, interval = "prediction",
   level = 0.975 ) )
print( predict( fitw2sls3$eq[[ 1 ]], se.pred = TRUE, interval = "prediction",
   level = 0.975 ) )

print( predict( fitw2sls4e, se.fit = TRUE, interval = "confidence",
   level = 0.25, useDfSys = TRUE ) )
print( predict( fitw2sls4e$eq[[ 2 ]], se.fit = TRUE, interval = "confidence",
   level = 0.25, useDfSys = TRUE ) )

print( predict( fitw2sls5, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )
print( predict( fitw2sls5$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )

print( predict( fitw2slsd1, se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99 ) )
print( predict( fitw2slsd1$eq[[ 2 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99 ) )

print( predict( fitw2slsd2e, se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData, useDfSys = TRUE ) )
print( predict( fitw2slsd2e$eq[[ 1 ]], se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData, useDfSys = TRUE ) )

print( predict( fitw2slsd3e, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.01, useDfSys = TRUE ) )
print( predict( fitw2slsd3e$eq[[ 2 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.01, useDfSys = TRUE ) )

# predict just one observation
smallData <- data.frame( price = 130, income = 150, farmPrice = 120,
   trend = 25 )

print( predict( fitw2sls1e, newdata = smallData ) )
print( predict( fitw2sls1e$eq[[ 1 ]], newdata = smallData ) )

print( predict( fitw2sls2, se.fit = TRUE, level = 0.9,
   newdata = smallData ) )
print( predict( fitw2sls2$eq[[ 1 ]], se.pred = TRUE, level = 0.99,
   newdata = smallData ) )

print( predict( fitw2sls3, interval = "prediction", level = 0.975,
   newdata = smallData ) )
print( predict( fitw2sls3$eq[[ 1 ]], interval = "confidence", level = 0.8,
   newdata = smallData ) )

print( predict( fitw2sls4e, se.fit = TRUE, interval = "confidence",
   level = 0.999, newdata = smallData ) )
print( predict( fitw2sls4e$eq[[ 2 ]], se.pred = TRUE, interval = "prediction",
   level = 0.75, newdata = smallData ) )

print( predict( fitw2sls5, se.fit = TRUE, interval = "prediction",
   newdata = smallData ) )
print( predict( fitw2sls5$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   newdata = smallData ) )

print( predict( fitw2slsd2e, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fitw2slsd2e$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )


## ************ correlation of predicted values ***************
print( correlation.systemfit( fitw2sls1e, 1, 2 ) )

print( correlation.systemfit( fitw2sls2, 2, 1 ) )

print( correlation.systemfit( fitw2sls3, 1, 2 ) )

print( correlation.systemfit( fitw2sls4e, 2, 1 ) )

print( correlation.systemfit( fitw2sls5, 1, 2 ) )

print( correlation.systemfit( fitw2slsd1, 2, 1 ) )

print( correlation.systemfit( fitw2slsd2e, 1, 2 ) )

print( correlation.systemfit( fitw2slsd3e, 2, 1 ) )


## ************ LOG-Likelihood values ***************
print( logLik( fitw2sls1e ) )
print( logLik( fitw2sls1e, residCovDiag = TRUE ) )

print( logLik( fitw2sls2 ) )
print( logLik( fitw2sls2, residCovDiag = TRUE ) )

print( logLik( fitw2sls3 ) )
print( logLik( fitw2sls3, residCovDiag = TRUE ) )

print( logLik( fitw2sls4e ) )
print( logLik( fitw2sls4e, residCovDiag = TRUE ) )

print( logLik( fitw2sls5 ) )
print( logLik( fitw2sls5, residCovDiag = TRUE ) )

print( logLik( fitw2slsd1 ) )
print( logLik( fitw2slsd1, residCovDiag = TRUE ) )

print( logLik( fitw2slsd2e ) )
print( logLik( fitw2slsd2e, residCovDiag = TRUE ) )

print( logLik( fitw2slsd3e ) )
print( logLik( fitw2slsd3e, residCovDiag = TRUE ) )


## ************** F tests ****************
# testing first restriction
print( linearHypothesis( fitw2sls1, restrm ) )
linearHypothesis( fitw2sls1, restrict )

print( linearHypothesis( fitw2slsd1e, restrm ) )
linearHypothesis( fitw2slsd1e, restrict )

# testing second restriction
restrOnly2m <- matrix(0,1,7)
restrOnly2q <- 0.5
restrOnly2m[1,2] <- -1
restrOnly2m[1,5] <-  1
restrictOnly2 <- "- demand_price + supply_price = 0.5"
# first restriction not imposed 
print( linearHypothesis( fitw2sls1e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitw2sls1e, restrictOnly2 )

print( linearHypothesis( fitw2slsd1, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitw2slsd1, restrictOnly2 )

# first restriction imposed
print( linearHypothesis( fitw2sls2, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitw2sls2, restrictOnly2 )

print( linearHypothesis( fitw2sls3, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitw2sls3, restrictOnly2 )

print( linearHypothesis( fitw2slsd2e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitw2slsd2e, restrictOnly2 )

print( linearHypothesis( fitw2slsd3e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitw2slsd3e, restrictOnly2 )

# testing both of the restrictions
print( linearHypothesis( fitw2sls1e, restr2m, restr2q ) )
linearHypothesis( fitw2sls1e, restrict2 )

print( linearHypothesis( fitw2slsd1, restr2m, restr2q ) )
linearHypothesis( fitw2slsd1, restrict2 )


## ************** Wald tests ****************
# testing first restriction
print( linearHypothesis( fitw2sls1, restrm, test = "Chisq" ) )
linearHypothesis( fitw2sls1, restrict, test = "Chisq" )

print( linearHypothesis( fitw2slsd1e, restrm, test = "Chisq" ) )
linearHypothesis( fitw2slsd1e, restrict, test = "Chisq" )

# testing second restriction
# first restriction not imposed
print( linearHypothesis( fitw2sls1e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitw2sls1e, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitw2slsd1, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitw2slsd1, restrictOnly2, test = "Chisq" )
# first restriction imposed
print( linearHypothesis( fitw2sls2, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitw2sls2, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitw2sls3, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitw2sls3, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitw2slsd2e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitw2slsd2e, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitw2slsd3e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitw2slsd3e, restrictOnly2, test = "Chisq" )

# testing both of the restrictions
print( linearHypothesis( fitw2sls1e, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitw2sls1e, restrict2, test = "Chisq" )

print( linearHypothesis( fitw2slsd1, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitw2slsd1, restrict2, test = "Chisq" )


## ****************** model frame **************************
print( mf <- model.frame( fitw2sls1e ) )
print( mf1 <- model.frame( fitw2sls1e$eq[[ 1 ]] ) )
print( attributes( mf1 )$terms )
print( mf2 <- model.frame( fitw2sls1e$eq[[ 2 ]] ) )
print( attributes( mf2 )$terms )

print( all.equal( mf, model.frame( fitw2sls2 ) ) )
print( all.equal( mf2, model.frame( fitw2sls2$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitw2sls3 ) ) )
print( all.equal( mf1, model.frame( fitw2sls3$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitw2sls4e ) ) )
print( all.equal( mf2, model.frame( fitw2sls4e$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitw2sls5 ) ) )
print( all.equal( mf1, model.frame( fitw2sls5$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitw2slsd1 ) ) )
print( all.equal( mf2, model.frame( fitw2slsd1$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitw2slsd2e ) ) )
print( all.equal( mf1, model.frame( fitw2slsd2e$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitw2slsd3e ) ) )
print( all.equal( mf2, model.frame( fitw2slsd3e$eq[[ 2 ]] ) ) )

fitw2sls1e$eq[[ 1 ]]$modelInst
fitw2sls1e$eq[[ 2 ]]$modelInst

fitw2sls4Sym$eq[[ 1 ]]$modelInst
fitw2sls4Sym$eq[[ 2 ]]$modelInst

fitw2sls5$eq[[ 1 ]]$modelInst
fitw2sls5$eq[[ 2 ]]$modelInst


## **************** model matrix ************************
# with x (returnModelMatrix) = TRUE
print( !is.null( fitw2sls1e$eq[[ 1 ]]$x ) )
print( mm <- model.matrix( fitw2sls1e ) )
print( mm1 <- model.matrix( fitw2sls1e$eq[[ 1 ]] ) )
print( mm2 <- model.matrix( fitw2sls1e$eq[[ 2 ]] ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitw2sls1 ) ) )
print( all.equal( mm1, model.matrix( fitw2sls1$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2sls1$eq[[ 2 ]] ) ) )
print( !is.null( fitw2sls1$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitw2sls2e$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitw2sls2e ) ) )
print( all.equal( mm1, model.matrix( fitw2sls2e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2sls2e$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitw2sls2Sym ) ) )
print( all.equal( mm1, model.matrix( fitw2sls2Sym$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2sls2Sym$eq[[ 2 ]] ) ) )
print( !is.null( fitw2sls2Sym$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitw2slsd3$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitw2slsd3 ) ) )
print( all.equal( mm1, model.matrix( fitw2slsd3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2slsd3$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitw2slsd3e ) ) )
print( all.equal( mm1, model.matrix( fitw2slsd3e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2slsd3e$eq[[ 2 ]] ) ) )
print( !is.null( fitw2slsd3e$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitw2sls4$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitw2sls4 ) ) )
print( all.equal( mm1, model.matrix( fitw2sls4$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2sls4$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitw2sls4e ) ) )
print( all.equal( mm1, model.matrix( fitw2sls4e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2sls4e$eq[[ 2 ]] ) ) )
print( !is.null( fitw2sls4e$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitw2sls5$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitw2sls5 ) ) )
print( all.equal( mm1, model.matrix( fitw2sls5$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2sls5$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitw2sls5e ) ) )
print( all.equal( mm1, model.matrix( fitw2sls5e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitw2sls5e$eq[[ 2 ]] ) ) )
print( !is.null( fitw2sls5e$eq[[ 1 ]]$x ) )

# matrices of instrumental variables
model.matrix( fitw2sls1, which = "z" )
model.matrix( fitw2sls1$eq[[ 1 ]], which = "z" )
model.matrix( fitw2sls1$eq[[ 2 ]], which = "z" )

# matrices of fitted regressors
model.matrix( fitw2sls5e, which = "xHat" )
model.matrix( fitw2sls5e$eq[[ 1 ]], which = "xHat" )
model.matrix( fitw2sls5e$eq[[ 2 ]], which = "xHat" )


## **************** formulas ************************
formula( fitw2sls1e )
formula( fitw2sls1e$eq[[ 1 ]] )

formula( fitw2sls2 )
formula( fitw2sls2$eq[[ 2 ]] )

formula( fitw2sls3 )
formula( fitw2sls3$eq[[ 1 ]] )

formula( fitw2sls4e )
formula( fitw2sls4e$eq[[ 2 ]] )

formula( fitw2sls5 )
formula( fitw2sls5$eq[[ 1 ]] )

formula( fitw2slsd1 )
formula( fitw2slsd1$eq[[ 2 ]] )

formula( fitw2slsd2e )
formula( fitw2slsd2e$eq[[ 1 ]] )

formula( fitw2slsd3e )
formula( fitw2slsd3e$eq[[ 2 ]] )


## **************** model terms *******************
terms( fitw2sls1e )
terms( fitw2sls1e$eq[[ 1 ]] )

terms( fitw2sls2 )
terms( fitw2sls2$eq[[ 2 ]] )

terms( fitw2sls3 )
terms( fitw2sls3$eq[[ 1 ]] )

terms( fitw2sls4e )
terms( fitw2sls4e$eq[[ 2 ]] )

terms( fitw2sls5 )
terms( fitw2sls5$eq[[ 1 ]] )

terms( fitw2slsd1 )
terms( fitw2slsd1$eq[[ 2 ]] )

terms( fitw2slsd2e )
terms( fitw2slsd2e$eq[[ 1 ]] )

terms( fitw2slsd3e )
terms( fitw2slsd3e$eq[[ 2 ]] )


## **************** terms of instruments *******************
fitw2sls1e$eq[[ 1 ]]$termsInst

fitw2sls2$eq[[ 2 ]]$termsInst

fitw2sls3$eq[[ 1 ]]$termsInst

fitw2sls4e$eq[[ 2 ]]$termsInst

fitw2sls5$eq[[ 1 ]]$termsInst

fitw2slsd1$eq[[ 2 ]]$termsInst

fitw2slsd2e$eq[[ 1 ]]$termsInst

fitw2slsd3e$eq[[ 2 ]]$termsInst


## **************** estfun ************************
library( "sandwich" )

estfun( fitw2sls1 )
round( colSums( estfun( fitw2sls1 ) ), digits = 7 )

estfun( fitw2sls1e )
round( colSums( estfun( fitw2sls1e ) ), digits = 7 )

estfun( fitw2slsd1e )
round( colSums( estfun( fitw2slsd1e ) ), digits = 7 )


## **************** bread ************************
bread( fitw2sls1 )

bread( fitw2sls1e )

bread( fitw2slsd1e )

###   $Id: systemfit.R 1178 2022-09-04 18:54:35Z arne $
###
###            Simultaneous Equation Estimation for R
###
### Copyright 2002-2004 Jeff D. Hamann <jeff.hamann@forestinformatics.com>
###                     Arne Henningsen <http://www.arne-henningsen.de>
###
### This file is part of the nlsystemfit library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 2, or at your option, any later version,
### incorporated herein by reference.
###
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
###
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


systemfit <- function(  formula,
                        method = "OLS",
                        inst=NULL,
                        data=list(),
                        restrict.matrix = NULL,
                        restrict.rhs = NULL,
                        restrict.regMat = NULL,
                        pooled = FALSE,
                        control = systemfit.control( ... ),
                        ... )
{

   ## determine whether we have panel date and thus a panel-like model
   panelLike <- inherits(data, c("pdata.frame", "plm.dim"))

   ## checking argument 'formula'
   if( panelLike ){
      if( !inherits( formula, "formula" ) ){
         stop( "argument 'formula' must be an object of class 'formula'",
            " for panel-like models" )
      }
   } else {
      # single-equation models
      if( inherits( formula, "formula" ) ){
         formula <- list( formula )
      } else if( inherits( formula, "list" ) ){
         if( !all( sapply( formula, function(x) inherits( x, "formula" ) ) ) ){
            stop( "the list of argument 'formula' must",
               " contain only objects of class 'formula'" )
         }
      } else {
         stop( "argument 'formula' must be an object of class 'formula'",
            " or a list of objects of class 'formula'" )
      }
   }

   ## checking argument 'method'
   if(!( method %in% c( "OLS", "WLS", "SUR", "2SLS", "W2SLS", "3SLS",
         "LIML", "FIML" ) ) ){
      stop( "The method must be 'OLS', 'WLS', 'SUR',",
         " '2SLS', 'W2SLS', or '3SLS'" )
   }

   ## prepare model and data for panel-like models
   if( panelLike ) {
      if( !is.null( restrict.regMat ) && pooled ){
         stop( "argument 'restrict.regMat' cannot be used for pooled estimation",
            " of panel-like data" )
      }
      result <- .systemfitPanel( formula = formula, inst = inst,
         data = data, pooled = pooled )
      data <- result$wideData
      formula <- result$eqnSystem
      inst <- result$instSystem
      if( pooled ){
         restrict.regMat <- result$restrict.regMat
      }
   }

   ## checking argument 'inst'
   if( method %in% c( "2SLS", "W2SLS", "3SLS" ) ){
      if( is.null( inst ) ) {
         stop( "The methods '2SLS', 'W2SLS', and '3SLS' need instruments" )
      } else if( inherits( inst, "formula" ) ){
         if( length( inst ) != 2 ){
            stop( "argument 'inst' must be a one-sided formula" )
         }
         inst <- lapply( c( 1:length( formula ) ), function(x) inst )
      } else if( inherits( inst, "list" ) ){
         if( length( inst ) != length( formula ) ){
            stop( "if different instruments are specified for each equation,",
               " the length of argument 'inst' must be equal to the number",
               " of equations" )
         }
         if( !is.null( names( inst ) ) && !is.null( names( formula ) ) ){
            if( any( names( inst ) != names( formula ) ) ){
               warning( "names of formulas for instruments (argument 'inst')",
                  " are not equal to names of equations (argument 'formula')" )
            }
         }
         if( !all( sapply( inst, function(x) inherits( x, "formula" ) ) ) ){
            stop( "the list of argument 'inst' must",
               " contain only objects of class 'formula'" )
         }
         if( !all( lapply( inst, length ) == 2 ) ){
            stop( "the list of argument 'inst' must",
               " contain only one-sided formulas" )
         }
      } else {
         stop( "argument 'inst' must be an object of class 'formula'",
            " or a list of objects of class 'formula'" )
      }
   } else {
      if( !is.null( inst ) ) {
         warning( "The methods 'OLS', 'WLS', and 'SUR' do not need instruments;",
            " ignoring argument 'inst'" )
         inst <- NULL
      }
   }

   ## default value of argument 'singleEqSigma'
   if( is.null( control$singleEqSigma ) ) {
      control$singleEqSigma <- ( is.null( restrict.matrix ) & is.null( restrict.regMat ) )
   }

   ## checking argument 'pooled'
   if( !is.logical( pooled ) || length( pooled ) != 1 ){
      stop( "argument 'pooled' must be logical" )
   }

  results <- list()               # results to be returned
  results$eq <- list()            # results for the individual equations
  iter    <- NULL                 # number of iterations
  nEq     <- length( formula )       # number of equations
  ssr     <- numeric( nEq ) # sum of squared residuals of each equation
  sigma   <- numeric( nEq ) # estimated sigma (std. dev. of residuals) of each equation

   if( is.null( names( formula ) ) ) {
      eqnLabels <- paste( "eq", c( 1:nEq ), sep = "" )
   } else {
      eqnLabels <- names( formula )
      if( sum( regexpr( " |_", eqnLabels ) != -1 ) > 0 ) {
         stop( "equation labels may not contain blanks (' ') or underscores ('_')" )
      }
   }

   results$call <- match.call() # get the original call

   ## prepare y vectors and X matrices for each equation
   modelFrame <- .prepareData.systemfit( data )
   # list of terms objects of each equation
   termsEq <- list()
   # terms and model frames for the individual equations
   modelFrameEq <- list()
   # list of evaluated model frames of each equation
   evalModelFrameEq <- list()
   # list for vectors of endogenous variables in each equation
   yVecEq  <- list()
   # list for matrices of regressors in each equation
   xMatEq  <- list()
   # number of exogenous variables /(unrestricted) coefficients in each equation
   nCoefEq <- numeric( nEq )
   # names of observations of each equation (with observations with NAs)
   obsNamesNaEq <- list()
   # names of coefficients
   coefNames  <- NULL
   # names of coefficients of each equation
   coefNamesEq <- list()
   # prepare data for individual equations
   for(i in 1:nEq ) {
      modelFrameEq[[ i ]] <- modelFrame
      modelFrameEq[[ i ]]$formula <- formula[[ i ]]
      evalModelFrameEq[[ i ]] <- eval( modelFrameEq[[ i ]] )
      termsEq[[ i ]] <- attr( evalModelFrameEq[[ i ]], "terms" )
      weights <- model.extract( evalModelFrameEq[[ i ]], "weights" )
      yVecEq[[i]] <- model.extract( evalModelFrameEq[[ i ]], "response" )
      xMatEq[[i]] <- model.matrix( termsEq[[ i ]], evalModelFrameEq[[ i ]] )
      obsNamesNaEq[[ i ]] <- rownames( xMatEq[[ i ]] )
      nCoefEq[i] <- ncol(xMatEq[[i]])
      cNamesEq <- NULL
      for(j in 1:nCoefEq[i]) {
         xjName <- colnames( xMatEq[[ i ]] )[ j ]
         if( panelLike && xjName != "(Intercept)" ){
            coefNames <- c( coefNames, xjName )
            cNamesEq <- c( cNamesEq, sub(
               paste( "^", eqnLabels[ i ], "_", sep = "" ), "", xjName ) )
         } else {
            coefNames <- c( coefNames,
               paste( eqnLabels[ i ], xjName, sep = "_" ) )
            cNamesEq <- c( cNamesEq, xjName )
         }
      }
      coefNamesEq[[ i ]] <- cNamesEq
   }
   rm( modelFrameEq, xjName, cNamesEq )

   ## prepare Z matrices of instruments for each equation
   if( !is.null( inst ) ) {
      # list of terms objects of instruments of each equation
      termsInst <- list()
      # model frame of instruments
      modelFrameInst <- list()
      # evaluated model frame of instruments
      evalModelFrameInst <- list()
      # list for matrices of instruments in each equation
      zMatEq  <- list()
      # prepare data for individual equations
      for(i in 1:nEq) {
         modelFrameInst[[ i ]] <- modelFrame
         modelFrameInst[[ i ]]$formula <- inst[[ i ]]
         evalModelFrameInst[[ i ]] <- eval( modelFrameInst[[ i ]] )
         termsInst[[ i ]] <- attr( evalModelFrameInst[[ i ]], "terms" )
         zMatEq[[i]] <- model.matrix( termsInst[[ i ]], evalModelFrameInst[[ i ]] )
      }
   }

   ## check if all endogenous variables, regressors, and instruments
   ## have the same number of observations (including observations with NAs)
   # total number of observations per equation (including NAs)
   nObsWithNa <- length( yVecEq[[ 1 ]] )
   for( i in 1:nEq ){
      if( nObsWithNa != length( yVecEq[[ i ]] ) ) {
         stop( "all equations must have the same number of observations",
            " (including observations with NAs)",
            " but the endogenous variable of equation 1 has ",
            nObsWithNa, " observations",
            " while the endogenous variable of equation ", i, " has ",
            length( yVecEq[[ i ]] ), " observations" )
      }
      if( nObsWithNa != nrow( xMatEq[[ i ]] ) ) {
         stop( "the regressors of each equation must have the same number",
            " of observations as the corresponding endogenous variable",
            " (including observations with NAs)",
            " but the regressors of equation ", i, " have ",
            nrow( xMatEq[[ i ]] ), " observations",
            " while the endogenous variable of this equation has ",
            length( yVecEq[[ i ]] ), " observations" )
      }
      if( !is.null( inst ) ) {
         if( nObsWithNa != nrow( zMatEq[[ i ]] ) ) {
            stop( "the instrumental variables of each equation must have",
               " the same number of observations as the corresponding regressors",
               " (including observations with NAs)",
               " but the instrumental variables of equation ", i, " have ",
               nrow( zMatEq[[ i ]] ), " observations",
               " while the regressors of this equation have ",
               nrow( xMatEq[[ i ]] ), " observations" )
         }
      }
   }

   ## determine valid observations of each equation
   # which observations in each equation have no NAs
   validObsEq <- matrix( NA, nrow = nObsWithNa, ncol = nEq )
   for( i in 1:nEq ){
      validObsEq[ , i ] <- !is.na( yVecEq[[ i ]] ) &
         rowSums( is.na( xMatEq[[ i ]] ) ) == 0
      if( !is.null( inst ) ) {
         validObsEq[ , i ] <- validObsEq[ , i ] &
            rowSums( is.na( zMatEq[[ i ]] ) ) == 0
      }
   }

   ## check if the system of equations is unbalanced
   # which observations have no NAs in all equations
   validObsAll <- rowSums( !validObsEq ) == 0
   unbalanced <- FALSE
   for( i in 1:nEq ) {
      if( any( validObsEq[ !validObsAll, i ] ) ) {
         unbalanced <- TRUE
      }
   }
   if( unbalanced ) {
      warning( "the estimation of systems of equations with unequal numbers",
         " of observations has not been thoroughly tested yet" )
   }
   rm( validObsAll, unbalanced )

   ## remove all observations with NAs
   for( i in 1:nEq ) {
      # vectors of endogenous variables
      yVecEq[[ i ]] <- yVecEq[[ i ]][ validObsEq[ , i ] ]
      # matrices of regressors
      attrAssign <- attributes( xMatEq[[ i ]] )$assign
      xMatEq[[ i ]] <- xMatEq[[ i ]][ validObsEq[ , i ], , drop = FALSE ]
      attributes( xMatEq[[ i ]] )$assign <- attrAssign
      # matrices of instrumental variables
      if( !is.null( inst ) ) {
         zMatEq[[ i ]] <- zMatEq[[ i ]][ validObsEq[ , i ], , drop = FALSE ]
      }
   }
   rm( attrAssign )

   ## prepare matrices for using the Matrix package
   if( control$useMatrix ){
      # attributes of the model matrices
      xMatEqAttr <- list()
      for( i in 1:nEq ) {
         xMatEqAttr[[ i ]] <- attributes( xMatEq[[i]] )
         xMatEq[[ i ]] <- as( xMatEq[[ i ]], "denseMatrix")
         if( !is.null( inst ) ) {
            zMatEq[[ i ]] <- as( zMatEq[[ i ]], "denseMatrix")
         }
      }
   }

   # number of observations in each equation
   nObsEq  <- numeric( nEq )
   # names of observations of each equation (without observations with NAs)
   obsNamesEq <- list()
   for( i in 1:nEq ){
      obsNamesEq[[ i ]] <- rownames( xMatEq[[ i ]] )
      nObsEq[i] <- length( yVecEq[[i]] )
   }

   # stacked vector of all endogenous variables
   yVecAll <- matrix( 0, 0, 1 )
   for( i in 1:nEq ) {
      yVecAll <- c(yVecAll,yVecEq[[i]])
   }

   # stacked matrices of all regressors
   xMatAll <- .stackMatList( xMatEq, way = "diag",
      useMatrix = control$useMatrix )

   # impose restrictions via argument 'restrict.regMat'
   if( !is.null( restrict.regMat ) ) {
      # checking matrix to modify (post-multiply) the regressor matrix (restrict.regMat)
      if( !is.matrix( restrict.regMat ) ) {
         stop( "argument 'restrict.regMat' must be a matrix" )
      }
      if( nrow( restrict.regMat ) != sum( nCoefEq ) ){
         stop( "argument 'restrict.regMat' must be a matrix with number of rows",
            " equal to the number of all regressors [in this model: ",
            sum( nCoefEq ), "]" )
      }
      # default names for modified regressors and their coefficients
      if( is.null( colnames( restrict.regMat ) ) ){
         colnames( restrict.regMat ) <- paste( "C", c( 1:ncol( restrict.regMat ) ), sep = "" )
      }
      # default rownames for matrix to modify regressors
      if( is.null( rownames( restrict.regMat ) ) ){
         rownames( restrict.regMat ) <- coefNames
      }
      # modify regressor matrix (by restrict.regMat)
      XU <- xMatAll
      xMatAll  <- XU %*% restrict.regMat
      if( control$useMatrix ){
         xMatAll <- as( xMatAll, "CsparseMatrix" )
      }
   }

   # fitted values of regressors for IV estimations
   if( !is.null( inst ) ) {
      # stacked matrices of all instruments
      zMatAll <- .stackMatList( zMatEq, way = "diag",
         useMatrix = control$useMatrix )
      # fitted values of all regressors
      xMatHatAll <- calcFittedRegMat( xMatAll = xMatAll, zMatEq = zMatEq, 
         nEq = nEq, nObsEq = nObsEq, 
         useMatrix = control$useMatrix, solvetol = control$solvetol )
   }

   # checking and modifying parameter restrictions
   coefNamesModReg <- if( is.null( restrict.regMat ) ) coefNames else colnames( restrict.regMat )
   if( is.character( restrict.matrix ) ) {
      R.restr <- car::makeHypothesis( coefNamesModReg, restrict.matrix, restrict.rhs )
      if( is.null( dim( R.restr ) ) ){
         R.restr <- t( R.restr )
      }
      q.restr <- R.restr[ , ncol( R.restr ), drop = FALSE ]
      R.restr <- R.restr[ , -ncol( R.restr ), drop = FALSE ]
   } else if( !is.null( restrict.matrix ) ) {
      if( is.null( dim( restrict.matrix ) ) ) {
         R.restr <- t( restrict.matrix )
      } else {
         R.restr <- restrict.matrix
      }
      if( is.null( restrict.rhs ) ) {
         q.restr <- matrix( 0, nrow( restrict.matrix ) ,1 )
      } else {
         if( is.null( dim( restrict.rhs ) ) ) {
            q.restr <- matrix( restrict.rhs, ncol = 1  )
         } else {
            q.restr <- restrict.rhs
         }
      }
   } else {
      R.restr <- NULL
      if( !is.null( restrict.rhs ) ) {
         warning( "ignoring argument 'restrict.rhs',",
            " because argument 'restrict.matrix' is not specified" )
      }
      q.restr <- restrict.rhs
   }

   # row names and column names of restriction matrix and vector
   if( !is.null( R.restr ) ){
      if( is.null( rownames( R.restr ) ) ) {
         rownames( R.restr ) <-
            car::printHypothesis( R.restr, q.restr, coefNamesModReg )
      }
      if( is.null( colnames( R.restr ) ) ) {
         colnames( R.restr ) <- coefNamesModReg
      }
      if( is.null( rownames( q.restr ) ) ) {
         rownames( q.restr ) <-
            car::printHypothesis( R.restr, q.restr, coefNamesModReg )
      }
      if( is.null( colnames( q.restr ) ) ) {
         colnames( q.restr ) <- "*rhs*"
      }
   }

   nObsAll  <- sum( nObsEq )  # total number of observations of all equations
   nCoefAll <- sum( nCoefEq ) # total number of exogenous variables/(unrestricted) coefficients in all equations
   nCoefLiAll <- nCoefAll     # total number of linear independent coefficients in all equations
   nCoefLiEq  <- nCoefEq      # total number of linear independent coefficients in each equation
   if(!is.null(restrict.regMat)) {
      nCoefLiAll <- nCoefLiAll - ( nrow( restrict.regMat ) - ncol( restrict.regMat ) )
      for(i in 1:nEq) {
         nCoefLiEq[i] <- ncol(xMatAll)
         for(j in 1: ncol(xMatAll) ) {
            if(sum(xMatAll[(1+sum(nObsEq[1:i])-nObsEq[i]):(sum(nObsEq[1:i])),j]^2)==0) {
               nCoefLiEq[i] <- nCoefLiEq[i]-1
            }
         }
      }
   }
   if(!is.null(R.restr)) {
      nCoefLiAll  <- nCoefLiAll - nrow(R.restr)
      if(is.null(restrict.regMat)) {
         for(j in 1:nrow(R.restr)) {
            for(i in 1:nEq) {  # search for restrictions that are NOT cross-equation
               if( sum( R.restr[ j, (1+sum(nCoefEq[1:i])-nCoefEq[i]):(sum(nCoefEq[1:i]))]^2) ==
                   sum(R.restr[j,]^2)) {
                  nCoefLiEq[i] <- nCoefLiEq[i]-1
               }
            }
         }
      }
    }
    df <- nObsEq - nCoefLiEq    # degress of freedom of each equation

  ## only for OLS, WLS and SUR estimation
  if( method %in% c( "OLS", "WLS", "SUR" ) ) {
    if(is.null(R.restr)) {
      coef <- solve( crossprod( xMatAll ), crossprod( xMatAll, yVecAll ), tol=control$solvetol )
               # estimated coefficients
    } else {
      W <- .prepareWmatrix( crossprod( xMatAll ), R.restr,
         useMatrix = control$useMatrix )
      V <- c( as.numeric( crossprod( xMatAll, yVecAll ) ), q.restr )
      if( method == "OLS" || control$residCovRestricted ){
         coef <- solve( W, V, tol=control$solvetol )[ 1:ncol(xMatAll) ]
      } else {
         coef <- solve( crossprod( xMatAll ), crossprod( xMatAll, yVecAll ),
            tol = control$solvetol )
      }
    }
  }

  ## only for OLS estimation
  if(method=="OLS") {
    resids <- yVecAll - xMatAll %*% coef                                        # residuals
    if(control$singleEqSigma) {
      rcov <- .calcResidCov( resids, methodResidCov = control$methodResidCov,
         validObsEq = validObsEq, nCoefEq = nCoefLiEq, xEq = xMatEq, diag = TRUE,
         centered = control$centerResiduals, useMatrix = control$useMatrix,
         solvetol = control$solvetol )    # residual covariance matrix
      coefCov <- .calcGLS( xMat = xMatAll, R.restr = R.restr, q.restr = q.restr,
         sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
         solvetol = control$solvetol )
                    # coefficient covariance matrix
    } else {
      s2 <- .calcSigma2( resids, nObs = nObsAll, nCoef = nCoefLiAll, methodResidCov = control$methodResidCov )
                           # sigma squared
      if(is.null(R.restr)) {
        coefCov <- s2 * solve( crossprod( xMatAll ), tol=control$solvetol )
                          # coefficient covariance matrix
      } else {
        coefCov <- s2 * solve( W, tol=control$solvetol )[1:ncol(xMatAll),1:ncol(xMatAll)]
                    # coefficient covariance matrix
      }
    }
  }

  ## only for WLS estimation
  if( method %in% c( "WLS" ) ||
      ( method %in% c( "SUR" ) && control$residCovWeighted ) ) {
    coefOld  <- coef # coefficients of previous step
    coefDiff <- coef # difference of coefficients between this and previous step
    iter  <- 0
    while((sum(coefDiff^2)/sum(coefOld^2))^0.5>control$tol & iter < control$maxiter^( method == "WLS" ) ) {
      iter  <- iter+1
      coefOld <- coef                # coefficients of previous step
      resids <- yVecAll - xMatAll %*% coef     # residuals
      rcov <- .calcResidCov( resids, methodResidCov = control$methodResidCov,
         validObsEq = validObsEq, nCoefEq = nCoefLiEq, xEq = xMatEq, diag = TRUE,
         centered = control$centerResiduals, useMatrix = control$useMatrix,
         solvetol = control$solvetol )
      coef  <- .calcGLS( xMat = xMatAll, yVec = yVecAll, R.restr = R.restr,
         q.restr = q.restr, sigma = rcov, validObsEq = validObsEq,
         useMatrix = control$useMatrix, solvetol = control$solvetol )
      coefDiff <- coef - coefOld # difference of coefficients between this and previous step
    }
    coefCov <- .calcGLS( xMat = xMatAll, R.restr = R.restr, q.restr = q.restr,
       sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
       solvetol = control$solvetol )
    resids <- yVecAll - xMatAll %*% coef                        # residuals
  }

  ## only for SUR estimation
  if( method %in% c( "SUR" ) ) {
    coefOld  <- coef # coefficients of previous step
    coefDiff <- coef # difference of coefficients between this and previous step
    iter  <- 0
    while((sum(coefDiff^2)/sum(coefOld^2))^0.5>control$tol & iter < control$maxiter) {
      iter  <- iter+1
      coefOld <- coef                           # coefficients of previous step
      resids <- yVecAll-xMatAll%*%coef                     # residuals
      rcov <- .calcResidCov( resids, methodResidCov = control$methodResidCov,
         validObsEq = validObsEq, nCoefEq = nCoefLiEq, xEq = xMatEq,
         centered = control$centerResiduals, useMatrix = control$useMatrix,
         solvetol = control$solvetol )
      coef <- .calcGLS( xMat = xMatAll, yVec = yVecAll,
         R.restr = R.restr, q.restr = q.restr,
         sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
         solvetol = control$solvetol )     # coefficients
      coefDiff <- coef - coefOld # difference of coefficients between this and previous step
    }
    coefCov <- .calcGLS( xMat = xMatAll, R.restr = R.restr, q.restr = q.restr,
       sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
       solvetol = control$solvetol )
            # final step coefficient covariance matrix
    resids <- yVecAll - xMatAll %*% coef                        # residuals
  }

  ## only for 2SLS, W2SLS and 3SLS estimation
  if( method %in% c( "2SLS", "W2SLS", "3SLS" ) ) {
    if(is.null(R.restr)) {
      coef <- solve( crossprod( xMatHatAll ), crossprod( xMatHatAll, yVecAll ), tol=control$solvetol )
         # 2nd stage coefficients
    } else {
      W <- .prepareWmatrix( crossprod(xMatHatAll), R.restr,
         useMatrix = control$useMatrix )
      V <- c( as.numeric( crossprod( xMatHatAll, yVecAll ) ), q.restr )
      if( method == "2SLS" || control$residCovRestricted ){
         coef <- solve( W, V, tol=control$solvetol )[ 1:ncol(xMatAll) ]
      } else {
         coef <- solve( crossprod( xMatHatAll ), crossprod( xMatHatAll, yVecAll ),
            tol = control$solvetol )
      }
    }
    b2 <- coef
  }

  ## only for 2SLS estimation
  if(method=="2SLS") {
    resids <- yVecAll - xMatAll %*% coef                        # residuals
    if(control$singleEqSigma) {
      rcov <- .calcResidCov( resids, methodResidCov = control$methodResidCov,
         validObsEq = validObsEq, nCoefEq = nCoefLiEq, xEq = xMatEq, diag = TRUE,
         centered = control$centerResiduals, useMatrix = control$useMatrix,
         solvetol = control$solvetol )
      coefCov <- .calcGLS( xMat = xMatHatAll, R.restr = R.restr, q.restr = q.restr,
         sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
         solvetol = control$solvetol )  # coefficient covariance matrix
    } else {
      s2 <- .calcSigma2( resids, nObs = nObsAll, nCoef = nCoefLiAll, methodResidCov = control$methodResidCov )
                           # sigma squared
      if(is.null(R.restr)) {
        coefCov <- s2 * solve( crossprod( xMatHatAll ), tol=control$solvetol )
                  # coefficient covariance matrix
      } else {
        coefCov <- s2 * solve( W, tol=control$solvetol )[1:ncol(xMatAll),1:ncol(xMatAll)]
                    # coeff. covariance matrix
      }
    }
  }

  ## only for W2SLS estimation
  if( method %in% c( "W2SLS" ) ||
         ( method %in% c( "3SLS" ) && control$residCovWeighted ) ) {
    coefOld  <- coef # coefficients of previous step
    coefDiff <- coef # difference of coefficients between this and previous step
    iter  <- 0
    while((sum(coefDiff^2)/sum(coefOld^2))^0.5>control$tol & iter < control$maxiter^( method == "W2LS" ) ) {
      iter  <- iter+1
      coefOld <- coef                           # coefficients of previous step
      resids <- yVecAll-xMatAll%*%coef                     # residuals
      rcov <- .calcResidCov( resids, methodResidCov = control$methodResidCov,
         validObsEq = validObsEq, nCoefEq = nCoefLiEq, xEq = xMatEq, diag = TRUE,
         centered = control$centerResiduals, useMatrix = control$useMatrix,
         solvetol = control$solvetol )
      coef <- .calcGLS( xMat = xMatHatAll, yVec = yVecAll, R.restr = R.restr, q.restr = q.restr,
         sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
         solvetol = control$solvetol )          # (unrestr.) coeffic.
      coefDiff <- coef - coefOld # difference of coefficients between this and previous step
    }
    coefCov <- .calcGLS( xMat = xMatHatAll, R.restr = R.restr, q.restr = q.restr,
       sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
       solvetol = control$solvetol )  # coefficient covariance matrix
    resids <- yVecAll - xMatAll %*% coef                        # residuals
  }

  ## only for 3SLS estimation
  if( method %in% c( "3SLS" ) ) {
    coefOld  <- coef # coefficients of previous step
    coefDiff <- coef # difference of coefficients between this and previous step
    iter  <- 0
    while((sum(coefDiff^2)/sum(coefOld^2))^0.5>control$tol & iter < control$maxiter) {
      iter  <- iter+1
      coefOld <- coef                           # coefficients of previous step
      resids <- yVecAll-xMatAll%*%coef                     # residuals
      rcov <- .calcResidCov( resids, methodResidCov = control$methodResidCov,
         validObsEq = validObsEq, nCoefEq = nCoefLiEq, xEq = xMatEq,
         centered = control$centerResiduals, useMatrix = control$useMatrix,
         solvetol = control$solvetol )
      if(control$method3sls=="GLS") {
         coef <- .calcGLS( xMat = xMatHatAll, yVec = yVecAll,
            R.restr = R.restr, q.restr = q.restr,
            sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
            solvetol = control$solvetol )  # (unrestr.) coeffic.
      }
      if(control$method3sls=="IV") {
         coef <- .calcGLS( xMat = xMatHatAll, xMat2 = xMatAll, yVec = yVecAll,
            R.restr = R.restr, q.restr = q.restr,
            sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
            solvetol = control$solvetol )   # (unrestr.) coeffic.
      }
      if(control$method3sls=="GMM") {
        ZtOmega <- .calcXtOmegaInv( xMat = zMatAll, sigma = rcov,
           validObsEq = validObsEq, invertSigma = FALSE, useMatrix = control$useMatrix,
           solvetol = control$solvetol )
        if(is.null(R.restr)) {
          coef <- as.numeric( solve( crossprod( xMatAll, zMatAll ) %*%
            solve( ZtOmega %*% zMatAll, crossprod( zMatAll, xMatAll ),
            tol=control$solvetol ), crossprod( xMatAll, zMatAll ) %*%
            solve( ZtOmega %*% zMatAll, crossprod( zMatAll, yVecAll ),
            tol=control$solvetol ), tol=control$solvetol ) )
        } else {
          W <- .prepareWmatrix( crossprod( xMatAll, zMatAll ) %*%
               solve( ZtOmega %*% zMatAll, crossprod( zMatAll, xMatAll ),
               tol=control$solvetol ), R.restr,
               useMatrix = control$useMatrix )
          V <- c( as.numeric( crossprod( xMatAll, zMatAll ) %*%
            solve( ZtOmega  %*% zMatAll, crossprod( zMatAll, yVecAll ),
            tol = control$solvetol ) ), q.restr )
          Winv <- solve( W, tol=control$solvetol )
          coef <- ( Winv %*% V )[1:ncol(xMatAll),]     # restricted coefficients
        }
      }
      if(control$method3sls=="Schmidt") {
        xMatHatOmegaInv <- .calcXtOmegaInv( xMat = xMatHatAll, sigma = rcov,
           validObsEq = validObsEq, useMatrix = control$useMatrix,
           solvetol = control$solvetol )
        if(is.null(R.restr)) {
          coef <- as.numeric( solve( crossprod( xMatHatAll, t( xMatHatOmegaInv ) ), 
            xMatHatOmegaInv %*% zMatAll %*%
            solve( crossprod( zMatAll ), crossprod( zMatAll, yVecAll),
            tol=control$solvetol ), tol=control$solvetol ) )
                           # (unrestr.) coeffic.
        } else {
          W <- .prepareWmatrix( crossprod( xMatHatAll, t( xMatHatOmegaInv ) ),
             R.restr, useMatrix = control$useMatrix )
          V <- c( as.numeric( xMatHatOmegaInv %*% zMatAll %*%
            solve( crossprod( zMatAll ), crossprod( zMatAll, yVecAll ),
            tol = control$solvetol ) ), q.restr )
          Winv <- solve( W, tol=control$solvetol )
          coef <- ( Winv %*% V )[1:ncol(xMatAll),]     # restricted coefficients
        }
      }
      if(control$method3sls=="EViews") {
         coef <- b2 + .calcGLS( xMat = xMatHatAll, yVec = ( yVecAll -  xMatAll %*% b2 ),
            R.restr = R.restr, q.restr = q.restr, sigma = rcov,
            validObsEq = validObsEq, useMatrix = control$useMatrix,
            solvetol = control$solvetol )  # (unrestr.) coeffic.
      }
      coefDiff <- coef - coefOld # difference of coefficients between this and previous step
    }
    if(control$method3sls=="GLS") {
       coefCov <- .calcGLS( xMat = xMatHatAll, R.restr = R.restr, q.restr = q.restr,
          sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
          solvetol = control$solvetol )  # coefficient covariance matrix
    }
    if(control$method3sls=="IV") {
       coefCov <- .calcGLS( xMat = xMatHatAll, xMat2 = xMatAll,
          R.restr = R.restr, q.restr = q.restr,
          sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
          solvetol = control$solvetol )
    }
    if(control$method3sls=="GMM") {
      if(is.null(R.restr)) {
        coefCov <- solve( crossprod( xMatAll, zMatAll ) %*%
           solve( ZtOmega %*% zMatAll, crossprod( zMatAll, xMatAll ),
           tol=control$solvetol ), tol=control$solvetol )
                # final step coefficient covariance matrix
      } else {
        coefCov <- Winv[1:ncol(xMatAll),1:ncol(xMatAll)] # coefficient covariance matrix
      }
    }
    if(control$method3sls=="Schmidt") {
      xMatHatOmegaInv <- .calcXtOmegaInv( xMat = xMatHatAll, sigma = rcov,
         validObsEq = validObsEq, useMatrix = control$useMatrix,
         solvetol = control$solvetol )
      PH <- zMatAll %*%  solve( crossprod( zMatAll ), t( zMatAll ),
         tol=control$solvetol )
      PHOmega <- .calcXtOmegaInv( xMat = t( PH ), sigma = rcov,
         validObsEq = validObsEq, invertSigma = FALSE, useMatrix = control$useMatrix,
         solvetol = control$solvetol )
      if(is.null(R.restr)) {
         coefCov <- solve( xMatHatOmegaInv %*% xMatHatAll,
            xMatHatOmegaInv %*% PHOmega %*% PH %*%
            crossprod( xMatHatOmegaInv, solve( xMatHatOmegaInv %*% xMatHatAll,
            tol=control$solvetol ) ), tol=control$solvetol )
                  # final step coefficient covariance matrix
      } else {
         VV <- .stackMatList(
            list( xMatHatOmegaInv %*% PHOmega %*% PH %*% t( xMatHatOmegaInv ),
            matrix( 0, nrow( R.restr ), nrow( R.restr ) ) ), "diag",
            useMatrix = control$useMatrix )
         coefCov <- ( Winv %*% VV %*% Winv )[ 1:ncol(xMatAll), 1:ncol(xMatAll) ]
                  # coefficient covariance matrix
      }
    }
    if(control$method3sls=="EViews") {
       coefCov <- .calcGLS( xMat = xMatHatAll,
          R.restr = R.restr, q.restr = q.restr,
          sigma = rcov, validObsEq = validObsEq, useMatrix = control$useMatrix,
          solvetol = control$solvetol )  # final step coefficient covariance matrix
    }
    resids <- yVecAll - xMatAll %*% coef                        # residuals
  }

  ## FIML estimation
  if( method == "FIML" ) {
    fimlResult <- .systemfitFiml( systemfitCall = results$call, nObsEq = nObsEq,
      validObsEq = validObsEq,
      nCoefEq = nCoefLiEq, yVec = yVecAll, xMat = xMatAll, xEq = xMatEq, methodResidCov = control$methodResidCov,
      centerResiduals = control$centerResiduals, solvetol = control$solvetol )
    #print( fimlResult )
    coef <- fimlResult$coefficients
    coefCov <- fimlResult$coefCov
    resids <- fimlResult$resids
  }

  ## for all estimation methods
  fitted.values <- xMatAll %*% coef   # fitted endogenous values
  if(!is.null(restrict.regMat)) {
    coef  <- restrict.regMat %*% coef
    coefCov <- restrict.regMat %*% coefCov %*% t(restrict.regMat)
  }


  ## equation wise results
  for(i in 1:nEq) {
    results$eq[[ i ]] <- list()
    results$eq[[ i ]]$eqnNo    <- i               # equation number
    results$eq[[ i ]]$eqnLabel <- eqnLabels[[i]]
    results$eq[[ i ]]$method   <- method

    results$eq[[ i ]]$residuals <- rep( NA, nObsWithNa )
    results$eq[[ i ]]$residuals[ validObsEq[ , i ] ] <-
      resids[ ( 1 + sum(nObsEq[1:i]) -nObsEq[i] ):( sum(nObsEq[1:i]) ) ]
    names( results$eq[[ i ]]$residuals ) <- obsNamesNaEq[[ i ]]

    results$eq[[ i ]]$coefficients <-
      drop( coef[(1+sum(nCoefEq[1:i])-nCoefEq[i]):(sum(nCoefEq[1:i]))] )
              # estimated coefficients of equation i
    results$eq[[ i ]]$coefCov  <- as.matrix(
      coefCov[(1+sum(nCoefEq[1:i])-nCoefEq[i]):(sum(nCoefEq[1:i])),
        (1+sum(nCoefEq[1:i])-nCoefEq[i]):(sum(nCoefEq[1:i]))] )
              # covariance matrix of estimated coefficients of equation i

    # set names
    names( results$eq[[ i ]]$coefficients )  <- coefNamesEq[[ i ]]
    colnames( results$eq[[ i ]]$coefCov ) <- coefNamesEq[[ i ]]
    rownames( results$eq[[ i ]]$coefCov ) <- coefNamesEq[[ i ]]

    results$eq[[ i ]]$fitted.values <- rep( NA, nObsWithNa )
    results$eq[[ i ]]$fitted.values[ validObsEq[ , i ] ] <-
      fitted.values[(1+sum(nObsEq[1:i])-nObsEq[i]):(sum(nObsEq[1:i]))]
    names( results$eq[[ i ]]$fitted.values ) <- obsNamesNaEq[[ i ]]

    results$eq[[ i ]]$terms    <- termsEq[[ i ]]
    results$eq[[ i ]]$rank     <- nCoefLiEq[i]
      # rank = number of linear independent coefficients
    results$eq[[ i ]]$nCoef.sys    <- nCoefAll
      # total number of coefficients of the entire system
    results$eq[[ i ]]$rank.sys     <- nCoefLiAll
      # rank = number of linear independent coefficients of the entire system
    results$eq[[ i ]]$df.residual  <- df[i]           # degrees of freedom of residuals
    results$eq[[ i ]]$df.residual.sys  <- nObsAll- nCoefLiAll
       # degrees of freedom of residuals of the whole system
    if( control$y ){
      results$eq[[ i ]]$y   <- yVecEq[[i]]     # vector of endogenous variables
      names( results$eq[[ i ]]$y ) <- obsNamesEq[[ i ]]
    }
    if( control$x ){
      results$eq[[ i ]]$x  <- as.matrix( xMatEq[[i]] )
      if( control$useMatrix ){
         attributes( results$eq[[ i ]]$x ) <- xMatEqAttr[[ i ]]
      }
      rownames( results$eq[[ i ]]$x ) <- obsNamesEq[[ i ]]
    }
    if( control$model ){
      results$eq[[ i ]]$model <- evalModelFrameEq[[ i ]]
         # model frame of this equation
      rownames( results$eq[[ i ]]$model ) <- obsNamesNaEq[[ i ]]
      if( !is.null( inst ) ) {
         results$eq[[ i ]]$modelInst <- evalModelFrameInst[[ i ]]
         rownames( results$eq[[ i ]]$modelInst ) <- obsNamesNaEq[[ i ]]
      }
    }
    if( method %in% c( "2SLS", "W2SLS", "3SLS" ) ) {
      results$eq[[ i ]]$inst         <- inst[[i]]
      results$eq[[ i ]]$termsInst <- termsInst[[i]]
      if(  control$z ) {
         results$eq[[ i ]]$z <- zMatEq[[i]]  # matrix of instrumental variables
         rownames( results$eq[[ i ]]$z ) <- obsNamesEq[[ i ]]
      }
    }
    class( results$eq[[ i ]] ) <- "systemfit.equation"
  }

  ## results of the total system
  # all estimated coefficients
  results$coefficients <- as.numeric( drop( coef ) )
  names( results$coefficients ) <- coefNames

  # coefficients covariance matrix
  results$coefCov <- as.matrix( coefCov )
  colnames( results$coefCov ) <- coefNames
  rownames( results$coefCov ) <- coefNames

  # residual covarance matrix used for estimation
  if( method %in% c( "WLS", "W2SLS", "SUR", "3SLS" ) ){
    results$residCovEst <- as.matrix( rcov )
    colnames( results$residCovEst ) <- eqnLabels
    rownames( results$residCovEst ) <- eqnLabels
  }

  # residual covarance matrix
  results$residCov <- .calcResidCov( resids, methodResidCov = control$methodResidCov,
      validObsEq = validObsEq, nCoefEq = nCoefLiEq, xEq = xMatEq,
      centered = control$centerResiduals, solvetol = control$solvetol )
  colnames( results$residCov ) <- eqnLabels
  rownames( results$residCov ) <- eqnLabels

  results$method  <- method
  results$rank    <- nCoefLiAll
     # rank = total number of linear independent coefficients of all equations
  results$df.residual <- nObsAll - nCoefLiAll
     # degrees of freedom of the whole system
  results$iter    <- iter
  results$restrict.matrix <- R.restr
  results$restrict.rhs <- q.restr
  results$restrict.regMat <- restrict.regMat
  results$control <- control
  results$panelLike <- panelLike
  class(results)  <- "systemfit"

  results
}

library( systemfit )
options( digits = 3 )

data( "Kmenta" )
useMatrix <- FALSE

demand <- consump ~ price + income
supply <- consump ~ price + farmPrice + trend
system <- list( demand = demand, supply = supply )
restrm <- matrix(0,1,7)  # restriction matrix "R"
restrm[1,3] <-  1
restrm[1,7] <- -1
restrict <- "demand_income - supply_trend = 0"
restr2m <- matrix(0,2,7)  # restriction matrix "R" 2
restr2m[1,3] <-  1
restr2m[1,7] <- -1
restr2m[2,2] <- -1
restr2m[2,5] <-  1
restr2q <- c( 0, 0.5 )    # restriction vector "q" 2
restrict2 <- c( "demand_income - supply_trend = 0",
   "- demand_price + supply_price = 0.5" )
tc <- matrix(0,7,6)
tc[1,1] <- 1
tc[2,2] <- 1
tc[3,3] <- 1
tc[4,4] <- 1
tc[5,5] <- 1
tc[6,6] <- 1
tc[7,3] <- 1
restr3m <- matrix(0,1,6)  # restriction matrix "R" 2
restr3m[1,2] <- -1
restr3m[1,5] <-  1
restr3q <- c( 0.5 )       # restriction vector "q" 2
restrict3 <- "- C2 + C5 = 0.5"

# It is not possible to estimate OLS with systemfit
# exactly as EViews does, because EViews uses
# methodResidCov == "geomean" for the coefficient covariance matrix and
# methodResidCov == "noDfCor" for the residual covariance matrix, while
# systemfit uses always the same formulas for both calculations.

## *******  single-equation OLS estimations  *********************
lmDemand <- lm( demand, data = Kmenta )
lmSupply <- lm( supply, data = Kmenta )

## *************** OLS estimation ************************
## ********** OLS estimation (default) ********************
fitols1 <- systemfit( system, "OLS", data = Kmenta, useMatrix = useMatrix )
print( summary( fitols1 ) )
nobs( fitols1 )
all.equal( coef( fitols1 ), c( coef( lmDemand ), coef( lmSupply ) ),
   check.attributes = FALSE )
all.equal( coef( summary( fitols1 ) ),
   rbind( coef( summary( lmDemand ) ), coef( summary( lmSupply ) ) ),
   check.attributes = FALSE )
all.equal( vcov( fitols1 ),
   as.matrix( bdiag( vcov( lmDemand ), vcov( lmSupply ) ) ),
   check.attributes = FALSE )

## ********** OLS estimation (no singleEqSigma=F) ******************
fitols1s <- systemfit( system, "OLS", data = Kmenta,
   singleEqSigma = FALSE, useMatrix = useMatrix )
print( summary( fitols1s ) )
all.equal( coef( fitols1s ), c( coef( lmDemand ), coef( lmSupply ) ),
   check.attributes = FALSE )

## ****************  OLS (useDfSys=T) ***********************
print( summary( fitols1, useDfSys = TRUE ) )

## ****************  OLS (methodResidCov="noDfCor") ***********************
fitols1r <- systemfit( system, "OLS", data = Kmenta,
   methodResidCov = "noDfCor", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitols1r ) )
all.equal( coef( fitols1r ), c( coef( lmDemand ), coef( lmSupply ) ),
   check.attributes = FALSE )

## ********  OLS (methodResidCov="noDfCor", singleEqSigma=F) ***********
fitols1rs <- systemfit( system, "OLS", data = Kmenta,
   methodResidCov = "noDfCor", singleEqSigma = FALSE,
   useMatrix = useMatrix )
print( summary( fitols1rs ) )
all.equal( coef( fitols1rs ), c( coef( lmDemand ), coef( lmSupply ) ),
   check.attributes = FALSE )

## ****************  OLS (methodResidCov="Theil" ) ***********************
fitols1r <- systemfit( system, "OLS", data = Kmenta,
   methodResidCov = "Theil", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitols1r ) )
all.equal( coef( fitols1r ), c( coef( lmDemand ), coef( lmSupply ) ),
   check.attributes = FALSE )

## ****************  OLS (methodResidCov="max") ***********************
fitols1r <- systemfit( system, "OLS", data = Kmenta,
   methodResidCov = "max", x = TRUE,
   useMatrix = useMatrix )
print( summary( fitols1r ) )

## ********  OLS (methodResidCov="max", singleEqSigma=F) ***********
fitols1rs <- systemfit( system, "OLS", data = Kmenta,
   methodResidCov = "max", singleEqSigma = FALSE,
   useMatrix = useMatrix )
print( summary( fitols1rs ) )


## ********* OLS with cross-equation restriction ************
## ****** OLS with cross-equation restriction (default) *********
fitols2 <- systemfit( system, "OLS", data = Kmenta,
   restrict.matrix = restrm, useMatrix = useMatrix )
print( summary( fitols2 ) )
# the same with symbolically specified restrictions
fitols2Sym <- systemfit( system, "OLS", data = Kmenta,
   restrict.matrix = restrict, useMatrix = useMatrix )
all.equal( fitols2, fitols2Sym )

## ****** OLS with cross-equation restriction (singleEqSigma=T) *******
fitols2s <- systemfit( system, "OLS", data = Kmenta,
   restrict.matrix = restrm, singleEqSigma = TRUE,
   useMatrix = useMatrix )
print( summary( fitols2s ) )

## ****** OLS with cross-equation restriction (useDfSys=F) *******
print( summary( fitols2, useDfSys = FALSE ) )

## ****** OLS with cross-equation restriction (methodResidCov="noDfCor") *******
fitols2r <- systemfit( system, "OLS", data = Kmenta,
   restrict.matrix = restrm, methodResidCov = "noDfCor",
   useMatrix = useMatrix )
print( summary( fitols2r ) )

## ** OLS with cross-equation restriction (methodResidCov="noDfCor",singleEqSigma=T) ***
fitols2rs <- systemfit( system, "OLS", data = Kmenta,
   restrict.matrix = restrm, methodResidCov = "noDfCor",
   x = TRUE, useMatrix = useMatrix )
print( summary( fitols2rs ) )

## *** OLS with cross-equation restriction via restrict.regMat ***
## *** OLS with cross-equation restriction via restrict.regMat (default) ***
fitols3 <- systemfit( system, "OLS", data = Kmenta, restrict.regMat = tc,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitols3 ) )

## *** OLS with cross-equation restriction via restrict.regMat (singleEqSigma=T) ***
fitols3s <- systemfit( system, "OLS", data = Kmenta,
   restrict.regMat = tc, singleEqSigma = TRUE, useMatrix = useMatrix )
print( summary( fitols3s ) )

## *** OLS with cross-equation restriction via restrict.regMat (useDfSys=F) ***
print( summary( fitols3, useDfSys = FALSE ) )

## *** OLS with cross-equation restriction via restrict.regMat (methodResidCov="noDfCor") ***
fitols3r <- systemfit( system, "OLS", data = Kmenta,
   restrict.regMat = tc, methodResidCov = "noDfCor",
   useMatrix = useMatrix )
print( summary( fitols3r ) )

## OLS with cross-equation restriction via restrict.regMat (methodResidCov="noDfCor",singleEqSigma=T)
fitols3rs <- systemfit( system, "OLS", data = Kmenta,
   restrict.regMat = tc, methodResidCov = "noDfCor", singleEqSigma = TRUE,
   useMatrix = useMatrix )
print( summary( fitols3rs ) )

## ********* OLS with 2 cross-equation restrictions ***********
## ********* OLS with 2 cross-equation restrictions (default) ***********
fitols4 <- systemfit( system, "OLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, useMatrix = useMatrix )
print( summary( fitols4 ) )
# the same with symbolically specified restrictions
fitols4Sym <- systemfit( system, "OLS", data = Kmenta,
   restrict.matrix = restrict2, useMatrix = useMatrix )
all.equal( fitols4, fitols4Sym )

## ****** OLS with 2 cross-equation restrictions (singleEqSigma=T) *******
fitols4s <- systemfit( system, "OLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, singleEqSigma = TRUE, x = TRUE,
   useMatrix = useMatrix )
print( summary( fitols4s ) )

## ****** OLS with 2 cross-equation restrictions (useDfSys=F) *******
print( summary( fitols4, useDfSys = FALSE ) )

## ****** OLS with 2 cross-equation restrictions (methodResidCov="noDfCor") *******
fitols4r <- systemfit( system, "OLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, methodResidCov = "noDfCor",
   useMatrix = useMatrix )
print( summary( fitols4r ) )

## OLS with 2 cross-equation restrictions (methodResidCov="noDfCor", singleEqSigma=T) *
fitols4rs <- systemfit( system, "OLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, methodResidCov = "noDfCor",
   singleEqSigma = TRUE, useMatrix = useMatrix )
print( summary( fitols4rs ) )

## ***** OLS with 2 cross-equation restrictions via R and restrict.regMat ****
## ***** OLS with 2 cross-equation restrictions via R and restrict.regMat (default) ****
fitols5 <- systemfit( system, "OLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, methodResidCov = "noDfCor",
   useMatrix = useMatrix )
print( summary( fitols5 ) )
# the same with symbolically specified restrictions
fitols5Sym <- systemfit( system, "OLS", data = Kmenta,
   restrict.matrix = restrict3, restrict.regMat = tc,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
all.equal( fitols5, fitols5Sym )

## ***** OLS with 2 cross-equation restrictions via R and restrict.regMat (singleEqSigma=T) ****
fitols5s <- systemfit( system, "OLS", data = Kmenta,restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, singleEqSigma = T,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitols5s ) )

## ***** OLS with 2 cross-equation restrictions via R and restrict.regMat (useDfSys=F) ****
fitols5o <- systemfit( system, "OLS", data = Kmenta,restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, useMatrix = useMatrix )
print( summary( fitols5o, useDfSys = FALSE ) )

## OLS with 2 cross-equation restr. via R and restrict.regMat (methodResidCov="noDfCor",singleEqSigma=T)
fitols5rs <- systemfit( system, "OLS", data = Kmenta,restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, methodResidCov = "noDfCor",
   singleEqSigma = TRUE, useMatrix = useMatrix )
print( summary( fitols5rs ) )


## *********** estimations with a single regressor ************
fitolsS1 <- systemfit(
   list( consump ~ price - 1, consump ~ price + trend ), "OLS",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitolsS1 ) )
fitolsS2 <- systemfit(
   list( consump ~ price - 1, consump ~ trend - 1 ), "OLS",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitolsS2 ) )
fitolsS3 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "OLS",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitolsS3 ) )
fitolsS4 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "OLS",
   data = Kmenta, restrict.matrix = matrix( c( 1, -1 ), nrow = 1 ),
   useMatrix = useMatrix )
print( summary( fitolsS4 ) )
fitolsS5 <- systemfit(
   list( consump ~ 1, farmPrice ~ 1 ), "OLS",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitolsS5 ) )


## **************** shorter summaries **********************
print( summary( fitols1, useDfSys = TRUE, equations = FALSE ) )

print( summary( fitols2r ), residCov = FALSE, equations = FALSE )

print( summary( fitols3s, useDfSys = FALSE ), residCov = TRUE )

print( summary( fitols4rs, residCov = FALSE, equations = FALSE ) )

print( summary( fitols5, equations = FALSE ), residCov = FALSE )


## ****************** residuals **************************
print( residuals( fitols1 ) )
print( residuals( fitols1$eq[[ 2 ]] ) )

print( residuals( fitols2r ) )
print( residuals( fitols2r$eq[[ 1 ]] ) )

print( residuals( fitols3s ) )
print( residuals( fitols3s$eq[[ 2 ]] ) )

print( residuals( fitols4rs ) )
print( residuals( fitols4rs$eq[[ 1 ]] ) )

print( residuals( fitols5 ) )
print( residuals( fitols5$eq[[ 2 ]] ) )


## *************** coefficients *********************
print( round( coef( fitols1rs ), digits = 6 ) )
print( round( coef( fitols1rs$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitols2s ), digits = 6 ) )
print( round( coef( fitols2s$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitols3 ), digits = 6 ) )
print( round( coef( fitols3, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitols3$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitols4r ), digits = 6 ) )
print( round( coef( fitols4r$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitols5 ), digits = 6 ) )
print( round( coef( fitols5, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitols5$eq[[ 2 ]] ), digits = 6 ) )


## *************** coefficients with stats *********************
print( round( coef( summary( fitols1rs, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitols1rs$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fitols2s ) ), digits = 6 ) )
print( round( coef( summary( fitols2s$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitols3, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitols3, useDfSys = FALSE ), modified.regMat = TRUE ),
   digits = 6 ) )
print( round( coef( summary( fitols3$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fitols4r, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitols4r$eq[[ 1 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fitols5 ) ), digits = 6 ) )
print( round( coef( summary( fitols5 ), modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fitols5$eq[[ 2 ]] ) ), digits = 6 ) )


## *********** variance covariance matrix of the coefficients *******
print( round( vcov( fitols1rs ), digits = 6 ) )
print( round( vcov( fitols1rs$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitols2s ), digits = 6 ) )
print( round( vcov( fitols2s$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitols3 ), digits = 6 ) )
print( round( vcov( fitols3, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitols3$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitols4r ), digits = 6 ) )
print( round( vcov( fitols4r$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitols5 ), digits = 6 ) )
print( round( vcov( fitols5, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitols5$eq[[ 2 ]] ), digits = 6 ) )


## *********** confidence intervals of coefficients *************
print( confint( fitols1, useDfSys = TRUE ) )
print( confint( fitols1$eq[[ 2 ]], level = 0.9, useDfSys = TRUE ) )

print( confint( fitols2r, level = 0.9 ) )
print( confint( fitols2r$eq[[ 1 ]], level = 0.99 ) )

print( confint( fitols3s, level = 0.99 ) )
print( confint( fitols3s$eq[[ 2 ]], level = 0.5 ) )

print( confint( fitols4rs, level = 0.5 ) )
print( confint( fitols4rs$eq[[ 1 ]], level = 0.25 ) )

print( confint( fitols5, level = 0.25 ) )
print( confint( fitols5$eq[[ 2 ]], level = 0.999 ) )

print( confint( fitols3, level = 0.999, useDfSys = FALSE ) )
print( confint( fitols3$eq[[ 1 ]], useDfSys = FALSE ) )


## *********** fitted values *************
print( fitted( fitols1 ) )
print( fitted( fitols1$eq[[ 2 ]] ) )

print( fitted( fitols2r ) )
print( fitted( fitols2r$eq[[ 1 ]] ) )

print( fitted( fitols3s ) )
print( fitted( fitols3s$eq[[ 2 ]] ) )

print( fitted( fitols4rs ) )
print( fitted( fitols4rs$eq[[ 1 ]] ) )

print( fitted( fitols5 ) )
print( fitted( fitols5$eq[[ 2 ]] ) )


## *********** predicted values *************
predictData <- Kmenta
predictData$consump <- NULL
predictData$price <- Kmenta$price * 0.9
predictData$income <- Kmenta$income * 1.1

print( predict( fitols1, se.fit = TRUE, interval = "prediction",
   useDfSys = TRUE ) )
print( predict( fitols1$eq[[ 2 ]], se.fit = TRUE, interval = "prediction",
   useDfSys = TRUE ) )

print( predict( fitols2r, se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )
print( predict( fitols2r$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )

print( predict( fitols3s, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )
print( predict( fitols3s$eq[[ 2 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )

print( predict( fitols4rs, se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99 ) )
print( predict( fitols4rs$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99 ) )

print( predict( fitols5, se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )
print( predict( fitols5$eq[[ 2 ]], se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )

# predict just one observation
smallData <- data.frame( price = 130, income = 150, farmPrice = 120,
   trend = 25 )

print( predict( fitols1, newdata = smallData ) )
print( predict( fitols1$eq[[ 1 ]], newdata = smallData ) )

print( predict( fitols2r, se.fit = TRUE, level = 0.9,
   newdata = smallData ) )
print( predict( fitols2r$eq[[ 1 ]], se.pred = TRUE, level = 0.99,
   newdata = smallData ) )

print( predict( fitols3s, interval = "prediction", level = 0.975,
   newdata = smallData ) )
print( predict( fitols3s$eq[[ 1 ]], interval = "confidence", level = 0.8,
   newdata = smallData ) )

print( predict( fitols4rs, se.fit = TRUE, interval = "confidence",
   level = 0.999, newdata = smallData ) )
print( predict( fitols4rs$eq[[ 2 ]], se.pred = TRUE, interval = "prediction",
   level = 0.75, newdata = smallData ) )

print( predict( fitols5, se.fit = TRUE, interval = "prediction",
   newdata = smallData ) )
print( predict( fitols5$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   newdata = smallData ) )

print( predict( fitols5rs, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fitols5rs$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )


## ************ correlation of predicted values ***************
print( correlation.systemfit( fitols1, 1, 2 ) )

print( correlation.systemfit( fitols2r, 2, 1 ) )

print( correlation.systemfit( fitols3s, 1, 2 ) )

print( correlation.systemfit( fitols4rs, 2, 1 ) )

print( correlation.systemfit( fitols5, 1, 2 ) )


## ************ Log-Likelihood values ***************
print( logLik( fitols1 ) )
print( logLik( fitols1, residCovDiag = TRUE ) )
all.equal( logLik( fitols1, residCovDiag = TRUE ),
   logLik( lmDemand ) + logLik( lmSupply ),
   check.attributes = FALSE )

print( logLik( fitols2r ) )
print( logLik( fitols2r, residCovDiag = TRUE ) )

print( logLik( fitols3s ) )
print( logLik( fitols3s, residCovDiag = TRUE ) )

print( logLik( fitols4rs ) )
print( logLik( fitols4rs, residCovDiag = TRUE ) )

print( logLik( fitols5 ) )
print( logLik( fitols5, residCovDiag = TRUE ) )


## ************** F tests ****************
# testing first restriction
print( linearHypothesis( fitols1, restrm ) )
linearHypothesis( fitols1, restrict )

print( linearHypothesis( fitols1s, restrm ) )
linearHypothesis( fitols1s, restrict )

print( linearHypothesis( fitols1, restrm ) )
linearHypothesis( fitols1, restrict )

print( linearHypothesis( fitols1r, restrm ) )
linearHypothesis( fitols1r, restrict )

# testing second restriction
restrOnly2m <- matrix(0,1,7)
restrOnly2q <- 0.5
restrOnly2m[1,2] <- -1
restrOnly2m[1,5] <-  1
restrictOnly2 <- "- demand_price + supply_price = 0.5"
# first restriction not imposed 
print( linearHypothesis( fitols1, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitols1, restrictOnly2 )

# first restriction imposed
print( linearHypothesis( fitols2, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitols2, restrictOnly2 )

print( linearHypothesis( fitols3, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitols3, restrictOnly2 )

# testing both of the restrictions
print( linearHypothesis( fitols1, restr2m, restr2q ) )
linearHypothesis( fitols1, restrict2 )


## ************** Wald tests ****************
# testing first restriction
print( linearHypothesis( fitols1, restrm, test = "Chisq" ) )
linearHypothesis( fitols1, restrict, test = "Chisq" )

print( linearHypothesis( fitols1s, restrm, test = "Chisq" ) )
linearHypothesis( fitols1s, restrict, test = "Chisq" )

print( linearHypothesis( fitols1, restrm, test = "Chisq" ) )
linearHypothesis( fitols1, restrict, test = "Chisq" )

print( linearHypothesis( fitols1r, restrm, test = "Chisq" ) )
linearHypothesis( fitols1r, restrict, test = "Chisq" )

# testing second restriction
# first restriction not imposed
print( linearHypothesis( fitols1, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitols1, restrictOnly2, test = "Chisq" )
# first restriction imposed
print( linearHypothesis( fitols2, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitols2, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitols3, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitols3, restrictOnly2, test = "Chisq" )

# testing both of the restrictions
print( linearHypothesis( fitols1, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitols1, restrict2, test = "Chisq" )


## ****************** model frame **************************
print( mf <- model.frame( fitols1 ) )
print( mf1 <- model.frame( fitols1$eq[[ 1 ]] ) )
print( attributes( mf1 )$terms )
print( mf2 <- model.frame( fitols1$eq[[ 2 ]] ) )
print( attributes( mf2 )$terms )

print( all.equal( mf, model.frame( fitols2r ) ) )
print( all.equal( mf1, model.frame( fitols2r$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitols3s ) ) )
print( all.equal( mf2, model.frame( fitols3s$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitols4rs ) ) )
print( all.equal( mf1, model.frame( fitols4rs$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitols5 ) ) )
print( all.equal( mf2, model.frame( fitols5$eq[[ 2 ]] ) ) )


## **************** model matrix ************************
# with x (returnModelMatrix) = TRUE
print( !is.null( fitols1r$eq[[ 1 ]]$x ) )
print( mm <- model.matrix( fitols1r ) )
print( mm1 <- model.matrix( fitols1r$eq[[ 1 ]] ) )
print( mm2 <- model.matrix( fitols1r$eq[[ 2 ]] ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitols1rs ) ) )
print( all.equal( mm1, model.matrix( fitols1rs$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols1rs$eq[[ 2 ]] ) ) )
print( !is.null( fitols1rs$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitols2rs$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitols2rs ) ) )
print( all.equal( mm1, model.matrix( fitols2rs$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols2rs$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitols2 ) ) )
print( all.equal( mm1, model.matrix( fitols2$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols2$eq[[ 2 ]] ) ) )
print( !is.null( fitols2$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitols3$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitols3 ) ) )
print( all.equal( mm1, model.matrix( fitols3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols3$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitols3r ) ) )
print( all.equal( mm1, model.matrix( fitols3r$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols3r$eq[[ 2 ]] ) ) )
print( !is.null( fitols3r$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitols4s$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitols4s ) ) )
print( all.equal( mm1, model.matrix( fitols4s$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols4s$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitols4Sym ) ) )
print( all.equal( mm1, model.matrix( fitols4Sym$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols4Sym$eq[[ 2 ]] ) ) )
print( !is.null( fitols4Sym$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitols5s$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitols5s ) ) )
print( all.equal( mm1, model.matrix( fitols5s$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols5s$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitols5 ) ) )
print( all.equal( mm1, model.matrix( fitols5$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitols5$eq[[ 2 ]] ) ) )
print( !is.null( fitols5$eq[[ 1 ]]$x ) )

try( model.matrix( fitols1, which = "z" ) )


## **************** formulas ************************
formula( fitols1 )
formula( fitols1$eq[[ 2 ]] )

formula( fitols2r )
formula( fitols2r$eq[[ 1 ]] )

formula( fitols3s )
formula( fitols3s$eq[[ 2 ]] )

formula( fitols4rs )
formula( fitols4rs$eq[[ 1 ]] )

formula( fitols5 )
formula( fitols5$eq[[ 2 ]] )


## **************** model terms *******************
terms( fitols1 )
terms( fitols1$eq[[ 2 ]] )

terms( fitols2r )
terms( fitols2r$eq[[ 1 ]] )

terms( fitols3s )
terms( fitols3s$eq[[ 2 ]] )

terms( fitols4rs )
terms( fitols4rs$eq[[ 1 ]] )

terms( fitols5 )
terms( fitols5$eq[[ 2 ]] )


## **************** estfun ************************
library( "sandwich" )

estfun( fitols1 )
round( colSums( estfun( fitols1 ) ), digits = 7 )

estfun( fitols1s )
round( colSums( estfun( fitols1s ) ), digits = 7 )

estfun( fitols1r )
round( colSums( estfun( fitols1r ) ), digits = 7 )

try( estfun( fitols2 ) )

try( estfun( fitols2Sym ) )

try( estfun( fitols3s ) )

try( estfun( fitols4r ) )

try( estfun( fitols4Sym ) )

try( estfun( fitols5 ) )

try( estfun( fitols5Sym ) )


## **************** bread ************************
bread( fitols1 )

bread( fitols1s )

bread( fitols1r )

try( bread( fitols2 ) )

## prepare summary results that belong to the whole system
summary.systemfit <- function( object, useDfSys = NULL,
      residCov = TRUE, equations = TRUE, ... ) {

   if( is.null( useDfSys ) ) {
      useDfSys <- length( coef( object ) ) != object$rank
         # TRUE if there are restrictions imposed
   }

   # number of equations
   nEq <- length( object$eq )
   # number of observations per equation
   nObsEq <- rep( NA, nEq )
   validObsEq <- matrix( NA, nrow = nrow( residuals( object ) ), ncol = nEq )
   for( i in 1:nEq ) {
      nObsEq[ i ] <- length( residuals( object$eq[[ i ]], na.rm = TRUE ) )
      validObsEq[ , i ] <- !is.na( residuals( object$eq[[ i ]] ) )
   }
   # total number of observations
   nObs <- sum( nObsEq )

   # preparing objects that will be returned
   result <- list()
   result$call <- object$call
   result$method <- object$method
   result$iter <- object$iter
   result$control <- object$control
   result$residuals <- residuals( object )
   result$residCovEst <- object$residCovEst
   result$residCov <- object$residCov
   if( !is.null( result$residCovEst ) ) {
      dimnames( result$residCovEst ) <- dimnames( result$residCov )
   }
   result$residCor <- cor( residuals( object ), use = "na.or.complete" )
   dimnames( result$residCor ) <- dimnames( result$residCov )
   result$detResidCov <- det( object$residCov, tol = object$control$solvetol )

   # now prepare summury results for the individual equations
   result$eq <- list()
   for( i in 1:length( object$eq ) ) {
       result$eq[[ i ]] <- summary( object$eq[[i]], useDfSys = useDfSys )
   }

   # coefficients, standard errors, ... 
   result$coefCov <- object$coefCov
   coef <- object$coefficients
   stdEr <- diag( result$coefCov )^0.5  # standard errors
   tStat <- coef / stdEr                # t-statistic
   if( useDfSys ) {             # p-values
      pVal <- 2 * ( 1 - pt( abs( tStat ), object$df.residual ) )
   } else {
      pVal <- NULL
      for( i in 1:length( object$eq ) ){
         pVal <- c( pVal, coef( result$eq[[ i ]] )[ , 4 ] )
      }
   }
   result$coefficients <- cbind( coef, stdEr, tStat, pVal )
   colnames( result$coefficients ) <- c( "Estimate", "Std. Error",
      "t value", "Pr(>|t|)" )
   result$df <- c( object$rank, object$df.residual )

   # coefficients of the modified regressor matrix
   if( !is.null( object$restrict.regMat ) ) {
      coefModReg <- coef( object, modified.regMat = TRUE )
      stdErModReg <- diag( vcov( object, modified.regMat = TRUE ) )^0.5  # standard errors
      tStatModReg <- coefModReg / stdErModReg    # t-statistic
      if( useDfSys ) {             # p-values
         pValModReg <- 2 * ( 1 - pt( abs( tStatModReg ), object$df.residual ) )
      } else {
         pValModReg <- rep( NA, length( coefModReg ) )
      }
      result$coefModReg <- cbind( coefModReg, stdErModReg, tStatModReg, pValModReg )
      colnames( result$coefModReg ) <- c( "Estimate", "Std. Error",
         "t value", "Pr(>|t|)" )
   }

   # R^2 values
   resid <- NULL
   response <- NULL
   responseMinusMean <- NULL
   for( i in 1:length( object$eq ) ) {
      resid <- c( resid, residuals( object$eq[[ i ]], na.rm = TRUE ) )
      responseEqI <- fitted( object$eq[[ i ]], na.rm = TRUE ) +
         residuals( object$eq[[ i ]], na.rm = TRUE )
      response <- c( response, responseEqI )
      responseMinusMean <- c( responseMinusMean,
         responseEqI - mean( responseEqI ) )
   }

   # OLS R^2 value of the entire system
   rss <- sum( resid^2 )
   tss <- sum( responseMinusMean^2 )
   result$ols.r.squared <- 1 - rss / tss

   # System R^2 value of McElroy (1977)
   # formula from Greene (2003, p. 345 )
   # (first formula, numerator modified to save memory)
   xMat <- matrix( resid, ncol = 1 )
   if( object$control$useMatrix ){
      object$residCov <- as( object$residCov, "symmetricMatrix")
      xMat <- as( xMat, "CsparseMatrix" )
   }
   rtOmega <- .calcXtOmegaInv( xMat = xMat,
      sigma = object$residCov, validObsEq = validObsEq,
      solvetol = object$control$solvetol,
      useMatrix = object$control$useMatrix )
   yCov <- .calcResidCov( response, methodResidCov = "noDfCor",
      validObsEq = validObsEq, centered = TRUE,
      solvetol = object$control$solvetol )
   residCovInv <- solve( object$residCov, tol = object$control$solvetol )
   denominator <- 0
   for( i in 1:length( object$eq ) ) {
      for( j in 1:length( object$eq ) ) {
         denominator <- denominator + residCovInv[ i, j ] * yCov[ i, j ] *
            ( nObsEq[ i ] * nObsEq[ j ] )^0.5
      }
   }
   result$mcelroy.r.squared <- drop( 1 - ( rtOmega %*% resid ) / denominator )

   result$printEquations <- equations
   result$printResidCov  <- residCov

   class( result ) <- "summary.systemfit"
   return( result )
}

## print summary results of the whole system
print.summary.systemfit <- function( x,
      digits = max( 3, getOption("digits") - 1 ),
      residCov = x$printResidCov, equations = x$printEquations, ... ) {

  table <- NULL
  labels <- NULL

  cat("\n")
  cat("systemfit results \n")
  cat("method: ")
  if(!is.null(x$iter)) if(x$iter>1) cat("iterated ")
  cat( paste( x$method, "\n\n"))
  if(!is.null(x$iter)) {
    if(x$iter>1) {
      if(x$iter<x$control$maxiter) {
        cat( paste( "convergence achieved after",x$iter,"iterations\n\n" ) )
      } else {
        cat( paste( "warning: convergence not achieved after", x$iter,
                    "iterations\n\n" ) )
      }
    }
  }

   table.sys <- cbind( round( sum( x$df ),         digits ),
                       round( x$df[2],             digits ),
                       round( sum( sapply( x$eq, function( x ) x$ssr ) ), digits ),
                       round( x$detResidCov,       digits ),
                       round( x$ols.r.squared,     digits ),
                       round( x$mcelroy.r.squared, digits ) )
   rownames( table.sys ) <- c( "system" )
   colnames( table.sys ) <- c( "N", "DF", "SSR", "detRCov", "OLS-R2", "McElroy-R2" )
   print( table.sys, quote = FALSE, right = TRUE, digits = digits )

   cat("\n")

  for(i in 1:length( x$eq ) ) {
    row <- NULL
    row <- cbind( round( sum( x$eq[[i]]$df ),  digits ),
                  round( x$eq[[i]]$df[2], digits ),
                  round( x$eq[[i]]$ssr,   digits ),
                  round( x$eq[[i]]$sigma^2, digits ),
                  round( x$eq[[i]]$sigma,   digits ),
                  round( x$eq[[i]]$r.squared,     digits ),
                  round( x$eq[[i]]$adj.r.squared, digits ))
    table  <- rbind( table, row )
    labels <- rbind( labels, x$eq[[i]]$eqnLabel )
  }
  rownames(table) <- c( labels )
  colnames(table) <- c("N","DF", "SSR", "MSE", "RMSE", "R2", "Adj R2" )
  print(table, quote = FALSE, right = TRUE, digits = digits )

  cat("\n")

   if( residCov ){
      if(!is.null(x$residCovEst)) {
         cat("The covariance matrix of the residuals used for estimation\n")
         print( x$residCovEst, digits = digits )
         cat("\n")
         if( min(eigen( x$residCov, only.values=TRUE)$values) < 0 ) {
            cat("warning: this covariance matrix is NOT positive semidefinit!\n")
            cat("\n")
         }
      }

      cat("The covariance matrix of the residuals\n")
      print( x$residCov, digits = digits )
      cat("\n")

      cat("The correlations of the residuals\n")
      print( x$residCor, digits = digits )
      cat("\n")
   }

   if( equations ){
      ## now print the individual equations
      for(i in 1:length( x$eq ) ) {
         print( x$eq[[i]], digits = digits )
      }
   } else {
      cat( "\nCoefficients:\n" )
      printCoefmat( coef( x ), digits = digits )
   }

  invisible( x )
}


## prepare summary results for a single equation
summary.systemfit.equation <- function( object, useDfSys = NULL, ... ) {

   if( is.null( useDfSys ) ) {
      useDfSys <- object$nCoef.sys != object$rank.sys
         # TRUE if there are restrictions imposed
   }

   # number of observations in this equation
   nObs <- length( residuals( object, na.rm = TRUE ) )

   # preparing objects that will be returned
   result <- list()
   result$eqnLabel <- object$eqnLabel
   result$eqnNo <- object$eqnNo
   result$terms <- object$terms
   result$instruments <- object$inst
   result$method <- object$method
   result$residuals <- object$residuals

   # coefficients, standard errors, ...
   result$coefCov <- object$coefCov
   coef <- object$coefficients
   stdEr <- diag( result$coefCov )^0.5  # standard errors
   tStat <- coef / stdEr                # t-statistic
   if( useDfSys ) {             # p-values
      pVal <- 2 * ( 1 - pt( abs( tStat ), object$df.residual.sys ) )
   } else {
      pVal <- 2 * ( 1 - pt( abs( tStat ), object$df.residual ) )
   }
   result$coefficients <- cbind( coef, stdEr, tStat, pVal )
   colnames( result$coefficients ) <- c( "Estimate", "Std. Error",
      "t value", "Pr(>|t|)" )
   result$df <- c( length( coef( object ) ), nObs - length( coef( object ) ) )
   result$ssr <- sum( residuals( object, na.rm = TRUE )^2 )
   result$sigma <- sqrt( result$ssr / df.residual( object ) )

   # R^2 values
   response <- fitted( object, na.rm = TRUE ) + residuals( object, na.rm = TRUE )
   rss <- sum( residuals( object, na.rm = TRUE )^2 )
   tss <- sum( ( response - mean( response ) )^2 )
   result$r.squared <- 1 - rss / tss
   result$adj.r.squared <- 1 - ( ( nObs - 1 ) / object$df.residual ) *
      ( 1 - result$r.squared )
   class( result ) <- "summary.systemfit.equation"
   return( result )
}


## print summary results for a single equation
print.summary.systemfit.equation <- function( x,
   digits = max( 3, getOption("digits") - 1 ), ... ) {

  cat("\n")
  cat( x$method, " estimates for '", x$eqnLabel,
         "' (equation ", x$eqnNo, ")\n", sep = "" )

  cat("Model Formula: ")
  print( formula( x$terms ) )
  if(!is.null(x$inst)) {
    cat("Instruments: ")
    print(x$inst)
  }
  cat("\n")

  printCoefmat( x$coefficients, digits = digits )

  cat(paste("\nResidual standard error:", round( x$sigma, digits ),
            "on", x$df[ 2 ], "degrees of freedom\n" ))

  cat( paste( "Number of observations:", round( sum( x$df ), digits ),
              "Degrees of Freedom:", round( x$df[ 2 ], digits ),"\n" ) )

  cat( paste( "SSR:", round( x$ssr, digits ),
              "MSE:", round( x$sigma^2, digits ),
              "Root MSE:", round(x$sigma, digits), "\n" ) )

  cat( paste( "Multiple R-Squared:", round( x$r.squared, digits ),
              "Adjusted R-Squared:", round( x$adj.r.squared, digits ),
              "\n" ) )
  cat("\n")
  invisible( x )
}

library( systemfit )
options( digits = 3 )

data( "Kmenta" )
useMatrix <- FALSE

demand <- consump ~ price + income
supply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
inst1  <- ~ income + farmPrice
instlist <- list( inst1, inst )
system <- list( demand = demand, supply = supply )
restrm <- matrix(0,1,7)  # restriction matrix "R"
restrm[1,3] <-  1
restrm[1,7] <- -1
restrict <- "demand_income - supply_trend = 0"
restr2m <- matrix(0,2,7)  # restriction matrix "R" 2
restr2m[1,3] <-  1
restr2m[1,7] <- -1
restr2m[2,2] <- -1
restr2m[2,5] <-  1
restr2q <- c( 0, 0.5 )  # restriction vector "q" 2
restrict2 <- c( "demand_income - supply_trend = 0",
   "- demand_price + supply_price = 0.5" )
tc <- matrix(0,7,6)
tc[1,1] <- 1
tc[2,2] <- 1
tc[3,3] <- 1
tc[4,4] <- 1
tc[5,5] <- 1
tc[6,6] <- 1
tc[7,3] <- 1
restr3m <- matrix(0,1,6)  # restriction matrix "R" 2
restr3m[1,2] <- -1
restr3m[1,5] <-  1
restr3q <- c( 0.5 )  # restriction vector "q" 2
restrict3 <- "- C2 + C5 = 0.5"

# It is not possible to estimate 2SLS with systemfit exactly
# as EViews does, because EViews uses
# methodResidCov == "geomean" for the coefficient covariance matrix and
# methodResidCov == "noDfCor" for the residual covariance matrix.
# systemfit uses always the same formulas for both calculations.

## *************** 2SLS estimation ************************
## ************ 2SLS estimation (default)*********************
fit2sls1 <- systemfit( system, "2SLS", data = Kmenta, inst = inst,
   x = TRUE, useMatrix = useMatrix )
print( summary( fit2sls1 ) )
nobs( fit2sls1 )

## *************** 2SLS estimation (singleEqSigma=F)*******************
fit2sls1s <- systemfit( system, "2SLS", data = Kmenta, inst = inst,
   singleEqSigma = FALSE, useMatrix = useMatrix )
print( summary( fit2sls1s ) )
nobs( fit2sls1s )

## ********************* 2SLS (useDfSys = TRUE) *****************
print( summary( fit2sls1, useDfSys = TRUE ) )
nobs( fit2sls1 )

## ********************* 2SLS (methodResidCov = "noDfCor" ) *****************
fit2sls1r <- systemfit( system, "2SLS", data = Kmenta, inst = inst,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fit2sls1r ) )
nobs( fit2sls1r )

## *************** 2SLS (methodResidCov="noDfCor", singleEqSigma=F) *************
fit2sls1rs <- systemfit( system, "2SLS", data = Kmenta, inst = inst,
   methodResidCov = "noDfCor", singleEqSigma = FALSE, useMatrix = useMatrix )
print( summary( fit2sls1rs ) )
nobs( fit2sls1rs )

## ********************* 2SLS with restriction ********************
## **************** 2SLS with restriction (default)********************
fit2sls2 <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, useMatrix = useMatrix )
print( summary( fit2sls2 ) )
nobs( fit2sls2 )
# the same with symbolically specified restrictions
fit2sls2Sym <- systemfit( system, "2SLS", data = Kmenta,
   restrict.matrix = restrict, inst = inst, useMatrix = useMatrix )
all.equal( fit2sls2, fit2sls2Sym )
nobs( fit2sls2Sym )

## ************* 2SLS with restriction (singleEqSigma=T) *****************
fit2sls2s <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, singleEqSigma = TRUE, x = TRUE,
   useMatrix = useMatrix )
print( summary( fit2sls2s ) )
nobs( fit2sls2s )

## ********************* 2SLS with restriction (useDfSys=T) **************
print( summary( fit2sls2, useDfSys = TRUE ) )
nobs( fit2sls2 )

## ********************* 2SLS with restriction (methodResidCov = "noDfCor") **************
fit2sls2r <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fit2sls2r ) )
nobs( fit2sls2r )

## ******** 2SLS with restriction (methodResidCov="noDfCor", singleEqSigma=TRUE) *********
fit2sls2rs <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, methodResidCov = "noDfCor", singleEqSigma = TRUE,
   useMatrix = useMatrix )
print( summary( fit2sls2rs ) )
nobs( fit2sls2rs )

## ********************* 2SLS with restriction via restrict.regMat ******************
## *************** 2SLS with restriction via restrict.regMat (default )***************
fit2sls3 <- systemfit( system, "2SLS", data = Kmenta, restrict.regMat = tc,
   inst = inst, methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fit2sls3, useDfSys = TRUE ) )
nobs( fit2sls3 )


## ***************** 2SLS with 2 restrictions *******************
## ************** 2SLS with 2 restrictions (default) **************
fit2sls4 <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, useMatrix = useMatrix )
print( summary( fit2sls4 ) )
nobs( fit2sls4 )
# the same with symbolically specified restrictions
fit2sls4Sym <- systemfit( system, "2SLS", data = Kmenta,
   restrict.matrix = restrict2, inst = inst, useMatrix = useMatrix )
all.equal( fit2sls4, fit2sls4Sym )
nobs( fit2sls4Sym )

## ************ 2SLS with 2 restrictions (singleEqSigma=T) **************
fit2sls4s <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, singleEqSigma = TRUE,
   useMatrix = useMatrix )
print( summary( fit2sls4s ) )
nobs( fit2sls4s )

## ***************** 2SLS with 2 restrictions (useDfSys=T) **************
print( summary( fit2sls4, useDfSys = TRUE ) )
nobs( fit2sls4 )

## ***************** 2SLS with 2 restrictions (methodResidCov="noDfCor") **************
fit2sls4r <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, methodResidCov = "noDfCor",
   x = TRUE, useMatrix = useMatrix )
print( summary( fit2sls4r ) )
nobs( fit2sls4r )

## ***** 2SLS with 2 restrictions (methodResidCov="noDfCor", singleEqSigma=T) *******
fit2sls4rs <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, methodResidCov = "noDfCor",
   singleEqSigma = TRUE, useMatrix = useMatrix )
print( summary( fit2sls4rs ) )
nobs( fit2sls4rs )

## ************* 2SLS with 2 restrictions via R and restrict.regMat ******************
## ******** 2SLS with 2 restrictions via R and restrict.regMat (default) *************
fit2sls5 <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   useMatrix = useMatrix )
print( summary( fit2sls5 ) )
nobs( fit2sls5 )
# the same with symbolically specified restrictions
fit2sls5Sym <- systemfit( system, "2SLS", data = Kmenta,
   restrict.matrix = restrict3, restrict.regMat = tc, inst = inst,
   useMatrix = useMatrix )
all.equal( fit2sls5, fit2sls5Sym )
nobs( fit2sls5Sym )

## ******* 2SLS with 2 restrictions via R and restrict.regMat (singleEqSigma=T) ******
fit2sls5s <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   singleEqSigma = TRUE, useMatrix = useMatrix )
print( summary( fit2sls5s ) )
nobs( fit2sls5s )

## ********** 2SLS with 2 restrictions via R and restrict.regMat (useDfSys=T) *******
print( summary( fit2sls5, useDfSys = TRUE ) )
nobs( fit2sls5 )

## ************* 2SLS with 2 restrictions via R and restrict.regMat (methodResidCov="noDfCor") *********
fit2sls5r <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fit2sls5r ) )
nobs( fit2sls5r )

## ** 2SLS with 2 restrictions via R and restrict.regMat (methodResidCov="noDfCor", singleEqSigma=T) **
fit2sls5rs <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   methodResidCov = "noDfCor", singleEqSigma = TRUE,
   x = TRUE, useMatrix = useMatrix )
print( summary( fit2sls5rs ) )
nobs( fit2sls5rs )

## *********** 2SLS estimation with different instruments **************
## ******* 2SLS estimation with different instruments (default) *********
fit2slsd1 <- systemfit( system, "2SLS", data = Kmenta, inst = instlist,
   useMatrix = useMatrix )
print( summary( fit2slsd1 ) )
nobs( fit2slsd1 )

## *********** 2SLS estimation with different instruments (singleEqSigma=F)*****
fit2slsd1s <- systemfit( system, "2SLS", data = Kmenta, inst = instlist,
   singleEqSigma = FALSE, useMatrix = useMatrix )
print( summary( fit2slsd1s ) )
nobs( fit2slsd1s )

## ********* 2SLS estimation with different instruments (useDfSys=T) *******
print( summary( fit2slsd1, useDfSys = TRUE ) )
nobs( fit2slsd1 )

## ********* 2SLS estimation with different instruments (methodResidCov="noDfCor") ******
fit2slsd1r <- systemfit( system, "2SLS", data = Kmenta, inst = instlist,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fit2slsd1r ) )
nobs( fit2slsd1r )

## 2SLS estimation with different instruments (methodResidCov="noDfCor",singleEqSigma=F)
fit2slsd1r <- systemfit( system, "2SLS", data = Kmenta, inst = instlist,
   methodResidCov = "noDfCor", singleEqSigma = FALSE,
   useMatrix = useMatrix )
print( summary( fit2slsd1r ) )
nobs( fit2slsd1r )

## **** 2SLS estimation with different instruments and restriction *******
## ** 2SLS estimation with different instruments and restriction (default) ****
fit2slsd2 <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = instlist, useMatrix = useMatrix )
print( summary( fit2slsd2 ) )
nobs( fit2slsd2 )

## 2SLS estimation with different instruments and restriction (singleEqSigma=T)
fit2slsd2s <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = instlist, singleEqSigma = TRUE, useMatrix = useMatrix )
print( summary( fit2slsd2s ) )
nobs( fit2slsd2s )

## **** 2SLS estimation with different instruments and restriction (useDfSys=F)
print( summary( fit2slsd2, useDfSys = FALSE ) )
nobs( fit2slsd2 )

## **** 2SLS estimation with different instruments and restriction (methodResidCov="noDfCor")
fit2slsd2r <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = instlist, methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fit2slsd2r ) )
nobs( fit2slsd2r )

## 2SLS estimation with different instr. and restr. (methodResidCov="noDfCor", singleEqSigma=T)
fit2slsd2rs <- systemfit( system, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = instlist, methodResidCov = "noDfCor", singleEqSigma = TRUE,
   useMatrix = useMatrix )
print( summary( fit2slsd2rs ) )
nobs( fit2slsd2rs )

## **** 2SLS estimation with different instruments and restriction via restrict.regMat *
## 2SLS estimation with different instruments and restriction via restrict.regMat (default)
fit2slsd3 <- systemfit( system, "2SLS", data = Kmenta, restrict.regMat = tc,
   inst = instlist, useMatrix = useMatrix )
print( summary( fit2slsd3 ) )
nobs( fit2slsd3 )

## **** 2SLS estimation with different instr. and restr. via restrict.regMat (methodResidCov="noDfCor")
fit2slsd3r <- systemfit( system, "2SLS", data = Kmenta, restrict.regMat = tc,
   inst = instlist, methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fit2slsd3r ) )
nobs( fit2slsd3r )


## *********** estimations with a single regressor ************
fit2slsS1 <- systemfit(
   list( consump ~ price - 1, price ~ consump + trend ), "2SLS",
   data = Kmenta, inst = ~ farmPrice + trend + income, useMatrix = useMatrix )
print( summary( fit2slsS1 ) )
nobs( fit2slsS1 )
fit2slsS2 <- systemfit(
   list( consump ~ price - 1, consump ~ trend - 1 ), "2SLS",
   data = Kmenta, inst = ~ farmPrice + price + income, useMatrix = useMatrix )
print( summary( fit2slsS2 ) )
nobs( fit2slsS2 )
fit2slsS3 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "2SLS",
   data = Kmenta, inst = instlist, useMatrix = useMatrix )
print( summary( fit2slsS3 ) )
nobs( fit2slsS3 )
fit2slsS4 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "2SLS",
   data = Kmenta, inst = ~ farmPrice + trend + income,
   restrict.matrix = matrix( c( 1, -1 ), nrow = 1 ), useMatrix = useMatrix )
print( summary( fit2slsS4 ) )
nobs( fit2slsS4 )
fit2slsS5 <- systemfit(
   list( consump ~ 1, price ~ 1 ), "2SLS",
   data = Kmenta, inst = ~ farmPrice, useMatrix = useMatrix )
print( summary( fit2slsS1 ) )


## **************** shorter summaries **********************
print( summary( fit2sls1, useDfSys = TRUE, residCov = FALSE ) )

print( summary( fit2sls1, equations = FALSE ) )

print( summary( fit2sls1rs, residCov = FALSE, equations = FALSE ) )

print( summary( fit2sls2Sym, useDfSys = FALSE ), equations = FALSE )

print( summary( fit2sls2 ), residCov = FALSE )

print( summary( fit2sls3, useDfSys = FALSE, residCov = FALSE,
   equations = FALSE ) )

print( summary( fit2sls4s ), equations = FALSE, residCov = FALSE )

print( summary( fit2sls5r, equations = FALSE, residCov = FALSE ) )

print( summary( fit2slsd1s ), residCov = FALSE, equations = FALSE )

print( summary( fit2slsd2, residCov = FALSE, equations = FALSE ) )

print( summary( fit2slsd3r ), residCov = FALSE, equations = FALSE )


## ****************** residuals **************************
print( residuals( fit2sls1 ) )
print( residuals( fit2sls1$eq[[ 1 ]] ) )

print( residuals( fit2sls2s ) )
print( residuals( fit2sls2s$eq[[ 2 ]] ) )

print( residuals( fit2sls3 ) )
print( residuals( fit2sls3$eq[[ 1 ]] ) )

print( residuals( fit2sls4r ) )
print( residuals( fit2sls4r$eq[[ 2 ]] ) )

print( residuals( fit2sls5rs ) )
print( residuals( fit2sls5rs$eq[[ 1 ]] ) )

print( residuals( fit2slsd1 ) )
print( residuals( fit2slsd1$eq[[ 2 ]] ) )

print( residuals( fit2slsd2r ) )
print( residuals( fit2slsd2r$eq[[ 1 ]] ) )


## *************** coefficients *********************
print( round( coef( fit2sls1s ), digits = 6 ) )
print( round( coef( fit2sls1s$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fit2sls2 ), digits = 6 ) )
print( round( coef( fit2sls2$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fit2sls3 ), digits = 6 ) )
print( round( coef( fit2sls3, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fit2sls3$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fit2sls4s ), digits = 6 ) )
print( round( coef( fit2sls4s$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fit2sls5r ), digits = 6 ) )
print( round( coef( fit2sls5r, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fit2sls5r$eq[[ 2 ]] ), digits = 6 ) )


## *************** coefficients with stats *********************
print( round( coef( summary( fit2sls1s ) ), digits = 6 ) )
print( round( coef( summary( fit2sls1s$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fit2sls2, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fit2sls2$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fit2sls3 ) ), digits = 6 ) )
print( round( coef( summary( fit2sls3 ), modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fit2sls3$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fit2sls4s ) ), digits = 6 ) )
print( round( coef( summary( fit2sls4s$eq[[ 2 ]] ) ), digits = 6 ) )

print( round( coef( summary( fit2sls5r, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fit2sls5r, useDfSys = FALSE ),
   modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fit2sls5r$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )


## *********** variance covariance matrix of the coefficients *******
print( round( vcov( fit2sls1s ), digits = 6 ) )
print( round( vcov( fit2sls1s$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fit2sls1r ), digits = 6 ) )
print( round( vcov( fit2sls1r$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fit2sls2 ), digits = 6 ) )
print( round( vcov( fit2sls2$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fit2sls3 ), digits = 6 ) )
print( round( vcov( fit2sls3, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fit2sls3$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fit2sls4s ), digits = 6 ) )
print( round( vcov( fit2sls4s$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fit2sls5r ), digits = 6 ) )
print( round( vcov( fit2sls5r, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fit2sls5r$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fit2slsd1 ), digits = 6 ) )
print( round( vcov( fit2slsd1$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fit2slsd2rs ), digits = 6 ) )
print( round( vcov( fit2slsd2rs$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fit2slsd3 ), digits = 6 ) )
print( round( vcov( fit2slsd3, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fit2slsd3$eq[[ 1 ]] ), digits = 6 ) )


## *********** confidence intervals of coefficients *************
print( confint( fit2sls1 ) )
print( confint( fit2sls1$eq[[ 1 ]], level = 0.9 ) )

print( confint( fit2sls2s, level = 0.9 ) )
print( confint( fit2sls2s$eq[[ 2 ]], level = 0.99 ) )

print( confint( fit2sls3, level = 0.99, useDfSys = TRUE ) )
print( confint( fit2sls3$eq[[ 1 ]], level = 0.5, useDfSys = TRUE ) )

print( confint( fit2sls4r, level = 0.5 ) )
print( confint( fit2sls4r$eq[[ 2 ]], level = 0.25 ) )

print( confint( fit2sls5rs, level = 0.25 ) )
print( confint( fit2sls5rs$eq[[ 1 ]], level = 0.975 ) )

print( confint( fit2slsd1, level = 0.975, useDfSys = TRUE ) )
print( confint( fit2slsd1$eq[[ 2 ]], level = 0.999, useDfSys = TRUE ) )

print( confint( fit2slsd2r, level = 0.999 ) )
print( confint( fit2slsd2r$eq[[ 1 ]] ) )


## *********** fitted values *************
print( fitted( fit2sls1, se.fit = TRUE, interval = "prediction" ) )
print( fitted( fit2sls1$eq[[ 1 ]] ) )

print( fitted( fit2sls2s ) )
print( fitted( fit2sls2s$eq[[ 2 ]] ) )

print( fitted( fit2sls3 ) )
print( fitted( fit2sls3$eq[[ 1 ]] ) )

print( fitted( fit2sls4r ) )
print( fitted( fit2sls4r$eq[[ 2 ]] ) )

print( fitted( fit2sls5rs ) )
print( fitted( fit2sls5rs$eq[[ 1 ]] ) )

print( fitted( fit2slsd1 ) )
print( fitted( fit2slsd1$eq[[ 2 ]] ) )

print( fitted( fit2slsd2r ) )
print( fitted( fit2slsd2r$eq[[ 1 ]] ) )


## *********** predicted values *************
predictData <- Kmenta
predictData$consump <- NULL
predictData$price <- Kmenta$price * 0.9
predictData$income <- Kmenta$income * 1.1

print( predict( fit2sls1, se.fit = TRUE, interval = "prediction" ) )
print( predict( fit2sls1$eq[[ 1 ]], se.fit = TRUE, interval = "prediction" ) )

print( predict( fit2sls2s, se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )
print( predict( fit2sls2s$eq[[ 2 ]], se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )

print( predict( fit2sls3, se.pred = TRUE, interval = "prediction",
   level = 0.975, useDfSys = TRUE ) )
print( predict( fit2sls3$eq[[ 1 ]], se.pred = TRUE, interval = "prediction",
   level = 0.975, useDfSys = TRUE ) )

print( predict( fit2sls4r, se.fit = TRUE, interval = "confidence",
   level = 0.25 ) )
print( predict( fit2sls4r$eq[[ 2 ]], se.fit = TRUE, interval = "confidence",
   level = 0.25 ) )

print( predict( fit2sls5rs, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )
print( predict( fit2sls5rs$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )

print( predict( fit2slsd1, se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99, useDfSys = TRUE ) )
print( predict( fit2slsd1$eq[[ 2 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99, useDfSys = TRUE ) )

print( predict( fit2slsd2r, se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )
print( predict( fit2slsd2r$eq[[ 1 ]], se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )

# predict just one observation
smallData <- data.frame( price = 130, income = 150, farmPrice = 120,
   trend = 25 )

print( predict( fit2sls1rs, newdata = smallData ) )
print( predict( fit2sls1rs$eq[[ 1 ]], newdata = smallData ) )

print( predict( fit2sls2, se.fit = TRUE, level = 0.9,
   newdata = smallData ) )
print( predict( fit2sls2$eq[[ 1 ]], se.pred = TRUE, level = 0.99,
   newdata = smallData ) )

print( predict( fit2sls3, interval = "prediction", level = 0.975,
   newdata = smallData ) )
print( predict( fit2sls3$eq[[ 1 ]], interval = "confidence", level = 0.8,
   newdata = smallData ) )

print( predict( fit2sls4r, se.fit = TRUE, interval = "confidence",
   level = 0.999, newdata = smallData ) )
print( predict( fit2sls4r$eq[[ 2 ]], se.pred = TRUE, interval = "prediction",
   level = 0.75, newdata = smallData ) )

print( predict( fit2sls5s, se.fit = TRUE, interval = "prediction",
   newdata = smallData ) )
print( predict( fit2sls5s$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   newdata = smallData ) )

print( predict( fit2slsd3, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fit2slsd3$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )


## ************ correlation of predicted values ***************
print( correlation.systemfit( fit2sls1, 1, 2 ) )

print( correlation.systemfit( fit2sls2s, 2, 1 ) )

print( correlation.systemfit( fit2sls3, 1, 2 ) )

print( correlation.systemfit( fit2sls4r, 2, 1 ) )

print( correlation.systemfit( fit2sls5rs, 1, 2 ) )

print( correlation.systemfit( fit2slsd1, 2, 1 ) )

print( correlation.systemfit( fit2slsd2r, 1, 2 ) )


## ************ Log-Likelihood values ***************
print( logLik( fit2sls1 ) )
print( logLik( fit2sls1, residCovDiag = TRUE ) )

print( logLik( fit2sls2s ) )
print( logLik( fit2sls2s, residCovDiag = TRUE ) )

print( logLik( fit2sls3 ) )
print( logLik( fit2sls3, residCovDiag = TRUE ) )

print( logLik( fit2sls4r ) )
print( logLik( fit2sls4r, residCovDiag = TRUE ) )

print( logLik( fit2sls5rs ) )
print( logLik( fit2sls5rs, residCovDiag = TRUE ) )

print( logLik( fit2slsd1 ) )
print( logLik( fit2slsd1, residCovDiag = TRUE ) )

print( logLik( fit2slsd2r ) )
print( logLik( fit2slsd2r, residCovDiag = TRUE ) )


## ************** F tests ****************
# testing first restriction
print( linearHypothesis( fit2sls1, restrm ) )
linearHypothesis( fit2sls1, restrict )

print( linearHypothesis( fit2sls1s, restrm ) )
linearHypothesis( fit2sls1s, restrict )

print( linearHypothesis( fit2sls1, restrm ) )
linearHypothesis( fit2sls1, restrict )

print( linearHypothesis( fit2sls1r, restrm ) )
linearHypothesis( fit2sls1r, restrict )

# testing second restriction
restrOnly2m <- matrix(0,1,7)
restrOnly2q <- 0.5
restrOnly2m[1,2] <- -1
restrOnly2m[1,5] <-  1
restrictOnly2 <- "- demand_price + supply_price = 0.5"
# first restriction not imposed 
print( linearHypothesis( fit2sls1, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit2sls1, restrictOnly2 )

# first restriction imposed
print( linearHypothesis( fit2sls2, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit2sls2, restrictOnly2 )

print( linearHypothesis( fit2sls2r, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit2sls2r, restrictOnly2 )

print( linearHypothesis( fit2sls3, restrOnly2m, restrOnly2q ) )
linearHypothesis( fit2sls3, restrictOnly2 )

# testing both of the restrictions
print( linearHypothesis( fit2sls1, restr2m, restr2q ) )
linearHypothesis( fit2sls1, restrict2 )


## ************** Wald tests ****************
# testing first restriction
print( linearHypothesis( fit2sls1, restrm, test = "Chisq" ) )
linearHypothesis( fit2sls1, restrict, test = "Chisq" )

print( linearHypothesis( fit2sls1s, restrm, test = "Chisq" ) )
linearHypothesis( fit2sls1s, restrict, test = "Chisq" )

print( linearHypothesis( fit2sls1, restrm, test = "Chisq" ) )
linearHypothesis( fit2sls1, restrict, test = "Chisq" )

print( linearHypothesis( fit2sls1r, restrm, test = "Chisq" ) )
linearHypothesis( fit2sls1r, restrict, test = "Chisq" )

# testing second restriction
# first restriction not imposed
print( linearHypothesis( fit2sls1, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit2sls1, restrictOnly2, test = "Chisq" )
# first restriction imposed
print( linearHypothesis( fit2sls2, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit2sls2, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit2sls2r, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit2sls2r, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fit2sls3, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fit2sls3, restrictOnly2, test = "Chisq" )

# testing both of the restrictions
print( linearHypothesis( fit2sls1, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fit2sls1, restrict2, test = "Chisq" )


## **************** model frame ************************
print( mf <- model.frame( fit2sls1 ) )
print( mf1 <- model.frame( fit2sls1$eq[[ 1 ]] ) )
print( attributes( mf1 )$terms )
print( mf2 <- model.frame( fit2sls1$eq[[ 2 ]] ) )
print( attributes( mf2 )$terms )

print( all.equal( mf, model.frame( fit2sls2s ) ) )
print( all.equal( mf2, model.frame( fit2sls2s$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fit2sls3 ) ) )
print( all.equal( mf1, model.frame( fit2sls3$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fit2sls4r ) ) )
print( all.equal( mf2, model.frame( fit2sls4r$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fit2sls5rs ) ) )
print( all.equal( mf1, model.frame( fit2sls5rs$eq[[ 1 ]] ) ) )

fit2sls1$eq[[ 1 ]]$modelInst
fit2sls1$eq[[ 2 ]]$modelInst

fit2sls2s$eq[[ 1 ]]$modelInst
fit2sls2s$eq[[ 2 ]]$modelInst

fit2sls5rs$eq[[ 1 ]]$modelInst
fit2sls5rs$eq[[ 2 ]]$modelInst


## **************** model matrix ************************
# with x (returnModelMatrix) = TRUE
print( !is.null( fit2sls1$eq[[ 1 ]]$x ) )
print( mm <- model.matrix( fit2sls1 ) )
print( mm1 <- model.matrix( fit2sls1$eq[[ 1 ]] ) )
print( mm2 <- model.matrix( fit2sls1$eq[[ 2 ]] ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit2sls1s ) ) )
print( all.equal( mm1, model.matrix( fit2sls1s$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit2sls1s$eq[[ 2 ]] ) ) )
print( !is.null( fit2sls1s$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit2sls2s$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit2sls2s ) ) )
print( all.equal( mm1, model.matrix( fit2sls2s$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit2sls2s$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit2sls2Sym ) ) )
print( all.equal( mm1, model.matrix( fit2sls2Sym$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit2sls2Sym$eq[[ 2 ]] ) ) )
print( !is.null( fit2sls2Sym$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit2sls3 ) ) )
print( all.equal( mm1, model.matrix( fit2sls3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit2sls3$eq[[ 2 ]] ) ) )
print( !is.null( fit2sls3$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit2sls4r$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit2sls4r ) ) )
print( all.equal( mm1, model.matrix( fit2sls4r$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit2sls4r$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit2sls4s ) ) )
print( all.equal( mm1, model.matrix( fit2sls4s$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit2sls4s$eq[[ 2 ]] ) ) )
print( !is.null( fit2sls4s$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fit2sls5rs$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fit2sls5rs ) ) )
print( all.equal( mm1, model.matrix( fit2sls5rs$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit2sls5rs$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fit2sls5r ) ) )
print( all.equal( mm1, model.matrix( fit2sls5r$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fit2sls5r$eq[[ 2 ]] ) ) )
print( !is.null( fit2sls5r$eq[[ 1 ]]$x ) )

# matrices of instrumental variables
model.matrix( fit2sls1, which = "z" )
model.matrix( fit2sls1$eq[[ 1 ]], which = "z" )
model.matrix( fit2sls1$eq[[ 2 ]], which = "z" )

# matrices of fitted regressors
model.matrix( fit2sls5r, which = "xHat" )
model.matrix( fit2sls5r$eq[[ 1 ]], which = "xHat" )
model.matrix( fit2sls5r$eq[[ 2 ]], which = "xHat" )


## **************** formulas ************************
formula( fit2sls1 )
formula( fit2sls1$eq[[ 1 ]] )

formula( fit2sls2s )
formula( fit2sls2s$eq[[ 2 ]] )

formula( fit2sls3 )
formula( fit2sls3$eq[[ 1 ]] )

formula( fit2sls4r )
formula( fit2sls4r$eq[[ 2 ]] )

formula( fit2sls5rs )
formula( fit2sls5rs$eq[[ 1 ]] )

formula( fit2slsd1 )
formula( fit2slsd1$eq[[ 2 ]] )

formula( fit2slsd2r )
formula( fit2slsd2r$eq[[ 1 ]] )


## **************** model terms *******************
terms( fit2sls1 )
terms( fit2sls1$eq[[ 1 ]] )

terms( fit2sls2s )
terms( fit2sls2s$eq[[ 2 ]] )

terms( fit2sls3 )
terms( fit2sls3$eq[[ 1 ]] )

terms( fit2sls4r )
terms( fit2sls4r$eq[[ 2 ]] )

terms( fit2sls5rs )
terms( fit2sls5rs$eq[[ 1 ]] )

terms( fit2slsd1 )
terms( fit2slsd1$eq[[ 2 ]] )

terms( fit2slsd2r )
terms( fit2slsd2r$eq[[ 1 ]] )


## **************** terms of instruments *******************
fit2sls1$eq[[ 1 ]]$termsInst

fit2sls2s$eq[[ 2 ]]$termsInst

fit2sls3$eq[[ 1 ]]$termsInst

fit2sls4r$eq[[ 2 ]]$termsInst

fit2sls5rs$eq[[ 1 ]]$termsInst

fit2slsd1$eq[[ 2 ]]$termsInst

fit2slsd2r$eq[[ 1 ]]$termsInst


## **************** estfun ************************
library( "sandwich" )

estfun( fit2sls1 )
round( colSums( estfun( fit2sls1 ) ), digits = 7 )

estfun( fit2sls1s )
round( colSums( estfun( fit2sls1s ) ), digits = 7 )

estfun( fit2sls1r )
round( colSums( estfun( fit2sls1r ) ), digits = 7 )


## **************** bread ************************
bread( fit2sls1 )

bread( fit2sls1s )

bread( fit2sls1r )

formula.systemfit <- function( x, ... ) {
   result <- list()
   eqnLabels <- NULL
   for( i in 1:length( x$eq ) ){
      result <- c( result, formula( x$eq[[ i ]] ) )
      eqnLabels <- c( eqnLabels, x$eq[[ i ]]$eqnLabel )
   }
   names( result ) <- eqnLabels
   return( result )
}

formula.systemfit.equation <- function( x, ... ) {
   result <- formula( x$terms )
   return( result )
}

###   $Id: nlsystemfit.r 1132 2015-06-08 20:51:21Z arne $
###
###            Simultaneous Nonlinear Least Squares for R
###
### Copyright 2003-2004 Jeff D. Hamann <jeff.hamann@forestinformatics.com>
###
### This file is part of the nlsystemfit library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 2, or at your option, any later version,
### incorporated herein by reference.
###
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
###
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA

### uses the Dennis + Schnabel Minimizer which is the one utilized by R's nlm()

### remove before release
# rm(list=ls(all=TRUE))

### this function is the "driver" function for the minimization...
knls <- function( theta, eqns, data, fitmethod="OLS", parmnames, instr=NULL, S=NULL ) {

  r  <- matrix()               # residuals equation wise
  r <- NULL

  gmm.resids <- matrix()
  gmm.resids <- NULL

  residi  <- list()               # residuals equation wise
  lhs <- list()
  rhs <- list()
  neqs <- length( eqns )
  nobs <- dim( data )[[1]]            # number of nonmissing observations

  ## GMM specific variables, in this case... g = 2, k = 3
#  V <- matrix( 0, g*k, g*k )          # V is a 6x6 matrix
  moments <- list()
  mn <- array()

  moments <- NULL
  mn <- NULL
  lhs <- NULL
  rhs <- NULL
  residi <- NULL
  # partial derivatives of the residuals with respect to the parameters
  dResidTheta <- NULL
  dResidThetai <- list()
  d2ResidTheta <- array( NA, c( 0, 0, 0 ) )
  d2ResidThetai <- list()

  ## get the values of the parameters
  for( i in 1:length( parmnames ) ) {
    name <- names( parmnames )[i]
    val <- theta[i]
    storage.mode( val ) <-  "double"
    assign( name, val )
  }

  ## build the residual vector...
  for( i in 1:length( eqns ) ) {
    lhs[[i]] <- as.matrix( eval( as.formula( eqns[[i]] )[[2]], envir = data ) )
    rhs[[i]] <- as.matrix( eval( as.formula( eqns[[i]] )[[3]], envir = data ) )
    residi[[i]] <- lhs[[i]] - rhs[[i]]
    r <- rbind( r, as.matrix( residi[[i]] ) )
    if( fitmethod == "GMM" ) {
      gmm.resids <- cbind( gmm.resids, as.matrix( residi[[i]] ) )
    }
    dResidThetai[[ i ]] <- - attributes( with( data, with( as.list( theta ),
      eval( deriv( eqns[[i]], names( parmnames )), envir = data ))))$gradient
    dResidTheta <- rbind( dResidTheta, dResidThetai[[ i ]] )
    d2ResidThetai[[ i ]] <- - attributes( with( data, with( as.list( theta ),
      eval( deriv3( eqns[[i]], names( parmnames ) ), envir = data ))))$hessian
    temp <- array( NA, c( dim( d2ResidTheta )[ 1 ] +
      dim( d2ResidThetai[[ i ]] )[ 1 ], dim( d2ResidThetai[[ i ]] )[ 2:3 ] ) )
    if( i > 1 ) {
      temp[ 1:dim( d2ResidTheta )[ 1 ], , ] <- d2ResidTheta
    }
    temp[ ( dim( d2ResidTheta )[ 1 ] + 1 ):( dim( temp )[ 1 ] ),
      , ] <- d2ResidThetai[[ i ]]
    d2ResidTheta <- temp
  }
  ## these are the objective functions for the various fitting methods
  if( fitmethod == "OLS" ) {
    obj <- crossprod( r )
    gradient <- 2 * t( r ) %*% dResidTheta
#print( d2ResidTheta )
#print( t( dResidTheta ) %*% dResidTheta )
    hessian <- matrix( NA, nrow = length( parmnames ), ncol = length( parmnames ) )
    for( i in 1:length( parmnames ) ) {
      hessian[ i, ] <- 2 * t( r ) %*% d2ResidTheta[ , , i ]
    }
    hessian <- hessian + 2 * t( dResidTheta ) %*% dResidTheta
    rownames( hessian ) <- colnames( dResidTheta )
    colnames( hessian ) <- colnames( dResidTheta )
#print( hessian - t( hessian ) )
    hessian <- NULL
    attributes( obj ) <- list( gradient = gradient, hessian = hessian )
  }
  if( fitmethod == "2SLS" ) {
    ## W is premultiplied == ( diag( neqs ) %x% W )
    ##obj <- ( t(r) %*% S %*% r )
    obj <- crossprod(t(crossprod(r,S)),r)
    attributes( obj ) <- list( gradient = 2 * t( r ) %*% S %*% dResidTheta )
  }
  if( fitmethod == "SUR" ) {
    ## S is premultiplied == ( qr.solve( S ) %x% diag( nobs ) )
    ##obj <- ( t(r) %*% S %*% r )
    obj <- crossprod(t(crossprod(r,S)),r)
    attributes( obj ) <- list( gradient = 2 * t( r ) %*% S %*% dResidTheta )
  }
  if( fitmethod == "3SLS" ) {
    ## S is premultiplied == ( qr.solve( S ) %x% W )
    ##obj <- ( t(r) %*% S %*% r )
    obj <- crossprod(t(crossprod(r,S)),r)
    attributes( obj ) <- list( gradient = 2 * t( r ) %*% S %*% dResidTheta )
  }
  if( fitmethod == "GMM" ) {
    ## this just can't be correct... or can it...
    ## S is a gx x gk matrix
    ## g = number of eqns, k = number of inst variables
    z <- as.matrix( model.frame( instr, data = data ) )
    for(t in 1:nobs) {
      moments <- rbind( moments, gmm.resids[t,] %x% z[t,] )
    }
    g <- length( eqns )                 # number of equations
    k <- dim( as.matrix( model.frame( instr, data = data ) ) )[[2]]
    gk <- g*k
    for( i in 1:gk ) {
      mn <- rbind( mn, mean( moments[,i] ) )
    }
    ##obj <- ( t(nobs*mn) %*% S %*% (nobs*mn) ) / nobs
    ##obj <- ( t(mn) %*% S %*% (mn) )
    obj <- crossprod(t(crossprod(mn,S)),mn)
  }

  ## it would be nice to place the gradient and/or hessian attributes...
  ## how can I make this work???
  ## attr( obj, "gradient" ) <- "hi mom"
  ## attr( obj, "hessian" ) <- hessian...

  return( obj )
}


nlsystemfit <- function( method="OLS",
                        eqns,
                        startvals,
                        eqnlabels=c(as.character(1:length(eqns))),
                        inst=NULL,
                        data=list(),
                        solvtol=.Machine$double.eps,
                        maxiter=1000, ... ) {

  ## some tests
  if(!(method=="OLS" | method=="SUR" | method=="2SLS" | method=="3SLS" | method=="GMM" )){
    stop("The method must be 'OLS', 'SUR', '2SLS', or '3SLS'")}
  if((method=="2SLS" | method=="3SLS" | method=="GMM") & is.null(inst)) {
    stop("The methods '2SLS', '3SLS' and 'GMM' need instruments!")}

  lhs <- list()
  rhs <- list()
  derivs <- list()

  results <- list()               # results to be returned
  results$eq <- list()            # results for the individual equations
  resulti <- list()               # results of the ith equation
  residi  <- list()               # residuals equation wise
  iter    <- NULL                 # number of iterations
  G       <- length( eqns )       # number of equations
  n       <- array( 0, c(G))      # number of observations in each equation
  k       <- array( 0, c(G) )     # number of (unrestricted) coefficients/regressors in each equation
  df       <- array( 0, c(G) )     # degrees of freedom in each equation
  instl   <- list()               # list of the instruments for each equation
  ssr     <- array( 0, c(G))      # sum of squared residuals of each equation
  mse     <- array( 0, c(G))      # mean square error (residuals) of each equation
  rmse    <- array( 0, c(G))      # root of mse
  r2      <- array( 0, c(G))      # R-squared value
  adjr2   <- array( 0, c(G))      # adjusted R-squared value
  nobs <- dim( data )[[1]]
  S <- matrix( 0, G, G )               # covariance matrix of the residuals
  X <- array()
  x <- list()

  resids <- array()
  resids <- NULL

  if( method == "OLS" ) {
    if( TRUE ) {
      est <- nlm( knls, startvals,
               typsize=abs(startvals),iterlim=maxiter,
               eqns=eqns,
               data=data,
               fitmethod=method,
               parmnames=startvals,
               ... )
    } else {
      est <- optim( fn = knls, par = startvals,
               eqns=eqns,
               data=data,
               fitmethod=method,
               parmnames=startvals )
    }
  }
  if( method == "2SLS" ) {
    ## just fit and part out the return structure
    z <- as.matrix( model.frame( inst, data = data ) )
    Wt <- z %*% qr.solve( crossprod( z ), tol=solvtol ) %*% t(z)
    W2 <- diag( length( eqns ) ) %x% Wt
    est <- nlm( knls, startvals,
               typsize=abs(startvals),iterlim=maxiter,
               eqns=eqns,
               data=data,
               fitmethod=method,
               parmnames=startvals,
               S=W2, ... )
  }
  if( method == "SUR" || method == "3SLS" || method == "GMM" ) {
    ## fit ols/2sls, build the cov matrix for estimation and refit
    if( method == "SUR" ) {
      estols <- nlm( knls, startvals,
                    typsize=abs(startvals),iterlim=maxiter,
                    eqns=eqns,
                    data=data,
                    fitmethod="OLS",
                    parmnames=startvals, ... )
    }
    if( method == "3SLS" || method == "GMM" ) {
      z <- as.matrix( model.frame( inst, data = data ) )
      W <- z %*% qr.solve( crossprod( z ), tol=solvtol ) %*% t(z)
      W2 <- ( diag( length( eqns ) ) %x% W )
      estols <- nlm( knls, startvals,
                    typsize=abs(startvals),iterlim=maxiter,
                    eqns=eqns,
                    data=data,
                    fitmethod="2SLS",
                    parmnames=startvals,
                    instr=inst,
                    S=W2, ... )
    }

    ## build the S matrix
    names( estols$estimate ) <- names( startvals )
    for( i in 1:length( estols$estimate ) ) {
      name <- names( estols$estimate )[i]
      val <- estols$estimate[i]
      storage.mode( val ) <-  "double"
      assign( name, val )
    }

    ## get the rank for the eqns, compute the first-stage
    ## cov matrix to finish the SUR and 3SLS methods
    for(i in 1:G) {
      lhs[[i]] <- as.matrix( eval( as.formula( eqns[[i]] )[[2]], envir = data ) )
      rhs[[i]] <- as.matrix( eval( as.formula( eqns[[i]] )[[3]], envir = data ) )
      residi[[i]] <- lhs[[i]] - rhs[[i]]
      derivs[[i]] <- deriv( as.formula( eqns[[i]] ), names( startvals ) )
      ## computing the jacobian to get the rank to get the number of variables...
      jacobian <- attr( eval( derivs[[i]], envir = data ), "gradient" )
      n[i]   <-  length( lhs[[i]] )
      k[i] <- qr( jacobian )$rank
      df[i] <- n[i] - k[i]
    }

    ## covariance matrix of the residuals from the first stage...
    Sols <- matrix( 0, G, G )
    rcovformula <- 1
    for(i in 1:G) {
      for(j in 1:G) {
        Sols[i,j] <- sum(residi[[i]]*residi[[j]])/(
                                                   sqrt((n[i]-rcovformula*k[i])*(n[j]-rcovformula*k[j])))
      }
    }

    if( method == "SUR" ) {
      Solsinv <- qr.solve( Sols, tol=solvtol ) %x% diag( nobs )
      est <- nlm( knls,estols$estimate,
                 typsize=abs(estols$estimate),iterlim=maxiter,
                 eqns=eqns, data=data, fitmethod=method, parmnames=startvals,
                 S=Solsinv, ... )
    }
    if( method == "3SLS" ) {
      z <- as.matrix( model.frame( inst, data = data ) )
      W <- z %*% qr.solve( crossprod( z ), tol=solvtol ) %*% t(z)
      Solsinv <- qr.solve( Sols, tol=solvtol ) %x% W
      est <- nlm( knls, estols$estimate,
                 typsize=abs(estols$estimate),iterlim=maxiter,
                 eqns=eqns, data=data, fitmethod=method, parmnames=startvals,
                 S=Solsinv, instr=z, ... )
    }
    if( method == "GMM" ) {
      resids <- NULL
      for(i in 1:G) {
        resids <- cbind( resids, residi[[i]] )
      }
      z <- as.matrix( model.frame( inst, data = data ) )
      moments <- list()
      moments <- NULL
      for(t in 1:nobs) {
        moments <- rbind( moments, resids[t,] %x% z[t,] )
      }
      v2sls <- qr.solve( var( moments ), tol=solvtol )
      est <- nlm( knls,estols$estimate,
                 typsize=abs(estols$estimate),iterlim=maxiter,
                 eqns=eqns, data=data, fitmethod="GMM", parmnames=startvals,
                 S=v2sls, instr=inst, ... )
    }
  }

  ## done with the fitting...
  ## now, part out the results from the nlm function
  ## to rebuild the equations and return object
  ## get the parameters for each of the equations and


  ## evaluate the residuals for eqn
  ## get the values of the final parameters
  if( TRUE ) {
    estimate <- est$estimate
  } else {
    estimate <- est$par
  }
  names( estimate ) <- names( startvals )
  for( i in 1:length( estimate ) ) {
    name <- names( estimate )[i]
    ### I wonder if I need to clear out the variables before assigning them for good measure...
    assign( name, NULL )
    val <- estimate[i]
    storage.mode( val ) <-  "double"
    assign( name, val )
  }

  ## get the rank for the eqns, compute the first-stage
  ## cov matrix to finish the SUR and 3SLS methods
  X <- NULL
  results$resids <- array()
  results$resids <- NULL

  ## you're working on parsing out the parameters and the estiamtes for the return structure...
  for(i in 1:G) {
    lhs[[i]] <- as.matrix( eval( as.formula( eqns[[i]] )[[2]], envir = data ) )
    rhs[[i]] <- as.matrix( eval( as.formula( eqns[[i]] )[[3]], envir = data ) )
    residi[[i]] <- lhs[[i]] - rhs[[i]]
    derivs[[i]] <- deriv( as.formula( eqns[[i]] ), names( startvals ) )

    ## computing the jacobian to get the rank to get the number of variables...
    jacobian <- attr( eval( derivs[[i]], envir = data ), "gradient" )
    n[i]   <-  length( lhs[[i]] )
    k[i] <- qr( jacobian )$rank
    df[i] <- n[i] - k[i]
    ssr[i] <- crossprod( residi[[i]] )
    mse[i] <- ssr[i] / ( n[i] - k[i] )
    rmse[i] <- sqrt( mse[i] )

    X <- rbind( X, jacobian )
    results$resids <- cbind( results$resids, as.matrix( residi[[i]] ) )
  }

  ## compute the final covariance matrix
  ## you really should use the code below to handle weights...
  rcovformula <- 1
  for(i in 1:G) {
    for(j in 1:G) {
      S[i,j] <- sum(residi[[i]]*residi[[j]])/(
                                              sqrt((n[i]-rcovformula*k[i])*(n[j]-rcovformula*k[j])))
    }
  }

### for when you get the weights working...
#     vardef <- 1
#     if( vardef == 1 ) {
#       D <- diag( G ) * 1 / sqrt( nrow( data ) )
#     }
#     if( vardef == 2 ) {
#       D <- diag( G ) * 1 / sqrt( sum( weights ) )
#     }
#     if( vardef == 3 ) {
#       D <- diag( G ) * 1 / sqrt( sum( weights ) - ( sum( n ) - sum( k ) ) )
#     }
#     if( vardef == 4 ) {
#       for(i in 1:G) {
#         D <- diag( G )
#         D[i,i] <- D[i,i] * 1 / sqrt( nrow( data ) - n[i] )
#       }
#     }
#     ## the docs have this, but the table contains the above equations
#     R <- crossprod( results$resids )
#     S <- D %*% R %*% D
#     SI <- qr.solve( S, tol=solvtol ) %x% diag( nrow( data ) )
#     covb <- qr.solve(t(X) %*% SI %*% X, tol=solvtol )



  ## get the variance-covariance matrix
  if( method == "OLS" ) {
    SI <- diag( diag( qr.solve( S, tol=solvtol ) ) ) %x% diag( nrow( data ) )
    covb <- qr.solve(t(X) %*% SI %*% X, tol=solvtol )
  }
  if( method == "2SLS" ) {
    Z <- model.matrix(inst, data = data )
    W <- Z %*% qr.solve( crossprod( Z ), tol=solvtol ) %*% t(Z)
    SW <- diag( diag( qr.solve( S, tol=solvtol ) ) ) %x% W
    covb <- qr.solve(t(X) %*% SW %*% X, tol=solvtol )
  }
  if( method == "SUR" ) {
    SI <- qr.solve( S, tol=solvtol ) %x% diag( nrow( data ) )
    covb <- qr.solve(t(X) %*% SI %*% X, tol=solvtol )
  }
  if( method == "3SLS" ) {
    Z <- model.matrix(inst, data = data )
    W <- Z %*% qr.solve( crossprod( Z ), tol=solvtol ) %*% t(Z)
    SW <- qr.solve( S, tol=solvtol ) %x% W
    covb <- qr.solve(t(X) %*% SW %*% X, tol=solvtol )
  }
  if( method == "GMM" ) {
#    print( "obtaining GMM(SE)" )
    z <- as.matrix( model.frame( inst, data = data ) )
    moments <- list()
    moments <- NULL
    resids <- NULL
    for(i in 1:G) {
      resids <- cbind( resids, residi[[i]] )
    }
    for(t in 1:nobs) {
      moments <- rbind( moments, resids[t,] %x% z[t,] )
    }
#    print( var( moments ) )
    Vinv <- qr.solve( var( moments ), tol=solvtol )
#    print( Vinv )
    Y <- diag( length( eqns ) ) %x% t(z)
#    print( "covb now..." )
#    print( dim( Y ) )
#    print( dim( X ) )
    covb <- qr.solve( t( Y %*% X ) %*% Vinv %*% ( Y %*% X  ), tol=solvtol )
#    print( covb )
  }
  colnames( covb ) <- rownames( covb )


  ## bind the standard errors to the parameter estimate matrix
  se2 <- sqrt( diag( covb ) )
  t.val <- estimate / se2
  prob  <- 2*( 1 - pt( abs( t.val ), sum( n ) - sum( k ) ) ) ### you better check this...

  results$method       <- method
  results$n <- sum( n )
  results$k <- sum( k )
  results$b <- estimate
  results$se <- se2
  results$t <- t.val
  results$p <- prob

  ## build the results structure...
  for(i in 1:G) {
    ## you may be able to shrink this up a little and write the values directly to the return structure...
    eqn.terms <- vector()
    eqn.est <- vector()
    eqn.se <- vector()
    jacob <- attr( eval( deriv( as.formula( eqns[[i]] ), names( startvals ) ),
       envir = data ), "gradient" )
    for( v in 1:length( estimate ) ) {
      if( qr( jacob[,v] )$rank > 0 ) {
        eqn.terms <- rbind( eqn.terms, name <- names( estimate )[v] )
        eqn.est <- rbind( eqn.est, estimate[v] )
        eqn.se <- rbind( eqn.se, se2[v] )
      }
    }


    ## build the "return" structure for the equations
    resulti$method       <- method
    resulti$i            <- i               # equation number
    resulti$eqnlabel     <- eqnlabels[[i]]
    resulti$formula      <- eqns[[i]]
    resulti$b <- as.vector( eqn.est )
    names( resulti$b )   <- eqn.terms
    resulti$se           <- eqn.se
    resulti$t            <- resulti$b / resulti$se
    resulti$p            <- 2*( 1-pt(abs(resulti$t), n[i] - k[i] ))
    resulti$n            <- n[i]            # number of observations
    resulti$k            <- k[i]            # number of coefficients/regressors
    resulti$df           <- df[i]           # degrees of freedom of residuals
    resulti$predicted    <- rhs[[i]]           # predicted values
    resulti$residuals    <- residi[[i]]     # residuals
    resulti$ssr          <- ssr[i]             # sum of squared errors/residuals
    resulti$mse          <- mse[i]             # estimated variance of the residuals (mean squared error)
    resulti$s2           <- mse[i]             #        the same (sigma hat squared)
    resulti$rmse         <- rmse[i]            # estimated standard error of the residuals
    resulti$s            <- rmse[i]            #        the same (sigma hat)

#     ## you'll need these to compute the correlations...
#     print( paste( "eqn ", i ) )
    coefNames <- rownames( covb )[ rownames( covb ) %in%
      strsplit( as.character( eqns[[ i ]] )[ 3 ], "[^a-zA-Z0-9.]" )[[ 1 ]] ]
    resulti$covb <- covb[ coefNames, coefNames ]

#     resulti$x <- model.frame( as.formula( eqns[[i]] )[[3]], data = data )
#     print( resulti$x )
#    print( model.frame( eval( eqns[[i]], envir = data ), data = data ) )



    ## fix this to allow for multiple instruments?
    if(method=="2SLS" | method=="3SLS" | method=="GMM") {
      resulti$inst         <- inst
      ##resulti$inst         <- inst[[i]]
      ##resulti$inst         <- instl[[i]]
      ## resulti$h            <- h[[i]]          # matrix of instrumental variables
    }

    resulti$r2     <- 1 - ssr[i] / ( ( crossprod( lhs[[i]]) ) - mean( lhs[[i]] )^2 * nobs )
    resulti$adjr2  <- 1 - ((n[i]-1)/df[i])*(1-resulti$r2)

    class(resulti)        <- "nlsystemfit.equation"
    results$eq[[i]]      <- resulti
  }

  results$solvtol <- solvtol
  results$covb <- covb
  results$rcov <- S
  results$rcor <- cor( results$resids )
  results$drcov <- det(results$rcov)          # det(rcov, tol=solvetol)

  if(method=="2SLS" || method=="3SLS") {
    ##      results$h       <- H            # matrix of all (diagonally stacked) instrumental variables
  }
  if(method=="SUR" || method=="3SLS" || method=="GMM" ) {
    results$rcovest <- Sols      # residual covarance matrix used for estimation
    ##results$mcelr2  <- mcelr2       # McElroy's R-squared value for the equation system
  }

  ## build the "return" structure for the whole system
  results$method  <- method
  results$g       <- G              # number of equations
  results$nlmest  <- est

  class(results)  <- "nlsystemfit.system"

  if( results$nlmest$code >= 4 ) {
    warning( "Estimation did not converge!" )
  }

  return( results )
}


## print the (summary) results that belong to the whole system
summary.nlsystemfit.system <- function(object,...) {
  summary.nlsystemfit.system <- object
  summary.nlsystemfit.system
}


## print the results that belong to the whole system
print.nlsystemfit.system <- function( x, digits=6,... ) {
  object <- x

  save.digits <- unlist(options(digits=digits))
  on.exit(options(digits=save.digits))

  table <- NULL
  labels <- NULL

  cat("\n")
  cat("nlsystemfit results \n")
  cat("method: ")

#  if(!is.null(object$iter)) if(object$iter>1) cat("iterated ")
  cat( paste( object$method, "\n\n"))
#   if(!is.null(object$iter)) {
#     if(object$iter>1) {
#       if(object$iter<object$maxiter) {
#         cat( paste( "convergence achieved after",object$iter,"iterations\n\n" ) )
#       } else {
#         cat( paste( "warning: convergence not achieved after",object$iter,"iterations\n\n" ) )
#       }
#     }
#   }

  cat( paste( "convergence achieved after",object$nlmest$iterations,"iterations\n" ) )
  cat( paste( "nlsystemfit objective function value:",object$nlmest$minimum,"\n\n" ) )


  for(i in 1:object$g) {
    row <- NULL
    row <- cbind( round( object$eq[[i]]$n,     digits ),
                  round( object$eq[[i]]$df,    digits ),
                  round( object$eq[[i]]$ssr,   digits ),
                  round( object$eq[[i]]$mse,   digits ),
                  round( object$eq[[i]]$rmse,  digits ),
                  round( object$eq[[i]]$r2,    digits ),
                  round( object$eq[[i]]$adjr2, digits ))
    table  <- rbind( table, row )
    labels <- rbind( labels, object$eq[[i]]$eqnlabel )
  }
  rownames(table) <- c( labels )
  colnames(table) <- c("N","DF", "SSR", "MSE", "RMSE", "R2", "Adj R2" )

  ##print.matrix(table, quote = FALSE, right = TRUE )
  ##prmatrix(table, quote = FALSE, right = TRUE )
  print(table, quote = FALSE, right = TRUE )
  cat("\n")

  ## check this code before release...
  if(!is.null(object$rcovest)) {
    cat("The covariance matrix of the residuals used for estimation\n")
    rcov <- object$rcovest
    rownames(rcov) <- labels
    colnames(rcov) <- labels
    print( rcov )
    cat("\n")
    if( min(eigen( object$rcovest, only.values=TRUE)$values) < 0 ) {
      cat("warning: this covariance matrix is NOT positive semidefinit!\n")
      cat("\n")
    }
  }

  cat("The covariance matrix of the residuals\n")
  rcov <- object$rcov
  rownames(rcov) <- labels
  colnames(rcov) <- labels
  print( rcov )
  cat("\n")

  cat("The correlations of the residuals\n")
  rcor <- object$rcor
  rownames(rcor) <- labels
  colnames(rcor) <- labels
  print( rcor )
  cat("\n")

  cat("The determinant of the residual covariance matrix: ")
  cat(object$drcov)
  cat("\n")

### check this code before release
#   cat("OLS R-squared value of the system: ")
#   cat(object$olsr2)
#   cat("\n")

#   if(!is.null(object$mcelr2)) {
#     cat("McElroy's R-squared value for the system: ")
#     cat(object$mcelr2)
#     cat("\n")
#   }

  ## now print the individual equations
  for(i in 1:object$g) {
    print( object$eq[[i]], digits )
  }

}


## print the (summary) results for a single equation
summary.nlsystemfit.equation <- function(object,...) {
  summary.nlsystemfit.equation <- object
  summary.nlsystemfit.equation
}


## print the results for a single equation
print.nlsystemfit.equation <- function( x, digits=6, ... ) {
  object <- x

  save.digits <- unlist(options(digits=digits))
  on.exit(options(digits=save.digits))

  cat("\n")
  cat( paste( object$method, " estimates for ", object$eqnlabel, " (equation ", object$i, ")\n", sep="" ) )

  cat("Model Formula: ")
  print(object$formula)
  if(!is.null(object$inst)) {
    cat("Instruments: ")
    print(object$inst)
  }
  cat("\n")

  Signif <- symnum(object$p, corr = FALSE, na = FALSE,
                   cutpoints = c(0,  .001,.01,.05, .1, 1),
                   symbols   = c("***","**","*","."," "))

  table <- cbind(round( object$b,  digits ),
                 round( object$se, digits ),
                 round( object$t,  digits ),
                 round( object$p,  digits ),
                 Signif)

  rownames(table) <- names(object$b)
  colnames(table) <- c("Estimate","Std. Error","t value","Pr(>|t|)","")

  ##print.matrix(table, quote = FALSE, right = TRUE )
  ##prmatrix(table, quote = FALSE, right = TRUE )
  print(table, quote = FALSE, right = TRUE )
  cat("---\nSignif. codes: ",attr(Signif,"legend"),"\n")

  cat(paste("\nResidual standard error:", round(object$s, digits),  ## s ist the variance, isn't it???
            "on", object$df, "degrees of freedom\n"))

  cat( paste( "Number of observations:", round(object$n, digits),
              "Degrees of Freedom:", round(object$df, digits),"\n" ) )

  cat( paste( "SSR:",     round(object$ssr,    digits),
              "MSE:", round(object$mse, digits),
              "Root MSE:",   round(object$rmse,  digits), "\n" ) )

   cat( paste( "Multiple R-Squared:", round(object$r2,    digits),
               "Adjusted R-Squared:", round(object$adjr2, digits),
               "\n" ) )
  cat("\n")
}

# from Model Selection and Inference: A Practical Information-Theoretic Approach
# Kenneth P. Burnham and David R. Anderson, 1998. Springer-Verlag, New York, New York.

## Akaike's Information Criterion
## AIC = n * log( sigmahat^2 ) + 2K
## n = number of obs
## sigmahat^2 = sum( error^2 ) / n == residual sums of squares
## K is the total number if estimated parameters, including the intercept and sigma^2 (nparams + 1)
## second order AIC
## AICc = AIC + (2K*(K+1))/(n-K-1)
## unless the sample size is large with repsect to the number of estiamted parameters, use AICc.

## calculate predicted values, its standard errors and the prediction intervals
predict.systemfit <- function( object, newdata = NULL,
                               se.fit=FALSE, se.pred=FALSE,
                               interval="none", level=0.95,
                               useDfSys = NULL, ... ) {

   if( is.null( useDfSys ) ) {
      useDfSys <- length( coef( object ) ) != object$rank
         # TRUE if there are restrictions imposed
   }

   for(i in 1:length( object$eq ) )  {
      predicted.i <- predict( object$eq[[ i ]], newdata = newdata,
         se.fit = se.fit, se.pred = se.pred, interval = interval,
         level = level, useDfSys = useDfSys )
      names( predicted.i ) <- paste( object$eq[[ i ]]$eqnLabel, ".",
         names( predicted.i ), sep = "" )
      if( i == 1 ) {
         predicted <- predicted.i
      } else {
         predicted <- cbind( predicted, predicted.i )
      }
   }
   names( predicted ) <- sub( "(?<!\\.se)\\.fit$", ".pred",
         names( predicted ), perl = TRUE )

   return( predicted )
}

## calculate predicted values, its standard errors and the prediction intervals
predict.systemfit.equation <- function( object, newdata = NULL,
                               se.fit=FALSE, se.pred=FALSE,
                               interval="none", level=0.95,
                               useDfSys = NULL, ... ) {

   if( is.null( useDfSys ) ) {
      useDfSys <- object$nCoef.sys != object$rank.sys
         # TRUE if there are restrictions imposed
   }

   if( is.null( newdata ) ) {
      xMatNoNa <-  model.matrix( object )
      xMat <- matrix( NA, nrow = length( residuals( object ) ),
         ncol = ncol( xMatNoNa ) )
      xMat[ !is.na( residuals( object ) ), ] <- xMatNoNa
      rm( xMatNoNa )
   } else {
      xMat <-  model.matrix( formula( delete.response( object$terms ) ),
         data = newdata )
   }

   # fitted values
   predicted <- data.frame( fit = drop( xMat %*% object$coefficients ) )

   # calculate variance covariance matrices
   if( se.fit | interval == "confidence" ) {
      yCovConf <- drop( xMat %*% object$coefCov %*% t( xMat ) )
   }
   if( se.pred | interval == "prediction" ) {
      sigmaSqr <- sum( residuals( object, na.rm = TRUE )^2 ) / df.residual( object )
      yCovPred <- drop( xMat %*% object$coefCov %*% t( xMat ) + sigmaSqr )
   }
   # standard errors of fitted values
   if( se.fit ) {
      if( length( yCovConf ) == 1 ) {
         predicted[[ "se.fit" ]] <- sqrt( yCovConf )
      } else {
         predicted[[ "se.fit" ]] <- sqrt( diag( yCovConf ) )
      }
   }
   # standard errors of prediction
   if( se.pred ) {
      if( length( yCovPred ) == 1 ) {
         predicted[[ "se.pred" ]] <- sqrt( yCovPred )
      } else {
         predicted[[ "se.pred" ]] <- sqrt( diag( yCovPred ) )
      }
   }

   # confidence intervals
   if( interval == "confidence" ) {
      if( useDfSys ) {
         tval   <- qt( 1 - ( 1- level )/2, object$df.residual.sys )
      } else {
         tval   <- qt( 1 - ( 1- level )/2, object$df.residual )
      }
      if(  length( yCovConf ) == 1 ) {
         stdErConf <- sqrt( yCovConf )
      } else {
         stdErConf <- sqrt( diag( yCovConf ) )
      }
      predicted[[ "lwr" ]] <- predicted$fit - ( tval * stdErConf )
      predicted[[ "upr" ]] <- predicted$fit + ( tval * stdErConf )
   }
   # prediction intervals
   if( interval == "prediction" ) {
      if( useDfSys ) {
         tval   <- qt( 1 - ( 1- level )/2, object$df.residual.sys )
      } else {
         tval   <- qt( 1 - ( 1- level )/2, object$df.residual )
      }
      if( length( yCovPred ) == 1 ) {
         stdErPred <- sqrt( yCovPred )
      } else {
         stdErPred <- sqrt( diag( yCovPred ) )
      }
      predicted[[ "lwr" ]] <- predicted$fit - ( tval * stdErPred )
      predicted[[ "upr" ]] <- predicted$fit + ( tval * stdErPred )
   }

   return( predicted )
}

.stackMatList <- function( matList, way, useMatrix = FALSE ){
   if( way == "diag" ){
      result <- matrix( 0, 0, 0 )
      for( i in 1:length( matList ) ){
         result <- rbind( 
            cbind( result, 
               matrix( 0, nrow( result ), ncol( matList[[ i ]] ) ) ),
            cbind( matrix( 0, nrow( matList[[ i ]] ), ncol( result ) ),
               as.matrix( matList[[ i ]] ) ) )
      }
   } else if( way == "below" ) {
      result <- NULL
      for( i in 1:length( matList ) ){
         result <- rbind( result, as.matrix( matList[[ i ]] ) )
      }
   }

   if( useMatrix ){
      result <- as( result, "CsparseMatrix" )
   }

   return( result )
}

.prepareWmatrix <- function( upperleft, R.restr, useMatrix = FALSE ){
   if( nrow( R.restr ) == 1 ){
      lowerRows <- c( R.restr, 0 )
   } else {
      lowerRows <- cbind2( R.restr,
         matrix( 0, nrow( R.restr ), nrow( R.restr ) ) )
   }
   result <- rbind2( cbind2( as.matrix( upperleft ), t(R.restr) ), lowerRows )

   if( useMatrix ){
      result <- as( result, "denseMatrix")
   }

   return( result )
}

logLik.systemfit <- function( object, residCovDiag = FALSE, ... ){

   if( length( residCovDiag ) != 1 || any( !is.logical( residCovDiag ) ) ) {
      stop( "argument 'residCovDiag' must be a single logical value" )
   } 
      
   resid <- residuals( object )
   residCov <- .calcResidCov( resid, "noDfCor" )
   if( residCovDiag ) {
      residCov <- diag( diag( residCov ) )
   }
   residCovInv <- solve( residCov )
   resid <- as.matrix( resid )
   nEq <- ncol( resid )

   result <- 0
   for( i in 1:nrow( resid ) ) {
      vEq <- !is.na( resid[ i, ] )
      if( sum( vEq ) == nEq ) {
         result <- result - ( nEq / 2 ) * log( 2 * pi ) -
            ( 1 / 2 ) * log( det( residCov ) ) -
            ( 1 / 2 ) * resid[ i, , drop = FALSE ] %*% residCovInv %*%
               t( resid[ i, , drop = FALSE ] )
      } else if( sum( vEq ) > 0 ) {
         nEq2 <- sum( vEq )
         residCov2 <- residCov[ vEq, vEq, drop = FALSE ] -
            residCov[ vEq, !vEq, drop = FALSE ] %*%
            solve( residCov[ !vEq, !vEq, drop = FALSE ] ) %*%
            residCov[ !vEq, vEq, drop = FALSE ]
         residCov2Inv <- solve( residCov2 )
         result <- result - ( nEq2 / 2 ) * log( 2 * pi ) -
            ( 1 / 2 ) * log( det( residCov2 ) ) -
            ( 1 / 2 ) * resid[ i, vEq, drop = FALSE ] %*%
               residCov2Inv %*% t( resid[ i, vEq, drop = FALSE ] )
      }
   }

   if( object$method %in% c( "OLS", "2SLS" ) ){
      nSigma <- 1
   } else if( object$method %in% c( "WLS", "W2SLS" ) ){
      nSigma <- nEq
   } else if( object$method %in% c( "SUR", "3SLS" ) ){
      nSigma <- nEq * ( nEq + 1 ) / 2
   } else {
      stop( "internal error: unknown estimation method '", object$method, "'" )
   }

   attributes( result )$nobs <- df.residual( object ) + object$rank
   attributes( result )$df <- object$rank + nSigma
   class( result ) <- "logLik"

   return( result )
}

library( "systemfit" )
options( digits = 5 )

data( "Kmenta" )
useMatrix <- FALSE

eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
eqSystem <- list( demand = eqDemand, supply = eqSupply )
restrm <- matrix(0,1,7)  # restriction matrix "R"
restrm[1,3] <-  1
restrm[1,7] <- -1
restr2m <- matrix(0,2,7)  # restriction matrix "R" 2
restr2m[1,3] <-  1
restr2m[1,7] <- -1
restr2m[2,2] <- -1
restr2m[2,5] <-  1
restr2q <- c( 0, 0.5 )  # restriction vector "q" 2
tc <- matrix(0,7,6)
tc[1,1] <- 1
tc[2,2] <- 1
tc[3,3] <- 1
tc[4,4] <- 1
tc[5,5] <- 1
tc[6,6] <- 1
tc[7,3] <- 1
restr3m <- matrix(0,1,6)  # restriction matrix "R" 2
restr3m[1,2] <- -1
restr3m[1,5] <-  1
restr3q <- c( 0.5 )  # restriction vector "q" 2


## ******************* unrestricted estimation *****************
## ******************** default estimation *********************
fit2sls1 <- systemfit( eqSystem, "2SLS", data = Kmenta, inst = inst,
   useMatrix = useMatrix )
fit3sls1 <- systemfit( eqSystem, "3SLS", data = Kmenta, inst = inst,
   useMatrix = useMatrix )
print( hausman.systemfit( fit2sls1, fit3sls1 ) )

## ************** 2SLS estimation with singleEqSigma = FALSE *****************
fit2sls1s <- systemfit( eqSystem, "2SLS", data = Kmenta, inst = inst,
   singleEqSigma = FALSE, useMatrix = useMatrix )
print( hausman.systemfit( fit2sls1s, fit3sls1 ) )

## ******************* estimations with methodResidCov = 0 *****************
fit2sls1r <- systemfit( eqSystem, "2SLS", data = Kmenta, inst = inst,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
fit3sls1r <- systemfit( eqSystem, "3SLS", data = Kmenta, inst = inst,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
print( hausman.systemfit( fit2sls1r, fit3sls1r ) )


## ********************* estimation with restriction ********************
## *********************** default estimation ***********************
fit2sls2 <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, useMatrix = useMatrix )
fit3sls2 <- systemfit( eqSystem, "3SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, useMatrix = useMatrix )
# print( hausman.systemfit( fit2sls2, fit3sls2 ) )

## ************* 2SLS estimation with singleEqSigma = TRUE *****************
fit2sls2s <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, singleEqSigma = TRUE, useMatrix = useMatrix )
# print( hausman.systemfit( fit2sls2s, fit3sls2 ) )

## ********************* estimations with methodResidCov = 0 **************
fit2sls2r <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, methodResidCov = "noDfCor", useMatrix = useMatrix )
fit3sls2r <- systemfit( eqSystem, "3SLS", data = Kmenta, restrict.matrix = restrm,
   inst = inst, methodResidCov = "noDfCor", useMatrix = useMatrix )
# print( hausman.systemfit( fit2sls2r, fit3sls2r ) )


## ****************** estimation with restriction via restrict.regMat ******************
## ********************** default estimation ********************
fit2sls3 <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.regMat = tc,
   inst = inst, useMatrix = useMatrix )
fit3sls3 <- systemfit( eqSystem, "3SLS", data = Kmenta, restrict.regMat = tc,
   inst = inst, useMatrix = useMatrix )
print( hausman.systemfit( fit2sls3, fit3sls3 ) )

## ******************* estimations with methodResidCov = 0 *******
fit2sls3r <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.regMat = tc,
   inst = inst, methodResidCov = "noDfCor", useMatrix = useMatrix )
fit3sls3r <- systemfit( eqSystem, "3SLS", data = Kmenta, restrict.regMat = tc,
   inst = inst, methodResidCov = "noDfCor", useMatrix = useMatrix )
print( hausman.systemfit( fit2sls3r, fit3sls3r ) )


## ***************** estimations with 2 restrictions *******************
## *********************** default estimations **************
fit2sls4 <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, useMatrix = useMatrix )
fit3sls4 <- systemfit( eqSystem, "3SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, useMatrix = useMatrix )
# print( hausman.systemfit( fit2sls4, fit3sls4 ) )

## ***************** estimations with methodResidCov = 0 **************
fit2sls4r <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, methodResidCov = "noDfCor",
   useMatrix = useMatrix )
fit3sls4r <- systemfit( eqSystem, "3SLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, inst = inst, methodResidCov = "noDfCor",
   useMatrix = useMatrix )
# print( hausman.systemfit( fit2sls4r, fit3sls4r ) )


## *********** estimations with 2 restrictions via R and restrict.regMat ***************
## ***************** default estimations *******************
fit2sls5 <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   useMatrix = useMatrix )
fit3sls5 <- systemfit( eqSystem, "3SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   useMatrix = useMatrix )
# print( hausman.systemfit( fit2sls5, fit3sls5 ) )

## ************* estimations with methodResidCov = 0 *********
fit2sls5r <- systemfit( eqSystem, "2SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
fit3sls5r <- systemfit( eqSystem, "3SLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, inst = inst,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
# print( hausman.systemfit( fit2sls5r, fit3sls5r ) )

library( "systemfit" )
library( "sandwich" )
options( warn = 1 )
options( digits = 3 )

data( "KleinI" )
eqConsump  <- consump ~ corpProf + corpProfLag + wages
eqInvest   <- invest ~ corpProf + corpProfLag + capitalLag
eqPrivWage <- privWage ~ gnp + gnpLag + trend
inst <- ~ govExp + taxes + govWage + trend + capitalLag + corpProfLag + gnpLag
system <- list( Consumption = eqConsump, Investment = eqInvest,
   PrivateWages = eqPrivWage )
restrict <- c( "Consumption_corpProf + Investment_capitalLag = 0" )
restrict2 <- c( restrict, "Consumption_corpProfLag - PrivateWages_trend = 0" )

for( dataNo in 1:5 ) {
   # set some values of some variables to NA
   if( dataNo == 2 ) {
      KleinI$gnpLag[ 7 ] <- NA
   } else if( dataNo == 3 ) {
      KleinI$wages[ 10 ] <- NA
   } else if( dataNo == 4 ) {
      KleinI$corpProf[ 13 ] <- NA
   } else if( dataNo == 5 ) {
      KleinI$invest[ 16 ] <- NA
   }

   # single-equation OLS
   lmConsump  <- lm( eqConsump, data = KleinI )
   lmInvest   <- lm( eqInvest, data = KleinI )
   lmPrivWage <- lm( eqPrivWage, data = KleinI )
   
for( methodNo in 1:5 ) {
   method <- c( "OLS", "2SLS", "SUR", "3SLS", "3SLS" )[ methodNo ]
   maxit <- ifelse( methodNo == 5, 500, 1 )

   cat( "> \n> # ", ifelse( maxit == 1, "", "I" ), method, "\n", sep = "" )
   if( method %in% c( "OLS", "WLS", "SUR" ) ) {
      kleinModel <- systemfit( system, method = method, data = KleinI,
         methodResidCov = ifelse( method == "OLS", "geomean", "noDfCor" ),
         maxit = maxit )
   } else {
      kleinModel <- systemfit( system, method = method, data = KleinI,
         inst = inst, methodResidCov = "noDfCor", maxit = maxit )
   }
   cat( "> summary\n" )
   print( summary( kleinModel ) )
   if( method == "OLS" ) {
      cat( "compare coef with single-equation OLS\n" )
      print( all.equal( coef( kleinModel ),
         c( coef( lmConsump ), coef( lmInvest ), coef( lmPrivWage ) ),
         check.attributes = FALSE ) )
   }
   cat( "> residuals\n" )
   print( residuals( kleinModel ) )
   cat( "> fitted\n" )
   print( fitted( kleinModel ) )
   cat( "> predict\n" )
   print( predict( kleinModel, se.fit = TRUE,
      interval = ifelse( methodNo %in% c( 1, 4 ), "prediction",  "confidence" ),
      useDfSys = methodNo %in% c( 1, 3, 5 ) ) )
   cat( "> model.frame\n" )
   if( methodNo == 1 ) {
      mfOls <- model.frame( kleinModel )
      print( mfOls )
   } else if( methodNo == 2 ) {
      mf2sls <- model.frame( kleinModel )
      print( mf2sls )
   cat( "> Frames of instrumental variables\n" )
      for( i in 1:3 ){
         print( kleinModel$eq[[ i ]]$modelInst )
      }
   } else if( methodNo == 3 ) {
      print( all.equal( mfOls, model.frame( kleinModel ) ) )
   } else {
      print( all.equal( mf2sls, model.frame( kleinModel ) ) )
   }
   cat( "> model.matrix\n" )
   if( methodNo == 1 ) {
      mmOls <- model.matrix( kleinModel )
      print( mmOls )
   } else {
      print( all.equal( mmOls, model.matrix( kleinModel ) ) )
   }
   if( methodNo == 2 ) {
      cat( "> matrix of instrumental variables\n" )
      print( model.matrix( kleinModel, which = "z" ) )
      cat( "> matrix of fitted regressors\n" )
      print( round( model.matrix( kleinModel, which = "xHat" ), digits = 7 ) )
   }
   cat( "> nobs\n" )
   print( nobs( kleinModel ) )
   cat( "> linearHypothesis\n" )
   print( linearHypothesis( kleinModel, restrict ) )
   print( linearHypothesis( kleinModel, restrict, test = "F" ) )
   print( linearHypothesis( kleinModel, restrict, test = "Chisq" ) )
   print( linearHypothesis( kleinModel, restrict2 ) )
   print( linearHypothesis( kleinModel, restrict2, test = "F" ) )
   print( linearHypothesis( kleinModel, restrict2, test = "Chisq" ) )
   cat( "> logLik\n" )
   print( logLik( kleinModel ) )
   print( logLik( kleinModel, residCovDiag = TRUE ) )
   if( method == "OLS" ) {
      cat( "compare log likelihood value with single-equation OLS\n" )
      print( all.equal( logLik( kleinModel, residCovDiag = TRUE ),
         logLik( lmConsump ) + logLik( lmInvest ) + logLik( lmPrivWage ),
         check.attributes = FALSE ) )
   }

   cat( "Estimating function\n" )
   print( round( estfun( kleinModel ), digits = 7 ) )
   print( all.equal( colSums( estfun( kleinModel ) ),
      rep( 0, ncol( estfun( kleinModel ) ) ), check.attributes = FALSE ) )

   cat( "> Bread\n" )
   print( bread( kleinModel ) )
}
}

## this function returns test statistic for
## the hausman test
# The m-statistic is then distributed with k degrees of freedom, where k
# is the rank of the matrix .A generalized inverse is used, as
# recommended by Hausman (1982).

hausman.systemfit <- function( results2sls, results3sls ) {

   result <- list()

   if( is.null( results2sls$restrict.regMat ) ) {
      result$q <- coef( results2sls ) - coef( results3sls )
      result$qVar <- vcov( results2sls ) - vcov( results3sls )
   } else {
      result$q <- coef( results2sls, modified.regMat = TRUE ) -
         coef( results3sls, modified.regMat = TRUE )
      result$qVar <- vcov( results2sls, modified.regMat = TRUE ) -
         vcov( results3sls, modified.regMat = TRUE )
   }

#    if( min( eigen( hausman$qVar )$values ) < 0 ) {
#       warning( "the matrix V is not 'positive definite'" )
#    }

   result$statistic <- crossprod( result$q, solve( result$qVar, result$q ) )
   names( result$statistic ) <- "Hausman"
   result$parameter <- nrow( result$qVar )
   names( result$parameter ) <- "df"
   result$p.value <- 1 - pchisq( result$statistic, result$parameter )
   result$method = paste( "Hausman specification test for consistency of",
      "the 3SLS estimation" )
   if( "data" %in% names( results2sls$call ) ) {
      result$data.name <- results2sls$call$data
   } else {
      result$data.name <- "unknown"
   }
   class( result ) <- "htest"
   return( result )
}

linearHypothesis.systemfit <- function( model,
      hypothesis.matrix, rhs = NULL, test = c( "FT", "F", "Chisq" ),
      vcov. = NULL, ... ){

   thisCall <- match.call()
   test <- match.arg( test )

   modelName <- deparse( substitute( model ) )

   if( test == "Chisq" ){
      result <- car::linearHypothesis.default( model,
         hypothesis.matrix = hypothesis.matrix, rhs = rhs, test = test,
         vcov. = vcov., ... )

      attributes( result )$heading[ 1 ] <-
         "Linear hypothesis test (Chi^2 statistic of a Wald test)\n\nHypothesis:"

      modelPos <- grep( "^Model 1: .*Model 2:", attributes( result )$heading )
      attributes( result )$heading[ modelPos[ 1 ] ] <-
         sub( "Model 2:.*$",
            paste( "Model 2: ", modelName, sep = "" ),
            attributes( result )$heading[ modelPos[ 1 ] ] )

   } else if ( test == "F" ) {
      result <- car::linearHypothesis.default( model,
         hypothesis.matrix = hypothesis.matrix, rhs = rhs, test = test,
         vcov. = vcov., ... )

      attributes( result )$heading[ 1 ] <-
         "Linear hypothesis test (F statistic of a Wald test)\n\nHypothesis:"

      modelPos <- grep( "^Model 1: .*Model 2:", attributes( result )$heading )
      attributes( result )$heading[ modelPos[ 1 ] ] <-
         sub( "Model 2:.*$",
            paste( "Model 2: ", modelName, sep = "" ),
            attributes( result )$heading[ modelPos[ 1 ] ] )

   } else if ( test == "FT" ) {
      if( is.character( hypothesis.matrix ) ) {
         R.restr <- car::makeHypothesis( names( coef( model ) ),
            hypothesis.matrix, rhs )
         if( is.null( dim( R.restr ) ) ){
            R.restr <- t( R.restr )
         }
         q.restr <- R.restr[ , ncol( R.restr ), drop = FALSE ]
         R.restr <- R.restr[ , -ncol( R.restr ), drop = FALSE ]
         rownames( R.restr ) <- hypothesis.matrix
      } else {
         R.restr <- hypothesis.matrix
         if( is.null( rhs ) ) {
            q.restr <- rep( 0, nrow( hypothesis.matrix ) )
         } else {
            q.restr <- rhs
         }
      }

      result <- as.data.frame( matrix( NA, nrow = 2, ncol = 4 ) )
      names( result ) <- c( "Res.Df", "Df", "F", "Pr(>F)" )
      rownames( result ) <- c( 1, 2 )

      ftest <- .ftest.systemfit( object = model,
         restrict.matrix = R.restr, restrict.rhs = q.restr,
         vcov. = vcov. )
      result[ 1, 1 ] <- ftest$df.residual.sys + ftest$nRestr
      result[ 2, 1 ] <- ftest$df.residual.sys
      result[ 2, 2 ] <- result[ 1, 1 ] - result[ 2, 1 ]
      result[ 2, 3 ] <- ftest$statistic
      result[ 2, 4 ] <- ftest$p.value

      title <- "Linear hypothesis test (Theil's F test)\n\nHypothesis:"
      topnote <- paste( "Model 1: restricted model",
         "\nModel 2: ", modelName, sep = "" )
      if( is.null( vcov. ) ){
         note <- ""
      } else {
         note <- "\nNote: Coefficient covariance matrix supplied.\n"
      }
      attributes( result )$heading <- c( title,
         car::printHypothesis( R.restr, q.restr, names( coef( model ) ) ),
         "", topnote, note )
      class( result ) <- c( "anova", "data.frame" )
   } else {
      stop( "unknown test statistic '", test, "'. Please use 'F', 'FT', or 'Chisq'" )
   }

   return( result )
}

linear.hypothesis.systemfit <- function( ... ) {
   .Deprecated( "linearHypothesis.systemfit", package = "systemfit" )
   return( linearHypothesis.systemfit( ... ) )
}

library( systemfit )
options( digits = 3 )

data( "Kmenta" )
useMatrix <- FALSE

demand <- consump ~ price + income
supply <- consump ~ price + farmPrice + trend
system <- list( demand = demand, supply = supply )
restrm <- matrix(0,1,7)  # restriction matrix "R"
restrm[1,3] <-  1
restrm[1,7] <- -1
restrict <- "demand_income - supply_trend = 0"
restr2m <- matrix(0,2,7)  # restriction matrix "R" 2
restr2m[1,3] <-  1
restr2m[1,7] <- -1
restr2m[2,2] <- -1
restr2m[2,5] <-  1
restr2q <- c( 0, 0.5 )  # restriction vector "q" 2
restrict2 <- c( "demand_income - supply_trend = 0",
   "- demand_price + supply_price = 0.5" )
tc <- matrix(0,7,6)
tc[1,1] <- 1
tc[2,2] <- 1
tc[3,3] <- 1
tc[4,4] <- 1
tc[5,5] <- 1
tc[6,6] <- 1
tc[7,3] <- 1
restr3m <- matrix(0,1,6)  # restriction matrix "R" 2
restr3m[1,2] <- -1
restr3m[1,5] <-  1
restr3q <- c( 0.5 )  # restriction vector "q" 2
restrict3 <- "- C2 + C5 = 0.5"


## *******  single-equation OLS estimations  *********************
lmDemand <- lm( demand, data = Kmenta )
lmSupply <- lm( supply, data = Kmenta )

## *************** WLS estimation ************************
fitwls1 <- systemfit( system, "WLS", data = Kmenta, useMatrix = useMatrix )
print( summary( fitwls1 ) )
all.equal( coef( fitwls1 ), c( coef( lmDemand ), coef( lmSupply ) ),
   check.attributes = FALSE )
all.equal( coef( summary( fitwls1 ) ),
   rbind( coef( summary( lmDemand ) ), coef( summary( lmSupply ) ) ),
   check.attributes = FALSE )
all.equal( vcov( fitwls1 ),
   as.matrix( bdiag( vcov( lmDemand ), vcov( lmSupply ) ) ),
   check.attributes = FALSE )

## *************** WLS estimation (EViews-like) ************************
fitwls1e <- systemfit( system, "WLS", data = Kmenta, methodResidCov = "noDfCor",
   x = TRUE, useMatrix = useMatrix )
print( summary( fitwls1e, useDfSys = TRUE ) )
all.equal( coef( fitwls1e ), c( coef( lmDemand ), coef( lmSupply ) ),
   check.attributes = FALSE )

## ************** WLS with cross-equation restriction ***************
fitwls2 <- systemfit( system, "WLS", data = Kmenta, restrict.matrix = restrm,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitwls2 ) )
# the same with symbolically specified restrictions
fitwls2Sym <- systemfit( system, "WLS", data = Kmenta,
   restrict.matrix = restrict, x = TRUE,
   useMatrix = useMatrix )
all.equal( fitwls2, fitwls2Sym )

## ************** WLS with cross-equation restriction (EViews-like) *******
fitwls2e <- systemfit( system, "WLS", data = Kmenta, restrict.matrix = restrm,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fitwls2e ) )

## ******* WLS with cross-equation restriction via restrict.regMat **********
fitwls3 <- systemfit( system,"WLS", data = Kmenta, restrict.regMat = tc,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitwls3 ) )

## ******* WLS with cross-equation restriction via restrict.regMat (EViews-like) *****
fitwls3e <- systemfit( system,"WLS", data = Kmenta, restrict.regMat = tc,
   methodResidCov = "noDfCor", useMatrix = useMatrix )
print( summary( fitwls3e ) )

## ***** WLS with 2 cross-equation restrictions ***************
fitwls4 <- systemfit( system,"WLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, useMatrix = useMatrix )
print( summary( fitwls4 ) )
# the same with symbolically specified restrictions
fitwls4Sym <- systemfit( system, "WLS", data = Kmenta,
   restrict.matrix = restrict2, useMatrix = useMatrix )
all.equal( fitwls4, fitwls4Sym )

## ***** WLS with 2 cross-equation restrictions (EViews-like) **********
fitwls4e <- systemfit( system,"WLS", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr2m, restrict.rhs = restr2q,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitwls4e ) )

## *********** WLS with 2 cross-equation restrictions via R and restrict.regMat ******
fitwls5 <- systemfit( system, "WLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitwls5 ) )
# the same with symbolically specified restrictions
fitwls5Sym <- systemfit( system, "WLS", data = Kmenta,
   restrict.matrix = restrict3, restrict.regMat = tc,
   x = TRUE, useMatrix = useMatrix )
all.equal( fitwls5, fitwls5Sym )

## *********** WLS with 2 cross-equation restrictions via R and restrict.regMat (EViews-like)
fitwls5e <- systemfit( system, "WLS", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   useMatrix = useMatrix )
print( summary( fitwls5e ) )

## *************** iterated WLS estimation *********************
fitwlsi1 <- systemfit( system, "WLS", data = Kmenta,
   maxit = 100, useMatrix = useMatrix )
print( summary( fitwlsi1, useDfSys = TRUE ) )

## *************** iterated WLS estimation (EViews-like) ************
fitwlsi1e <- systemfit( system, "WLS", data = Kmenta, methodResidCov = "noDfCor",
   maxit = 100, x = TRUE, useMatrix = useMatrix )
print( summary( fitwlsi1e, useDfSys = TRUE ) )

## ****** iterated WLS with cross-equation restriction ***************
fitwlsi2 <- systemfit( system, "WLS", data = Kmenta, restrict.matrix = restrm,
   maxit = 100, x = TRUE, useMatrix = useMatrix )
print( summary( fitwlsi2 ) )

## ****** iterated WLS with cross-equation restriction (EViews-like) ********
fitwlsi2e <- systemfit( system, "WLS", data = Kmenta, restrict.matrix = restrm,
   methodResidCov = "noDfCor", maxit = 100, useMatrix = useMatrix )
print( summary( fitwlsi2e ) )

## ******* iterated WLS with cross-equation restriction via restrict.regMat **********
fitwlsi3 <- systemfit( system, "WLS", data = Kmenta, restrict.regMat = tc,
   maxit = 100, x = TRUE, useMatrix = useMatrix )
print( summary( fitwlsi3 ) )

## ******* iterated WLS with cross-equation restriction via restrict.regMat (EViews-like) ***
fitwlsi3e <- systemfit( system, "WLS", data = Kmenta, restrict.regMat = tc,
   methodResidCov = "noDfCor", maxit = 100, useMatrix = useMatrix )
print( summary( fitwlsi3e ) )
nobs( fitwlsi3e )

## ******* iterated WLS with 2 cross-equation restrictions ***********
fitwlsi4 <- systemfit( system, "WLS", data = Kmenta, restrict.matrix = restr2m,
   restrict.rhs = restr2q, maxit = 100, useMatrix = useMatrix )
print( summary( fitwlsi4 ) )

## ******* iterated WLS with 2 cross-equation restrictions (EViews-like) *****
fitwlsi4e <- systemfit( system, "WLS", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr2m, restrict.rhs = restr2q, maxit = 100,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitwlsi4e ) )

## ***** iterated WLS with 2 cross-equation restrictions via R and restrict.regMat ******
fitwlsi5 <- systemfit( system, "WLS", data = Kmenta, restrict.matrix = restr3m,
   restrict.rhs = restr3q, restrict.regMat = tc, maxit = 100,
   x = TRUE, useMatrix = useMatrix )
print( summary( fitwlsi5 ) )

## *** iterated WLS with 2 cross-equation restrictions via R and restrict.regMat (EViews-like)
fitwlsi5e <- systemfit( system, "WLS", data = Kmenta, methodResidCov = "noDfCor",
   restrict.matrix = restr3m, restrict.rhs = restr3q, restrict.regMat = tc,
   maxit = 100, useMatrix = useMatrix )
print( summary( fitwlsi5e ) )


## *********** estimations with a single regressor ************
fitwlsS1 <- systemfit(
   list( consump ~ price - 1, consump ~ price + trend ), "WLS",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitwlsS1 ) )
fitwlsS2 <- systemfit(
   list( consump ~ price - 1, consump ~ trend - 1 ), "WLS",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitwlsS2 ) )
fitwlsS3 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "WLS",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitwlsS3 ) )
fitwlsS4 <- systemfit(
   list( consump ~ trend - 1, price ~ trend - 1 ), "WLS",
   data = Kmenta, restrict.matrix = matrix( c( 1, -1 ), nrow = 1 ),
   useMatrix = useMatrix )
print( summary( fitwlsS4 ) )
fitwlsS5 <- systemfit(
   list( consump ~ 1, price ~ 1 ), "WLS",
   data = Kmenta, useMatrix = useMatrix )
print( summary( fitwlsS5) )


## **************** shorter summaries **********************
print( summary( fitwls1 ), residCov = FALSE, equations = FALSE )

print( summary( fitwls2e, useDfSys = FALSE, residCov = FALSE ),
   equations = FALSE )

print( summary( fitwls3 ), residCov = FALSE )

print( summary( fitwls4e, residCov = FALSE, equations = FALSE ) )

print( summary( fitwls5, useDfSys = FALSE ), residCov = FALSE )

print( summary( fitwlsi1e, useDfSys = TRUE, equations = FALSE ) )

print( summary( fitwlsi2, equations = FALSE, residCov = FALSE ),
   residCov = TRUE )

print( summary( fitwlsi3e ), equations = FALSE, residCov = FALSE )

print( summary( fitwlsi4, equations = FALSE ), equations = TRUE )

print( summary( fitwlsi5e, useDfSys = FALSE, residCov = FALSE ) )


## ****************** residuals **************************
print( residuals( fitwls1 ) )
print( residuals( fitwls1$eq[[ 2 ]] ) )

print( residuals( fitwls2e ) )
print( residuals( fitwls2e$eq[[ 1 ]] ) )

print( residuals( fitwls3 ) )
print( residuals( fitwls3$eq[[ 2 ]] ) )

print( residuals( fitwls4e ) )
print( residuals( fitwls4e$eq[[ 1 ]] ) )

print( residuals( fitwls5 ) )
print( residuals( fitwls5$eq[[ 2 ]] ) )

print( residuals( fitwlsi1e ) )
print( residuals( fitwlsi1e$eq[[ 1 ]] ) )

print( residuals( fitwlsi2 ) )
print( residuals( fitwlsi2$eq[[ 2 ]] ) )

print( residuals( fitwlsi3e ) )
print( residuals( fitwlsi3e$eq[[ 1 ]] ) )

print( residuals( fitwlsi4 ) )
print( residuals( fitwlsi4$eq[[ 2 ]] ) )

print( residuals( fitwlsi5e ) )
print( residuals( fitwlsi5e$eq[[ 1 ]] ) )


## *************** coefficients *********************
print( round( coef( fitwls1e ), digits = 6 ) )
print( round( coef( fitwls1e$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitwlsi2 ), digits = 6 ) )
print( round( coef( fitwlsi2$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitwls3e ), digits = 6 ) )
print( round( coef( fitwls3e, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitwls3e$eq[[ 1 ]] ), digits = 6 ) )

print( round( coef( fitwls4 ), digits = 6 ) )
print( round( coef( fitwls4$eq[[ 2 ]] ), digits = 6 ) )

print( round( coef( fitwlsi5 ), digits = 6 ) )
print( round( coef( fitwlsi5, modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( fitwlsi5$eq[[ 1 ]] ), digits = 6 ) )


## *************** coefficients with stats *********************
print( round( coef( summary( fitwls1e ) ), digits = 6 ) )
print( round( coef( summary( fitwls1e$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitwlsi2 ) ), digits = 6 ) )
print( round( coef( summary( fitwlsi2$eq[[ 2 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitwls3e ) ), digits = 6 ) )
print( round( coef( summary( fitwls3e ), modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fitwls3e$eq[[ 1 ]] ) ), digits = 6 ) )

print( round( coef( summary( fitwls4, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitwls4$eq[[ 2 ]], useDfSys = FALSE ) ),
   digits = 6 ) )

print( round( coef( summary( fitwlsi5, useDfSys = FALSE ) ), digits = 6 ) )
print( round( coef( summary( fitwlsi5, useDfSys = FALSE ),
   modified.regMat = TRUE ), digits = 6 ) )
print( round( coef( summary( fitwlsi5$eq[[ 1 ]], useDfSys = FALSE ) ),
   digits = 6 ) )


## *********** variance covariance matrix of the coefficients *******
print( round( vcov( fitwls1e ), digits = 6 ) )
print( round( vcov( fitwls1e$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitwls2 ), digits = 6 ) )
print( round( vcov( fitwls2$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitwls3e ), digits = 6 ) )
print( round( vcov( fitwls3e, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitwls3e$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitwls4 ), digits = 6 ) )
print( round( vcov( fitwls4$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitwls5 ), digits = 6 ) )
print( round( vcov( fitwls5, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitwls5$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitwlsi1 ), digits = 6 ) )
print( round( vcov( fitwlsi1$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitwlsi2e ), digits = 6 ) )
print( round( vcov( fitwlsi2e$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitwlsi3 ), digits = 6 ) )
print( round( vcov( fitwlsi3, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitwlsi3$eq[[ 2 ]] ), digits = 6 ) )

print( round( vcov( fitwlsi4e ), digits = 6 ) )
print( round( vcov( fitwlsi4e$eq[[ 1 ]] ), digits = 6 ) )

print( round( vcov( fitwlsi5e ), digits = 6 ) )
print( round( vcov( fitwlsi5e, modified.regMat = TRUE ), digits = 6 ) )
print( round( vcov( fitwlsi5e$eq[[ 2 ]] ), digits = 6 ) )


## *********** confidence intervals of coefficients *************
print( confint( fitwls1 ) )
print( confint( fitwls1$eq[[ 2 ]], level = 0.9 ) )

print( confint( fitwls2e, level = 0.9 ) )
print( confint( fitwls2e$eq[[ 1 ]], level = 0.99 ) )

print( confint( fitwls3, level = 0.99 ) )
print( confint( fitwls3$eq[[ 2 ]], level = 0.5 ) )

print( confint( fitwls4e, level = 0.5 ) )
print( confint( fitwls4e$eq[[ 1 ]], level = 0.25 ) )

print( confint( fitwls5, level = 0.25 ) )
print( confint( fitwls5$eq[[ 2 ]], level = 0.975 ) )

print( confint( fitwlsi1e, level = 0.975, useDfSys = TRUE ) )
print( confint( fitwlsi1e$eq[[ 1 ]], level = 0.999, useDfSys = TRUE ) )

print( confint( fitwlsi2, level = 0.999 ) )
print( confint( fitwlsi2$eq[[ 2 ]], level = 0.1 ) )

print( confint( fitwlsi3e, level = 0.1 ) )
print( confint( fitwlsi3e$eq[[ 1 ]], level = 0.01 ) )

print( confint( fitwlsi4, level = 0.01 ) )
print( confint( fitwlsi4$eq[[ 2 ]], level = 0.33 ) )

print( confint( fitwlsi5e, level = 0.33 ) )
print( confint( fitwlsi5e$eq[[ 1 ]] ) )


## *********** fitted values *************
print( fitted( fitwls1 ) )
print( fitted( fitwls1$eq[[ 2 ]] ) )

print( fitted( fitwls2e ) )
print( fitted( fitwls2e$eq[[ 1 ]] ) )

print( fitted( fitwls3 ) )
print( fitted( fitwls3$eq[[ 2 ]] ) )

print( fitted( fitwls4e ) )
print( fitted( fitwls4e$eq[[ 1 ]] ) )

print( fitted( fitwls5 ) )
print( fitted( fitwls5$eq[[ 2 ]] ) )

print( fitted( fitwlsi1e ) )
print( fitted( fitwlsi1e$eq[[ 1 ]] ) )

print( fitted( fitwlsi2 ) )
print( fitted( fitwlsi2$eq[[ 2 ]] ) )

print( fitted( fitwlsi3e ) )
print( fitted( fitwlsi3e$eq[[ 1 ]] ) )

print( fitted( fitwlsi4 ) )
print( fitted( fitwlsi4$eq[[ 2 ]] ) )

print( fitted( fitwlsi5e ) )
print( fitted( fitwlsi5e$eq[[ 1 ]] ) )


## *********** predicted values *************
predictData <- Kmenta
predictData$consump <- NULL
predictData$price <- Kmenta$price * 0.9
predictData$income <- Kmenta$income * 1.1

print( predict( fitwls1, se.fit = TRUE, interval = "prediction" ) )
print( predict( fitwls1$eq[[ 2 ]], se.fit = TRUE, interval = "prediction" ) )

print( predict( fitwls2e, se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )
print( predict( fitwls2e$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   level = 0.999, newdata = predictData ) )

print( predict( fitwls3, se.pred = TRUE, interval = "prediction",
   level = 0.975 ) )
print( predict( fitwls3$eq[[ 2 ]], se.pred = TRUE, interval = "prediction",
   level = 0.975 ) )

print( predict( fitwls4e, se.fit = TRUE, interval = "confidence",
   level = 0.25 ) )
print( predict( fitwls4e$eq[[ 1 ]], se.fit = TRUE, interval = "confidence",
   level = 0.25 ) )

print( predict( fitwls5, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )
print( predict( fitwls5$eq[[ 2 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = predictData ) )

print( predict( fitwlsi1e, se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99, useDfSys = TRUE ) )
print( predict( fitwlsi1e$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.99, useDfSys = TRUE ) )

print( predict( fitwlsi2, se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )
print( predict( fitwlsi2$eq[[ 2 ]], se.fit = TRUE, interval = "prediction",
   level = 0.9, newdata = predictData ) )

print( predict( fitwlsi3e, interval = "prediction", level = 0.925 ) )
print( predict( fitwlsi3e$eq[[ 1 ]], interval = "prediction", level = 0.925 ) )

print( predict( fitwlsi4, interval = "confidence", newdata = predictData ) )
print( predict( fitwlsi4$eq[[ 2 ]], interval = "confidence",
   newdata = predictData ) )

print( predict( fitwlsi5e, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.01 ) )
print( predict( fitwlsi5e$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.01 ) )

# predict just one observation
smallData <- data.frame( price = 130, income = 150, farmPrice = 120,
   trend = 25 )

print( predict( fitwls1, newdata = smallData ) )
print( predict( fitwls1$eq[[ 1 ]], newdata = smallData ) )

print( predict( fitwls2e, se.fit = TRUE, level = 0.9,
   newdata = smallData ) )
print( predict( fitwls2e$eq[[ 1 ]], se.pred = TRUE, level = 0.99,
   newdata = smallData ) )

print( predict( fitwls3, interval = "prediction", level = 0.975,
   newdata = smallData ) )
print( predict( fitwls3$eq[[ 1 ]], interval = "confidence", level = 0.8,
   newdata = smallData ) )

print( predict( fitwls4e, se.fit = TRUE, interval = "confidence",
   level = 0.999, newdata = smallData ) )
print( predict( fitwls4e$eq[[ 2 ]], se.pred = TRUE, interval = "prediction",
   level = 0.75, newdata = smallData ) )

print( predict( fitwls5, se.fit = TRUE, interval = "prediction",
   newdata = smallData ) )
print( predict( fitwls5$eq[[ 1 ]], se.pred = TRUE, interval = "confidence",
   newdata = smallData ) )

print( predict( fitwlsi3e, se.fit = TRUE, se.pred = TRUE,
   interval = "prediction", level = 0.5, newdata = smallData ) )
print( predict( fitwlsi3e$eq[[ 1 ]], se.fit = TRUE, se.pred = TRUE,
   interval = "confidence", level = 0.25, newdata = smallData ) )


## ************ correlation of predicted values ***************
print( correlation.systemfit( fitwls1, 2, 1 ) )

print( correlation.systemfit( fitwls2e, 1, 2 ) )

print( correlation.systemfit( fitwls3, 2, 1 ) )

print( correlation.systemfit( fitwls4e, 1, 2 ) )

print( correlation.systemfit( fitwls5, 2, 1 ) )

print( correlation.systemfit( fitwlsi1e, 1, 2 ) )

print( correlation.systemfit( fitwlsi2, 2, 1 ) )

print( correlation.systemfit( fitwlsi3e, 1, 2 ) )

print( correlation.systemfit( fitwlsi4, 2, 1 ) )

print( correlation.systemfit( fitwlsi5e, 1, 2 ) )


## ************ Log-Likelihood values ***************
print( logLik( fitwls1 ) )
print( logLik( fitwls1, residCovDiag = TRUE ) )
all.equal( logLik( fitwls1, residCovDiag = TRUE ),
   logLik( lmDemand ) + logLik( lmSupply ),
   check.attributes = FALSE )

print( logLik( fitwls2e ) )
print( logLik( fitwls2e, residCovDiag = TRUE ) )

print( logLik( fitwls3 ) )
print( logLik( fitwls3, residCovDiag = TRUE ) )

print( logLik( fitwls4e ) )
print( logLik( fitwls4e, residCovDiag = TRUE ) )

print( logLik( fitwls5 ) )
print( logLik( fitwls5, residCovDiag = TRUE ) )

print( logLik( fitwlsi1e ) )
print( logLik( fitwlsi1e, residCovDiag = TRUE ) )

print( logLik( fitwlsi2 ) )
print( logLik( fitwlsi2, residCovDiag = TRUE ) )

print( logLik( fitwlsi3e ) )
print( logLik( fitwlsi3e, residCovDiag = TRUE ) )

print( logLik( fitwlsi4 ) )
print( logLik( fitwlsi4, residCovDiag = TRUE ) )

print( logLik( fitwlsi5e ) )
print( logLik( fitwlsi5e, residCovDiag = TRUE ) )


## ************** F tests ****************
# testing first restriction
print( linearHypothesis( fitwls1, restrm ) )
linearHypothesis( fitwls1, restrict )

print( linearHypothesis( fitwlsi1e, restrm ) )
linearHypothesis( fitwlsi1e, restrict )

# testing second restriction
restrOnly2m <- matrix(0,1,7)
restrOnly2q <- 0.5
restrOnly2m[1,2] <- -1
restrOnly2m[1,5] <-  1
restrictOnly2 <- "- demand_price + supply_price = 0.5"
# first restriction not imposed 
print( linearHypothesis( fitwls1e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitwls1e, restrictOnly2 )

print( linearHypothesis( fitwlsi1, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitwlsi1, restrictOnly2 )

# first restriction imposed
print( linearHypothesis( fitwls2, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitwls2, restrictOnly2 )

print( linearHypothesis( fitwls3, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitwls3, restrictOnly2 )

print( linearHypothesis( fitwlsi2e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitwlsi2e, restrictOnly2 )

print( linearHypothesis( fitwlsi3e, restrOnly2m, restrOnly2q ) )
linearHypothesis( fitwlsi3e, restrictOnly2 )

# testing both of the restrictions
print( linearHypothesis( fitwls1e, restr2m, restr2q ) )
linearHypothesis( fitwls1e, restrict2 )

print( linearHypothesis( fitwlsi1, restr2m, restr2q ) )
linearHypothesis( fitwlsi1, restrict2 )


## ************** Wald tests ****************
# testing first restriction
print( linearHypothesis( fitwls1, restrm, test = "Chisq" ) )
linearHypothesis( fitwls1, restrict, test = "Chisq" )

print( linearHypothesis( fitwlsi1e, restrm, test = "Chisq" ) )
linearHypothesis( fitwlsi1e, restrict, test = "Chisq" )

# testing second restriction
# first restriction not imposed
print( linearHypothesis( fitwls1e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitwls1e, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitwlsi1, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitwlsi1, restrictOnly2, test = "Chisq" )

# first restriction imposed
print( linearHypothesis( fitwls2, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitwls2, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitwls3, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitwls3, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitwlsi2e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitwlsi2e, restrictOnly2, test = "Chisq" )

print( linearHypothesis( fitwlsi3e, restrOnly2m, restrOnly2q, test = "Chisq" ) )
linearHypothesis( fitwlsi3e, restrictOnly2, test = "Chisq" )

# testing both of the restrictions
print( linearHypothesis( fitwls1e, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitwls1e, restrict2, test = "Chisq" )

print( linearHypothesis( fitwlsi1, restr2m, restr2q, test = "Chisq" ) )
linearHypothesis( fitwlsi1, restrict2, test = "Chisq" )


## ****************** model frame **************************
print( mf <- model.frame( fitwls1 ) )
print( mf1 <- model.frame( fitwls1$eq[[ 1 ]] ) )
print( attributes( mf1 )$terms )
print( mf2 <- model.frame( fitwls1$eq[[ 2 ]] ) )
print( attributes( mf2 )$terms )

print( all.equal( mf, model.frame( fitwls2e ) ) )
print( all.equal( mf1, model.frame( fitwls2e$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitwls3 ) ) )
print( all.equal( mf2, model.frame( fitwls3$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitwls4e ) ) )
print( all.equal( mf1, model.frame( fitwls4e$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitwls5 ) ) )
print( all.equal( mf2, model.frame( fitwls5$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitwlsi1e ) ) )
print( all.equal( mf1, model.frame( fitwlsi1e$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitwlsi2 ) ) )
print( all.equal( mf2, model.frame( fitwlsi2$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitwlsi3e ) ) )
print( all.equal( mf1, model.frame( fitwlsi3e$eq[[ 1 ]] ) ) )

print( all.equal( mf, model.frame( fitwlsi4 ) ) )
print( all.equal( mf2, model.frame( fitwlsi4$eq[[ 2 ]] ) ) )

print( all.equal( mf, model.frame( fitwlsi5e ) ) )
print( all.equal( mf1, model.frame( fitwlsi5e$eq[[ 1 ]] ) ) )


## **************** model matrix ************************
# with x (returnModelMatrix) = TRUE
print( !is.null( fitwls1e$eq[[ 1 ]]$x ) )
print( mm <- model.matrix( fitwlsi1e ) )
print( mm1 <- model.matrix( fitwlsi1e$eq[[ 1 ]] ) )
print( mm2 <- model.matrix( fitwlsi1e$eq[[ 2 ]] ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitwlsi1 ) ) )
print( all.equal( mm1, model.matrix( fitwlsi1$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwlsi1$eq[[ 2 ]] ) ) )
print( !is.null( fitwls1$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitwls2$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitwls2 ) ) )
print( all.equal( mm1, model.matrix( fitwls2$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwls2$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitwls2e ) ) )
print( all.equal( mm1, model.matrix( fitwls2e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwls2e$eq[[ 2 ]] ) ) )
print( !is.null( fitwls2e$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitwlsi3$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitwlsi3 ) ) )
print( all.equal( mm1, model.matrix( fitwlsi3$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwlsi3$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitwlsi3e ) ) )
print( all.equal( mm1, model.matrix( fitwlsi3e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwlsi3e$eq[[ 2 ]] ) ) )
print( !is.null( fitwlsi3e$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitwls4e$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitwls4e ) ) )
print( all.equal( mm1, model.matrix( fitwls4e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwls4e$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitwls4Sym ) ) )
print( all.equal( mm1, model.matrix( fitwls4Sym$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwls4Sym$eq[[ 2 ]] ) ) )
print( !is.null( fitwls4Sym$eq[[ 1 ]]$x ) )

# with x (returnModelMatrix) = TRUE
print( !is.null( fitwls5$eq[[ 1 ]]$x ) )
print( all.equal( mm, model.matrix( fitwls5 ) ) )
print( all.equal( mm1, model.matrix( fitwls5$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwls5$eq[[ 2 ]] ) ) )

# with x (returnModelMatrix) = FALSE
print( all.equal( mm, model.matrix( fitwls5e ) ) )
print( all.equal( mm1, model.matrix( fitwls5e$eq[[ 1 ]] ) ) )
print( all.equal( mm2, model.matrix( fitwls5e$eq[[ 2 ]] ) ) )
print( !is.null( fitwls5e$eq[[ 1 ]]$x ) )


## **************** formulas ************************
formula( fitwls1 )
formula( fitwls1$eq[[ 2 ]] )

formula( fitwls2e )
formula( fitwls2e$eq[[ 1 ]] )

formula( fitwls3 )
formula( fitwls3$eq[[ 2 ]] )

formula( fitwls4e )
formula( fitwls4e$eq[[ 1 ]] )

formula( fitwls5 )
formula( fitwls5$eq[[ 2 ]] )

formula( fitwlsi1e )
formula( fitwlsi1e$eq[[ 1 ]] )

formula( fitwlsi2 )
formula( fitwlsi2$eq[[ 2 ]] )

formula( fitwlsi3e )
formula( fitwlsi3e$eq[[ 1 ]] )

formula( fitwlsi4 )
formula( fitwlsi4$eq[[ 2 ]] )

formula( fitwlsi5e )
formula( fitwlsi5e$eq[[ 1 ]] )


## **************** model terms *******************
terms( fitwls1 )
terms( fitwls1$eq[[ 2 ]] )

terms( fitwls2e )
terms( fitwls2e$eq[[ 1 ]] )

terms( fitwls3 )
terms( fitwls3$eq[[ 2 ]] )

terms( fitwls4e )
terms( fitwls4e$eq[[ 1 ]] )

terms( fitwls5 )
terms( fitwls5$eq[[ 2 ]] )

terms( fitwlsi1e )
terms( fitwlsi1e$eq[[ 1 ]] )

terms( fitwlsi2 )
terms( fitwlsi2$eq[[ 2 ]] )

terms( fitwlsi3e )
terms( fitwlsi3e$eq[[ 1 ]] )

terms( fitwlsi4 )
terms( fitwlsi4$eq[[ 2 ]] )

terms( fitwlsi5e )
terms( fitwlsi5e$eq[[ 1 ]] )


## **************** estfun ************************
library( "sandwich" )

estfun( fitwls1 )
round( colSums( estfun( fitwls1 ) ), digits = 7 )

estfun( fitwlsi1e )
round( colSums( estfun( fitwlsi1e ) ), digits = 7 )


## **************** bread ************************
bread( fitwls1 )

bread( fitwlsi1e )

\documentclass[article,nojss]{jss}
\usepackage[utf8]{inputenc}
\usepackage{amsmath}
\usepackage{float}

% \usepackage{lineno}
% 
% \linenumbers

\newcommand{\bHat}{\hat{\beta}}
\newcommand{\COVHat}{\widehat{\COV}}
\newcommand{\lHat}{\hat{\lambda}}
\newcommand{\OHat}{\widehat{\Omega}}
\newcommand{\SHat}{\widehat{\Sigma}}
\newcommand{\sHat}{\hat{\sigma}}
\newcommand{\uHat}{\hat{u}}
\newcommand{\XHat}{\widehat{X}}
\newcommand{\tr}{\mathrm{tr}}
\newcommand{\LR}{\mathit{LR}}
\clubpenalty=10000
\widowpenalty=10000
\setlength{\emergencystretch}{3em}
% \setlength{\overfullrule}{5pt}

\newcommand{\codeD}[2]{\code{#1}\hspace{0pt}\code{.#2}}
\newcommand{\codeDD}[3]{\code{#1}\hspace{0pt}\code{.#2}\hspace{0pt}\code{.#3}}
\newcommand{\codeDDD}[4]{\code{#1}\hspace{0pt}\code{.#2}\hspace{0pt}%
\code{.#3}\hspace{0pt}\code{.#4}}

\hyphenation{systemfit}

\author{Arne Henningsen\\University of Copenhagen
   \And Jeff D.\ Hamann\\ Forest Informatics, Inc.}
\Plainauthor{Arne Henningsen, Jeff D. Hamann}
        
\title{\pkg{systemfit}: A Package for Estimating Systems of Simultaneous Equations in \proglang{R}}
\Plaintitle{systemfit: A Package for Estimating Systems of Simultaneous Equations in R}
\Shorttitle{\pkg{systemfit}: Estimating Systems of Simultaneous Equations in \proglang{R}}

%% an abstract and keywords
\Abstract{
This introduction to the \proglang{R} package \pkg{systemfit}
is a slightly modified version of \cite{henningsen07a},
published in the \emph{Journal of Statistical Software}.

Many statistical analyses (e.g., in econometrics, biostatistics and
experimental design) are based on models containing systems of
structurally related equations.
The \pkg{systemfit} package provides the capability
to estimate systems of linear equations
within the \proglang{R} programming environment.
For instance, this package can be used for 
``ordinary least squares'' (OLS),
``seemingly unrelated regression'' (SUR),
and the instrumental variable (IV) methods
``two-stage least squares'' (2SLS) and
``three-stage least squares'' (3SLS),
where SUR and 3SLS estimations can optionally be iterated.
Furthermore, the \pkg{systemfit} package provides tools
for several statistical tests.
It has been tested on a variety of datasets
and its reliability is demonstrated.
}

\Keywords{\proglang{R}, system of simultaneous equations,
  seemingly unrelated regression, two-stage least squares,
  three-stage least squares, instrumental variables}
\Plainkeywords{R, system of simultaneous equations,
  seemingly unrelated regression, two-stage least squares,
  three-stage least squares, instrumental variables}

%% at least one keyword must be supplied


%% publication information
%% NOTE: This needs to filled out ONLY IF THE PAPER WAS ACCEPTED.
%% If it was not (yet) accepted, leave them commented.
\Volume{23}
\Issue{4}
\Month{December}
\Year{2007}
\Submitdate{2006-03-15}
\Acceptdate{2007-10-24}

%% The address of (at least) one author should be given
%% in the following format:
\Address{
   Arne Henningsen\\
   Institute of Food and Resource Economics\\
   University of Copenhagen\\
   Rolighedsvej 25\\
   D-1958 Frederiksberg C, Denmark\\
   E-mail: \email{arne.henningsen@gmail.com}\\
   URL: \url{http://www.arne-henningsen.name/}\\
   \\
   Jeff D.\ Hamann\\
   Forest Informatics, Inc.\\
   PO Box 1421\\
   Corvallis, Oregon 97339-1421, United States of America\\
   E-mail: \email{jeff.hamann@forestinformatics.com}\\
   URL: \url{http://www.forestinformatics.com/}\\
}
%% It is also possible to add a telephone and fax number
%% before the e-mail in the following format:
%% Telephone: +43/1/31336-5053
%% Fax: +43/1/31336-734

%% for those who use Sweave please include the following line (with % symbols):
%% need no \usepackage{Sweave.sty}

%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\begin{document}

% initialisation stuff

<<include=FALSE>>=
library(knitr)
opts_chunk$set(
engine='R'
)
@

%\VignetteIndexEntry{Systemfit}
%\VignetteDepends{plm,sem}
%\VignetteKeywords{R, system of simultaneous equations,
% seemingly unrelated regression, two-stage least squares,
% three-stage least squares, instrumental variables}
%\VignettePackage{systemfit}
%\VignetteEngine{knitr::knitr} 
<<echo=FALSE>>=
options( prompt = "R> ", ctinue = "+  " )
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Introduction}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Many theoretical models that are econometrically estimated
consist of more than one equation.
The disturbance terms of these equations are likely
to be contemporaneously correlated,
because unconsidered factors
that influence the disturbance term in one equation
probably influence the disturbance terms in other equations, too.
Ignoring this contemporaneous correlation
and estimating these equations separately
leads to inefficient estimates of the coefficients.
However, estimating all equations simultaneously
with a ``generalized least squares'' (GLS) estimator,
which takes the covariance structure of the residuals into account,
leads to efficient estimates.
This estimation procedure is generally called
``seemingly unrelated regression'' \citep[SUR,][]{zellner62}.
Another reason to estimate a system of equations simultaneously are
cross-equation restrictions on the coefficients.%
\footnote{
Especially the economic theory suggests many cross-equation
restrictions on the coefficients (e.g., the symmetry restriction in demand models).
}
Estimating the coefficients under cross-equation restrictions
and testing these restrictions
requires a simultaneous estimation approach.

Furthermore, these models can contain variables
that appear on the left-hand side in one equation
and on the right-hand side of another equation.
Ignoring the endogeneity of these variables can lead to inconsistent
estimates.
This simultaneity bias can be corrected for by
applying a ``two-stage least squares'' (2SLS) estimation
to each equation.
Combining this estimation method with the SUR method results
in a simultaneous estimation of the system of equations
by the ``three-stage least squares'' (3SLS) method
\citep{zellner62b}.

% For all of the methods developed in the package, the disturbances of
% the individual equations are assumed to be independent and identically
% distributed (iid).  
% In the future, we would like to add the ability to fit equations were
% the disturbances are serially correlated (wikins 1969).

The \pkg{systemfit} package provides the capability to estimate
systems of linear equations in \proglang{R}
\citep{r-project-07}.
Currently, the estimation methods
``ordinary least squares'' (OLS),
``weighted least squares'' (WLS),
``seemingly unrelated regression'' (SUR),
``two-stage least squares'' (2SLS),
``weighted two-stage least squares'' (W2SLS), and
``three-stage least squares'' (3SLS)
are implemented.%
\footnote{
In this context,
the term ``weighted'' in ``weighted least squares'' (WLS) and
``weighted two-stage least squares'' (W2SLS) means
that the \emph{equations} might have different weights
and \emph{not}
that the \emph{observations} have different weights.
}
The WLS, SUR, W2SLS, and 3SLS estimates can be based
either on one-step (OLS or 2SLS) (co)variances
or these estimations can be iterated,
where the (co)variances are calculated from the estimates of the previous step.
Furthermore,
the \pkg{systemfit} package
provides statistical tests for restrictions on the coefficients
and for testing the consistency of the 3SLS estimation.

Although systems of linear equations can be estimated
with several other statistical and econometric software packages
(e.g., \proglang{SAS}, \proglang{EViews}, \proglang{TSP}),
\pkg{systemfit} has several advantages.
First, all estimation procedures are publicly available in the source code.
Second, the estimation algorithms can be easily modified to meet specific
requirements.
Third, the (advanced) user can control estimation details generally
not available in other software packages by overriding reasonable defaults.

% This paper is organized as follows:
In Section~\ref{sec:statistics} we introduce the statistical
background of estimating equation systems.
The implementation of the statistical procedures in \proglang{R} is
briefly explained in Section~\ref{sec:code}.
Section~\ref{sec:Usage} demonstrates how to run
\pkg{systemfit} and how some of the features presented in the second
section can be used.
In Section~\ref{sec:reliability} we replicate several textbook results
with the \pkg{systemfit} package.
Finally, a summary and outlook are presented in
Section~\ref{sec:Summmary}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Statistical background}\label{sec:statistics}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

In this section we give a short overview of the statistical background
that the \pkg{systemfit} package is based on.
More detailed descriptions of simultaneous equations systems
are available for instance in
\citet[Chapter~7]{theil71},
\citet[Part~4]{judge82},
\citet[Part~5]{judge85},
\citet{srivastava87},
\citet[Chapters 14--15]{greene03}, and
\citet[Chapter~10]{zivot06}.

After introducing notations and assumptions,
we provide the formulas to estimate systems of linear equations.
We then demonstrate how to estimate coefficients under linear restrictions.
Finally, we present additional relevant issues about estimation of
equation systems.

Consider a system of $G$ equations, where the $i$th equation is of
the form 
\begin{equation}
   y_{i} = X_i \beta_i + u_i, \quad i = 1, 2, \ldots, G ,
   \label{eq:model}
\end{equation}
where $y_i$ is a vector of the dependent variable,
$X_i$ is a matrix of the exogenous variables,
$\beta_i$ is the coefficient vector and
$u_i$ is a vector of the disturbance terms of the $i$th equation.

We can write the ``stacked'' system as
\begin{equation}
   \left[ \begin{array}{c}
      y_1 \\ y_2\\ \vdots\\ y_G
   \end{array} \right] = 
   \left[ \begin{array}{cccc}
      X_1 & 0 & \cdots & 0\\
      0 & X_2 & \cdots & 0\\
      \vdots & \vdots & \ddots & \vdots\\
      0 & 0 & \cdots & X_G
   \end{array}\right]
   \left[ \begin{array}{c}
      \beta_1 \\ \beta_2 \\ \vdots\\ \beta_G
   \end{array} \right] +
   \left[ \begin{array}{c}
      u_1 \\ u_2 \\ \vdots\\ u_G
   \end{array} \right]
   \label{eq:model-array}
\end{equation}

or more simply as
\begin{equation}
   y = X \beta + u .
   \label{eq:model-matrices}
\end{equation}   

We assume that there is no correlation of the disturbance terms 
across observations, so that
\begin{equation}
   \E \left[ u_{it} \, u_{js} \right] = 0
   \; \forall \; t \neq s ,
\end{equation}
where $i$ and $j$ indicate the equation number 
and $t$ and $s$ denote the observation number,
where the number of observations is the same for all equations.

However, we explicitly allow for contemporaneous correlation, i.e.,
\begin{equation}
   \E \left[ u_{it} \, u_{jt} \right] = \sigma_{ij} .
\end{equation}

Thus, the covariance matrix of all disturbances is
\begin{equation}
   \E \left[ u \, u^\top \right] = \Omega = \Sigma \otimes I_T ,
\end{equation}
where $\Sigma = \left[ \sigma_{ij} \right]$ is the (contemporaneous)
disturbance covariance matrix,
$\otimes$ is the Kronecker product,
$I_T$ is an identity matrix of dimension $T$,
and $T$ is the number of observations in each equation.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Estimation with only exogenous regressors}
\label{sec:Estimation-ols-wls-sur}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

If all regressors are exogenous,
the system of equations (Equation~\ref{eq:model})
can be consistently estimated by ordinary least squares (OLS),
weighted least squares (WLS), and seemingly unrelated regression (SUR).
These estimators can be obtained by
\begin{equation}
   \bHat = \left( X^\top \OHat^{-1} X \right)^{-1} X^\top \OHat^{-1} y .
   \label{eq:ols-wls-sur}
\end{equation}
The covariance matrix of these estimators
can be estimated by
\begin{equation}
   \COVHat \left[ \bHat \right] = \left( X^\top \OHat^{-1} X \right)^{-1} .
   \label{eq:cov-ols-wls-sur}
\end{equation}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Ordinary least squares (OLS)}

The ordinary least squares (OLS) estimator is based on the assumption
that the disturbance terms are not contemporaneously correlated
$(\sigma_{ij} = 0 \; \forall \; i \neq j)$
and have the same variance in each equation
$( \sigma_i^2 = \sigma_j^2 \, \forall \, i, j)$.
In this case, $\OHat$ in Equation~\ref{eq:ols-wls-sur}
is equal to $I_{G \cdot T}$ and thus, cancels out.
The OLS estimator is efficient,
as long as the disturbances are not contemporaneously correlated.

If the whole system is treated as one single equation,
$\OHat$ in Equation~\ref{eq:cov-ols-wls-sur} is $\sHat^2 I_{G \cdot T}$,
where $\sHat^2$ is an estimator for the variance of all disturbances
$(\sigma^2 = \E [ u_{it}^2 ])$.
If the disturbance terms of the individual equations
are allowed to have different variances,
$\OHat$ in Equation~\ref{eq:cov-ols-wls-sur} is $\SHat \otimes I_T$,
where $\sHat_{ij} = 0 \; \forall \; i \neq j$ and
$\sHat_{ii} = \sHat_i^2$ is the estimated variance
of the disturbance term in the $i$th equation.

If the estimated coefficients are not constrained by cross-equation restrictions,
the simultaneous
OLS estimation of the system leads to the same estimated coefficients
as an equation-wise OLS estimation.
The covariance matrix of the coefficients from an equation-wise
OLS estimation is equal to the covariance matrix obtained
by Equation~\ref{eq:cov-ols-wls-sur}
with $\OHat$ equal to $\SHat \otimes I_T$.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Weighted least squares (WLS)}

The weighted least squares (WLS) estimator allows for different variances
of the disturbance terms in the different equations
$( \sigma_i^2 \neq \sigma_j^2 \, \forall \, i \neq j)$,
but assumes
that the disturbance terms are not contemporaneously correlated.
In this case, $\OHat$ in Equations~\ref{eq:ols-wls-sur}
and~\ref{eq:cov-ols-wls-sur}
is $\SHat \otimes I_T$,
where $\sHat_{ij} = 0 \; \forall \; i \neq j$ and
$\sHat_{ii} = \sHat_i^2$ is the estimated variance
of the disturbance terms in the $i$th equation.
Theoretically, $\sHat_{ii}$ should be the variance of
the (true) disturbances $( \sigma_{ii} )$.
However, they are not known in most empirical applications.
Therefore, true variances are generally replaced by estimated variances
$( \sHat_{ii} )$
that are calculated from the residuals of a first-step OLS estimation
(see Section~\ref{sec:residcov}).%
\footnote{%
Note that $\OHat$ in Equation~\ref{eq:ols-wls-sur}
is not the same $\OHat$ as in Equation~\ref{eq:cov-ols-wls-sur}.
The first is calculated from the residuals of a first-step OLS estimation;
the second is calculated from the residuals of this WLS estimation.
The same applies to the SUR, W2SLS, and 3SLS estimations
described in the following sections.
}

The WLS estimator is (asymptotically) efficient only
if the disturbance terms are not contemporaneously correlated.
If the estimated coefficients are not constrained by cross-equation restrictions,
they are equal to OLS estimates.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Seemingly unrelated regression (SUR)}

If the disturbances are contemporaneously correlated, a generalized
least squares (GLS) estimation leads to an efficient estimator for the coefficients.
In this case, the GLS estimator is generally called ``seemingly unrelated regression''
(SUR) estimator \citep{zellner62}.
However, the true covariance matrix of the disturbance terms
is generally unknown.
The textbook solution for this problem is
a feasible generalized least squares (FGLS) estimation.
As the FGLS estimator is based on
an estimated covariance matrix of the disturbance terms,
it is only asymptotically efficient.
In case of a SUR estimator, $\OHat$ in Equations~\ref{eq:ols-wls-sur}
and~\ref{eq:cov-ols-wls-sur}
is $\SHat \otimes I_T$,
where $\SHat$ is the estimated covariance matrix of the disturbance terms.

It should be noted that while an unbiased OLS or WLS estimation requires only that
the regressors and the disturbance terms of each single 
equation are uncorrelated
$( \E \left[ u_i^\top X_i \right] = 0 \; \forall \; i )$,
a consistent SUR estimation requires that all disturbance terms and all 
regressors are uncorrelated
$( \E \left[ u_i^\top X_j \right] = 0 \; \forall \; i, j )$.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Estimation with endogenous regressors}
\label{sec:Estimation-2sls-w2sls-3sls}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

If the regressors of one or more equations are correlated
with the disturbances ($\E \left[ u_i^\top X_i \right] \neq 0$),
OLS, WLS, and SUR estimates are biased.
This can be circumvented by a
two-stage least squares (2SLS),
weighted two-stage least squares (W2SLS),
or a three-stage least squares (3SLS) estimation
with instrumental variables (IV).
The instrumental variables for each equation $Z_i$ 
can be either different or identical for all equations.
They must not be correlated with
the disturbance terms of the corresponding equation 
($\E \left[ u_i^\top Z_i \right] = 0$).

At the first stage new (``fitted'') regressors are obtained by
\begin{equation}
   \XHat_i = Z_i \left( Z_i^\top Z_i \right)^{-1} Z_i^\top X_i .
\end{equation}
Then, these ``fitted'' regressors are substituted for the original regressors
in Equation~\ref{eq:ols-wls-sur}
to obtain unbiased 2SLS, W2SLS, or 3SLS estimates of $\beta$ by
\begin{equation}
   \bHat
   = \left( \XHat^\top \OHat^{-1} \XHat \right)^{-1} \XHat^\top \OHat^{-1} y ,
   \label{eq:2sls-w2sls-3sls}
\end{equation}
where
\begin{equation}
   \XHat =
   \left[ \begin{array}{cccc}
      \XHat_1 & 0       & \cdots & 0\\
      0       & \XHat_2 & \cdots & 0\\
      \vdots  & \vdots  & \ddots & \vdots\\
      0       & 0       & \cdots & \XHat_G
   \end{array}\right] .
\end{equation}
An estimator of the covariance matrix of the estimated coefficients
can be obtained from Equation~\ref{eq:cov-ols-wls-sur} analogously.
Hence, we get
\begin{equation}
   \COVHat \left[ \bHat \right]
   = \left( \XHat^\top \OHat^{-1} \XHat \right)^{-1} .
   \label{eq:cov-2sls-w2sls-3sls}
\end{equation}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Two-stage least squares (2SLS)}

The two-stage least squares (2SLS) estimator
is based on the same assumptions about the disturbance terms as the OLS estimator.
Accordingly, $\OHat$ in Equation~\ref{eq:2sls-w2sls-3sls}
is equal to $I_{G \cdot T}$ and thus, cancels out.
Like for the OLS estimator,
the whole system can be treated either as one single equation
with $\OHat$ in Equation~\ref{eq:cov-2sls-w2sls-3sls}
equal to $\sHat^2 I_{G \cdot T}$,
or the disturbance terms of the individual equations
are allowed to have different variances
with $\OHat$ in Equation~\ref{eq:cov-2sls-w2sls-3sls}
equal to $\SHat \otimes I_T$,
where $\sHat_{ij} = 0 \; \forall \; i \neq j$ and
$\sHat_{ii} = \sHat_i^2$.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Weighted two-stage least squares (W2SLS)}

The weighted two-stage least squares (W2SLS) estimator allows
for different variances of the disturbance terms in the different equations.
Hence, $\OHat$ in Equations~\ref{eq:2sls-w2sls-3sls}
and~\ref{eq:cov-2sls-w2sls-3sls}
is $\SHat \otimes I_T$,
where $\sHat_{ij} = 0 \; \forall \; i \neq j$ and
$\sHat_{ii} = \sHat_i^2$.
If the estimated coefficients are not constrained by cross-equation restrictions,
they are equal to 2SLS estimates.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Three-stage least squares (3SLS)}

If the disturbances are contemporaneously correlated,
a feasible generalized least squares (FGLS) version of the two-stage least squares
estimation leads to consistent and asymptotically more efficient estimates.
This estimation procedure is generally called ``three-stage least
squares'' \citep[3SLS,][]{zellner62b}.
The standard 3SLS estimator and its covariance matrix are obtained by
Equations~\ref{eq:2sls-w2sls-3sls} and~\ref{eq:cov-2sls-w2sls-3sls}
with $\OHat$ equal to $\SHat \otimes I_T$,
where $\SHat$ is the estimated covariance matrix of the disturbance terms.

While an unbiased 2SLS or W2SLS estimation requires only that
the instrumental variables and the disturbance terms of each single 
equation are uncorrelated
$( \E \left[ u_i^\top Z_i \right]) = 0 \; \forall \; i )$,
\cite{schmidt90} points out that the 3SLS estimator is only consistent
if all disturbance terms and all instrumental variables are uncorrelated
$( \E \left[ u_i^\top Z_j \right]) = 0 \; \forall \; i, j )$.
Since there might be occasions where this cannot be avoided,
\cite{schmidt90} analyses other approaches to obtain 3SLS estimators.

One of these approaches based on instrumental variable estimation
(3SLS-IV) is
\begin{equation}
   \bHat_\text{3SLS-IV} = \left( \XHat^\top \OHat^{-1} X
   \right)^{-1} \XHat^\top \OHat^{-1} y .
   \label{eq:3slsIv}
\end{equation}
An estimator of the covariance matrix of the estimated 3SLS-IV coefficients is
\begin{equation}
   \COVHat \left[ \bHat_\text{3SLS-IV} \right] = \left( \XHat^\top \OHat^{-1}
   X \right)^{-1} .
\end{equation}
Another approach based on the generalized method of moments (GMM)
estimator (3SLS-GMM) is
\begin{equation}
   \bHat_\text{3SLS-GMM} = \left( X^\top Z \left( Z^\top \OHat Z \right)^{-1}
   Z^\top X \right)^{-1} X^\top Z \left( Z^\top \OHat Z \right)^{-1} Z^\top y
   \label{eq:3slsGmm}
\end{equation}
with
\begin{equation}
   Z =
   \left[ \begin{array}{cccc}
      Z_1 & 0 & \cdots & 0\\
      0 & Z_2 & \cdots & 0\\
      \vdots & \vdots & \ddots & \vdots\\
      0 & 0 & \cdots & Z_G
   \end{array}\right] .
\end{equation}
An estimator of the covariance matrix of the estimated 3SLS-GMM coefficients is
\begin{equation}
   \COVHat \left[ \bHat_\text{3SLS-GMM} \right] =
   \left( X^\top Z \left( Z^\top \OHat Z \right)^{-1} Z^\top X \right)^{-1} .
\end{equation}
A fourth approach developed by \cite{schmidt90} himself is
\begin{equation}
   \bHat_\text{3SLS-Schmidt} = \left( \XHat^\top \OHat^{-1} \XHat
   \right)^{-1} \XHat^\top \OHat^{-1}
   Z \left( Z^\top Z \right)^{-1} Z^\top y .
   \label{eq:3slsSchmidt}
\end{equation}
An estimator of the covariance matrix of these estimated coefficients is
\begin{align}
   \COVHat \left[ \bHat_\text{3SLS-Schmidt} \right] = &
   \left( \XHat^\top \OHat^{-1}  \XHat \right)^{-1}
   \XHat^\top \OHat^{-1} Z \left( Z^\top Z \right)^{-1} Z^\top \OHat Z \\
   & \left( Z^\top Z \right)^{-1} Z^\top \OHat^{-1} \XHat
   \left( \XHat^\top \OHat^{-1}  \XHat \right)^{-1} .
   \nonumber
\end{align}
The econometrics software \proglang{EViews} uses
\begin{equation}
   \bHat_\text{3SLS-EViews} = \bHat_\text{2SLS} +
   \left( \XHat^\top \OHat^{-1} \XHat \right)^{-1}
   \XHat^\top \OHat^{-1} \left( y - X \bHat_\text{2SLS} \right) ,
   \label{eq:3slsEViews}
\end{equation}
where $\bHat_\text{2SLS}$ is the two-stage least squares estimator
as defined above.
\proglang{EViews} uses the standard 3SLS formula
(Equation~\ref{eq:cov-2sls-w2sls-3sls}) to
calculate an estimator of the covariance matrix of the estimated coefficients.


If the same instrumental variables are used in all equations 
($Z_1 = Z_2 = \ldots = Z_G$),
all the above mentioned approaches lead to identical estimates.
However, if this is not the case, the results depend on the 
method used \citep{schmidt90}.
The only reason to use different instruments for different equations
is a correlation of the instruments of one equation with the
disturbance terms of another equation.
Otherwise, one could simply use all instruments in every equation
\citep{schmidt90}.
In this case, only the 3SLS-GMM (Equation~\ref{eq:3slsGmm})
and the 3SLS estimator developed by \cite{schmidt90} 
(Equation~\ref{eq:3slsSchmidt}) are consistent.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Estimation under linear restrictions on the coefficients}
\label{sec:Restrictions}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

In many empirical applications,
it is desirable to estimate the coefficients under linear restrictions.
For instance, in econometric demand and production analysis,
it is common to estimate the coefficients
under homogeneity and symmetry restrictions
that are derived from the underlying theoretical model.

There are two different methods to estimate the coefficients
under linear restrictions.
First, a matrix $M$ can be specified that
\begin{equation}
   \beta = M \cdot \beta^\text{M} \label{eq:T-restr} ,
\end{equation}
where $\beta^\text{M}$ is a vector of restricted (linear independent) coefficients,
and $M$ is a matrix with the number of rows equal to the number of
unrestricted coefficients ($\beta$) and
the number of columns equal to the number of restricted coefficients
($\beta^\text{M}$).
$M$ can be used to map each unrestricted coefficient to one or more
restricted coefficients.

The second method to estimate the coefficients under linear restrictions
constrains the coefficients by
\begin{equation}
   R \beta^\text{R} = q ,
   \label{eq:restr-R}
\end{equation}
where $\beta^\text{R}$ is the vector of the restricted coefficients,
and $R$ and $q$ are a matrix and vector, respectively,
that specify the restrictions \citep[see][p.~100]{greene03}.
Each linear independent restriction is represented by one row of $R$
and the corresponding element of~$q$.

The first method is less flexible than the second%
\footnote{
While restrictions like $\beta_1 = 2 \beta_2$ can be specified by
both methods,
restrictions like $\beta_1 + \beta_2 = 4$ can be specified only
by the second method.
}, 
but is preferable if the coefficients are estimated
under many equality constraints across different equations of the system.
Of course, these restrictions can be also specified using
the latter method.
However, while the latter method increases the dimension of the 
matrices to be inverted during estimation, the first reduces it. 
Thus, in some cases the latter way leads to estimation problems
(e.g., (near) singularity of the matrices to be inverted),
while the first does not.

These two methods can be combined.
In this case, the restrictions specified using the latter method are
imposed on the linear independent coefficients
that are restricted by the first method, so that
\begin{equation}
   R \beta^\text{MR} = q ,
\end{equation}
where $\beta^\text{MR}$ is the vector of the restricted $\beta^\text{M}$ coefficients.


\subsubsection{Calculation of restricted estimators}

If the first method (Equation~\ref{eq:T-restr}) is chosen
to estimate the coefficients under these restrictions,
the matrix of regressors $X$ is (post-)\hspace{0pt}multiplied
by the $M$ matrix,
so that
\begin{equation}
    X^\text{M} = X \cdot M .
\end{equation}
Then, $X^\text{M}$ is substituted for $X$ and a standard estimation as described
in the previous section is done
(Equations~\ref{eq:ols-wls-sur}--\ref{eq:3slsEViews}).
This results in the linear independent coefficient estimates $\bHat^\text{M}$ and
their covariance matrix.
The original coefficients can be obtained by Equation~\ref{eq:T-restr}
and the estimated covariance matrix of the original coefficients
can be obtained by
\begin{equation}
   \COVHat \left[ \bHat \right]
   = M \cdot \COVHat \left[ \bHat^\text{M} \right] \cdot M^\top .
\end{equation}

The implementation of the second method to estimate the coefficients
under linear restrictions (Equation~\ref{eq:restr-R})
is described for each estimation method in the following sections.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Restricted OLS, WLS, and SUR estimation}

The OLS, WLS, and SUR estimators restricted by $R \beta^\text{R} = q$ can be obtained by
\begin{equation}
   \left[ \begin{array}{c}
      \bHat^\text{R} \\[0.2em] \lHat
   \end{array} \right]
   =
   \left[ \begin{array}{cc}
      X^\top \OHat^{-1} X & R^\top \\
      R & 0
   \end{array} \right]^{-1}
   \cdot
   \left[ \begin{array}{c}
      X^\top \OHat^{-1} y \\ q
   \end{array} \right] ,
   \label{eq:ols-wls-sur-r}
\end{equation}
where $\lambda$ is a vector of the Lagrangean multipliers of the restrictions
and $\OHat$ is defined as in Section~\ref{sec:Estimation-ols-wls-sur}.
An estimator of the covariance matrix of the estimated coefficients is
\begin{equation}
   \COVHat
   \left[ \begin{array}{c}
      \bHat^\text{R} \\[0.2em] \lHat
   \end{array} \right] 
   = 
   \left[ \begin{array}{cc}
      X^\top \OHat^{-1} X & R^\top \\
      R & 0
   \end{array} \right]^{-1} .
   \label{eq:cov-ols-wls-sur-r}
\end{equation}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Restricted 2SLS, W2SLS, and 3SLS estimation}

The 2SLS, W2SLS, and standard 3SLS estimators
restricted by $R \beta^\text{R} = q$ can be obtained by
\begin{equation}
   \left[ \begin{array}{c}
      \bHat^\text{R} \\[0.2em] \lHat
   \end{array} \right]
   =
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} \XHat & R^\top \\
      R & 0
   \end{array} \right]^{-1}
   \cdot
   \left[ \begin{array}{c}
      \XHat^\top \OHat^{-1} y \\ q
   \end{array} \right] ,
   \label{eq:2sls-w2sls-3sls-r}
\end{equation}
where $\OHat$ is defined as in Section~\ref{sec:Estimation-2sls-w2sls-3sls}.
An estimator of the covariance matrix of the estimated coefficients is
\begin{equation}
   \COVHat
   \left[ \begin{array}{c}
      \bHat^\text{R} \\[0.2em] \lHat
   \end{array} \right] 
   = 
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} \XHat & R^\top \\
      R & 0
   \end{array} \right]^{-1} .
   \label{eq:cov-2sls-w2sls-3sls-r}
\end{equation}

The 3SLS-IV estimator restricted by $R \beta^\text{R} = q$ can be obtained by
\begin{equation}
   \left[ \begin{array}{c}
      \bHat^\text{R}_\text{3SLS-IV} \\[0.2em] \lHat
   \end{array} \right]
   =
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} X & R^\top \\
      R & 0
   \end{array} \right]^{-1}
   \cdot
   \left[ \begin{array}{c}
      \XHat^\top \OHat^{-1} y \\ q
   \end{array} \right] ,
   \label{eq:3slsIvR}
\end{equation}
where
\begin{equation}
   \COVHat
   \left[ \begin{array}{c}
      \bHat^\text{R}_\text{3SLS-IV} \\[0.2em] \lHat
   \end{array} \right] 
   = 
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} X & R^\top \\
      R & 0
   \end{array} \right]^{-1} .
\end{equation}
The restricted 3SLS-GMM estimator can be obtained by
\begin{equation}
   \left[ \begin{array}{c}
      \bHat^\text{R}_\text{3SLS-GMM} \\[0.2em] \lHat
   \end{array} \right]
   =
   \left[ \begin{array}{cc}
      X^\top Z \left( Z^\top \OHat Z \right)^{-1} Z^\top X & R^\top \\
      R & 0
   \end{array} \right]^{-1}
   \cdot
   \left[ \begin{array}{c}
      X^\top Z \left( Z \OHat Z \right)^{-1} Z^\top y \\ q
   \end{array} \right] ,
   \label{eq:3slsGmmR}
\end{equation}
where
\begin{equation}
   \COVHat
   \left[ \begin{array}{c}
      \bHat^\text{R}_\text{3SLS-GMM} \\[0.2em] \lHat
   \end{array} \right] 
   = 
   \left[ \begin{array}{cc}
      X^\top Z \left( Z^\top \OHat Z \right)^{-1} Z^\top X & R^\top \\
      R & 0
   \end{array} \right]^{-1} .
\end{equation}
The restricted 3SLS estimator based on the suggestion of
\cite{schmidt90} is
\begin{equation}
   \left[ \begin{array}{c}
      \bHat^\text{R}_\text{3SLS-Schmidt} \\[0.2em] \lHat
   \end{array} \right]
   =
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} \XHat & R^\top \\
      R & 0
   \end{array} \right]^{-1}
   \cdot
   \left[ \begin{array}{c}
      \XHat^\top \OHat^{-1} Z \left( Z^\top Z \right)^{-1} Z^\top y \\ q
   \end{array} \right] ,
   \label{eq:3slsSchmidtR}
\end{equation}
where
\begin{eqnarray}
   \COVHat
   \left[ \begin{array}{c}
      \bHat^\text{R}_\text{3SLS-Schmidt} \\[0.2em] \lHat
   \end{array} \right] 
   & = & 
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} \XHat & R^\top \\
      R & 0
   \end{array} \right]^{-1}
   \\
   & & \cdot
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} Z \left( Z^\top Z \right)^{-1} Z^\top \OHat
      Z \left( Z^\top Z \right)^{-1} Z^\top \OHat^{-1} \XHat & 0^\top \\
      0 & 0
   \end{array} \right]
   \nonumber \\
   & & \cdot
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} \XHat & R^\top \\
      R & 0
   \end{array} \right]^{-1} .
   \nonumber
\end{eqnarray}
The econometrics software \proglang{EViews} calculates the restricted 3SLS estimator by
\begin{equation}
   \left[ \begin{array}{c}
      \bHat^\text{R}_\text{3SLS-EViews} \\[0.2em] \lHat
   \end{array} \right]
   =
   \left[ \begin{array}{cc}
      \XHat^\top \OHat^{-1} \XHat & R^\top \\
      R & 0
   \end{array} \right]^{-1}
   \cdot
   \left[ \begin{array}{c}
      \XHat^\top \OHat^{-1} \left( y - X \bHat^\text{R}_\text{2SLS} \right)
      \\ q 
   \end{array} \right] ,
   \label{eq:3slsEViewsR}
\end{equation}
where $\bHat^\text{R}_\text{2SLS}$ is the restricted 2SLS estimator calculated
by Equation~\ref{eq:2sls-w2sls-3sls-r}.
\proglang{EViews} uses the standard formula of the restricted 3SLS
estimator (Equation~\ref{eq:cov-2sls-w2sls-3sls-r}) to calculate an estimator
for the covariance matrix of the estimated coefficients.


If the same instrumental variables are used in all equations 
($Z_1 = Z_2 = \ldots = Z_G$),
all the above mentioned approaches lead to identical coefficient estimates
and identical covariance matrices of the estimated coefficients.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%\section{Other issues and tools}\label{sec:Other}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Residual covariance matrix}\label{sec:residcov}

Since the (true) disturbances ($u_i$) of the estimated equations
are generally not known,
their covariance matrix cannot be determined.
Therefore, this covariance matrix is generally calculated from
estimated residuals ($\uHat_i$)
that are obtained from a first-step OLS or 2SLS estimation.
Then, in a second step, the estimated residual covariance matrix
can be employed for a WLS, SUR, W2SLS, or 3SLS estimation.
In many cases, the residual covariance matrix is calculated by
\begin{equation}
   \sHat_{ij} = \frac{ \uHat_i^\top \uHat_j }{ T },
   \label{eq:rcovNoDfCor}
\end{equation}
where $T$ is the number of observations in each equation.
However, in finite samples this estimator is biased,
because it is not corrected for degrees of freedom.
The usual single-equation procedure to correct for degrees of freedom
cannot always be applied, because the number of regressors in each equation
might differ.
Two alternative approaches to calculate the residual covariance
matrix are
\begin{equation}
   \sHat_{ij} = \frac{ \uHat_i^\top \uHat_j }
   { \sqrt{ \left( T - K_i \right) \cdot \left( T - K_j \right) } }
   \label{eq:rcovGeomean}
\end{equation}
and
\begin{equation}
   \sHat_{ij} = \frac{ \uHat_i^\top \uHat_j }
   { T - \max \left( K_i , K_j \right) } \; ,
   \label{eq:rcovMax}
\end{equation}
where $K_i$ and $K_j$ are the number of regressors in equation
$i$ and $j$, respectively.
However, these formulas yield unbiased estimators only if $K_i = K_j$
\citep[p.~469]{judge85}.
% Greene (2003, p. 344) says that the second is unbiased if i = j or K_i = K_j,
% whereas the first is unbiased only if i = j. 
% However, if K_i = K_j the first and the second are equal.
% Why is the first biased if K_i = K_j ???????????


A further approach to obtain a residual covariance
matrix is
\begin{eqnarray}
   \sHat_{ij} & = &
   \frac{ \uHat_i^\top \uHat_j }
   { T - K_i - K_j + \tr \left[ X_i \left( X_i^\top X_i \right)^{-1}
   X_i^\top X_j \left( X_j^\top X_j \right)^{-1} X_j^\top \right] }
   \label{eq:rcovTheil} \\
   & = &
   \frac{ \uHat_i^\top \uHat_j }
   { T - K_i - K_j + \tr \left[ \left( X_i^\top X_i \right)^{-1}
   X_i^\top X_j \left( X_j^\top X_j \right)^{-1} X_j^\top X_i \right] }
\end{eqnarray} 
\citep[p.~309]{zellner62c}.
This yields an unbiased estimator for all elements of $\Sigma$,
but even if $\SHat$ is an unbiased estimator of $\Sigma$,
its inverse $\SHat^{-1}$ is not an unbiased estimator
of $\Sigma^{-1}$ \citep[p.~322]{theil71}.
Furthermore, the covariance matrix calculated by Equation~\ref{eq:rcovTheil}
is not necessarily positive semidefinite \citep[p.~322]{theil71}.
Hence, ``it is doubtful whether [this formula] is really superior to
[Equation~\ref{eq:rcovNoDfCor}]'' \citep[p.~322]{theil71}.


The WLS, SUR, W2SLS and 3SLS coefficient estimates are consistent
if the residual covariance matrix is calculated
using the residuals from a first-step OLS or 2SLS estimation.
There exists also an alternative slightly different approach
that consists of three steps.%
\footnote{
For instance, this approach is applied by
the command \code{TSCS} of the software \proglang{LIMDEP} that carries out SUR estimations
in which all coefficient vectors are constrained to be equal
\citep{greene06}.
}
In a first step, an OLS or 2SLS estimation is applied to obtain residuals
to calculate a (first-step) residual covariance matrix.
In a second step, the first-step residual covariance matrix
is used to estimate the model by WLS or W2SLS
and new residuals are obtained to calculate
a (second-step) residual covariance matrix.
Finally, in the third step,
the second-step residual covariance matrix is used
to estimate the model by SUR or 3SLS.
If the estimated coefficients are not constrained by cross-equation restrictions,
OLS and WLS estimates as well as 2SLS and W2SLS estimates are identical.
Hence, in this case both approaches generate the same results.

It is also possible to iterate WLS, SUR, W2SLS and 3SLS estimations.
At each iteration the residual covariance matrix is calculated
from the residuals of the previous iteration.
If Equation~\ref{eq:rcovNoDfCor} is applied to calculate the
residual covariance matrix,
an iterated SUR estimation converges to maximum
likelihood \citep[p.~345]{greene03}.

In some uncommon cases,
for instance in pooled estimations,
where the coefficients are restricted to be equal in all equations,
the means of the residuals of each equation are not equal to zero
$( \overline{ \uHat }_i \neq 0 )$.
Therefore, it might be argued
that the residual covariance matrix should be calculated
by subtracting the means from the residuals
and substituting $\uHat_i - \overline{ \uHat }_i$
for $\uHat_i$ in Equations~\ref{eq:rcovNoDfCor}--\ref{eq:rcovTheil}.

If the coefficients are estimated under any restrictions,
the residual covariance matrix
for a WLS, SUR, W2SLS, or 3SLS estimation
can be obtained either from a restricted or from an unrestricted
first-step estimation.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Degrees of freedom}
\label{sec:degreesOfFreedom}

To our knowledge the question about how to determine the degrees
of freedom for single-coefficient $t$ tests is not comprehensively
discussed in the literature.
While sometimes the degrees of freedom of the entire system
(total number of observations in all equations minus
total number of estimated coefficients)
are applied,
in other cases the degrees of freedom of each single equation
(number of observations in the equations minus
number of estimated coefficients in the equation)
are used.
Asymptotically, this distinction does not make a difference.
However, in many empirical applications, the number of observations
of each equation is rather small, and
therefore, it matters.

If a system of equations is estimated by an unrestricted OLS and
the covariance matrix of the coefficients is calculated
with $\OHat$ in Equation~\ref{eq:cov-ols-wls-sur}
equal to $\SHat \otimes I_T$,
the estimated coefficients and their standard errors are identical
to an equation-wise OLS estimation.
In this case, it is reasonable to use the degrees of freedom of
each single equation,
because this yields the same $P$ values as the equation-wise
OLS estimation.

In contrast, if a system of equations is estimated with many
cross-equation restrictions and
the covariance matrix of an OLS estimation is calculated
with $\OHat$ in Equation~\ref{eq:cov-ols-wls-sur}
equal to $\sHat^2 I_{G \cdot T}$,
the system estimation is similar to a single equation estimation.
Therefore, in this case, it seems to be reasonable to use the degrees
of freedom of the entire system.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Goodness of fit}

The goodness of fit of each single equation can be measured by the
traditional $R^2$ values
\begin{equation}
   R_i^2 = 1 - \frac{ \uHat_i^\top \uHat_i }
   { ( y_i - \overline{y_i} )^\top ( y_i - \overline{y_i} ) } \; ,
\end{equation}
where $R_i^2$ is the $R^2$ value of the $i$th equation
and $\overline{y_i}$ is the mean value of $y_i$.

The goodness of fit of the whole system can be measured by the
McElroy's $R^2$ value 
% also: \citep[p.~345]{greene03}
\begin{equation}
   R_*^2 = 1 - \frac{ \uHat^\top \OHat^{-1} \uHat }
   { y^\top \left( \SHat^{-1} \otimes
   \left( I_T - \frac{\iota \iota^\top}{T} \right) \right) y } ,
\end{equation}
where $\iota$ is a column vector of $T$ ones \citep{mcelroy77}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Testing linear restrictions}
\label{sec:testingRestrictions}

Linear restrictions can be tested by an $F$ test, two Wald tests and
a likelihood ratio (LR) test.

The $F$ statistic for systems of equations is
\begin{equation}
F = \frac{
   ( R \bHat - q )^\top
   ( R ( X^\top ( \Sigma \otimes I )^{-1} X )^{-1} R^\top )^{-1}
   ( R \bHat - q ) /
   j
}{
   \uHat^\top ( \Sigma \otimes I )^{-1} \uHat /
   ( G \cdot T - K )
} \; ,
\label{eq:f-test}
\end{equation}
where $j$ is the number of restrictions,
$K$ is the total number of estimated coefficients,
and all other variables are as defined before
\citep[p.~314]{theil71}.
Under the null hypothesis, $F$ is $F$ distributed
with $j$ and $G \cdot T - K$ degrees of freedom.

However, $F$ in Equation~\ref{eq:f-test} cannot be computed,
because $\Sigma$ is generally unknown.
As a solution, \citet[p.~314]{theil71} proposes
to replace the unknown $\Sigma$ in Equation~\ref{eq:f-test}
by the estimated covariance matrix $\SHat$.
\begin{equation}
\hat{F} = \frac{
   ( R \bHat - q )^\top
   ( R ( X^\top ( \SHat \otimes I )^{-1} X )^{-1} R^\top )^{-1}
   ( R \bHat - q ) /
   j
}{
   \uHat^\top ( \SHat \otimes I )^{-1} \uHat /
   ( G \cdot T - K )
}
\label{eq:f-test-theil}
\end{equation}
Asymptotically, $\hat{F}$ has the same distribution as $F$
in Equation~\ref{eq:f-test},
because the numerator of Equation~\ref{eq:f-test-theil}
converges in probability to the numerator of Equation~\ref{eq:f-test}
and the denominator of Equation~\ref{eq:f-test-theil}
converges in probability to the denominator of Equation~\ref{eq:f-test}
\citep[p.~402]{theil71}.

Furthermore, the denominators of both Equations~\ref{eq:f-test}
and~\ref{eq:f-test-theil} converge in probability to~$1$.
Taking this into account and applying Equation~\ref{eq:cov-ols-wls-sur},
we obtain the usual $F$~statistic of the Wald test.
\begin{equation}
\hat{\hat{F}} = \frac{
   ( R \bHat - q )^\top
   ( R \, \COVHat \left[ \bHat \right] R^\top )^{-1}
   ( R \bHat - q )
}{
   j
}
\label{eq:f-test-wald}
\end{equation}
Under the null hypotheses, also $\hat{\hat{F}}$ is
asymptotically $F$ distributed
with $j$ and $G \cdot T - K$ degrees of freedom.

Multiplying Equation~\ref{eq:f-test-wald} with $j$,
we obtain the usual $\chi^2$ statistic for the Wald test
\begin{equation}
W =
   ( R \bHat - q )^\top
   ( R \, \COVHat [ \bHat ] \, R^\top )^{-1}
   ( R \bHat - q ) .
\label{eq:chi2-test-wald}
\end{equation}
Asymptotically, $W$ has a $\chi^2$
distribution with $j$ degrees of freedom
under the null hypothesis
\citep[p.~347]{greene03}.

The likelihood-ratio (LR) statistic for systems of equations is
\begin{equation}
\LR = T \cdot \left(
   \log \left| \SHat_r \right|
   - \log \left| \SHat_u \right|
   \right) ,
\label{eq:lr-test}
\end{equation}
where $T$ is the number of observations per equation, and 
$\SHat_r$ and $\SHat_u$ are
the residual covariance matrices calculated by Equation~\ref{eq:rcovNoDfCor}
of the restricted and unrestricted estimation, respectively.
Asymptotically, $\LR$ has a $\chi^2$
distribution with $j$ degrees of freedom
under the null hypothesis
\citep[p.~349]{greene03}.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Hausman test}
\label{sec:hausman}

\citet{hausman78} developed a test for misspecification.
The null hypothesis of the test is
that the instrumental variables of each equation are
uncorrelated with the disturbance terms of all other equations
($\E \left[ u_i^\top Z_j \right] = 0 \, \forall \, i \neq j$).
Under this null hypothesis, both the 2SLS and the 3SLS estimator are consistent,
but the 3SLS estimator is (asymptotically) more efficient.
Under the alternative hypothesis, the 2SLS estimator is consistent
but the 3SLS estimator is inconsistent,
i.e., the instrumental variables of each equation are
uncorrelated with the disturbances of the same equation
($\E \left[ u_i^\top Z_i \right] = 0 \, \forall \, i$),
but the instrumental variables of at least one equation
are correlated with the disturbances of another equation
($\E \left[ u_i^\top Z_j \right] \neq 0 \, \exists \, i \neq j$).
The Hausman test statistic is
\begin{equation}
  m = \left( \bHat_\text{2SLS} - \bHat_\text{3SLS} \right)^\top
      \left( \COVHat \left[ \bHat_\text{2SLS} \right] -
             \COVHat \left[ \bHat_\text{3SLS} \right] \right)
      \left( \bHat_\text{2SLS} - \bHat_\text{3SLS} \right) ,
\label{eq:hausman}
\end{equation}
where $\bHat_\text{2SLS}$ and $\COVHat \left[ \bHat_\text{2SLS} \right]$ are the estimated
coefficient and covariance matrix from a 2SLS estimation, and
$\bHat_\text{3SLS}$ and $\COVHat \left[ \bHat_\text{3SLS} \right]$ are the estimated
coefficients and covariance matrix from a 3SLS estimation.
Under the null hypothesis, this test statistic has a
$\chi^2$ distribution with degrees of freedom equal to the number of
estimated coefficients.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Source code}\label{sec:code}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

The source code of the \pkg{systemfit} package is publicly available
for download from the Comprehensive \proglang{R} Archive Network (CRAN,
\url{http://CRAN.R-project.org/}).
The basic functionality of this package is provided by the
function \code{systemfit}.
Moreover, this package provides
tools for statistical tests,
functions (methods) to show, extract or calculate results,
some convenience functions,
and internal helper functions.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection[The basic function systemfit]{The basic function \code{systemfit}}

The \code{systemfit} function estimates systems of linear equations
by different estimation methods.
Where possible,
the user interface and the returned object of this function
follow the function \code{lm}
--- the basic tool for linear regressions in \proglang{R} ---
to make the usage of \code{systemfit} as easy as possible
for \proglang{R} users.

The econometric estimation is done by applying
the formulas in Sections~\ref{sec:Estimation-ols-wls-sur}
and~\ref{sec:Estimation-2sls-w2sls-3sls}
or --- if the coefficients are estimate under linear restrictions ---
by the formulas in Section~\ref{sec:Restrictions}.
If the restrictions on the coefficients are specified symbolically,
function \code{makeHypothesis} of the \pkg{car} package \citep{r-car-1.2-1, fox02a}
is used to create the restriction matrix.

The \code{systemfit} function returns a list of class \code{systemfit}
that contains the results that belong to the entire system of equations.
One special element of this list is called \code{eq},
which is a list that contains one object for each estimated equation.
These objects are lists of class \codeD{systemfit}{equation}
and contain the results that belong only to the regarding equation.
A complete description is available in the documentation of this function
that is included in the package.
A comparison of the elements returned by \code{lm} and by \code{systemfit}
is available in appendix~\ref{sec:returned-object}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Statistical tests}

The \code{linearHypothesis} and
\code{lrtest} methods for \code{systemfit} objects
as well as the function \codeD{hausman}{systemfit}
apply the statistical tests described in
Sections~\ref{sec:testingRestrictions} and~\ref{sec:hausman}.

The \code{linearHypothesis} method for {systemfit} objects
can be used to test linear restrictions on the estimated coefficients
by \citeauthor{theil71}'s $F$ test or by usual Wald tests.
Internally, \citeauthor{theil71}'s $F$ statistic is computed
by the hidden function \codeD{.ftest}{systemfit}
and the Wald tests are computed by
the default \code{linearHypothesis} method of the \pkg{car} package
\citep{r-car-1.2-1, fox02a}.
The \code{lrtest} method for \code{systemfit} objects is a wrapper function
to the default \code{lrtest} method of the \pkg{lmtest} package
\citep{r-lmtest},
which computes the likelihood-ratio (LR) test statistic.
All these functions return an object of class \code{anova}
that contains --- amongst others ---
the empirical test statistic, the degrees of freedom,
and the corresponding $P$ value.

The function \codeD{hausman}{systemfit}
tests the consistency of the 3SLS estimator.
It returns an object of class \code{htest},
which contains --- amongst others ---
the empirical test statistic,
the degrees of freedom, and the $P$ value.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Other methods and functions}

The \pkg{systemfit} package provides several methods
for objects both of classes \code{systemfit}
and \codeD{systemfit}{equation}:
\code{print} methods print the estimation results,
\code{summary} methods calculate summary results,
\code{confint} methods compute confidence intervals for the coefficients,
\code{predict} methods calculate predicted values,
\code{coef} methods extract the estimated coefficients,
\code{vcov} methods extract their covariance matrix,
\code{fitted} methods extract the fitted values,
\code{residuals} methods extract the residuals,
\code{formula} methods extract the formula(s),
\code{terms} methods extract the model terms,
\codeD{model}{frame} methods extract the model frame,
and \codeD{model}{matrix} methods extract the model matrix.
Some methods can be applied to objects of class \code{systemfit} only:
a \code{correlation} method calculates
the correlations between the predictions of two equations,
an \code{se.ratio} method computes the ratios of the standard errors
of the predictions between two models,
and a \code{logLik} method extracts the log likelihood value.
The package provides \code{print} methods
to print objects of classes
\codeD{summary}{systemfit}, \codeDD{summary}{systemfit}{equation},
and \codeD{confint}{systemfit}
that are returned by the above mentioned \code{summary}
and \code{confint} methods.
There exist also two \code{coef} methods
to extract the estimated coefficients,
their standard errors, $t$ values, and $P$ values
from objects of classes
\codeD{summary}{systemfit} and \codeDD{summary}{systemfit}{equation}.%
\footnote{%
There does not exist a special method to extract
the degrees of freedom of the residuals from \code{systemfit} objects,
because the default method of \code{df.residual}
works for these objects.
}

The convenience function
\code{createSystemfitModel} creates a model for \code{systemfit}
by random numbers;
\codeD{systemfit}{control} sets suitable default values
for the technical control parameters for \code{systemfit}.

Finally, the package includes some internal (hidden) helper functions:
\codeD{.prepareData}{systemfit},
\code{.stackMatList}, and \code{.prepareWmatrix}
for preparing the data matrices;
\code{.calcXtOmegaInv} and \code{.calcGLS}
for calculating the GLS estimator;
\code{.calcResidCov} and \code{.calcSigma2}
for calculating the (co)variances of the residuals;
and \codeD{.ftest}{systemfit}
for calculating \citeauthor{theil71}'s $F$ statistic.
If \code{systemfit} is applied to
a (classical) ``seemingly unrelated regression'' analysis with panel data,
it calls the hidden internal function \code{.systemfitPanel},
which reshapes the data, creates the formulas to be estimated,
and --- if requested --- specifies restrictions
to ensure that the coefficients of all individuals are equal.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Efficiency of computations}
\label{sec:code-efficiency}
We have followed \cite{bates04} to make the code of \pkg{systemfit}
faster and more stable.
First, if a formula contains an inverse of a matrix
that is post-multiplied by a vector or matrix,
we use \code{solve(A,b)} instead of \code{solve(A) \%*\% b}.
Second, we calculate crossproducts
by \code{crossprod(X)} or \code{crossprod(X,y)}
instead of \code{t(X) \%*\% X} or \code{t(X) \%*\% y},
respectively.

The matrix $\Omega^{-1}$ that is used to compute the estimated
coefficients and their covariance matrix is of size
$( G \cdot T ) \times ( G \cdot T )$
(see Sections~\ref{sec:Estimation-ols-wls-sur},
\ref{sec:Estimation-2sls-w2sls-3sls}, and~\ref{sec:Restrictions}).
In case of large data sets, $\Omega^{-1}$ becomes computationally infeasible.
Therefore, we use the following transformation and compute $X^\top \Omega^{-1}$
by dividing the $X$ matrix into submatrices,
doing some calculations with these submatrices,
adding up some of these submatrices, and
finally putting the submatrices together, so that
\begin{equation}
X^\top \Omega^{-1}
%= X^\top \left( \Sigma^{-1} \otimes I \right)
= \sum_{i=1} \left[ \begin{array}{c}
   \sigma^{1i} {X^1} \\
   \sigma^{2i} {X^2} \\
   \vdots \\
   \sigma^{Gi} {X^G} \\
\end{array} \right]^\top ,
\label{eq:omegaInv}
\end{equation}
where $\sigma^{ij}$ are the elements of the matrix $\Sigma^{-1}$,
and $X^i$ is a submatrix of $X$ that contains the rows
that belong to the $i$'s equation.
This computation is done inside the internal (hidden) function
\code{.calcXtOmegaInv}.

Since version 1.0, the \code{systemfit} function by default
uses the \pkg{Matrix} package \citep{r-matrix-07}
for all computations where matrices are involved.
The \pkg{Matrix} package provides classes for different types of matrices.
For instance, we choose class \code{dgeMatrix}
(``real matrices in general storage mode''),
for matrices $X_i$ in Equation~\ref{eq:model-array},
class \code{dgCMatrix} (``general, numeric, sparse matrices
in the (sorted) compressed sparse column format'')
for matrix $X$ in Equation~\ref{eq:model-matrices},
and class \code{dspMatrix}
(``symmetric real matrices in packed storage (one triangle only)'')
for the residual covariance matrix $\SHat$.
If the \pkg{Matrix} package is used,
the possibly huge matrix $\Omega^{-1}$ is no longer a problem,
because it is a sparse matrix
that can be stored in a compressed format (class \code{dgCMatrix}).
Hence, we no longer need the algorithm in Equation~\ref{eq:omegaInv}.
We have tested different ways to calculate a GLS estimator
like in Equation~\ref{eq:ols-wls-sur}
and we found
that the following code is the fastest:
<<eval=FALSE>>=
sigmaInv <- solve( residCov )
xtOmegaInv <- crossprod( xMat, kronecker( sigmaInv, Diagonal( nObs ) ) )
coef <- solve( xtOmegaInv %*% xMat, xtOmegaInv %*%  yVec )
@
In this code snippet,
\code{residCov} is the residual covariance matrix $\SHat$,
\code{nObs} is the number of observations in each equation $T$,
\code{xMat} is the matrix $X$ and
\code{yVec} is the vector $y$ in Equation~\ref{eq:ols-wls-sur}.

By default, the \code{systemfit} function uses the \pkg{Matrix} package
to perform GLS estimations,
because using this package considerably decreases
the computation time for many common models.
However, the estimation of small models with small data sets
gets slower by using the \pkg{Matrix} package
(see appendix~\ref{sec:timings}).
While this increase in computation time
is often imperceptible to human beings,
it might matter in some cases
such as iterated estimations or Monte Carlo studies.
Therefore, the user can opt for not using the \pkg{Matrix} package,
but Equation~\ref{eq:omegaInv} with standard \proglang{R} matrices.


\subsection[Overlap with other functions and packages in R]
   {Overlap with other functions and packages in \proglang{R}}

Single-equation models can be fitted in \proglang{R}
by OLS with function \code{lm} (package \pkg{stats})
and by 2SLS with function \code{tsls}
(package \pkg{sem}, \citealp{r-sem-2.0}).
This is also possible with the \code{systemfit} function,
but \code{systemfit} is specialized in estimating systems of equation,
i.e., more than one equation.
Its capability to estimate single-equation models is just a side-effect.

Function \code{sem} (package \pkg{sem}, \citealp{r-sem-2.0})
can be used to estimate structural equation models in \proglang{R}
by limited information maximum likelihood (LIML)
and full information maximum likelihood (FIML)
assuming normal or multinormal errors, respectively.
A special feature of this function is the estimation
of models with unobserved (``latent'') variables,
which is not possible with \code{systemfit}.
While \code{sem} cannot be used to consistently estimate
systems of simultaneous equations with some endogenous regressors,
it can be used to estimate systems of equations,
where all regressors are exogenous.
However, the latter is rather cumbersome
(see appendix~\ref{sec:sem}).
Hence, \code{systemfit} is the only function in \proglang{R}
that can be used to estimate systems of simultaneous equations
and it is the most convenient function to estimate
systems of equations with purely exogenous regressors.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section[Using systemfit]{Using \pkg{systemfit}}\label{sec:Usage}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

In this section we demonstrate how to use the \pkg{systemfit} package.
First, we show the standard usage of \code{systemfit} by a simple example.
Second, several options that can be specified by the user are presented.
Then, the usage of \code{systemfit}
for a (classical) ``seemingly unrelated regression'' analysis
with panel data is described.
Finally, we demonstrate how to apply some statistical tests.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection[Standard usage of systemfit]{Standard usage of \code{systemfit}}
\label{sec:standard-usage}

As described in the previous section,
systems of equations can be econometrically estimated
with the function \code{systemfit}.
The only mandatory argument is \code{formula}.
Typically, it is a list of formulas to be estimated,
but it may also be a single formula for estimating a single-equation model.
Each formula is a standard regression formula in \proglang{R}
(see documentation of \code{formula}).

The following demonstration uses an example taken from
\citet[p.~685]{kmenta86}.
We want to estimate a small model of the US food market:
\begin{align}
\texttt{consump} & = \beta_1 + \beta_2 \cdot \texttt{price}
   + \beta_3 \cdot \texttt{income} \\
\texttt{consump} & = \beta_4 + \beta_5 \cdot \texttt{price}
   + \beta_6 \cdot \texttt{farmPrice} + \beta_7 \cdot \texttt{trend}
\end{align}
The first equation represents the demand side of the food market.
Variable \code{consump} (food consumption per capita) is the dependent variable.
The regressors are \code{price} (ratio of food prices to general consumer prices)
and \code{income} (disposable income) as well as a constant.
The second equation specifies the supply side of the food market.
Variable \code{consump} is the dependent variable of this equation as well.
The regressors are again \code{price} (ratio of food prices to general 
consumer prices) and a constant as well as 
\code{farmPrice} (ratio of preceding year's prices received by farmers
to general consumer prices) and 
\code{trend} (a time trend in years).
These equations can be estimated by OLS in \proglang{R} by
<<message=FALSE>>=
library( "systemfit" )
data( "Kmenta" )
attach( Kmenta )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
eqSystem <- list( demand = eqDemand, supply = eqSupply )
fitols <- systemfit( eqSystem )
print( fitols )
@

The first line loads the \pkg{systemfit} package.
The second line loads example data that are included with the package.
They are attached to the \proglang{R} search path in line three.
In the fourth and fifth line, the demand and supply equations are specified,
respectively.%
\footnote{
A regression constant is always implied if not explicitly omitted.
}
In the sixth line, these equations are concatenated into a list and
are labeled \code{demand} and \code{supply}, respectively.%
\footnote{
If no labels are provided, the equations are numbered consecutively
( \code{eq1}, \code{eq2}, \ldots ).
}
Finally, in the last two lines, the regression is performed
and the estimation results are printed.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection[User options of systemfit]{User options of \code{systemfit}}
\label{sec:user-options}

The user can modify the default estimation method by providing
additional optional arguments,
e.g., to specify instrumental variables
or restrictions on the coefficients.
All optional arguments are described in the following:


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Estimation method}

The optional argument \code{method} is a string
that determines the estimation method.
It must be either \code{"OLS"}, \code{"WLS"}, \code{"SUR"},
\code{"2SLS"}, \code{"W2SLS"}, or \code{"3SLS"}.
These methods correspond to the estimation methods
described in Sections~\ref{sec:Estimation-ols-wls-sur},
\ref{sec:Estimation-2sls-w2sls-3sls}, and~\ref{sec:Restrictions}.
The following command estimates the model described above as
``seemingly unrelated regression''.
<<>>=
fitsur <- systemfit( eqSystem, method = "SUR" )
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Instrumental variables}
The instruments for a 2SLS, W2SLS or 3SLS estimation can be specified
by the argument \code{inst}.
If the same instruments should be used for all equations,
\code{inst} must be a one-sided formula.%
\footnote{
A one-sided formula is a standard formula in \proglang{R}
without a dependent variable.}
If different instruments should be used for each equation,
\code{inst} must be a list that contains a one-sided formula
for each equation.
The following example uses instrumental variables
to estimate the model described above
by ``three-stage least squares'' (3SLS).
While the first command specifies the same instruments for all equations,
the second uses different instruments:
<<>>=
fit3sls  <- systemfit( eqSystem, method = "3SLS",
   inst = ~ income + farmPrice + trend )
fit3sls2 <- systemfit( eqSystem, method = "3SLS",
   inst = list( ~ farmPrice + trend, ~ income + farmPrice + trend ) )
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Data}
Having all data in the global environment or attached to the search path
is often inconvenient.
Therefore, \code{systemfit} has the argument \code{data}
to specify a data frame
that contains the variables of the model.
In the following example, we use this argument to specify
that the data for the estimation
should be taken from the data frame \code{Kmenta}.
Hence, we no longer need to attach this data frame before calling \code{systemfit}:
<<>>=
fitsur <- systemfit( eqSystem, method = "SUR", data = Kmenta )
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Restrictions on the coefficients}
As outlined in Section~\ref{sec:Restrictions}, restrictions on the coefficients
can be specified in two ways.
One way is to use the matrix $R$ and the vector $q$
(see Section~\ref{sec:Restrictions}).
These restrictions can be specified symbolically
by argument \codeD{restrict}{matrix}
as in the generic function
\code{linearHypothesis} of the \pkg{car} package \citep{r-car-1.2-1, fox02a}.
This argument must be a vector of character strings,
where each element represents one linear restriction,
and each element must be either a linear combination of coefficients,
or a linear equation in the coefficients
(see documentation of function \code{linearHypothesis}
in the \pkg{car} package, \citealp{r-car-1.2-1, fox02a}).
We illustrate this by estimating the model under
the restriction $\beta_2 + \beta_6 = 0$.
Since the name of $\beta_2$
(coefficient of variable \code{price} in equation \code{demand})
is \code{demand\_price}
and the name of $\beta_6$
(coefficient of variable \code{farmPrice} in equation \code{supply})
is \code{supply\_farmPrice},
this restriction can be specified by
<<tidy=FALSE>>=
restrict <- "demand_price + supply_farmPrice = 0"
fitsurRmat <- systemfit(eqSystem, method = "SUR",
   restrict.matrix = restrict)
@
\label{code:Rmat}

Alternatively, the restrictions via matrix $R$ and vector $q$
can be specified numerically.
The matrix $R$ can be specified with argument \codeD{restrict}{matrix}
and the vector $q$ with argument \codeD{restrict}{rhs}.
<<tidy=FALSE>>=
Rmat <- matrix(0, nrow = 1, ncol = 7)
Rmat[1, 2] <- 1
Rmat[1, 6] <- 1
qvec <- c(0)
fitsurRmatNum <- systemfit(eqSystem, method = "SUR",
   restrict.matrix = Rmat, restrict.rhs = qvec)
@
The first line creates a $1 \times 7$ matrix of zeros,
where 1 is the number of restrictions and
7 is the number of unrestricted coefficients.
The following two lines specify this matrix in a way
that the multiplication with the coefficient vector
results in $ \beta_2 + \beta_6 $.
The fourth line creates a vector with a single element
that contains the right hand side of the restriction, i.e., zero.
Finally the coefficients are estimated under the restriction
$\beta_2 + \beta_6 = 0$.

The other way to specify restrictions on the coefficients is to modify the regressor matrix
by post-multiplying it with a matrix, say $M$
(see Section~\ref{sec:Restrictions}).
This kind of restriction can be specified
by setting argument \codeD{restrict}{regMat} equal to the matrix $M$.
We convert the restriction specified above to $\beta_2 = - \beta_6$,
and set $\beta_1 = \beta^\text{M}_1$, \ldots, $\beta_5 = \beta^\text{M}_5$,
$\beta_6 = - \beta^\text{M}_2$, and $\beta_7 =  \beta^\text{M}_6$.
We can do this in \proglang{R} by
<<tidy=FALSE>>=
modRegMat <- matrix(0, nrow = 7, ncol = 6)
modRegMat[1:5, 1:5] <- diag(5)
modRegMat[6, 2] <- -1
modRegMat[7, 6] <- 1
fitsurRegMat <- systemfit(eqSystem, method = "SUR",
   restrict.regMat = modRegMat)
@
The first line creates a $7 \times 6$ matrix of zeros,
where 7 is the number of unrestricted coefficients and
6 is the number of restricted coefficients.
The following three lines specify the matrix $M$ (\code{modRegMat})
as described before.
Finally the coefficients are estimated under the restriction
$\beta^\text{M}_2 = \beta_2 = - \beta_6$.

Of course, the estimation results do not depend on the method
that was used to specify this restriction:
<<>>=
all.equal( coef( fitsurRmat ), coef( fitsurRmatNum ) )
all.equal( coef( fitsurRmat ), coef( fitsurRegMat ) )
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Iteration control}
The estimation methods WLS, SUR, W2SLS and 3SLS need a
covariance matrix of the residuals
that can be calculated from a first-step OLS or 2SLS estimation
(see Section~\ref{sec:residcov}).
This procedure can be iterated and at each iteration the covariance
matrix is calculated from the previous step estimation.
This iteration is repeated until the maximum number of iterations
is reached or the coefficient estimates have converged.
The maximum number of iterations is specified by argument \code{maxiter}.
Its default value is one, which means no iteration.
The convergence criterion is
\begin{equation}
   \sqrt{ \frac{ \sum_i (\beta_{i,g} - \beta_{i,g-1})^2 }
   { \sum_i \beta_{i,g-1}^2 }}
      < \texttt{tol} ,
\end{equation}
where $\beta_{i,g}$ is the $i$th coefficient of the $g$th iteration.
The default value of the convergence criterion (argument \code{tol})
is $10^{-5}$.

In the following example,
we estimate the model described above by iterated SUR:
<<>>=
fitsurit <- systemfit( eqSystem, method = "SUR", maxiter = 500 )
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Residual covariance matrix}
It was explained in Section~\ref{sec:residcov} that several different
methods have been proposed to calculate the residual
covariance matrix.
The user can specify, which method \code{systemfit} should use.
Possible values of the argument \code{methodResidCov} are presented in
Table~\ref{tab:methodResidCov}.
By default, \code{systemfit} uses Equation~\ref{eq:rcovGeomean}.

\begin{table}[H]
\centering
\begin{tabular}{|c|c|}
\hline
argument & equation \\
\code{methodResidCov} & \\
\hline
\code{"noDfCor"} & \ref{eq:rcovNoDfCor} \\
\hline
\code{"geomean"} & \ref{eq:rcovGeomean} \\
\hline
\code{"max"}     & \ref{eq:rcovMax} \\
\hline
\code{"Theil"}   & \ref{eq:rcovTheil} \\
\hline
\end{tabular}
\caption{Possible values of argument \code{methodResidCov}}
\label{tab:methodResidCov}
\end{table}

Furthermore, the user can specify
whether the means should be subtracted from the residuals
before Equations~\ref{eq:rcovNoDfCor}, \ref{eq:rcovGeomean}, \ref{eq:rcovMax},
or~\ref{eq:rcovTheil} are applied
to calculate the residual covariance matrix
(see Section~\ref{sec:residcov}).
The corresponding argument is called \code{centerResiduals}.
It must be either \code{TRUE} (subtract the means) or
\code{FALSE} (take the unmodified residuals).
The default value of \code{centerResiduals} is \code{FALSE}.

Moreover, if the coefficients are estimated under restrictions,
the user can use argument \code{residCovRestricted} to specify
whether the residual covariance matrix
for a WLS, SUR, W2SLS, or 3SLS estimation
should be obtained from a restricted or from an unrestricted
first-step estimation
(see Section~\ref{sec:residcov}).
If this argument is \code{TRUE} (the default),
the residual covariance matrix is obtained from a restricted
OLS or 2SLS estimation.
If it is \code{FALSE},
the residual covariance matrix is obtained from an unrestricted
first-step estimation.

Finally, argument \code{residCovWeighted} can be used to decide,
whether the residual covariance matrix for a SUR (3SLS) estimation
should be obtained from a WLS (W2SLS) estimation
instead of from an OLS (2SLS) estimation
(see Section~\ref{sec:residcov}).
By default, \code{residCovWeighted} is \code{FALSE},
which means that the residuals of an OLS (2SLS) estimation are used
to compute the residual covariance matrix.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{3SLS formula}
As discussed in Sections~\ref{sec:Estimation-2sls-w2sls-3sls} and~\ref{sec:Restrictions},
there exist several different
methods to perform a 3SLS estimation.
The user can specify the method by argument \code{method3sls}.
Possible values are presented in Table~\ref{tab:method3sls}.
The default value is \code{"GLS"}.

\begin{table}[H]
\centering
\begin{tabular}{|c|c|c|}
\hline
argument           & equation       & equation \\
\code{method3sls} & (unrestricted) & (restricted) \\
\hline
\code{"GLS"}     & \ref{eq:2sls-w2sls-3sls} & \ref{eq:2sls-w2sls-3sls-r} \\
\hline
\code{"IV"}      & \ref{eq:3slsIv}      & \ref{eq:3slsIvR} \\
\hline
\code{"GMM"}     & \ref{eq:3slsGmm}     & \ref{eq:3slsGmmR} \\
\hline
\code{"Schmidt"} & \ref{eq:3slsSchmidt} & \ref{eq:3slsSchmidtR} \\
\hline
\code{"EViews"}  & \ref{eq:3slsEViews}  & \ref{eq:3slsEViewsR} \\
\hline
\end{tabular}
\caption{Possible values of argument \code{method3sls}}
\label{tab:method3sls}
\end{table}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Sigma squared}
In case of OLS or 2SLS estimations, argument \code{singleEqSigma}
can be used to specify,
whether different $\sigma^2$s for each single equation
or the same $\sigma^2$ for all equations should be used.
If argument \code{singleEqSigma} is \code{TRUE},
$\OHat$ in Equation~\ref{eq:cov-ols-wls-sur} or~\ref{eq:cov-2sls-w2sls-3sls}
is set to $\SHat \otimes I_T$.
In contrast, if argument \code{singleEqSigma} is \code{FALSE},
$\OHat$ in Equation~\ref{eq:cov-ols-wls-sur} or~\ref{eq:cov-2sls-w2sls-3sls}
is set to $\sHat^2 I_{G \cdot T}$.
In case of an unrestricted regression,
argument \code{singleEqSigma} is \code{TRUE} by default.
However, if the coefficients are estimated under restrictions,
this argument is \code{FALSE} by default.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{System options}
Furthermore, two options regarding some internal calculations are available.
First, argument \code{solvetol} specifies the tolerance level
for detecting linear dependencies when inverting a matrix or
calculating a determinant (using functions \code{solve} and \code{det}).
The default value depends on the used computer system and is equal to the
default tolerance level of \code{solve} and \code{det}.

Second, argument \code{useMatrix} specifies
whether the \pkg{Matrix} package \citep{r-matrix-07}
should be used for all computations where matrices are involved
(see Section~\ref{sec:code-efficiency}).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Returned data objects}

Finally, the user can decide
whether \code{systemfit} should return some data objects.
Argument \code{model} indicates
whether a data frame with the data of the model
should be returned.
Its default value is \code{TRUE},
i.e., the model frame is returned.
Arguments \code{x}, \code{y},
and \code{z} specify
whether the model matrices ($X_i$),
the responses ($y_i$), and the matrices of instrumental variables ($Z_i$),
respectively, should be returned.
These three arguments are \code{FALSE} by default,
i.e., these data objects are not returned.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection[Summary results with summary.systemfit]
   {Summary results with \code{summary.systemfit}}

The \code{summary} method can be used
to compute and print summary results of objects
returned by \code{systemfit}.
<<>>=
summary( fitsur )
@
First, the estimation method is reported
and a few summary statistics for the entire system
and for each equation are given.
Then, the covariance matrix used for estimation
and the covariance matrix as well as the correlation matrix
of the (final) residuals are printed.
Finally, the estimation results of each equation are reported:
the formula of the estimated equation,
the estimated coefficients, their standard errors, $t$ values, $P$ values
and codes indicating their statistical significance,
as well as some other statistics like
the standard error of the residuals and
the $R^2$ value of the equation.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection[Degrees of freedom for t tests]
   {Degrees of freedom for $t$ tests}

The \code{summary} method for \code{systemfit} objects has an optional argument \code{useDfSys}.
It selects the approach
that is applied by \code{systemfit}
to determine the degrees of freedom of $t$ tests of the estimated coefficients
(Section~\ref{sec:degreesOfFreedom}).
If argument \code{useDfSys} is \code{TRUE},
the degrees of freedom of the whole system are taken.
In contrast, if \code{useDfSys} is \code{FALSE},
the degrees of freedom of the single equation are taken.
If the coefficients are estimated under restrictions,
argument \code{useDfSys} is \code{TRUE} by default.
However, if no restrictions on the coefficient are specified,
the default value of \code{useDfSys} is \code{FALSE}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsubsection{Reduce amount of printed output}

The optional arguments \code{residCov} and \code{equations}
can be used reduce the amount of the printed output.
Argument \code{residCov} specifies
whether the covariance matrix and
the correlation matrix
of the residuals are printed.
Argument \code{equations} specifies
whether summary results of each equation are printed.
By default, both arguments are \code{TRUE}.
The following command returns a sparse summary output:
<<>>=
summary( fitsur, residCov = FALSE, equations = FALSE )
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Panel data}

The \code{systemfit} function can also be used
for a (classical) ``seemingly unrelated regression'' analysis
with panel data.
For this type of analysis,
the data must be provided in a transformed data frame of class \codeD{pdata}{frame}%
\footnote{
Generally, panel data can be either in ``long format''
(different individuals are arranged below each other)
or in ``wide format''
(different individuals are arranged next to each other).
For this analysis, the data must be in ``long format''.
}
which can be created with the function \codeD{pdata}{frame}
from the \proglang{R} package \pkg{plm}
\citep{r-plm-0.3-1}.
In contrast to the previously described usage of \code{systemfit},
argument \code{formula} must be a single equation (object of class \code{formula}).
This formula is estimated for all individuals.

We demonstrate the application of \code{systemfit}
to panel data
using an example taken from \citet[p.~340]{greene03}
that is based on \citet{grunfeld58}.
We want to estimate a model for gross investment of 5 US firms
in the years 1935--1954:
\begin{equation}
\texttt{invest}_{it} = \beta_1 + \beta_2 \cdot \texttt{value}_{it} +
   \beta_3 \cdot \texttt{capital}_{it}
\end{equation}
where \code{invest} is the gross investment of firm $i$ in year $t$,
\code{value} is the market value of the firm at the end of the
previous year, and
\code{capital} is the capital stock of the firm at the end of the
previous year.

This model can be estimated by
<<message=FALSE,eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
data( "GrunfeldGreene" )
library( "plm" )
GGPanel <- pdata.frame( GrunfeldGreene, c( "firm", "year" ) )
greeneSur <- systemfit( invest ~ value + capital, method = "SUR",
   data = GGPanel )
@
The first line loads the example data set \code{GrunfeldGreene}
that is included in the \pkg{systemfit} package.
The second line loads the \pkg{plm} package
and the following line specifies a data frame of class \codeD{pdata}{frame},
where the variables \code{firm} and \code{year} indicate
the individual (cross-section) and time identifier, respectively.
Finally, a seemingly unrelated regression is performed.

The optional argument \code{pooled} is a logical variable indicating
whether the coefficients are restricted to be equal for all
individuals.
By default, this argument is set to \code{FALSE}.
The following command does a seemingly unrelated regression
of the same model as before,
but with coefficients restricted to be equal for all individuals.
<<eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
greeneSurPooled <- systemfit( invest ~ value + capital, method = "SUR",
   data = GGPanel, pooled = TRUE )
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Testing linear restrictions}

As described in Section~\ref{sec:testingRestrictions},
linear restrictions can be tested by an $F$ test, two Wald tests and
an LR test.
The \pkg{systemfit} package provides
the method \code{linearHypothesis} for the $F$ and Wald tests
as well as the method \code{lrtest} for LR tests.

We will now test the restriction $\beta_2 = -\beta_6$
that was specified by the matrix \code{Rmat} and the vector \code{qvec}
in the example above (p.~\pageref{code:Rmat}).
<<>>=
linearHypothesis( fitsur, Rmat, qvec, test = "FT" )

linearHypothesis( fitsur, Rmat, qvec, test = "F" )

linearHypothesis( fitsur, Rmat, qvec, test = "Chisq" )

lrtest( fitsurRmat, fitsur )
@
The linear restrictions are tested by \citeauthor{theil71}'s $F$ test
(Equation~\ref{eq:f-test-theil}) first,
second by the $F$ statistic of a Wald test
(Equation~\ref{eq:f-test-wald}),
third by the $\chi^2$ statistic of a Wald test
(Equation~\ref{eq:chi2-test-wald}),
and finally by an LR test
(Equation~\ref{eq:lr-test}).

The first argument of the \code{linearHypothesis} method for \code{systemfit} objects
must be an unrestricted regression
returned by \code{systemfit}.
The second and third arguments are the restriction matrix $R$ and
the optional vector $q$, as described in Section~\ref{sec:Restrictions}.
Analogously to the argument \codeD{restrict}{matrix}
of the \code{systemfit} function,
the restrictions can be specified either in matrix form or symbolically.
The optional argument \code{test} must be
a character string, \code{"FT"}, \code{"F"}, or \code{"Chisq"},
specifying whether to compute
\citeauthor{theil71}'s finite-sample $F$ test
(with approximate $F$ distribution)
the finite-sample Wald test (with approximate $F$ distribution)
or the large-sample Wald test (with asymptotic $\chi^2$ distribution).

All arguments of the \code{lrtest} method for \code{systemfit} objects
must be fitted model objects returned by \code{systemfit}.
It consecutively compares all provided fitted model objects.

All tests print a short description of the test
and the tested model objects first.
Then, a small table is printed,
where each row belongs to one (unrestricted or restricted) model.
The second row reports (amongst others)
the degree(s) of freedom of the test,
the empirical test statistic,
and the marginal level of significance ($P$ value).
Although all tests check the same hypothesis,
there is some variation of the $P$ values.
However, all tests suggest the same decision:
The null hypothesis $\beta_2 = -\beta_6$ cannot be rejected at any
reasonable level of significance.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Hausman test}
A Hausman test, which is described in Section~\ref{sec:hausman},
can be carried out with following commands:
<<>>=
fit2sls  <- systemfit( eqSystem, method = "2SLS",
   inst = ~ income + farmPrice + trend, data = Kmenta )
fit3sls <- systemfit( eqSystem, method = "3SLS",
   inst = ~ income + farmPrice + trend, data = Kmenta )
hausman.systemfit( fit2sls, fit3sls )
@
<<echo=FALSE>>=
hausmantest <- hausman.systemfit( fit2sls, fit3sls )
@
First of all, the model is estimated by 2SLS and then by 3SLS.
Finally, in the last line
the test is carried out by the command \codeD{hausman}{systemfit}.
This function requires two arguments:
the result of a 2SLS estimation and
the result of a 3SLS estimation.
The Hausman test statistic is
\Sexpr{round( hausmantest$statistic, digits = 3)},
which has a $\chi^2$ distribution
with \Sexpr{hausmantest$parameter} degrees of freedom
under the null hypothesis.
The corresponding $P$ value is
\Sexpr{round( hausmantest$p.value, digits = 3 )}.
This shows that the null hypothesis is not rejected
at any reasonable level of significance.
Hence, we can assume that the 3SLS estimator is consistent.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Replication of textbook results}\label{sec:reliability}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

In this section, we reproduce results from several textbook examples
using the \pkg{systemfit} package for several reasons.
First, a comparison of \pkg{systemfit}'s results
with results published in the literature
confirms the reliability of the \pkg{systemfit} package.
Second, this section helps teachers and students
become familiar with using the \pkg{systemfit} package.
Third, the section encourages reproducible research,
which should be a general goal in scientific analysis
\citep{buckheit95,schwab00}.
For instance, by preparing this section,
the exact estimation methods of the replicated analyses have been discovered
and a few errors in \citet{greene03} have been found
\citep[see][]{greene06a}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection[Kmenta (1986): Example on p. 685 (food market)]
   {Kmenta (1986): Example on p.~685 (food market)}

First, we reproduce an example taken from \citet[p.~685]{kmenta86}.
The data are available from Table~13-1 (p.~687),
and the results are presented in Table~13-2 (p.~712) of this book.

Before starting the estimation,
we load the data and specify the two formulas of the model
as well as the instrumental variables.
Then the equation system is estimated by OLS, 2SLS, 3SLS, and
iterated 3SLS.
After each estimation, we provide the commands
to print the estimated coefficients.
<<echo=FALSE>>=
library( "systemfit" )
@
<<results='hide'>>=
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )
@
OLS estimation:
<<results='hide'>>=
fitOls <- systemfit( system, data = Kmenta )
round( coef( summary( fitOls ) ), digits = 4 )
@
2SLS estimation:
<<results='hide'>>=
fit2sls <- systemfit( system, method = "2SLS", inst = inst, data = Kmenta )
round( coef( summary( fit2sls ) ), digits = 4 )
@
3SLS estimation:
<<results='hide'>>=
fit3sls <- systemfit( system, method = "3SLS", inst = inst, data = Kmenta )
round( coef( summary( fit3sls ) ), digits = 4 )
@
Iterated 3SLS estimation:
<<results='hide'>>=
fitI3sls <- systemfit( system, method = "3SLS", inst = inst, data = Kmenta,
   maxit = 250 )
round( coef( summary( fitI3sls ) ), digits = 4 )
@

The above commands return exactly the same
coefficients and standard errors as published in \citet[p.~712]{kmenta86}
except for two minor exceptions:
two standard errors of the 2SLS estimation deviate by $0.0001$.
However, this difference is likely due to rounding errors
in \code{systemfit} or \citet{kmenta86} and
is so small that it empirically does not matter.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Greene (2003): Example 15.1 (Klein's model I)}
\label{sec:KleinsModel}

Second, we try to replicate Klein's ``Model I'' \citep{klein50}
that is described in \citet[p.~381]{greene03}.
The data are available from the online complements to \citet{greene03},
Table~F15.1
(\url{http://pages.stern.nyu.edu/~wgreene/Text/tables/TableF15-1.txt}),
and the estimation results are presented in Table~15.3 (p.~412).

Initially, the data are loaded and three equations
as well as the instrumental variables are specified.
As in the example before, the equation system is estimated by
OLS, 2SLS, 3SLS, and iterated 3SLS, and
commands to print the estimated coefficients are presented.
<<>>=
data( "KleinI" )
eqConsump  <- consump ~ corpProf + corpProfLag + wages
eqInvest   <- invest ~ corpProf + corpProfLag + capitalLag
eqPrivWage <- privWage ~ gnp + gnpLag + trend
inst <- ~ govExp + taxes + govWage + trend + capitalLag + corpProfLag + gnpLag
system <- list( Consumption = eqConsump, Investment = eqInvest,
   PrivateWages = eqPrivWage )
@
OLS estimation:
<<results='hide'>>=
kleinOls <- systemfit( system, data = KleinI )
round( coef( summary( kleinOls ) ), digits = 3 )
@
2SLS estimation:
<<results='hide'>>=
klein2sls <- systemfit( system, method = "2SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor" )
round( coef( summary( klein2sls ) ), digits = 3 )
@
3SLS estimation:
<<results='hide'>>=
klein3sls <- systemfit( system, method = "3SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor" )
round( coef( summary( klein3sls ) ), digits = 3 )
@
iterated 3SLS estimation:
<<results='hide'>>=
kleinI3sls <- systemfit( system, method = "3SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor", maxit = 500 )
round( coef( summary( kleinI3sls ) ), digits = 3 )
@

Again, these commands return almost the same results
as published in \citet{greene03}.%
\footnote{
There are two typos in Table~15.3 (p.~412).
Please take a look at the errata
\citep{greene06a}.
}
There are only two minor deviations,
where these values differ merely in the last digit.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Greene (2003): Example 14.1 (Grunfeld's investment data)}
\label{sec:grunfeld-greene}

Third, we reproduce Example~14.1 of \citet[p.~340]{greene03}
that is based on \citet{grunfeld58}.
The data are available from the online complements to \citet{greene03},
Table~F13.1
(\url{http://pages.stern.nyu.edu/~wgreene/Text/tables/TableF13-1.txt}).
Several different versions of the ``Grunfeld'' data set can be found,
whereas the version of \citet{greene03}
is considered incorrect \citep{cummins01}.
However, we use this incorrect version to replicate the results
in \citet{greene03}, Tables~14.1 and~14.2 (p.~351).%
\footnote{
A correct version of this data set with five additional firms
is available as data set \code{Grunfeld} in the \pkg{Ecdat} package
\citep{r-Ecdat-0.1-5}.
}

First, we load the data and the \pkg{plm} package,
indicate the individual (cross-section) and time identifiers,
and specify the formula to be estimated.
Then, the system is estimated by OLS, pooled OLS, SUR, and pooled SUR.
After each estimation, we show the commands to print the estimated coefficients,
the $\sigma^2$ values of the OLS estimations, and
the residual covariance matrix as well as the residual correlation matrix
of the SUR estimations.
<<message=FALSE,eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
data( "GrunfeldGreene" )
library( "plm" )
GGPanel <- pdata.frame( GrunfeldGreene, c( "firm", "year" ) )
formulaGrunfeld <- invest ~ value + capital
@
OLS estimation (Table 14.2):
<<results='hide',eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
greeneOls <- systemfit( formulaGrunfeld,
   data = GGPanel )
round( coef( summary( greeneOls ) ), digits = 4 )
round( sapply( greeneOls$eq, function(x){return(summary(x)$ssr/20)} ), digits = 3 )
@
pooled OLS (Table 14.2):
<<results='hide',eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
greeneOlsPooled <- systemfit( formulaGrunfeld,
   data = GGPanel, pooled = TRUE )
round( coef( summary( greeneOlsPooled$eq[[1]] ) ), digits = 4 ) #$
sum( sapply( greeneOlsPooled$eq, function(x){return(summary(x)$ssr)}) )/100
@
SUR estimation (Table~14.1):
<<results='hide',eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
greeneSur <- systemfit( formulaGrunfeld, method = "SUR",
   data = GGPanel, methodResidCov = "noDfCor" )
round( coef( summary( greeneSur ) ), digits = 4 )
round( greeneSur$residCov, digits = 3 ) #$
round( summary( greeneSur )$residCor, digits = 3 ) #$
@
pooled SUR estimation (Table~14.1):
<<results='hide',eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
greeneSurPooled <- systemfit( formulaGrunfeld, method = "SUR",
   data = GGPanel, pooled = TRUE, methodResidCov = "noDfCor",
   residCovWeighted = TRUE )
round( coef( summary( greeneSurPooled$eq[[1]] ) ), digits = 4 ) #$

round( greeneSurPooled$residCov, digits = 3 ) #$
round( cov( residuals( greeneSurPooled ) ), digits = 3 )
round( summary( greeneSurPooled )$residCor, digits = 3 ) #$
@

These commands return
nearly the same results as published in \citet{greene03}.%
\footnote{
There are several typos and errors in Table~14.1 (p.~412).
Please take a look at the errata of this book
\citep{greene06a}.
}
We present two different commands to print
the residual covariance matrix of the pooled SUR estimation.
The first calculates the covariance matrix without centering the residuals
(see Section~\ref{sec:residcov});
the returned values are equal to those published in \citet[p.~351]{greene03}.
The second command calculates the residual covariance matrix
after centering the residuals;
these returned values are equal to those published in the errata
\citep{greene06a}.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection[Theil (1971): Example on p. 295ff (Grunfeld's investment data)]
   {Theil (1971): Example on p.~295ff (Grunfeld's investment data)}

Finally, we estimate an example taken from \citet[p.~295ff]{theil71}
that is also based on \citet{grunfeld58}.
The data are available in Table~7.1 of \citet[p.~296]{theil71}.
They are a subset of the data set published by \citet{greene03}
(see Section~\ref{sec:grunfeld-greene}).

After extracting the data from the \code{GrunfeldGreene} data set,
the individual (cross-section) and time identifiers are indicated.
Then, the formula is specified,
and the model is estimated by OLS and SUR.
Commands to print the estimated coefficients are reported
after each estimation.
<<eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
GrunfeldTheil <- subset( GrunfeldGreene,
   firm %in% c( "General Electric", "Westinghouse" ) )
GTPanel <- pdata.frame( GrunfeldTheil, c( "firm", "year" ) )
formulaGrunfeld <- invest ~ value + capital
@
OLS estimation (page 295)
<<results='hide',eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
theilOls <- systemfit( formulaGrunfeld,
   data = GTPanel )
round( coef( summary( theilOls ) ), digits = 3 )
@
SUR estimation (page 300)
<<results='hide',eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
theilSur <- systemfit( formulaGrunfeld, method = "SUR",
   data = GTPanel, methodResidCov = "noDfCor" )
round( coef( summary( theilSur ) ), digits = 3 )
@

These commands
return exactly the same results as published in
\citet[pp.~295, 300]{theil71}.

Now, we apply an $F$ test to check whether the slope parameters are equal
for General Electric and Westinghouse (pages~313--315).
Then we re-estimate the model under these restrictions on the coefficients.

$F$ test (page 313--315)%
\footnote{%
The same restriction can be specified also symbolically by
\code{RMatrix <- c("General.Electric\_value = Westinghouse\_value",
"General.Electric\_capital = Westinghouse\_capital")}
}
<<results='hide',eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
RMatrix <- rbind( c( 0, 1, 0, 0, -1, 0 ), c( 0, 0, 1, 0, 0, -1 ) )
linearHypothesis( theilSur, RMatrix )
@

Restricted SUR estimation (page~316)
<<results='hide',tidy=FALSE,eval = requireNamespace("plm", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'plm' package is available
theilSurRestr <- systemfit(formulaGrunfeld, method = "SUR",
   data = GTPanel, methodResidCov = "noDfCor", restrict.matrix = RMatrix,
   residCovRestricted = FALSE)
round(coef(summary(theilSurRestr)), digits = 3)
@

The method \code{linearHypothesis} returns the same value of the $F$ statistic
as published in \citet[p.~315]{theil71}.
Hence, we arrive at the the same conclusion:
we accept the null hypothesis (restrictions on the coefficients are true)
at the 5~percent significance level.

Also the results of the restricted SUR estimation are identical
to the results published in
\citet[p.~316]{theil71}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Summary and outlook}\label{sec:Summmary}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\nopagebreak
In this article, we have described some of the basic features of the
\pkg{systemfit} package for estimating systems of linear
equations.
Many details of the estimation can be controlled by the user.
Furthermore, the package provides some statistical tests
for restrictions on the coefficients and consistency of 3SLS estimation.
It has been tested on a variety of datasets and has produced satisfactory
for a few years.
While the \pkg{systemfit} package performs the basic fitting methods,
more sophisticated tools exist.
We hope to implement missing functionalities
in the near future.
% Some of these are discussed in the following.

\subsubsection*{Unbalanced datasets}
Currently, the \pkg{systemfit} package requires
that all equations have the same number of observations.
However, many data sets have unbalanced observations.%
\footnote{
For instance,
forestry datasets typically contain many observations of inexpensive
variables (stem diameter, tree count) and few expensive variables such
as stem height or volume.
}
Simply dropping data points that do not contain observations for all
equations may reduce the number of observations considerably, and
thus, the information utilized in the estimation.
Hence, it is our intention to include the capability for estimations
with unbalanced data sets as described in \citet{schmidt77} in future
releases of \pkg{systemfit}.

\subsubsection*{Serial correlation and heteroscedasticity}
For all of the methods developed in the package, the disturbances of
the individual equations are assumed to be independent and identically
distributed (iid).
The package could be enhanced by the inclusion of methods to fit
equations with serially correlated and heteroscedastic disturbances
\citep{parks67}. 

\subsubsection*{Estimation methods}
In the future, we wish to include more sophisticated estimation
methods such as limited information maximum likelihood (LIML),
full information maximum likelihood (FIML), generalized methods of
moments (GMM) and spatial econometric methods
\citep{paelinck79,anselin88}.

\subsubsection*{Non-linear estimation}
Finally, the \pkg{systemfit} package provides a function to estimate
systems of non-linear equations.
However, the function \code{nlsystemfit} is currently under
development and the results are not yet always reliable due to
convergence difficulties.


\section*{Acknowledgments}
We thank Achim Zeileis, John Fox, Ott Toomet, William H.\ Greene,
two anonymous referees
and several users of \pkg{systemfit}
for their comments and suggestions
that helped us to improve the \pkg{systemfit} package as well as this paper.
Of course, any remaining errors are the authors'.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\bibliography{systemfit} % a subset of my big bibtex file
%\bibliography{agrarpol}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\clearpage

\begin{appendix}


\section[Object returned by systemfit]{Object returned by \code{systemfit}}
\label{sec:returned-object}

\code{systemfit} returns a list of class \code{systemfit}
that contains the results that belong to the entire system
of equations.
One special element of this list is called \code{eq}.
It is a list that contains one object for each estimated equation.
These objects are of the class \codeD{systemfit}{equation}
and contain the results that belong only to the regarding equation.


\begin{table}[htbp]
\centering
\setlength{\tabcolsep}{4mm}
{\ttfamily
\begin{tabular}{|l|l|l|}
\hline
\textbf{\code{lm}} & \textbf{\code{systemfit}} & \textbf{\code{systemfit.equation}} \\
\hline
coefficients    & coefficients   & coefficients \\
                & coefCov        & coefCov \\
fitted.values   &                & fitted.values \\
residuals       &                & residuals \\
                & residCov       & \\
                & residCovEst    & \\
rank            & rank           & rank \\
                &                & rank.sys \\
                &                & nCoef.sys \\
df.residual     & df.residual    & df.residual \\
                &                & df.residual.sys \\
call            & call           & \\
terms           &                & terms \\
                &                & inst \\
weights         &                & \\
contrasts       &                & \\
xlevels         &                & \\
offset          &                & \\
model\textnormal{*} &            & model\textnormal{*} \\
x\textnormal{**} &               & x\textnormal{**} \\
y\textnormal{**} &               & y\textnormal{**} \\
                &                & z\textnormal{**} \\
                & iter           & \\
                & eq             & \\
                &                & eqnLabel \\
                &                & eqnNo \\
                & method         & method \\
                & panelLike      & \\
                & restrict.matrix& \\
                & restrict.rhs   & \\
                & restrict.regMat& \\
                & control        & \\
\hline
\end{tabular}
}
\caption{Elements returned by \code{systemfit} and \code{lm}
(* if requested by the user with default \code{TRUE},
** if requested by the user with default \code{FALSE}).}
\label{tab:compare-lm}
\end{table}


The elements returned by \code{systemfit}
are similar to those returned by \code{lm},
the basic tool for linear regressions in \proglang{R}.
While some counterparts of elements returned by \code{lm}
can be found directly in objects of class \code{systemfit},
other counterparts are available for each equation
in objects of class \codeD{systemfit}{equation}.
This is demonstrated in Table~\ref{tab:compare-lm}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Computation times}\label{sec:timings}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Theoretically, one would expect
that the calculations with the \pkg{Matrix} package are
faster and more robust than calculations with the traditional method.
To test this hypothesis, we use function \code{createSystemfitModel}
to create a medium-sized multi-equation model with 8~equations,
10~regressors in each equation (without constant),
and 750~observations.
Then, we estimated this model with and without using the \pkg{Matrix} package.
Finally, the results are compared.
<<message=FALSE>>=
library( "systemfit" )
set.seed( 1 )
systemfitModel <- createSystemfitModel( nEq = 8, nReg = 10, nObs = 750 )
system.time(
   fitMatrix <- systemfit( systemfitModel$formula, method = "SUR",
      data = systemfitModel$data )
)
system.time(
   fitTrad <- systemfit( systemfitModel$formula, method = "SUR",
      data = systemfitModel$data, useMatrix = FALSE )
)
all.equal( fitMatrix, fitTrad )
@
The returned computation times clearly show
that using the \pkg{Matrix} package makes the estimation faster.
The comparison of the estimation results shows
that both methods return the same results.
The only differences between the returned objects
are --- as expected --- the \code{call}
and the stored control variable \code{useMatrix}.

However, the estimation of rather small models is much slower
with the \pkg{Matrix} package than without this package.
Moreover, the differences in computation time accumulate,
if the estimation is iterated.
<<>>=
smallModel <- createSystemfitModel( nEq = 3, nReg = 4, nObs = 50 )
system.time(
   fitSmallMatrix <- systemfit( smallModel$formula, method = "SUR",
      data = smallModel$data, maxit = 500 )
)
system.time(
   fitSmallTrad <- systemfit( smallModel$formula, method = "SUR",
      data = smallModel$data, maxit = 500, useMatrix = FALSE )
)
all.equal( fitSmallMatrix, fitSmallTrad )
@
As mentioned above, the usage of the \pkg{Matrix} package
clearly increases the computation times
for iterated (SUR) estimations of small models with small data sets.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section[Estimating systems of equations with sem]
   {Estimating systems of equations with \code{sem}}
\label{sec:sem}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
<<echo=FALSE>>=
options( width = 75 )
@

This section compares the commands to estimate a system of equations
by \code{sem} and \code{systemfit}.
This comparison uses Klein's ``Model I'' (see Section~\ref{sec:KleinsModel}).
Before starting the estimation,
we load the \pkg{sem} and \pkg{systemfit} package
as well as the required data set.
<<message=FALSE,eval = requireNamespace("sem", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'sem' package is available
library( "sem" )
library( "systemfit" )
data( "KleinI" )
@

First, we estimate the system by limited information maximum likelihood
(LIML) with \code{sem}:
<<tidy=FALSE,eval = requireNamespace("sem", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'sem' package is available
limlRam <- matrix(c(
   "consump  <-  corpProf",    "consump_corpProf",    NA,
   "consump  <-  corpProfLag", "consump_corpProfLag", NA,
   "consump  <-  wages",       "consump_wages",       NA,
   "invest   <-  corpProf",    "invest_corpProf",     NA,
   "invest   <-  corpProfLag", "invest_corpProfLag",  NA,
   "invest   <-  capitalLag",  "invest_capitalLag",   NA,
   "privWage <-  gnp",         "privWage_gnp",        NA,
   "privWage <-  gnpLag",      "privWage_gnpLag",     NA,
   "privWage <-  trend",       "privWage_trend",      NA,
   "consump  <-> consump",     "s11", NA,
   "privWage <-> privWage",    "s22", NA,
   "invest   <-> invest",      "s33", NA),
   ncol = 3, byrow = TRUE)
class(limlRam) <- "mod"
exogVar <- c("corpProf", "corpProfLag", "wages", "capitalLag", "trend",
   "gnp", "gnpLag")
endogVar <- c("consump", "invest", "privWage")
allVar <- c(exogVar, endogVar)

limlResult <- sem(model = limlRam, S = cov(KleinI[ -1, allVar ]),
   N = (nrow(KleinI) - 1), fixed.x = exogVar)
print(limlResult)
@

Theoretically, the LIML results should be identical to OLS results.
Therefore, we re-estimate this model by OLS with \code{systemfit}.
<<tidy=FALSE>>=
eqConsump <- consump ~ corpProf + corpProfLag + wages
eqInvest <- invest ~ corpProf + corpProfLag + capitalLag
eqPrivWage <- privWage ~ gnp + gnpLag + trend
system <- list(consump = eqConsump, invest = eqInvest,
   privWage = eqPrivWage)
olsResult <- systemfit(system, data = KleinI)
print(olsResult)
@
As expected, the results are identical.

Now, we estimate the system by full information maximum likelihood
(FIML) with \code{sem}:
<<tidy=FALSE,eval = requireNamespace("sem", quietly = TRUE)>>=
### this code chunk is evaluated only if the 'sem' package is available
fimlRam <- rbind(limlRam,
   c("consump  <-> invest",   "s12", NA),
   c("consump  <-> privWage", "s13", NA),
   c("privWage <-> invest",   "s23", NA))
class(fimlRam) <- "mod"

fimlResult <- sem(model = fimlRam, S = cov(KleinI[ -1, allVar ]),
   N = (nrow(KleinI) - 1), fixed.x = exogVar)
print(fimlResult)
@

Theoretically, results of an iterated SUR estimation should
converge to FIML results.
Therefore, we re-estimate this model by iterated SUR with \code{systemfit}.
<<>>=
surResult <- systemfit( system, method = "SUR", data = KleinI,
   methodResidCov = "noDfCor", maxit = 500 )
print( surResult )
@
As expected, the results are rather similar.


\end{appendix}

\end{document}


%%% Local Variables: 
%%% mode: latex
%%% TeX-master: t
%%% End: 

\name{summary.systemfit}
\alias{summary.systemfit}
\alias{summary.systemfit.equation}
\alias{print.summary.systemfit}
\alias{print.summary.systemfit.equation}
\title{Summary of systemfit estimation}

\description{
   These functions create and print summary results
   of the estimated equation system.
}

\usage{
   \method{summary}{systemfit}( object, useDfSys = NULL,
      residCov = TRUE, equations = TRUE, ... )

   \method{summary}{systemfit.equation}( object, useDfSys = NULL, ... )

   \method{print}{summary.systemfit}( x,
      digits = max( 3, getOption("digits") - 1 ),
      residCov = x$printResidCov, equations = x$printEquations, ... )

   \method{print}{summary.systemfit.equation}( x,
      digits = max( 3, getOption("digits") - 1 ), ... )
}

\arguments{
   \item{object}{an object of class \code{systemfit} or
      \code{systemfit.equation}.}
   \item{x}{an object of class \code{summary.systemfit} or
      \code{summary.systemfit.equation}.}
   \item{useDfSys}{logical. Use the degrees of freedom of the whole system
      (in place of the degrees of freedom of the single equation)
      to calculate prob values for the t-test of individual coefficients.
      If it not specified (\code{NULL}), it is set to \code{TRUE}
      if restrictions on the coefficients are imposed and \code{FALSE} otherwise.}
   \item{digits}{number of digits to print.}
   \item{residCov}{logical. If \code{TRUE},
      the residual correlation matrix, the residual covariance matrix,
      and its determinant are printed.}
   \item{equations}{logical. If \code{TRUE},
      summary results of each equation are printed.
      If \code{FALSE}, just the coefficients are printed.}
   \item{...}{not used by user.}
}

\value{
   Applying \code{summary} on an object of class \code{systemfit}
   returns a list of class \code{summary.systemfit}.
   Applying \code{summary} on an object of class
   \code{systemfit.equation}
   returns a list of class
   \code{summary.systemfit.equation}.
   An object of class \code{summary.systemfit}
   contains all results that belong to the whole system.
   This list contains one special object: \code{eq}.
   This is a list and contains objects of class
   \code{summary.systemfit.equation}.
   These objects contain the results that belong to each of the eatimated equations.

   The objects of classes \code{summary.systemfit} and
   \code{summary.systemfit.equation}
   have the following components
   (elements that are marked with a \eqn{*} are available only in objects of
   class \code{summary.systemfit};
   elements that are marked with a \eqn{+} are available only in objects of
   class \code{summary.systemfit.equation}):

   \item{method}{estimation method.}
   \item{residuals}{residuals.}
   \item{coefficients}{a matrix with columns for the estimated coefficients,
      their standard errors, t-statistic and corresponding (two-sided) p-values.}
   \item{df}{degrees of freedom, a 2-vector, where the first element is the number
      of coefficients and the second element is the number of observations minus
      the number of coefficients.}
   \item{coefCov}{estimated covariance matrix of the coefficients.}
   \item{call*}{the matched call of \code{systemfit}.}
   \item{ols.r.squared*}{OLS \eqn{R^2} value of the entire system.}
   \item{mcelroy.r.squared*}{McElroy's \eqn{R^2} value for the system.}
   \item{iter*}{number of iteration steps (only if the estimation is iterated).}
   \item{control*}{list of control parameters used for the estimation.}
   \item{residCov*}{estimated residual covariance matrix.}
   \item{residCovEst*}{residual covariance matrix used for estimation (only SUR and 3SLS).}
   \item{residCor*}{correlation matrix of the residuals.}
   \item{detResidCov*}{determinant of \code{residCov}.}
   \item{eqnLabel+}{equation label.}
   \item{eqnNo+}{equation number.}
   \item{terms+}{the 'terms' object used for the respective equation.}
   \item{r.squared+}{\eqn{R^2} value of the respective equation.}
   \item{adj.r.squared+}{adjusted \eqn{R^2} value of the respective equation.}
   \item{sigma+}{estimated standard error of the residuals
      of the respective equation.}
   \item{ssr+}{sum of squared residuals of the respective equation.}
   \item{printResidCov*}{argument \code{residCov}.}
   \item{printEquations*}{argument \code{equations}.}
}

\author{Jeff D. Hamann \email{jeff.hamann@forestinformatics.com},\cr
   Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{\code{\link{systemfit}}, \code{\link{print.systemfit}}}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS on each of the equations in the system
fitols <- systemfit( system, data = Kmenta )

## results of the system
summary( fitols )

## short results of the system
summary( fitols, residCov = FALSE, equations = FALSE )

## results of the first equation
summary( fitols$eq[[1]] )

## results of the second equation
summary( fitols$eq[[2]] )
}

\keyword{models}

\name{logLik.systemfit}
\alias{logLik.systemfit}
\title{Log-Likelihood value of systemfit object}

\description{
   This method calculates the log-likelihood value
   of a fitted object returned by \code{\link{systemfit}}.
}

\usage{
   \method{logLik}{systemfit}( object, residCovDiag = FALSE, ... )
}

\arguments{
   \item{object}{an object of class \code{systemfit}.}
   \item{residCovDiag}{logical.
      If this argument is set to \code{TRUE},
      the residual covaraince matrix
      that is used for calculating the log-likelihood value
      is assumed to be diagonal,
      i.e. all covariances are set to zero.
      This may be desirable for models
      estimated by OLS, 2SLS, WLS, and W2SLS.}
   \item{...}{currently not used.}
}

\details{
   The residual covariance matrix
   that is used for calculating the log-likelihood value 
   is calculated based on the actually obtained (final) residuals
   (not correcting for degrees of freedom).
   In case of systems of equations with unequal numbers of observations,
   the calculation of the residual covariance matrix
   is only based on the residuals/observations
   that are available in all equations.
}

\value{
   A numeric scalar (the log-likelihood value) with 2 attributes:
   \code{nobs} (total number of observations in all equations) and
   \code{df} (number of free parameters, i.e. coefficients
   + elements of the residual covariance matrix).
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}}, \code{\link{logLik}}
}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform a SUR estimation
fitsur <- systemfit( system, "SUR", data = Kmenta )

## residuals of all equations
logLik( fitsur )
}

\keyword{models}

\name{linearHypothesis.systemfit}
\alias{linearHypothesis.systemfit}

\title{Test Linear Hypothesis}

\description{
   Testing linear hypothesis on the coefficients of a system of equations
   by an F-test or Wald-test.
}

\usage{
   \method{linearHypothesis}{systemfit}( model,
      hypothesis.matrix, rhs = NULL, test = c( "FT", "F", "Chisq" ),
      vcov. = NULL, ... )
}

\arguments{
   \item{model}{a fitted object of type \code{systemfit}.}
   \item{hypothesis.matrix}{matrix (or vector) giving linear combinations
      of coefficients by rows,
      or a character vector giving the hypothesis in symbolic form
      (see documentation of \code{\link[car]{linearHypothesis}}
      in package "car" for details).}
   \item{rhs}{optional right-hand-side vector for hypothesis,
      with as many entries as rows in the hypothesis matrix;
      if omitted, it defaults to a vector of zeroes.}
   \item{test}{character string, "\code{FT}", "\code{F}", or "\code{Chisq}",
      specifying whether to compute
      Theil's finite-sample F test (with approximate F distribution),
      the finite-sample Wald test (with approximate F distribution),
      or the large-sample Wald test
      (with asymptotic Chi-squared distribution).}
   \item{vcov.}{a function for estimating the covariance matrix
      of the regression coefficients or an estimated covariance matrix
      (function \code{vcov} is used by default).}
   \item{\dots}{further arguments passed to
      \code{\link[car]{linearHypothesis.default}} (package "car").}
}

\details{
   Theil's \eqn{F} statistic for sytems of equations is
   \deqn{F = \frac{
      ( R \hat{b} - q )'
      ( R ( X' ( \Sigma \otimes I )^{-1} X )^{-1} R' )^{-1}
      ( R \hat{b} - q ) /
      j
   }{
      \hat{e}' ( \Sigma \otimes I )^{-1} \hat{e} /
      ( M \cdot T - K )
   } }
   where \eqn{j} is the number of restrictions,
   \eqn{M} is the number of equations,
   \eqn{T} is the number of observations per equation,
   \eqn{K} is the total number of estimated coefficients, and
   \eqn{\Sigma} is the estimated residual covariance matrix.
   Under the null hypothesis, \eqn{F} has an approximate \eqn{F} distribution
   with \eqn{j} and \eqn{M \cdot T - K} degrees of freedom
   (Theil, 1971, p. 314).

   The \eqn{F} statistic for a Wald test is
   \deqn{ F = \frac{
      ( R \hat{b} - q )'
      ( R \, \widehat{Cov} [ \hat{b} ] R' )^{-1}
      ( R \hat{b} - q )
   }{
      j
   } }
   Under the null hypothesis, \eqn{F} has an approximate \eqn{F} distribution
   with \eqn{j} and \eqn{M \cdot T - K} degrees of freedom
   (Greene, 2003, p. 346).

   The \eqn{\chi^2} statistic for a Wald test is
   \deqn{ W =
      ( R \hat{b} - q )'
      ( R \widehat{Cov} [ \hat{b} ] R' )^{-1}
      ( R \hat{b} - q )
   }
   Asymptotically, \eqn{W} has a \eqn{\chi^2}
   distribution with \eqn{j} degrees of freedom
   under the null hypothesis
   (Greene, 2003, p. 347).
}

\value{
   An object of class \code{anova},
   which contains the residual degrees of freedom in the model,
   the difference in degrees of freedom,
   the test statistic (either F or Wald/Chisq)
   and the corresponding p value.
   See documentation of \code{\link[car]{linearHypothesis}}
   in package "car".
}

\references{
  Greene, W. H. (2003)
  \emph{Econometric Analysis, Fifth Edition}, Prentice Hall.

   Theil, Henri (1971)
   \emph{Principles of Econometrics}, John Wiley & Sons, New York.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{\code{\link{systemfit}}, \code{\link[car]{linearHypothesis}}
   (package "car"),
   \code{\link{lrtest.systemfit}}}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## unconstrained SUR estimation
fitsur <- systemfit( system, method = "SUR", data=Kmenta )

# create hypothesis matrix to test whether beta_2 = \beta_6
R1 <- matrix( 0, nrow = 1, ncol = 7 )
R1[ 1, 2 ] <- 1
R1[ 1, 6 ] <- -1
# the same hypothesis in symbolic form
restrict1 <- "demand_price - supply_farmPrice = 0"

## perform Theil's F test
linearHypothesis( fitsur, R1 )  # rejected
linearHypothesis( fitsur, restrict1 )

## perform Wald test with F statistic
linearHypothesis( fitsur, R1, test = "F" )  # rejected
linearHypothesis( fitsur, restrict1 )

## perform Wald-test with chi^2 statistic
linearHypothesis( fitsur, R1, test = "Chisq" )  # rejected
linearHypothesis( fitsur, restrict1, test = "Chisq" )

# create hypothesis matrix to test whether beta_2 = - \beta_6
R2 <- matrix( 0, nrow = 1, ncol = 7 )
R2[ 1, 2 ] <- 1
R2[ 1, 6 ] <- 1
# the same hypothesis in symbolic form
restrict2 <- "demand_price + supply_farmPrice = 0"

## perform Theil's F test
linearHypothesis( fitsur, R2 )  # accepted
linearHypothesis( fitsur, restrict2 )

## perform Wald test with F statistic
linearHypothesis( fitsur, R2, test = "F" )  # accepted
linearHypothesis( fitsur, restrict2 )

## perform Wald-test with chi^2 statistic
linearHypothesis( fitsur, R2, test = "Chisq" )  # accepted
linearHypothesis( fitsur, restrict2, test = "Chisq" )
}

\keyword{models}

%    $Id: print.systemfit.Rd 986 2008-10-16 07:57:24Z arne $

\name{print.systemfit}
\alias{print.systemfit}
\alias{print.systemfit.equation}
\title{Print results of systemfit estimation}

\description{
   These functions print a few results of the estimated equation system.
}

\usage{
   \method{print}{systemfit}( x,
      digits = max( 3, getOption("digits") - 1 ), \dots )

   \method{print}{systemfit.equation}( x,
      digits = max( 3, getOption("digits") - 1 ), ... )
}

\arguments{
   \item{x}{an object of class \code{systemfit} or \code{systemfit.equation}.}
   \item{digits}{number of digits to print.}
   \item{\dots}{other arguments.}
}

\author{Jeff D. Hamann \email{jeff.hamann@forestinformatics.com},\cr
   Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{\code{\link{systemfit}}, \code{\link{summary.systemfit}}}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS on each of the equations in the system
fitols <- systemfit( system, data = Kmenta )

## results of the whole system
print( fitols )

## results of the first equation
print( fitols$eq[[1]] )

## results of the second equation
print( fitols$eq[[2]] )
}

\keyword{models}

%  $Id: lrtest.systemfit.Rd 1055 2010-08-12 20:11:21Z arne $

\name{lrtest.systemfit}
\alias{lrtest.systemfit}

\title{Likelihood Ratio test for Equation Systems}

\description{
   Testing linear hypothesis on the coefficients of a system of equations
   by a Likelihood Ratio test.
}

\usage{
   \method{lrtest}{systemfit}( object, ... )
}

\arguments{
   \item{object}{a fitted model object of class \code{systemfit}.}
   \item{\dots}{further fitted model objects of class \code{systemfit}.}
}

\details{
   \code{lrtest.systemfit} consecutively compares
   the fitted model object \code{object}
   with the models passed in \code{...}.

   The LR-statistic for sytems of equations is
   \deqn{
   LR = T \cdot \left(
      log \left| \hat{ \hat{ \Sigma } }_r \right|
      - log \left| \hat{ \hat{ \Sigma } }_u \right|
      \right)
   }
   where \eqn{T} is the number of observations per equation, and
   \eqn{\hat{\hat{\Sigma}}_r} and \eqn{\hat{\hat{\Sigma}}_u} are
   the residual covariance matrices calculated by formula "0"
   (see \code{\link{systemfit}})
   of the restricted and unrestricted estimation, respectively.
   Asymptotically, \eqn{LR} has a \eqn{\chi^2}
   distribution with \eqn{j} degrees of freedom
   under the null hypothesis
   (Green, 2003, p. 349).
}

\value{
   An object of class \code{anova},
   which contains the log-likelihood value,
   degrees of freedom, the difference in degrees of freedom,
   likelihood ratio Chi-squared statistic and corresponding p value.
   See documentation of \code{\link[lmtest]{lrtest}}
   in package "lmtest".
}

\references{
  Greene, W. H. (2003)
  \emph{Econometric Analysis, Fifth Edition}, Prentice Hall.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{\code{\link{systemfit}}, \code{\link[lmtest]{lrtest}}
   (package "lmtest"),
   \code{\link{linearHypothesis.systemfit}}}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## unconstrained SUR estimation
fitsur <- systemfit( system, "SUR", data = Kmenta )

# create restriction matrix to impose \eqn{beta_2 = \beta_6}
R1 <- matrix( 0, nrow = 1, ncol = 7 )
R1[ 1, 2 ] <- 1
R1[ 1, 6 ] <- -1

## constrained SUR estimation
fitsur1 <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = R1 )

## perform LR-test
lrTest1 <- lrtest( fitsur1, fitsur )
print( lrTest1 )   # rejected

# create restriction matrix to impose \eqn{beta_2 = - \beta_6}
R2 <- matrix( 0, nrow = 1, ncol = 7 )
R2[ 1, 2 ] <- 1
R2[ 1, 6 ] <- 1

## constrained SUR estimation
fitsur2 <- systemfit( system, "SUR", data = Kmenta, restrict.matrix = R2 )

## perform LR-test
lrTest2 <- lrtest( fitsur2, fitsur )
print( lrTest2 )   # accepted
}

\keyword{models}

\name{terms.systemfit}
\alias{terms.systemfit}
\alias{terms.systemfit.equation}
\title{Model Terms of systemfit Objects}

\description{
   This method extracts the model terms
   from fitted objects returned by \code{\link{systemfit}}.
}

\usage{
   \method{terms}{systemfit}( x, ... )
   \method{terms}{systemfit.equation}( x, ... )
}

\arguments{
   \item{x}{an object of class \code{systemfit}.}
   \item{...}{currently not used.}
}

\value{
   \code{terms.systemfit.equation} returns the model terms
   of a single equation of a \code{systemfit} object.
   \code{terms.systemfit.equation} returns a list of model terms:
   one model term object for each equation
   of the \code{systemfit} object.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}}, \code{\link{terms}}
}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform a SUR estimation
fitsur <- systemfit( system, "SUR", data = Kmenta )

## model terms of the second equation
terms( fitsur$eq[[ 2 ]] )

## all model terms of the system
terms( fitsur )
}

\keyword{models}

%    $Id: systemfit.Rd 1171 2022-06-20 07:42:53Z arne $


\name{systemfit}
\alias{systemfit}

\title{Linear Equation System Estimation}

\description{
  Fits a set of linear structural equations using Ordinary Least
  Squares (OLS), Weighted Least Squares (WLS), Seemingly Unrelated Regression (SUR),
  Two-Stage Least Squares (2SLS), Weighted Two-Stage Least Squares (W2SLS)
  or Three-Stage Least Squares (3SLS).
}

\usage{
systemfit( formula, method = "OLS",
           inst=NULL, data=list(),
           restrict.matrix = NULL, restrict.rhs = NULL, restrict.regMat = NULL,
           pooled = FALSE, control = systemfit.control( ... ), ... )
}

\arguments{
   \item{formula}{an object of class \code{formula}
      (for single-equation models)
      or (typically) a list of objects of class \code{formula}
      (for multiple-equation models);
      if argument \code{data} is of class \code{pdata.frame}
      (created with \code{pdata.frame()}),
      this argument must be a single object of class \code{formula}
      that represents the formula to be estimated for all individuals.}
   \item{method}{the estimation method, one of "OLS", "WLS", "SUR",
      "2SLS", "W2SLS", or "3SLS" (see details);
      iterated estimation methods can be specified by setting control parameter
      \code{maxiter} larger than 1 (e.g. 500).}
   \item{inst}{one-sided model formula specifying the instrumental variables
      (including exogenous explanatory variables)
      or a list of one-sided model formulas if different instruments should
      be used for the different equations (only needed for 2SLS, W2SLS, and
      3SLS estimations).}
   \item{data}{an optional data frame
      containing the variables in the model.
      By default the variables are taken from the environment from which
      systemfit is called.}
   \item{restrict.matrix}{an optional j x k matrix to impose linear
      restrictions on the coefficients by \code{restrict.matrix} * \eqn{b} = \code{restrict.rhs}
      (j = number of restrictions, k = number of all coefficients,
      \eqn{b} = vector of all coefficients)
      or a character vector giving the restrictions in symbolic form
      (see documentation of \code{\link[car]{linearHypothesis}}
      in package "car" for details).
      The number and the names of the coefficients can be obtained
      by estimating the system without restrictions and applying
      the \code{coef} method to the returned object.}
   \item{restrict.rhs}{an optional vector with j elements to impose linear
      restrictions (see \code{restrict.matrix}); default is a vector
      that contains j zeros.}
   \item{restrict.regMat}{an optional matrix to impose restrictions
      on the coefficients by post-multiplying the regressor matrix
      with this matrix (see details).}
   \item{control}{list of control parameters.
      The default is constructed by the function \code{\link{systemfit.control}}.
      See the documentation of \code{\link{systemfit.control}} for details.}
   \item{pooled}{logical, restrict coefficients to be equal in all equations
      (only for panel-like data).}
   \item{...}{arguments passed to \code{\link{systemfit.control}}.}
}

\details{
   The estimation of systems of equations with unequal numbers
   of observations has not been thoroughly tested yet.
   Currently, \code{systemfit} calculates the residual covariance matrix
   only from the residuals/observations that are available
   in all equations.

   If argument \code{data} is of class \code{pdata.frame}
   (created with \code{pdata.frame()} and thus, contains panel data in long format),
   argument \code{formula} must be a single equation
   that is applied to all individuals.
   In this case, argument \code{pooled} specifies
   whether the coefficients are restricted to be equal for all
   individuals.

   If argument \code{restrict.regMat} is specified,
   the regressor matrix \eqn{X} is post-multiplied by this matrix:
   \eqn{X^{*} = X \cdot} \code{restrict.regMat}.
   Then, this modified regressor matrix \eqn{X^{*}} is used for the
   estimation of the coefficient vector \eqn{b^{*}}.
   This means that the coefficients of the original regressors (\eqn{X}),
   vector \eqn{b},
   can be represented by \eqn{b =} \code{restrict.regMat} \eqn{\cdot b^{*}}.
   If \code{restrict.regMat} is a non-singular quadratic matrix,
   there are no restrictions on the coefficients imposed,
   but the coefficients \eqn{b^{*}} are linear combinations of
   the original coefficients \eqn{b}.
   If \code{restrict.regMat} has less columns than rows,
   linear restrictions are imposed on the coefficients \eqn{b}.
   However, imposing linear restrictions
   by the \code{restrict.regMat} matrix is less flexible than
   by providing the matrix \code{restrict.matrix} and the
   vector \code{restrict.rhs}.
   The advantage of imposing restrictions on the coefficients
   by the matrix \code{restrict.regMat} is that the matrix,
   which has to be inverted during the estimation,
   gets smaller by this procedure, while it gets larger
   if the restrictions are imposed by \code{restrict.matrix} and \code{restrict.rhs}.

   In the context of multi-equation models, 
   the term \dQuote{weighted} in \dQuote{weighted least squares} (WLS) and
   \dQuote{weighted two-stage least squares} (W2SLS) means
   that the \emph{equations} might have different weights
   and \emph{not}
   that the \emph{observations} have different weights.

   It is important to realize the limitations on estimating the residuals
   covariance matrix imposed by the number of observations \eqn{T}
   in each equation.
   With \eqn{g} equations we estimate \eqn{g*(g+1)/2} elements using
   \eqn{T*g} observations total.
   Beck and Katz (1995,1993) discuss the issue and the resulting overconfidence
   when the ratio of \eqn{T/g} is small (e.g. 3).
   Even for \eqn{T/g=5} the estimate is unstable both numerically
   and statistically
   and the 95% confidence region of the estimate of the variance is
   approximately \eqn{[0.5*\sigma^2, 3*\sigma^2]}{[0.5*s^2, 3*s^2]},
   which is inadequate precision if the covariance matrix will be used
   for simulation of asset return paths either for investment or risk
   management decisions.
   For a starter on models with large cross-sections see Reichlin (2002).
   [This paragraph has been provided by Stephen C. Bond -- Thanks!]
}

\value{
  \code{systemfit} returns a list of the class \code{systemfit} and
  contains all results that belong to the whole system.
  This list contains one special object: "eq". It is a list and contains
  one object for each estimated equation. These objects are of the class
  \code{systemfit.equation} and contain the results that belong only to the
  regarding equation.

  The objects of the class \code{systemfit} and
  \code{systemfit.equation} have the following components (the elements of
  the latter are marked with an asterisk (\eqn{*})):

  \item{call}{the matched call.}
  \item{method}{estimation method.}
  \item{rank}{total number of linear independent coefficients
      = number of coefficients minus number of linear restrictions.}
  \item{df.residual}{degrees of freedom of the whole system.}
  \item{iter}{number of iteration steps.}
  \item{coefficients}{vector of all estimated coefficients.}
  \item{coefCov}{estimated covariance matrix of \code{coefficients}.}
  \item{residCov}{estimated residual covariance matrix.}
  \item{residCovEst}{residual covariance matrix used for estimation
      (only WLS, W2SLS, SUR and 3SLS).}
  \item{restrict.matrix}{the restriction matrix.}
  \item{restrict.rhs}{the restriction vector.}
  \item{restrict.regMat}{matrix used to impose restrictions
      on the coefficients by post-multiplying the regressor matrix
      with this matrix.}
  \item{control}{list of control parameters used for the estimation.}
  \item{panelLike}{logical. Was this an analysis with panel-like data?}

  ## elements of the class systemfit.eq
  \item{eq}{a list that contains the results that belong to the individual equations.}
  \item{eqnLabel*}{the label of this equation.}
  \item{eqnNo*}{the number of this equation.}
  \item{terms*}{the 'terms' object used for the ith equation.}
  \item{inst*}{instruments of the ith equation (only 2SLS, W2SLS, and 3SLS).}
  \item{termsInst*}{the 'terms' object of the instruments 
     used for the ith equation (only 2SLS, W2SLS, and 3SLS).}
  \item{rank*}{number of linear independent coefficients in the ith equation
     (differs from the number of coefficients only if there are
     restrictions that are not cross-equation).}
  \item{nCoef.sys*}{total number of coefficients in all equations.}
  \item{rank.sys*}{total number of linear independent coefficients
      in all equations.}
  \item{df.residual*}{degrees of freedom of the ith equation.}
  \item{df.residual.sys*}{degrees of freedom of the whole system.}
  \item{coefficients*}{estimated coefficients of the ith equation.}
  \item{covb*}{estimated covariance matrix of \code{coefficients}.}
  \item{model*}{if requested (the default), the model frame of the ith equation.}
  \item{modelInst*}{if requested (the default), the model frame 
      of the instrumental variables of the ith equation 
      (only 2SLS, W2SLS, and 3SLS).}
  \item{x*}{if requested, the model matrix of the ith equation.}
  \item{y*}{if requested, the response of the ith equation.}
  \item{z*}{if requested, the matrix of instrumental variables
      of the ith equation (only 2SLS, W2SLS, and 3SLS).}
  \item{fitted.values*}{vector of fitted values of the ith equation.}
  \item{residuals*}{vector of residuals of the ith equation.}
}

\references{
   Beck, N.; J.N. Katz (1995)
   What to do (and not to do) with Time-Series Cross-Section Data,
   \emph{The American Political Science Review}, 89, pp. 634-647.

   Beck, N.; J.N. Katz; M.R. Alvarez; G. Garrett; P. Lange (1993)
   Government Partisanship, Labor Organization, and Macroeconomic Performance:
   a Corrigendum, \emph{American Political Science Review}, 87,
   pp. 945-48.

  Greene, W. H. (2003)
  \emph{Econometric Analysis, Fifth Edition}, Prentice Hall.

  Judge, George G.; W. E. Griffiths; R. Carter Hill; Helmut Luetkepohl and Tsoung-Chao Lee (1985)
  \emph{The Theory and Practice of Econometrics, Second Edition}, Wiley.

  Kmenta, J. (1997)
  \emph{Elements of Econometrics, Second Edition}, University of
  Michigan Publishing.

   Reichlin, L. (2002)
   \emph{Factor models in large cross-sections of time series},
   Working Paper, ECARES and CEPR.

  Schmidt, P. (1990)
  \emph{Three-Stage Least Squares with different Instruments for different equations},
  Journal of Econometrics 43, p. 389-394.

  Theil, H. (1971)
  \emph{Principles of Econometrics}, Wiley, New York.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com},\cr
  Jeff D. Hamann \email{jeff.hamann@forestinformatics.com}
}

\seealso{\code{\link{lm}} and \code{\link{nlsystemfit}}}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## OLS estimation
fitols <- systemfit( system, data=Kmenta )
print( fitols )

## OLS estimation with 2 restrictions
Rrestr <- matrix(0,2,7)
Rrestr[1,3] <-  1
Rrestr[1,7] <- -1
Rrestr[2,2] <- -1
Rrestr[2,5] <-  1
qrestr <- c( 0, 0.5 )
fitols2 <- systemfit( system, data = Kmenta,
                      restrict.matrix = Rrestr, restrict.rhs = qrestr )
print( fitols2 )

## OLS estimation with the same 2 restrictions in symbolic form
restrict <- c( "demand_income - supply_trend = 0",
   "- demand_price + supply_price = 0.5" )
fitols2b <- systemfit( system, data = Kmenta, restrict.matrix = restrict )
print( fitols2b )

# test whether both restricted estimators are identical
all.equal( coef( fitols2 ), coef( fitols2b ) )

## OLS with restrictions on the coefficients by modifying the regressor matrix
## with argument restrict.regMat
modReg <- matrix( 0, 7, 6 )
colnames( modReg ) <- c( "demIntercept", "demPrice", "demIncome",
   "supIntercept", "supPrice2", "supTrend" )
modReg[ 1, "demIntercept" ] <- 1
modReg[ 2, "demPrice" ]     <- 1
modReg[ 3, "demIncome" ]    <- 1
modReg[ 4, "supIntercept" ] <- 1
modReg[ 5, "supPrice2" ]    <- 1
modReg[ 6, "supPrice2" ]    <- 1
modReg[ 7, "supTrend" ]     <- 1
fitols3 <- systemfit( system, data = Kmenta, restrict.regMat = modReg )
print( fitols3 )


## iterated SUR estimation
fitsur <- systemfit( system, "SUR", data = Kmenta, maxit = 100 )
print( fitsur )

## 2SLS estimation
inst <- ~ income + farmPrice + trend
fit2sls <- systemfit( system, "2SLS", inst = inst, data = Kmenta )
print( fit2sls )

## 2SLS estimation with different instruments in each equation
inst1 <- ~ income + farmPrice
inst2 <- ~ income + farmPrice + trend
instlist <- list( inst1, inst2 )
fit2sls2 <- systemfit( system, "2SLS", inst = instlist, data = Kmenta )
print( fit2sls2 )

## 3SLS estimation with GMM-3SLS formula
inst <- ~ income + farmPrice + trend
fit3sls <- systemfit( system, "3SLS", inst = inst, data = Kmenta,
   method3sls = "GMM" )
print( fit3sls )


## Examples how to use systemfit() with panel-like data
## Repeating the SUR estimations in Greene (2003, p. 351)
data( "GrunfeldGreene" )
if( requireNamespace( 'plm', quietly = TRUE ) ) {
library( "plm" )
GGPanel <- pdata.frame( GrunfeldGreene, c( "firm", "year" ) )
formulaGrunfeld <- invest ~ value + capital
# SUR
greeneSur <- systemfit( formulaGrunfeld, "SUR",
   data = GGPanel, methodResidCov = "noDfCor" )
summary( greeneSur )
# SUR Pooled
greeneSurPooled <- systemfit( formulaGrunfeld, "SUR",
   data = GGPanel, pooled = TRUE, methodResidCov = "noDfCor",
   residCovWeighted = TRUE )
summary( greeneSurPooled )
}

## Further examples are in the documentation to the data sets
## 'KleinI' and 'GrunfeldGreene'.
}

\keyword{models}
\keyword{regression}

% 	$Id: ppine.Rd 486 2007-10-10 07:58:53Z henningsena $	


\name{ppine}
\alias{ppine}
\docType{data}
\title{Tree Growth Data for Ponderosa Pine}
\usage{data(ppine)}
\description{
A subset of tree growth observations from a Ponderosa pine growth database.

The \code{ppine} data frame has 166 rows and 8 columns.
}

\format{
  This data frame contains the following columns:
  \describe{
    \item{elev}{
      Altitude of the plot, in feet above mean sea level.
    }
    \item{smi}{
      Summer moisture index is the ratio of growing season heating
      degree days to growing season precipitation.
    }
    \item{dbh}{
      Diameter of the tree at breast height (4.5 feet).
    }
    \item{tht}{
      Total stem height for the tree.
    }
    \item{cr}{
      Crown ratio code. The scale is from 1 to 9 where a crown class of
      one represents a crown ratio between 0 and 15 percent. A crown
      ratio code of 2 represents a crown ratio value between 16 and
      25\%,...,8=76-85\%, 9 >=85\%.
    }
    \item{ba}{
      Plot basal area at the beginning of the growth period.
    }
    \item{dg}{
      Five-year diameter increment.
    }
    \item{hg}{
      Five-year height increment.
    }
  }
}

\details{
 The exogenous variables are \code{elev}, \code{smi}, \code{dbh},
 \code{tht}, \code{cr}, and \code{ba}; the endogenous variables
 \code{dg} and \code{hg}. There are no lagged  variables in the dataset
 and the observations are for a single remeasurement.

 The data was provided by the USDA Forest Service Intermountain
 Research Station.

 
%  The data was provided by the USDA Forest Service Intermountain
%  Research Station from installations on USDA Forest Service, Spokane Indian
%  Reservation and the Nez Perce Indian Reservation lands.
}

\source{
  William R. Wykoff \email{wwykoff@fs.fed.us} \emph{Rocky Mountain Research Station, 1221 South Main Street, Moscow, ID 83843}
}


\examples{
data(ppine)
}

\keyword{datasets}

\name{createSystemfitModel}
\alias{createSystemfitModel}
\title{Create a Model for systemfit}

\description{
   This function creates a model that can be estimated by \code{systemfit}.
   The data, disturbances, and --- if not provided by the user ---
   the coefficients as well as the disturbance covariance matrix
   are generated by random numbers.
}

\usage{
createSystemfitModel( nEq, nRegEq, nObs, coef = NULL, sigma = NULL )
}

\arguments{
   \item{nEq}{the number of equations.}
   \item{nRegEq}{the number of regressors in each equation
      (without the intercept).}
   \item{nObs}{the number of observations.}
   \item{coef}{an optional vector of coefficients.}
   \item{sigma}{an optional covariance matrix of the disturbance terms.}
}

\value{
   \code{createSystemfitModel} returns a list with following elements:

   \item{formula}{a list of the model equations
      (objects of class \code{formula}).}
   \item{data}{a \code{data.frame} that contains the data.}
   \item{coef}{a vector of (true) coefficients.}
   \item{sigma}{the covariance matrix of the disturbance terms.}
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{\code{\link{systemfit}}}


\examples{
## create a model by random numbers
systemfitModel <- createSystemfitModel( 3, 4, 100 )

## estimate this model by "SUR"
fitsur <- systemfit( systemfitModel$formula, "SUR", data = systemfitModel$data )

## compare the "true" and the estimated coefficients
cbind( systemfitModel$coef, coef( fitsur ) )
}

\keyword{models}

\name{model.matrix.systemfit}
\alias{model.matrix.systemfit}
\alias{model.matrix.systemfit.equation}
\title{Construct Design Matrices for Systems of Equations}

\description{
   These functions create design matrices from objects
   returned by \code{\link{systemfit}}.
}

\usage{
   \method{model.matrix}{systemfit}( object, which = "x", \dots )

   \method{model.matrix}{systemfit.equation}( object, which = "x", \dots )
}

\arguments{
   \item{object}{an object of class \code{systemfit} or
      \code{systemfit.equation}.}
   \item{which}{character string: 
      \code{"x"} indicates the usual model matrix of the regressors,
      \code{"xHat"} indicates the model matrix of the fitted regressors,
      \code{"z"} indicates the matrix of instrumental variables.}
   \item{\dots}{currently ignored.}
}

\value{
   \code{model.matrix.systemfit} returns a design matrix to estimate
      the specified system of equations.

   \code{model.matrix.systemfit.equation} returns a design matrix to estimate
      the specified formula of the respective equation.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}}, \code{\link{model.matrix}}, and
   \code{\link{model.frame.systemfit}}
}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS of the system
fitols <- systemfit( system, data = Kmenta )

## design matrix of the entire system
model.matrix( fitols )

## design matrix of the first equation
model.matrix( fitols$eq[[ 1 ]] )
}

\keyword{models}

\name{GrunfeldGreene}
\alias{GrunfeldGreene}
\docType{data}
\title{Grunfeld Data as published by Greene (2003)}
\description{
   Panel data on 5 US firms for the years 1935-1954.
}

\usage{
   data("GrunfeldGreene")
}
\format{
A data frame containing 20 annual observations on 3 variables for 5 firms.
\describe{
   \item{invest}{gross investment.}
   \item{value}{market value of the firm (at the end of the previous year).}
   \item{capital}{capital stock of the firm (at the end of the previous year).}
   \item{firm}{name of the firm ("General Motors", "Chrysler",
      "General Electric", "Westinghouse" or "US Steel").}
   \item{year}{year.}
}
}

\details{
There exist several different versions of this data set,
and this version is considered incorrect
(see \url{https://web.archive.org/web/20170426034143/http://web.stanford.edu/~clint/bench/grunfeld.htm} for details).
However, we provide this incorrect version to replicate the results
published in Theil (1971) and Greene (2003).
A correct version of this data set with 5 additional firms
is available in the \code{Ecdat} package
(data set \code{Grunfeld}).
}

\source{
   Greene (2003), Appendix F, Data Sets Used in Applications, Table F13.1.
   \url{https://pages.stern.nyu.edu/~wgreene/Text/econometricanalysis.htm}
   (a subset of this data set is available in Theil (1971), p. 296).
}

\references{
Greene, W.H. (2003). \emph{Econometric Analysis}, 5th edition. Prentice Hall,
Upper Saddle River (NJ).

Grunfeld, Y. (1958). \emph{The Determinants of Corporate Investment},
Unpublished Ph.D. Dissertation, University of Chicago.

Theil, Henri (1971). \emph{Principles of Econometrics},
   John Wiley & Sons, New York.
}

\examples{
## Repeating the OLS and SUR estimations in Greene (2003, pp. 351)
data( "GrunfeldGreene" )
if( requireNamespace( 'plm', quietly = TRUE ) ) {
library( "plm" )
GGPanel <- pdata.frame( GrunfeldGreene, c( "firm", "year" ) )
formulaGrunfeld <- invest ~ value + capital
# OLS
greeneOls <- systemfit( formulaGrunfeld, "OLS",
   data = GGPanel )
summary( greeneOls )
sapply( greeneOls$eq, function(x){return(summary(x)$ssr/20)} ) # sigma^2
# OLS Pooled
greeneOlsPooled <- systemfit( formulaGrunfeld, "OLS",
   data = GGPanel, pooled = TRUE )
summary( greeneOlsPooled )
sum( sapply( greeneOlsPooled$eq, function(x){return(summary(x)$ssr)}) )/97 # sigma^2
# SUR
greeneSur <- systemfit( formulaGrunfeld, "SUR",
   data = GGPanel, methodResidCov = "noDfCor" )
summary( greeneSur )
# SUR Pooled
greeneSurPooled <- systemfit( formulaGrunfeld, "SUR",
   data = GGPanel, pooled = TRUE, methodResidCov = "noDfCor",
   residCovWeighted = TRUE )
summary( greeneSurPooled )

## Repeating the OLS and SUR estimations in Theil (1971, pp. 295, 300)
GrunfeldTheil <- subset( GrunfeldGreene,
   firm \%in\% c( "General Electric", "Westinghouse" ) )
GTPanel <- pdata.frame( GrunfeldTheil, c( "firm", "year" ) )
formulaGrunfeld <- invest ~ value + capital
# OLS
theilOls <- systemfit( formulaGrunfeld, "OLS",
   data = GTPanel )
summary( theilOls )
# SUR
theilSur <- systemfit( formulaGrunfeld, "SUR",
   data = GTPanel, methodResidCov = "noDfCor" )
summary( theilSur )
}
}

\keyword{datasets}

\name{KleinI}
\alias{KleinI}
\docType{data}
\title{Klein Model I}
\description{
   Data for Klein's (1950) Model I of the US economy.
}

\usage{data("KleinI")}

\format{
A data frame containing annual observations from 1920 to 1941
   \describe{
      \item{year}{Year.}
      \item{consump}{Consumption.}
      \item{corpProf}{Corporate profits.}
      \item{corpProfLag}{Corporate profits of the previous year.}
      \item{privWage}{Private wage bill.}
      \item{invest}{Investment.}
      \item{capitalLag}{Capital stock of the previous year.}
      \item{gnp}{Gross national product.}
      \item{gnpLag}{Gross national product of the previous year.}
      \item{govWage}{Government wage bill.}
      \item{govExp}{Government spending.}
      \item{taxes}{Taxes.}
      \item{wages}{Sum of private and government wage bill.}
      \item{trend}{time trend measured as years from 1931.}
   }
}

\source{
   Greene (2003), Appendix F, Data Sets Used in Applications, Table F15.1.

   \url{https://pages.stern.nyu.edu/~wgreene/Text/econometricanalysis.htm}
}

\references{
Greene, W.H. (2003). \emph{Econometric Analysis}, 5th edition. Prentice Hall,
Upper Saddle River (NJ).

Klein, L. (1950). \emph{Economic Fluctuations in the United States, 1921--1941}.
John Wiley, New York.
}

\examples{
## Repeating the estimations of Klein's (1950) Model I
## in Greene (2003, pp. 381 and 412)
data( "KleinI" )
eqConsump  <- consump ~ corpProf + corpProfLag + wages
eqInvest   <- invest ~ corpProf + corpProfLag + capitalLag
eqPrivWage <- privWage ~ gnp + gnpLag + trend
inst <- ~ govExp + taxes + govWage + trend + capitalLag + corpProfLag + gnpLag
system <- list( Consumption = eqConsump, Investment = eqInvest,
   PrivateWages = eqPrivWage )
# OLS
kleinOls <- systemfit( system, data = KleinI )
summary( kleinOls )
# 2SLS
klein2sls <- systemfit( system, "2SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor" )
summary( klein2sls )
# 3SLS
klein3sls <- systemfit( system, "3SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor" )
summary( klein3sls )
# I3SLS
kleinI3sls <- systemfit( system, "3SLS", inst = inst, data = KleinI,
   methodResidCov = "noDfCor", maxit = 500 )
summary( kleinI3sls )
}
\keyword{datasets}

\name{confint.systemfit}
\alias{confint.systemfit}
\alias{confint.systemfit.equation}
\title{Confidence intervals of coefficients}

\description{
   These functions calculate the confidence intervals of the
   coefficients from an object returned by \code{\link{systemfit}}.
}

\usage{
   \method{confint}{systemfit}( object, parm = NULL, level = 0.95,
      useDfSys = NULL, \dots )

   \method{confint}{systemfit.equation}( object, parm, level = 0.95,
      useDfSys = NULL, \dots )
}

\arguments{
   \item{object}{an object of class \code{systemfit} or \code{systemfit.equation}.}
   \item{parm}{not used yet.}
   \item{level}{confidence level.}
   \item{useDfSys}{logical. Use the degrees of freedom of the whole system
      (in place of the degrees of freedom of the single equation)
      to calculate the confidence intervals of the coefficients.
      If it not specified (\code{NULL}), it is set to \code{TRUE}
      if restrictions on the coefficients are imposed and \code{FALSE} otherwise.}
   \item{\dots}{other arguments.}
}

\value{
   An object of class \code{confint.systemfit}, which is a matrix
   with columns giving lower and upper confidence limits
   for each estimated coefficient. These will be labelled as
   (1-level)/2 and 1 - (1-level)/2 in \% (by default 2.5\% and 97.5\%).
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}},
   \code{\link{print.confint.systemfit}}, \code{\link{confint}}
}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS on each of the equations in the system
fitols <- systemfit( system, data = Kmenta )

## confidence intervals of all coefficients
confint( fitols )

## confidence intervals of the coefficients of the first equation
confint( fitols$eq[[1]] )

## confidence intervals of the coefficients of the second equation
confint( fitols$eq[[2]] )
}

\keyword{models}
\keyword{regression}

%  $Id: Kmenta.Rd 486 2007-10-10 07:58:53Z henningsena $

\name{Kmenta}
\alias{Kmenta}
\docType{data}
\title{Partly Artificial Data on the U. S. Economy}
\description{
   These are partly contrived data from Kmenta (1986), constructed
   to illustrate estimation of a simultaneous-equation model.
}

\usage{
   data("Kmenta")
}

\format{
This data frame contains 20 annual observations of 5 variables:
\describe{
   \item{consump}{food consumption per capita.}
   \item{price}{ratio of food prices to general consumer prices.}
   \item{income}{disposable income in constant dollars.}
   \item{farmPrice}{ratio of preceding year's prices received by farmers
      to general consumer prices.}
   \item{trend}{time trend in years.}
}
}

\details{
   The exogenous variables \code{income}, \code{farmPrice},
   and \code{trend} are based on real data;
   the endogenous variables \code{price} and \code{consump}
   were generated by simulation.
}

\source{
   Kmenta (1986), Table 13-1, p. 687.
}

\references{
Kmenta, J. (1986). \emph{Elements of Econometrics}, Second Edition,
Macmillan, New York.
}

\examples{
## Replicating the estimations in Kmenta (1986), p. 712, Tab 13-2
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## OLS estimation
fitOls <- systemfit( system, data = Kmenta )
summary( fitOls )

## 2SLS estimation
fit2sls <- systemfit( system, "2SLS", inst = inst, data = Kmenta )
summary( fit2sls )

## 3SLS estimation
fit3sls <- systemfit( system, "3SLS", inst = inst, data = Kmenta )
summary( fit3sls )

## I3LS estimation
fitI3sls <- systemfit( system, "3SLS", inst = inst, data = Kmenta,
   maxit = 250 )
summary( fitI3sls )
}

\keyword{datasets}

%  $Id: nlsystemfit.Rd 548 2008-09-12 08:22:23Z henningsena $


\name{nlsystemfit}
\alias{nlsystemfit}

\title{Nonlinear Equation System Estimation}

\description{
  Fits a set of structural nonlinear equations using Ordinary Least
  Squares (OLS), Seemingly Unrelated Regression (SUR),
  Two-Stage Least Squares (2SLS), Three-Stage Least Squares (3SLS).
}


\usage{ nlsystemfit( method="OLS", eqns, startvals,
                     eqnlabels=c(as.character(1:length(eqns))), inst=NULL,
                     data=list(), solvtol=.Machine$double.eps,
                     maxiter=1000, ... )
}


\arguments{
  \item{method}{the estimation method, one of "OLS", "SUR", "2SLS",
    "3SLS".}
  \item{eqns}{a list of structural equations to be estimated.}
  \item{startvals}{a list of starting values for the coefficients.}
  \item{eqnlabels}{an optional list of character vectors of names
    for the equation labels.}
  \item{inst}{one-sided model formula specifying instrumental variables
    or a list of one-sided model formulas if different instruments should
    be used for the different equations (only needed for 2SLS, 3SLS and
    GMM estimations).}
  \item{data}{an optional data frame containing the variables in the model.
    By default the variables are taken from the environment from which
    nlsystemfit is called.}
  \item{solvtol}{tolerance for detecting linear dependencies in the columns
    of X in the \code{\link{qr}} function calls.}
  \item{maxiter}{the maximum number of iterations for the \code{\link{nlm}} function.}
  \item{...}{arguments passed to \code{\link{nlm}}.}
}

\details{

  The nlsystemfit function relies on \code{\link{nlm}} to perform the
  minimization of the objective functions and the \code{\link{qr}} set
  of functions.

  A system of nonlinear equations can be written as:

  \deqn{\epsilon_{t} = q( y_t, x_t, \theta )}
  \deqn{z_{t} = Z( x_t )}

  where \eqn{\epsilon_{t}} are the residuals from the y observations and
  the function evaluated at the coefficient estimates.

  The objective functions for the methods are:

  \tabular{lccc}{
    %    \hline
    Method \tab Instruments \tab Objective Function \tab Covariance of
    \eqn{\theta}\cr %\hline
    OLS \tab no \tab \eqn{r'r} \tab \eqn{(X(diag(S)^{-1}\bigotimes
      I)X)^{-1}}\cr %\hline
    SUR \tab no \tab \eqn{r'(diag(S)_{OLS}^{-1}\bigotimes I)r} \tab
    \eqn{(X(S^{-1}\bigotimes I)X)^{-1}}\cr %\hline
    2SLS \tab yes \tab \eqn{r'(I \bigotimes W)r} \tab
    \eqn{(X(diag(S)^{-1}\bigotimes I)X)^{-1}}\cr %\hline
    3SLS \tab yes \tab \eqn{r'(S_{2SLS}^{-1} \bigotimes W)r} \tab
    \eqn{(X(diag(S)^{-1}\bigotimes W)X)^{-1}} %\hline
  }

  where, r is a column vector for the residuals for each equation, S is
  variance-covariance matrix between the equations
  (\eqn{\hat{\sigma}_{ij} = (\hat{e}_i' \hat{e}_j) /
    \sqrt{(T - k_i)*(T - k_j)}}), X is matrix of the
  partial derivates with respect to the coefficients, W is a matrix of the
  instrument variables \eqn{Z(Z'Z)^{-1}Z}, Z is a matrix of the
  instrument variables, and I is an nxn identity matrix.

  The SUR and 3SLS methods requires two solutions. The first solution
  for the SUR is an OLS solution to obtain the variance-covariance
  matrix. The 3SLS uses the variance-covatiance from a 2SLS solution,
  then fits all the equations simultaneously.

  The user should be aware that the function is \bold{VERY} sensative to
  the starting values and the nlm function may not converge. The nlm
  function will be called with the \code{typsize} argument set the
  absolute values of the starting values for the OLS and 2SLS
  methods. For the SUR and 3SLS methods, the \code{typsize} argument is
  set to the absolute values of the resulting OLS and 2SLS coefficient
  estimates from the nlm result structre. In addition, the starting
  values for the SUR and 3SLS methods are obtained from the OLS and 2SLS
  coefficient estimates to shorten the number of iterations. The number of
  iterations reported in the summary are only those used in the last
  call to nlm, thus the number of iterations in the OLS portion of the
  SUR fit and the 2SLS portion of the 3SLS fit are not included.
}

\value{
  \code{nlsystemfit} returns a list of the class \code{nlsystemfit.system} and
  contains all results that belong to the whole system.
  This list contains one special object: "eq". It is a list and contains
  one object for each estimated equation. These objects are of the class
  \code{nlsystemfit.equation} and contain the results that belong only to the
  regarding equation.

  The objects of the class \code{nlsystemfit.system} and
  \code{nlsystemfit.equation} have the following components (the elements of
  the latter are marked with an asterisk (\eqn{*})):

  \item{eq}{a list object that contains a list object for each equation.}
  \item{method}{estimation method.}
  \item{resids}{an \eqn{n \times g} matrix of the residuals.}
  \item{g}{number of equations.}
  \item{n}{total number of observations.}
  \item{k}{total number of coefficients.}
  \item{b}{vector of all estimated coefficients.}
  \item{se}{estimated standard errors of \code{b}.}
  \item{t}{t values for \code{b}.}
  \item{p}{p values for \code{b}.}
  \item{bcov}{estimated covariance matrix of \code{b}.}
  \item{rcov}{estimated residual covariance matrix.}
  \item{drcov}{determinant of \code{rcov}.}
  \item{rcovest}{residual covariance matrix used for estimation (only SUR and 3SLS).}
  \item{rcor}{estimated residual correlation matrix.}
  \item{nlmest}{results from the nlm function call}
  \item{solvetol}{tolerance level when inverting a matrix or calculating a determinant.}

  ## elements of the class nlsystemfit.eq
  \item{eq}{a list that contains the results that belong to the individual equations.}
  \item{eqnlabel*}{the equation label of the ith equation (from the labels list).}
  \item{formula*}{model formula of the ith equation.}
  \item{n*}{number of observations of the ith equation.}
  \item{k*}{number of coefficients/regressors in the ith equation.}
  \item{df*}{degrees of freedom of the ith equation.}
  \item{b*}{estimated coefficients of the ith equation.}
  \item{se*}{estimated standard errors of \code{b}.}
  \item{t*}{t values for \code{b}.}
  \item{p*}{p values for \code{b}.}
  \item{covb*}{estimated covariance matrix of \code{b}.}
  \item{predicted*}{vector of predicted values of the ith equation.}
  \item{residuals*}{vector of residuals of the ith equation.}
  \item{ssr*}{sum of squared residuals of the ith equation.}
  \item{mse*}{estimated variance of the residuals (mean of squared errors) of the ith equation.}
  \item{s2*}{estimated variance of the residuals (\eqn{\hat{\sigma}^2}) of the ith equation.}
  \item{rmse*}{estimated standard error of the residulas (square root of mse) of the ith equation.}
  \item{s*}{estimated standard error of the residuals (\eqn{\hat{\sigma}}) of the ith equation.}
  \item{r2*}{R-squared (coefficient of determination).}
  \item{adjr2*}{adjusted R-squared value.}
}

\references{

  Gallant, R. H. (1987)
  \emph{Nonlinear Equation Estimation}, John Wiley and Sons, 610 pp.

  SAS Institute (1999)
  \emph{SAS/ETS User's Guide, Version 8}, Cary NC: SAS Institute 1546 pp.

}

\author{Jeff D. Hamann \email{jeff.hamann@forestinformatics.com} }

\seealso{\code{\link{systemfit}}, \code{\link{nlm}}, and \code{\link{qr}}}

\examples{
library( systemfit )
data( ppine )

hg.formula <- hg ~ exp( h0 + h1*log(tht) + h2*tht^2 + h3*elev + h4*cr)
dg.formula <- dg ~ exp( d0 + d1*log(dbh) + d2*hg + d3*cr + d4*ba  )
labels <- list( "height.growth", "diameter.growth" )
inst <- ~ tht + dbh + elev + cr + ba
start.values <- c(h0=-0.5, h1=0.5, h2=-0.001, h3=0.0001, h4=0.08,
                  d0=-0.5, d1=0.009, d2=0.25, d3=0.005, d4=-0.02 )
model <- list( hg.formula, dg.formula )

model.ols <- nlsystemfit( "OLS", model, start.values, data=ppine, eqnlabels=labels )
print( model.ols )

model.sur <- nlsystemfit( "SUR", model, start.values, data=ppine, eqnlabels=labels )
print( model.sur )

model.2sls <- nlsystemfit( "2SLS", model, start.values, data=ppine,
   eqnlabels=labels, inst=inst )
print( model.2sls )

model.3sls <- nlsystemfit( "3SLS", model, start.values, data=ppine,
                                    eqnlabels=labels, inst=inst )
print( model.3sls )
}

\keyword{models}
\keyword{regression}
\keyword{nonlinear}

%  $Id: correlation.systemfit.Rd 474 2007-07-27 07:22:28Z henningsena $

\name{correlation.systemfit}
\alias{correlation.systemfit}

\title{Correlation between Predictions from Equation i and j}

\description{
  \code{correlation} returns a vector of the correlations
  between the predictions of two equations in a set of equations. The
  correlation between the predictions is defined as,

  \deqn{
    r_{ijk} = \frac{x'_{ik}C_{ij}x_{jk}}{\sqrt{(x'_{ik}C_{ii}x_{ik})(x'_{jk}C_{jj}x_{jk})}}
  }

  where \eqn{r_{ijk}} is the correlation between the predicted values of
  equation i and j and  \eqn{C_{ij}} is the cross-equation variance-covariance
  matrix between equations i and j.

}

\usage{
correlation.systemfit( results, eqni, eqnj )
}

\arguments{
  \item{results}{an object of type \code{systemfit}.}
  \item{eqni}{index for equation i}
  \item{eqnj}{index for equation j}
}

\value{
  \code{correlation} returns a vector of the correlations between the
  predicted values in equation i and equation j.
}

\references{

  Greene, W. H. (1993)
  \emph{Econometric Analysis, Second Edition}, Macmillan.

  Hasenauer, H; Monserud, R and T. Gregoire. (1998)
  Using Simultansous Regression Techniques with Individual-Tree Growth Models.
  \emph{Forest Science}. 44(1):87-95

  Kmenta, J. (1997)
  \emph{Elements of Econometrics, Second Edition}, University of
  Michigan Publishing

}

\author{Jeff D. Hamann \email{jeff.hamann@forestinformatics.com}}

\seealso{\code{\link{systemfit}}}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform 2SLS on each of the equations in the system
fit2sls <- systemfit( system, "2SLS", inst = inst, data = Kmenta )
print( fit2sls )
print( fit2sls$rcov )

## perform the 3SLS
fit3sls <- systemfit( system, "3SLS", inst = inst, data = Kmenta )
print( fit3sls )
print( "covariance of residuals used for estimation (from 2sls)" )
print( fit3sls$rcovest )
print( "covariance of residuals" )
print( fit3sls$rcov )

## examine the correlation between the predicted values
## of suppy and demand by plotting the correlation over
## the value of q
r12 <- correlation.systemfit( fit3sls, 1, 2 )
plot( Kmenta$consump, r12, main="correlation between predictions from supply and demand" )
}

\keyword{models}

\name{systemfit.control}
\alias{systemfit.control}

\title{Create list of control parameters for systemfit}

\description{
   Create a list of control pararameters for function \code{\link{systemfit}}.
   All control parameters that are not passed to this function
   are set to default values.
}

\usage{
systemfit.control(
      maxiter = 1,
      tol = 1e-5,
      methodResidCov = "geomean",
      centerResiduals = FALSE,
      residCovRestricted = TRUE,
      residCovWeighted = FALSE,
      method3sls = "GLS",
      singleEqSigma = NULL,
      useMatrix = TRUE,
      solvetol = .Machine$double.eps,
      model = TRUE,
      x = FALSE,
      y = FALSE,
      z = FALSE )
}

\arguments{
   \item{maxiter}{maximum number of iterations for WLS, SUR, W2SLS and
      3SLS estimations.}
   \item{tol}{tolerance level indicating when to stop the iteration (only
      WLS, SUR, W2SLS and 3SLS estimations).}
   \item{methodResidCov}{method for calculating the estimated residual covariance
      matrix, one of "noDfCor", "geomean", "max", or "Theil" (see details).}
   \item{centerResiduals}{logical. Subtract the means from the residuals
      of each equation before calculating the estimated residual covariance matrix.}
   \item{residCovRestricted}{logical. If 'FALSE'
      the residual covariance matrix for a WLS, SUR, W2SLS, or 3SLS estimation
      is obtained from an unrestricted first-step estimation.}
   \item{residCovWeighted}{logical. If 'TRUE'
      the residual covariance matrix for a SUR or 3SLS estimation
      is obtained from a WLS or W2SLS estimation.}
   \item{method3sls}{method for calculating the 3SLS estimator,
      one of "GLS", "IV", "GMM", "Schmidt", or "EViews" (see details).}
   \item{singleEqSigma}{logical. use different \eqn{\sigma^2}s for each
      single equation to calculate the covariance matrix and the
      standard errors of the coefficients (only OLS and 2SLS)?
      If \code{singleEqSigma} is \code{NULL}, it is automatically determined:
      It is set to \code{TRUE}, if restrictions on the coefficients are imposed,
      and it is set to \code{FALSE} otherwise.}
   \item{useMatrix}{logical. Use package \code{Matrix} for matrix calculations?}
   \item{solvetol}{tolerance level for detecting linear dependencies
      when inverting a matrix or calculating a determinant (see
      \code{\link{solve}} and \code{\link{det}}).}
   \item{model, x, y, z}{
      logical. If 'TRUE' the corresponding components of the fit
      (the model frame, the model matrix, the response, and
      the matrix of instruments, respectively) are returned.}
}

\details{
   If the estimation is iterated
      (WLS, SUR, W2SLS or 3SLS estimation with \code{maxiter}>1),
      the convergence criterion is
   \deqn{\sqrt{ \frac{ \sum_i (b_{i,g} - b_{i,g-1})^2 }{ \sum_i b_{i,g-1}^2 }}
      < \code{tol}}
   (\eqn{b_{i,g}} is the ith coefficient of the gth iteration step).

   The method for calculating the estimated covariance matrix of the residuals
   (\eqn{\hat{\Sigma}}) can be one of the following
   (see Judge et al., 1985, p. 469): \cr
   if methodResidCov='noDfCor':
      \deqn{\hat{\sigma}_{ij} = \frac{\hat{e}_i' \hat{e}_j}{T}}
   if methodResidCov='geomean':
      \deqn{\hat{\sigma}_{ij} = \frac{\hat{e}_i' \hat{e}_j}
         {\sqrt{(T - k_i)*(T - k_j)}}}
   if methodResidCov='Theil':
      \deqn{\hat{\sigma}_{ij} = \frac{\hat{e}_i' \hat{e}_j}{T - k_i - k_j
      + tr[X_i(X_i'X_i)^{-1}X_i'X_j(X_j'X_j)^{-1}X_j']}}
   if methodResidCov='max':
      \deqn{\hat{\sigma}_{ij} = \frac{\hat{e}_i' \hat{e}_j}
         {T - \max( k_i, k_j)}}
   If \eqn{ i = j}, the formulas 'geomean', 'Theil', and 'max' are equal.
   All these three formulas yield unbiased estimators
   for the diagonal elements of the residual covariance matrix.
   If \eqn{i \neq j}, only formula 'Theil' yields an unbiased estimator for the residual
   covariance matrix, but it is not neccessarily positive semidefinit. Thus, it is
   doubtful whether formula 'Theil' is really superior to formula 'noDfCor'
   (Theil, 1971, p. 322).

   The methods for calculating the 3SLS estimator lead to identical results
   if the same instruments are used in all equations. If different instruments
   are used in the different equations, only the GMM-3SLS estimator ("GMM")
   and the 3SLS estimator proposed by Schmidt (1990) ("Schmidt") are consistent,
   whereas "GMM" is efficient relative to "Schmidt" (see Schmidt, 1990).

   If \code{residCovWeighted} is \code{TRUE},
   \code{\link{systemfit}} does a OLS or 2SLS estimation in a first step.
   It uses the residuals from the first-step estimation
   to calculate the residual covariance matrix
   that is used in a second-step WLS or W2SLS estimation.
   Then, it uses the residuals from the second-step estimation
   to calculate the residual covariance matrix
   that is used in a final SUR or 3SLS estimation.
   This three-step method is the default method of command "TSCS"
   in the software LIMDEP that carries out "SUR" estimations
   in which all coefficient vectors are constrained to be equal
   (personal information from W.H. Greene, 2006/02/16).
   If no cross-equation restrictions are imposed,
   \code{residCovWeighted} has no effect on the estimation results.
}

\value{
   A list of the above components.
}

\references{
  Judge, George G.; W. E. Griffiths; R. Carter Hill; Helmut Luetkepohl and Tsoung-Chao Lee (1985)
  \emph{The Theory and Practice of Econometrics, Second Edition}, Wiley.

  Schmidt, P. (1990)
  \emph{Three-Stage Least Squares with different Instruments for different equations},
  Journal of Econometrics 43, p. 389-394.

  Theil, H. (1971)
  \emph{Principles of Econometrics}, Wiley, New York.
}

\author{
  Arne Henningsen \email{arne.henningsen@googlemail.com}
}

\seealso{\code{\link{systemfit}}}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
eqSystem <- list( demand = eqDemand, supply = eqSupply )

## SUR estimation: calculation of residual covariance
## matrix without correction for degrees of freedom
fitsur <- systemfit( eqSystem, "SUR", data = Kmenta,
    control = systemfit.control( methodResidCov = "noDfCor" ) )
print( fitsur )
}

\keyword{models}
\keyword{regression}

\name{residuals.systemfit}
\alias{residuals.systemfit}
\alias{residuals.systemfit.equation}
\title{Residuals of systemfit object}

\description{
   These functions extract the residuals
   from an object returned by
   \code{\link{systemfit}}.
}

\usage{
   \method{residuals}{systemfit}( object, \dots )

   \method{residuals}{systemfit.equation}( object, na.rm = FALSE, \dots )
}

\arguments{
   \item{object}{an object of class \code{systemfit} or
      \code{systemfit.equation}.}
   \item{na.rm}{a logical value indicating whether \code{NA} values
      (corresponding to observations that were not included in the estimation)
      should be removed from the vector of residuals
      before it is returned.}
   \item{\dots}{other arguments.}
}

\value{
   \code{residuals.systemfit} returns a data.frame of residuals,
   where each column contains the residuals of one equation.

   \code{residuals.systemfit.equation} returns a vector of residuals.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}}, \code{\link{residuals}}
}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS on each of the equations in the system
fitols <- systemfit( system, data = Kmenta )

## residuals of all equations
residuals( fitols )

## residuals of the first equation
residuals( fitols$eq[[1]] )

## residuals of the second equation
residuals( fitols$eq[[2]] )
}

\keyword{models}

\name{estfun.systemfit}
\alias{estfun.systemfit}

\title{Extract Gradients of the Objective Function at each Observation}

\description{
   Extract the gradients of the objective function 
   with respect to the coefficients 
   evaluated at each observation 
   (\sQuote{Empirical Estimating Function}, 
   see \code{\link[sandwich]{estfun}}).
}

\usage{
\method{estfun}{systemfit}( x, residFit = TRUE, ... )
}

\arguments{
   \item{x}{an object of class \code{systemfit}.}
   \item{residFit}{logical. 
      If \code{FALSE}, the residuals are calculated based on observed regressors.
      If \code{TRUE}, the residuals are calculated based on fitted regressors.
      This argument is ignored if no instrumental variable are used.}
   \item{\dots}{further arguments (currently ignored).}
}

\value{
   Matrix of gradients of the objective function
   with respect to the coefficients
   evaluated at each observation.
}

\section{Warnings}{
   The \pkg{sandwich} package must be loaded before this method
   can be used.
   
   In specific estimations with the 3SLS method,
   not all columns of the matrix returned by the \code{estfun} method
   sum up to zero,
   which indicates that an inappropriate estimating function is returned.
   This can be either with argument \code{residFit} set to \code{TRUE} 
   or with this argument set to \code{FALSE}
   or even in both cases.
   This problem depends on the formula used for the 3SLS estimation
   and seems to be related to unbalanced systems and
   systems where different instruments are used in different equations.
}

\author{
   Arne Henningsen
}

\seealso{\code{\link[sandwich]{estfun}}, \code{\link{systemfit}}.}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )
inst <- ~ income + farmPrice + trend

## OLS estimation
fitols <- systemfit( system, "OLS", data = Kmenta )

## obtain the estimation function
library( "sandwich" )
estfun( fitols )

## this is only true for OLS models
all.equal( estfun( fitols ),
   unlist( residuals( fitols ) ) * model.matrix( fitols ) )

# each column should sum up to (approximately) zero
colSums( estfun( fitols ) )


## 2SLS estimation
fit2sls <- systemfit( system, "2SLS", inst = inst, data = Kmenta )

## obtain the estimation function
estfun( fit2sls )

## this is only true for 2SLS models
all.equal( estfun( fit2sls ),
   drop( rep( Kmenta$consump, 2 ) -  model.matrix( fit2sls, which = "xHat" ) \%*\% 
   coef( fit2sls ) ) * model.matrix( fit2sls, which = "xHat" ) )
all.equal( estfun( fit2sls, residFit = FALSE ),
   unlist( residuals( fit2sls ) ) * model.matrix( fit2sls, which = "xHat" ) )

# each column should sum up to (approximately) zero
colSums( estfun( fit2sls ) )
colSums( estfun( fit2sls, residFit = FALSE ) )


## iterated SUR estimation
fitsur <- systemfit( system, "SUR", data = Kmenta, maxit = 100 )

## obtain the estimation function
estfun( fitsur )

## this should be true for SUR and WLS models
all.equal( estfun( fitsur ),
   unlist( residuals( fitsur ) ) * 
   ( ( solve( fitsur$residCovEst ) \%x\% diag( nrow( Kmenta ) ) ) \%*\% 
      model.matrix( fitsur ) ), check.attributes = FALSE )

# each column should sum up to (approximately) zero
colSums( estfun( fitsur ) )


## 3SLS estimation
fit3sls <- systemfit( system, "3SLS", inst = inst, data = Kmenta )

## obtain the estimation function
estfun( fit3sls )
estfun( fit3sls, residFit = FALSE )

## this should be true for 3SLS and W2SLS models
all.equal( estfun( fit3sls ),
   drop( rep( Kmenta$consump, 2 ) -  
   model.matrix( fit2sls, which = "xHat" ) \%*\% coef( fit3sls ) ) * 
   ( ( solve( fit3sls$residCovEst ) \%x\% diag( nrow( Kmenta ) ) ) \%*\% 
      model.matrix( fit3sls, which = "xHat" ) ), check.attributes = FALSE )
all.equal( estfun( fit3sls, residFit = FALSE ),
   unlist( residuals( fit3sls ) ) * 
   ( ( solve( fit3sls$residCovEst ) \%x\% diag( nrow( Kmenta ) ) ) \%*\% 
      model.matrix( fit3sls, which = "xHat" ) ), check.attributes = FALSE )

# each column should sum up to (approximately) zero
colSums( estfun( fit3sls ) )
colSums( estfun( fit3sls, residFit = FALSE ) )
}

\keyword{methods}

\name{coef.systemfit}
\alias{coef.systemfit}
\alias{coef.systemfit.equation}
\alias{coef.summary.systemfit}
\alias{coef.summary.systemfit.equation}
\title{Coefficients of systemfit object}

\description{
   These functions extract the coefficients from an object returned by
   \code{\link{systemfit}}.
}

\usage{
   \method{coef}{systemfit}( object, modified.regMat = FALSE, \dots )

   \method{coef}{systemfit.equation}( object, \dots )

   \method{coef}{summary.systemfit}( object, modified.regMat = FALSE, \dots )

   \method{coef}{summary.systemfit.equation}( object, \dots )
}

\arguments{
   \item{object}{an object of class \code{systemfit},
      \code{systemfit.equation}, \code{summary.systemfit}, or
      \code{summary.systemfit.equation}.}
   \item{modified.regMat}{logical. If \code{TRUE},
      the coefficients of the modified regressor matrix
      (original regressor matrix post-multiplied by \code{restrict.regMat})
      rather than the coefficients of the original regressor matrix
      are returned.}
   \item{\dots}{other arguments.}
}

\value{
   \code{coef.systemfit} returns a vector of all estimated coefficients.

   \code{coef.systemfit.equation} returns a vector of the estimated coefficients
      of a single equation.

   \code{coef.summary.systemfit} returns a matrix of all estimated coefficients,
      their standard errors, t-values, and p-values.

   \code{coef.summary.systemfit.equation} returns a matrix of the estimated
      coefficients of a single equation, their standard errors, t-values,
      and p-values.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}}, \code{\link{coef}}
}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS on each of the equations in the system
fitols <- systemfit( system, data = Kmenta )

## all coefficients
coef( fitols )
coef( summary( fitols ) )

## coefficients of the first equation
coef( fitols$eq[[1]] )
coef( summary( fitols$eq[[1]] ) )

## coefficients of the second equation
coef( fitols$eq[[2]] )
coef( summary( fitols$eq[[2]] ) )

## estimation with restriction by modifying the regressor matrix
modReg <- matrix( 0, 7, 6 )
colnames( modReg ) <- c( "demIntercept", "demPrice", "demIncome",
   "supIntercept", "supPrice2", "supTrend" )
modReg[ 1, "demIntercept" ] <- 1
modReg[ 2, "demPrice" ]     <- 1
modReg[ 3, "demIncome" ]    <- 1
modReg[ 4, "supIntercept" ] <- 1
modReg[ 5, "supPrice2" ]    <- 1
modReg[ 6, "supPrice2" ]    <- 1
modReg[ 7, "supTrend" ]     <- 1
fitols3 <- systemfit( system, data = Kmenta, restrict.regMat = modReg )
coef( fitols3, modified.regMat  = TRUE )
coef( fitols3 )
}

\keyword{models}

\name{fitted.systemfit}
\alias{fitted.systemfit}
\alias{fitted.systemfit.equation}
\title{Fitted values}

\description{
   These functions extract the fitted values
   from an object returned by \code{\link{systemfit}}.
}

\usage{
   \method{fitted}{systemfit}( object, \dots )

   \method{fitted}{systemfit.equation}( object, na.rm = FALSE, \dots )
}

\arguments{
   \item{object}{an object of class \code{systemfit} or
      \code{systemfit.equation}.}
   \item{na.rm}{a logical value indicating whether \code{NA} values
      (corresponding to observations that were not included in the estimation)
      should be removed from the vector of fitted values
      before it is returned.}
   \item{\dots}{other arguments.}
}

\value{
   \code{fitted.systemfit} returns a data.frame of all fitted values,
   where each column contains the fitted values of one equation.

   \code{fitted.systemfit.equation} returns a vector of the fitted values
      of a single equation.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}}, \code{\link{fitted}}
}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS on each of the equations in the system
fitols <- systemfit( system, data = Kmenta )

## all fitted values
fitted( fitols )

## fitted values of the first equation
fitted( fitols$eq[[1]] )

## fitted values of the second equation
fitted( fitols$eq[[2]] )
}

\keyword{models}

\name{bread.systemfit}
\alias{bread.systemfit}

\title{Bread for Sandwiches}

\description{
   Extract the estimator for the bread of sandwhiches
   (see \code{\link[sandwich]{bread}}).
}

\usage{
\method{bread}{systemfit}( x, ... )
}

\arguments{
   \item{x}{an object of class \code{systemfit}.}
   \item{\dots}{further arguments (currently ignored).}
}

\value{
   Quadratic symmetric matrix,
   which is an estimator for the expectation of the negative derivative 
   of the estimating function (see \code{\link{estfun.systemfit}}).
}

\section{Warnings}{
   The \pkg{sandwich} package must be loaded before this method
   can be used.
   
   This method might not be suitable for specific formulas 
   for 3SLS estimations in case of unbalanced systems
   or different instruments for different equations.
}

\author{
   Arne Henningsen
}

\seealso{\code{\link[sandwich]{bread}}, \code{\link{systemfit}}.}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )
inst <- ~ income + farmPrice + trend

## OLS estimation
fitols <- systemfit( system, "OLS", data = Kmenta )

## obtain the bread
library( "sandwich" )
bread( fitols )

## this is only true for OLS models
all.equal( bread( fitols ),
   solve( crossprod( model.matrix( fitols ) ) / 40 ) )


## 2SLS estimation
fit2sls <- systemfit( system, "2SLS", inst = inst, data = Kmenta )

## obtain the bread
bread( fit2sls )

## this is only true for 2SLS models
all.equal( bread( fit2sls ),
   solve( crossprod( model.matrix( fit2sls, which = "xHat" ) ) / 40 ) )


## iterated SUR estimation
fitsur <- systemfit( system, "SUR", data = Kmenta, maxit = 100 )

## obtain the bread
bread( fitsur )

## this should be true for SUR and WLS models
all.equal( bread( fitsur ),
   solve( t( model.matrix( fitsur ) ) \%*\%
      (  ( solve( fitsur$residCovEst ) \%x\% diag( nrow( Kmenta ) ) ) \%*\% 
      model.matrix( fitsur ) ) / 40 ), check.attributes = FALSE )


## 3SLS estimation
fit3sls <- systemfit( system, "3SLS", inst = inst, data = Kmenta )

## obtain the bread
bread( fit3sls )

## this should be true for 3SLS and W2SLS models
all.equal( bread( fit3sls ),
   solve( t( model.matrix( fit3sls, which = "xHat" ) ) \%*\%
      (  ( solve( fit3sls$residCovEst ) \%x\% diag( nrow( Kmenta ) ) ) \%*\% 
      model.matrix( fit3sls, which = "xHat" ) ) / 40 ), check.attributes = FALSE )
}

\keyword{methods}

\name{vcov.systemfit}
\alias{vcov.systemfit}
\alias{vcov.systemfit.equation}
\title{Variance covariance matrix of coefficients}

\description{
   These functions extract the variance covariance matrix of the
   coefficients from an object returned by
   \code{\link{systemfit}}.
}

\usage{
   \method{vcov}{systemfit}( object, modified.regMat = FALSE, \dots )

   \method{vcov}{systemfit.equation}( object, \dots )
}

\arguments{
   \item{object}{an object of class \code{systemfit} or
      \code{systemfit.equation}.}
   \item{modified.regMat}{logical. If \code{TRUE},
      the covariance matrix of the coefficients of the modified regressor matrix
      (original regressor matrix post-multiplied by \code{restrict.regMat})
      rather than the covariance matrix of the coefficients
      of the original regressor matrix
      is returned.}
   \item{\dots}{other arguments.}
}

\value{
   \code{vcov.systemfit} returns the variance covariance matrix
      of all estimated coefficients.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}}, \code{\link{vcov}}
}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS on each of the equations in the system
fitols <- systemfit( system, data = Kmenta )

## variance covariance matrix of all coefficients
vcov( fitols )

## variance covariance matrix of the coefficients in the first equation
vcov( fitols$eq[[1]] )

## variance covariance matrix of the coefficients in the second equation
vcov( fitols$eq[[2]] )

## estimation with restriction by modifying the regressor matrix
modReg <- matrix( 0, 7, 6 )
colnames( modReg ) <- c( "demIntercept", "demPrice", "demIncome",
   "supIntercept", "supPrice2", "supTrend" )
modReg[ 1, "demIntercept" ] <- 1
modReg[ 2, "demPrice" ]     <- 1
modReg[ 3, "demIncome" ]    <- 1
modReg[ 4, "supIntercept" ] <- 1
modReg[ 5, "supPrice2" ]    <- 1
modReg[ 6, "supPrice2" ]    <- 1
modReg[ 7, "supTrend" ]     <- 1
fitsur <- systemfit( system, "SUR", data = Kmenta, restrict.regMat = modReg )
vcov( fitsur, modified.regMat  = TRUE )
vcov( fitsur )
}

\keyword{models}

% 	$Id: systemfit-internal.Rd 365 2007-06-08 08:23:42Z henningsena $	

\name{systemfit-internal}
\alias{knls}

\title{Internal systemfit functions}
\description{
  Internal systemfit functions
}
\usage{
knls( theta, eqns, data, fitmethod="OLS", parmnames, instr=NULL, S=NULL )
}

\details{
  These functions are no the called by the user.
  The knls is the driver function that is passed to the nlm call in
  \code{\link{nlsystemfit}}. 
}

%\seealso{\code{\link{nlm}},\code{\link{qr}}, and \code{\link{systemfit}} }


\keyword{internal}

\name{predict.systemfit}
\alias{predict.systemfit}
\alias{predict.systemfit.equation}

\title{Predictions from System Estimation}

\description{
   Returns the predicted values, their standard errors and the
   confidence limits of prediction.
}

\usage{
\method{predict}{systemfit}( object, newdata = NULL,
                             se.fit = FALSE, se.pred = FALSE,
                             interval = "none", level=0.95,
                             useDfSys = NULL, ... )

\method{predict}{systemfit.equation}( object, newdata = NULL,
                             se.fit = FALSE, se.pred = FALSE,
                             interval = "none", level=0.95,
                             useDfSys = NULL, ... )
}

\arguments{
   \item{object}{an object of class \code{systemfit} or
      \code{systemfit.equation}.}
   \item{newdata}{An optional data frame in which to look for variables with
      which to predict.  If it is \code{NULL}, the fitted values are returned.}
   \item{se.fit}{return the standard error of the fitted values?}
   \item{se.pred}{return the standard error of prediction?}
   \item{interval}{Type of interval calculation ("none", "confidence"
      or "prediction")}
   \item{level}{Tolerance/confidence level.}
   \item{useDfSys}{logical. Use the degrees of freedom of the whole system
      (in place of the degrees of freedom of the single equation)
      to calculate the confidence or prediction intervals.
      If it not specified (\code{NULL}), it is set to \code{TRUE}
      if restrictions on the coefficients are imposed and \code{FALSE} otherwise.}
   \item{...}{additional optional arguments.}
}

\details{
   The variance of the fitted values
   (used to calculate the standard errors of the fitted values
   and the "confidence interval") is calculated by
   \eqn{Var[E[y^0]-\hat{y}^0]=x^0 \; Var[b] \; {x^0}'}\cr
   The variances of the predicted values
   (used to calculate the standard errors of the predicted values
   and the "prediction intervals") is calculated by
   \eqn{Var[y^0-\hat{y}^0]=\hat{\sigma}^2+x^0 \; Var[b] \; {x^0}'}
}

\value{
   \code{predict.systemfit} returns a dataframe that
   contains for each equation the predicted values
   ("<eqnLable>.pred") and if requested
   the standard errors of the fitted values ("<eqnLable>.se.fit"),
   the standard errors of the prediction ("<eqnLable>.se.pred"),
   and the lower ("<eqnLable>.lwr") and upper ("<eqnLable>.upr")
   limits of the confidence or prediction interval(s).

   \code{predict.systemfit.equation} returns a dataframe that
   contains the predicted values ("fit") and if requested
   the standard errors of the fitted values ("se.fit"),
   the standard errors of the prediction ("se.pred"),
   and the lower ("lwr") and upper ("upr")
   limits of the confidence or prediction interval(s).
}

\references{

  Greene, W. H. (2003)
  \emph{Econometric Analysis, Fifth Edition}, Macmillan.

  Gujarati, D. N. (1995)
  \emph{Basic Econometrics, Third Edition}, McGraw-Hill.

  Kmenta, J. (1997)
  \emph{Elements of Econometrics, Second Edition}, University of
  Michigan Publishing.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}


\seealso{
   \code{\link{systemfit}}, \code{\link{predict}}
}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## OLS estimation
fitols <- systemfit( system, data=Kmenta )

## predicted values and limits
predict( fitols )

## predicted values of the first equation
predict( fitols$eq[[1]] )

## predicted values of the second equation
predict( fitols$eq[[2]] )
}

\keyword{models}

\name{model.frame.systemfit}
\alias{model.frame.systemfit}
\alias{model.frame.systemfit.equation}
\title{Extracting the Data of a systemfit Object}

\description{
   These functions return the data used by \code{\link{systemfit}}
   to estimate a system of equations.
}

\usage{
   \method{model.frame}{systemfit}( formula, \dots )

   \method{model.frame}{systemfit.equation}( formula, \dots )
}

\arguments{
   \item{formula}{an object of class \code{systemfit} or
      \code{systemfit.equation}.}
   \item{\dots}{currently ignored.}
}

\value{
   \code{model.frame.systemfit} returns a simple data frame
      (without a 'terms' attribute) that contains all variables
      used to estimate the entire system of equations.

   \code{model.frame.systemfit.equation} returns a model frame
      (data frame with a 'terms' attribute) that contains
      all variables used to estimate the respective equation.
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{
   \code{\link{systemfit}}, \code{\link{model.frame}}, and
   \code{\link{model.matrix.systemfit}}
}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS of the system
fitols <- systemfit( system, data = Kmenta )

## data used to estimate the entire system
model.frame( fitols )

## data used to estimate the first equation
model.frame( fitols$eq[[ 1 ]] )
}

\keyword{models}

%  $Id: summary.nlsystemfit.system.Rd 154 2006-02-14 14:15:11Z henningsena $

\name{summary.nlsystemfit}
\alias{summary.nlsystemfit.system}
\alias{summary.nlsystemfit.equation}
\title{Summary of nlsystemfit estimation}

\description{
   These functions print a summary of the estimated equation system.
}

\usage{
   \method{summary}{nlsystemfit.system}( object, ... )

   \method{summary}{nlsystemfit.equation}( object, ... )
}

\arguments{
   \item{object}{an object of class \code{nlsystemfit.system} or
      \code{nlsystemfit.equation}.}
   \item{...}{not used by user.}
}

\author{Jeff D. Hamann \email{jeff.hamann@forestinformatics.com}}

\seealso{\code{\link{nlsystemfit}}, \code{\link{print.nlsystemfit.system}}}

\examples{
library( systemfit )
data( ppine )

hg.formula <- hg ~ exp( h0 + h1*log(tht) + h2*tht^2 + h3*elev + h4*cr)
dg.formula <- dg ~ exp( d0 + d1*log(dbh) + d2*hg + d3*cr + d4*ba  )
labels <- list( "height.growth", "diameter.growth" )
inst <- ~ tht + dbh + elev + cr + ba
start.values <- c(h0=-0.5, h1=0.5, h2=-0.001, h3=0.0001, h4=0.08,
                  d0=-0.5, d1=0.009, d2=0.25, d3=0.005, d4=-0.02 )
model <- list( hg.formula, dg.formula )

model.ols <- nlsystemfit( "OLS", model, start.values, data=ppine, eqnlabels=labels )
print( model.ols )

model.3sls <- nlsystemfit( "3SLS", model, start.values, data=ppine,
                                    eqnlabels=labels, inst=inst )
print( model.3sls )
}

\keyword{models}
\keyword{regression}
\keyword{nonlinear}

%  $Id: se.ratio.systemfit.Rd 437 2007-06-30 07:44:12Z henningsena $


\name{se.ratio.systemfit}
\alias{se.ratio.systemfit}

\title{Ratio of the Standard Errors}

\description{
  \code{se.ratio.systemfit} returns a vector of the ratios of the
  standard errors of the predictions for two equations.
}

\usage{
se.ratio.systemfit( resultsi, resultsj, eqni )
}

\arguments{
  \item{resultsi}{an object of type \code{systemfit}.}
  \item{resultsj}{an object of type \code{systemfit}.}
  \item{eqni}{index for equation to obtain the ratio of standard errors.}
}

\value{
  \code{se.ratio} returns a vector of the standard errors of the ratios
  for the predictions between the predicted values in equation i and
  equation j.

}

\references{

  Hasenauer, H; Monserud, R and T. Gregoire. (1998)
  Using Simultaneous Regression Techniques with Individual-Tree Growth Models.
  \emph{Forest Science}. 44(1):87-95

}

\author{Jeff D. Hamann \email{jeff.hamann@forestinformatics.com}}

\seealso{\code{\link{systemfit}} and \code{\link{correlation.systemfit}}}


\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform 2SLS on each of the equations in the system
fit2sls <- systemfit( system, "2SLS", inst = inst, data = Kmenta )
fit3sls <- systemfit( system, "3SLS", inst = inst, data = Kmenta )

## print the results from the fits
print( fit2sls )
print( fit3sls )
print( "covariance of residuals used for estimation (from 2sls)" )
print( fit3sls$residCovEst )
print( "covariance of residuals" )
print( fit3sls$residCov )

## examine the improvement of 3SLS over 2SLS by computing
## the ratio of the standard errors of the estimates
improve.ratio <- se.ratio.systemfit( fit2sls, fit3sls, 2 )
print( "summary values for the ratio in the std. err. for the predictions" )
print( summary( improve.ratio ) )
}
\keyword{models}

\name{print.confint.systemfit}
\alias{print.confint.systemfit}
\title{Print confidence intervals of coefficients}

\description{
  This function prints the confidence intervals of the coefficients
  of the estimated equation system.
}

\usage{
   \method{print}{confint.systemfit}( x, digits=3, \dots )
}

\arguments{
  \item{x}{an object of type \code{confint.systemfit}.}
  \item{digits}{number of digits to print.}
  \item{\dots}{other arguments.}
}

\author{Arne Henningsen \email{arne.henningsen@googlemail.com}}

\seealso{\code{\link{systemfit}}, \code{\link{confint.systemfit}}
  and  \code{\link{confint.systemfit.equation}}}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform OLS on each of the equations in the system
fitols <- systemfit( system, data = Kmenta )

## calculate and print the confidence intervals
## of all coefficients
ci <- confint( fitols )
print( ci, digits=4 )

## calculate and print the confidence intervals
## of the coefficients of the second equation
ci2 <- confint( fitols$eq[[2]] )
print( ci2, digits=4 )
}

\keyword{models}
\keyword{regression}

%  $Id: hausman.systemfit.Rd 986 2008-10-16 07:57:24Z arne $

\name{hausman.systemfit}
\alias{hausman.systemfit}

\title{Hausman Test}

\description{
  \code{hausman.systemfit} returns the Hausman statistic for a specification test.
}

\usage{
   hausman.systemfit( results2sls, results3sls )
}

\arguments{
  \item{ results2sls }{result of a \emph{2SLS} (limited information) estimation
     returned by \code{\link{systemfit}}.}
  \item{ results3sls }{result of a \emph{3SLS} (full information) estimation
     returned by \code{\link{systemfit}}.}
}

\details{
   The null hypotheses of the test is that all exogenous variables are
   uncorrelated with all disturbance terms.
   Under this hypothesis both the 2SLS and the 3SLS estimator are consistent
   but only the 3SLS estimator is (asymptotically) efficient.
   Under the alternative hypothesis the 2SLS estimator is consistent
   but the 3SLS estimator is inconsistent.

   The Hausman test statistic is

   \deqn{m = ( b_2 - b_3 )' ( V_2 - V_3 ) ( b_2 - b_3 ) }

   where $b_2$ and $V_2$ are the estimated coefficients and their variance
   covariance matrix of a \emph{2SLS} estimation and
   $b_3$ and $V_3$ are the estimated coefficients and their variance
   covariance matrix of a \emph{3SLS} estimation.
}

\value{
   \code{hausman.systemfit} returns a list of the class
    \code{htest} that contains following elements:
   \item{q}{vector of the differences between the estimated coefficients.}
   \item{qVar}{variance covariance matrix of \code{q} (difference between the
     variance covariance matrices of the estimated coefficients).}
   \item{statistic}{the Hausman test statistic.}
   \item{parameter}{degrees of freedom.}
   \item{p.value}{P-value of the test.}
   \item{method}{character string describing this test.}
   \item{data.name}{name of the data.frame used for estimation.}
}

\references{

  Greene, W. H. (1993)
  \emph{Econometric Analysis, Fifth Edition}, Macmillan.

  Hausman, J. A. (1978)
  Specification Tests in Econometrics. \emph{Econometrica}. 46:1251-1271.

  Kmenta, J. (1997)
  \emph{Elements of Econometrics, Second Edition}, University of
  Michigan Publishing

}

\author{Jeff D. Hamann \email{jeff.hamann@forestinformatics.com},\cr
  Arne Henningsen \email{arne.henningsen@googlemail.com}
}

\seealso{\code{\link{systemfit}}}

\examples{
data( "Kmenta" )
eqDemand <- consump ~ price + income
eqSupply <- consump ~ price + farmPrice + trend
inst <- ~ income + farmPrice + trend
system <- list( demand = eqDemand, supply = eqSupply )

## perform the estimations
fit2sls <- systemfit( system, "2SLS", inst = inst, data = Kmenta )
fit3sls <- systemfit( system, "3SLS", inst = inst, data = Kmenta )

## perform the Hausman test
h <- hausman.systemfit( fit2sls, fit3sls )
print( h )
}

\keyword{models}
